/*
 * @(#)StandardAktionen.java   03.11.11
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.aktionen;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;
import de.inovat.buv.inovat.lib.gui.widget.DatumZeitOhneSekunden;
import de.inovat.buv.inovat.lib.gui.widget.ZeitOhneSekunden;

import org.eclipse.nebula.paperclips.core.BigPrint;
import org.eclipse.nebula.paperclips.core.border.LineBorder;
import org.eclipse.nebula.paperclips.core.grid.CellBackgroundProvider;
import org.eclipse.nebula.paperclips.core.grid.DefaultGridLook;
import org.eclipse.nebula.paperclips.core.grid.GridColumn;
import org.eclipse.nebula.paperclips.core.grid.GridPrint;
import org.eclipse.nebula.paperclips.core.ImagePrint;
import org.eclipse.nebula.paperclips.core.page.PagePrint;
import org.eclipse.nebula.paperclips.core.Print;
import org.eclipse.nebula.paperclips.core.ScalePrint;
import org.eclipse.nebula.paperclips.core.text.LineBreakPrint;
import org.eclipse.nebula.paperclips.core.text.TextPrint;
import org.eclipse.nebula.paperclips.core.text.TextStyle;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.printing.Printer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.forms.widgets.Section;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse stellt die statischen Methoden fr die Erstellung der verschiedenen
 * Druckauftrge zu Verfgung. Das Ergebnis kann im Druckvorschau angezeigt werden.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 13 $ / $Date: 2013-09-24 14:02:53 +0200 (Di, 24 Sep 2013) $ / ($Author: LG $)
 */
public class DruckAuftrag {
    public static final FontData FONT_TITEL         = new FontData("Arial", 14, SWT.BOLD);
    public static final FontData FONT_TABELLE_TITEL = new FontData("Arial", 10, SWT.BOLD);
    public static final FontData FONT_TABELLE       = new FontData("Arial", 10, SWT.NORMAL);
    public static final FontData FONT_LEGENDE       = new FontData("Arial", 12, SWT.BOLD);

    /** Maximale Anzahl der Spalten fr eine Seite. */
    private static final int MAX_SPALTEN_ANZAHL_PRO_SEITE = 12;
    public static final RGB  RGB_WEIS                     = new RGB(255, 255, 255);
    public static final RGB  RGB_SCHWARZ                  = new RGB(0, 0, 0);
    public static final RGB  RGB_GRAU                     = new RGB(200, 200, 200);

    //~ METHODEN ==============================================================

    /**
     * Ermittle alle Knder des BaumElements rekursiv.
     *
     * @param item BaumElement.
     *
     * @return Alle Knder des BaumElements rekursiv.
     */
    private static List<TreeItem> ermittleBaumAlleKinder(TreeItem item) {
        List<TreeItem> listeKinder = new ArrayList<TreeItem>();

        //
        for (TreeItem kind : item.getItems()) {
            listeKinder.add(kind);

            // Rekursiv:
            if (kind.getItemCount() > 0) {
                listeKinder.addAll(ermittleBaumAlleKinder(kind));
            }
        }

        return listeKinder;
    }

    /**
     * Ermittle den Druck-Auftrag fr die GUI-Tabelle.
     * Es werden Titel, Legende und Tabelle im Dokument erstellt.
     * </p>
     * Legende besteht aus maximum zwei Tabellen: erste ist ohne Farbe,
     * die zweite ist mit der Farbe dargestellt.
     * </p>
     * @param titel Titel des Dokuments.
     * @param anzSpaltenLegendeOhneFarbe Anzahl der Spalten in der Legende ohne die Farbe.
     * @param listeLegendeOhneFarbe Liste der Legende ohne die Farbe. Die Elemente werden in der
     * tabellarischen Form dargestellt. Liste kann null enthalten (wird leeres Element dargestellt).
     * Falls der Parameter gleich null ist, wird die Tabelle nicht erstellt.
     * @param anzSpaltenLegendeMitFarbe Anzahl der Spalten in der Legende mit der Farbe.
     * @param listeLegendeMitFarbe Liste der Legende mit der Farbe. Die Elemente werden in der
     * tabellarischen Form dargestellt. Die Farbe wird von der 1.Spalte bernommen.
     * Liste kann null enthalten (wird leeres Element dargestellt).
     * Falls der Parameter gleich null ist, wird die Tabelle nicht erstellt.
     *
     * @return Druck-Auftrag fr die GUI-Tabelle.
     */
    public static PagePrint ermittleDruckAuftrag(String titel, int anzSpaltenLegendeOhneFarbe, List<Control> listeLegendeOhneFarbe, int anzSpaltenLegendeMitFarbe, List<Control> listeLegendeMitFarbe) {
        return ermittleDruckAuftragAllgemein(titel, anzSpaltenLegendeOhneFarbe, listeLegendeOhneFarbe, anzSpaltenLegendeMitFarbe, listeLegendeMitFarbe, null);
    }

    /**
     * Ermittle den Druck-Auftrag fr die GUI-Tabelle.
     * Es werden Titel, Legende und Tabelle im Dokument erstellt.
     * </p>
     * Legende besteht aus maximum zwei Tabellen: erste ist ohne Farbe,
     * die zweite ist mit der Farbe dargestellt.
     * </p>
     * @param titel Titel des Dokuments.
     * @param anzSpaltenLegendeOhneFarbe Anzahl der Spalten in der Legende ohne die Farbe.
     * @param listeLegendeOhneFarbe Liste der Legende ohne die Farbe. Die Elemente werden in der
     * tabellarischen Form dargestellt. Liste kann null enthalten (wird leeres Element dargestellt).
     * Falls der Parameter gleich null ist, wird die Tabelle nicht erstellt.
     * @param anzSpaltenLegendeMitFarbe Anzahl der Spalten in der Legende mit der Farbe.
     * @param listeLegendeMitFarbe Liste der Legende mit der Farbe. Die Elemente werden in der
     * tabellarischen Form dargestellt. Die Farbe wird von der 1.Spalte bernommen.
     * Liste kann null enthalten (wird leeres Element dargestellt).
     * Falls der Parameter gleich null ist, wird die Tabelle nicht erstellt.
     * @param tabelleGui SWT-Tabelle.
     *
     * @return Druck-Auftrag fr die GUI-Tabelle.
     */
    public static PagePrint ermittleDruckAuftrag(String titel, int anzSpaltenLegendeOhneFarbe, List<Control> listeLegendeOhneFarbe, int anzSpaltenLegendeMitFarbe, List<Control> listeLegendeMitFarbe, final Table tabelleGui) {
        return ermittleDruckAuftragAllgemein(titel, anzSpaltenLegendeOhneFarbe, listeLegendeOhneFarbe, anzSpaltenLegendeMitFarbe, listeLegendeMitFarbe, tabelleGui);
    }

    /**
     * Ermittle den Druck-Auftrag fr die GUI-Tabelle.
     * Es werden Titel, Legende und Tabelle im Dokument erstellt.
     * </p>
     * Legende besteht aus maximum zwei Tabellen: erste ist ohne Farbe,
     * die zweite ist mit der Farbe dargestellt.
     * </p>
     * @param titel Titel des Dokuments.
     * @param anzSpaltenLegendeOhneFarbe Anzahl der Spalten in der Legende ohne die Farbe.
     * @param listeLegendeOhneFarbe Liste der Legende ohne die Farbe. Die Elemente werden in der
     * tabellarischen Form dargestellt. Liste kann null enthalten (wird leeres Element dargestellt).
     * Falls der Parameter gleich null ist, wird die Tabelle nicht erstellt.
     * @param anzSpaltenLegendeMitFarbe Anzahl der Spalten in der Legende mit der Farbe.
     * @param listeLegendeMitFarbe Liste der Legende mit der Farbe. Die Elemente werden in der
     * tabellarischen Form dargestellt. Die Farbe wird von der 1.Spalte bernommen.
     * Liste kann null enthalten (wird leeres Element dargestellt).
     * Falls der Parameter gleich null ist, wird die Tabelle nicht erstellt.
     * @param treeGui SWT-Baum.
     *
     * @return Druck-Auftrag fr die GUI-Tabelle.
     */
    public static PagePrint ermittleDruckAuftrag(String titel, int anzSpaltenLegendeOhneFarbe, List<Control> listeLegendeOhneFarbe, int anzSpaltenLegendeMitFarbe, List<Control> listeLegendeMitFarbe, final Tree treeGui) {
        return ermittleDruckAuftragAllgemein(titel, anzSpaltenLegendeOhneFarbe, listeLegendeOhneFarbe, anzSpaltenLegendeMitFarbe, listeLegendeMitFarbe, treeGui);
    }

    /**
     * Ermittle den Druck-Auftrag fr die GUI-Tabelle.
     * Es werden Titel, Legende und Tabelle im Dokument erstellt.
     * </p>
     * Legende besteht aus maximum zwei Tabellen: erste ist ohne Farbe,
     * die zweite ist mit der Farbe dargestellt.
     * </p>
     * @param titel Titel des Dokuments.
     * @param anzSpaltenLegendeOhneFarbe Anzahl der Spalten in der Legende ohne die Farbe.
     * @param listeLegendeOhneFarbe Liste der Legende ohne die Farbe. Die Elemente werden in der
     * tabellarischen Form dargestellt. Liste kann null enthalten (wird leeres Element dargestellt).
     * Falls der Parameter gleich null ist, wird die Tabelle nicht erstellt.
     * @param anzSpaltenLegendeMitFarbe Anzahl der Spalten in der Legende mit der Farbe.
     * @param listeLegendeMitFarbe Liste der Legende mit der Farbe. Die Elemente werden in der
     * tabellarischen Form dargestellt. Die Farbe wird von der 1.Spalte bernommen.
     * Liste kann null enthalten (wird leeres Element dargestellt).
     * Falls der Parameter gleich null ist, wird die Tabelle nicht erstellt.
     * @param tabellenObjekt SWT-Tabelle oder null. Kann von folgenden Typen sein:
     * {@link Tree},
     * {@link Table},
     *
     * @return Druck-Auftrag fr die GUI-Tabelle.
     */
    private static PagePrint ermittleDruckAuftragAllgemein(String titel, int anzSpaltenLegendeOhneFarbe, List<Control> listeLegendeOhneFarbe, int anzSpaltenLegendeMitFarbe, List<Control> listeLegendeMitFarbe, final Object tabellenObjekt) {
        final GridPrint grid = new GridPrint();

        // ----------------------------------------------------
        // Titel:
        erstelleTitel(grid, titel);

        // ----------------------------------------------------
        // Legende:
        if (listeLegendeOhneFarbe != null) {
            final GridPrint tabelleLegende = ermittleDruckAuftragTabelleAllgemein(listeLegendeOhneFarbe, anzSpaltenLegendeOhneFarbe, false, null, FONT_LEGENDE, false, false);

            grid.add(tabelleLegende);
            erstelleLeereZeile(grid, FONT_LEGENDE);
        }

        if (listeLegendeMitFarbe != null) {
            final GridPrint tabelleLegende = ermittleDruckAuftragTabelleAllgemein(listeLegendeMitFarbe, anzSpaltenLegendeMitFarbe, false, null, FONT_LEGENDE, true, false);

            grid.add(tabelleLegende);
            erstelleLeereZeile(grid, FONT_LEGENDE);
        }

        // ----------------------------------------------------
        // Tabelle:
        // Anzahl der Spalten:
        int anzSpalten = 0;

        if (tabellenObjekt instanceof Table) {
            Table table = (Table) tabellenObjekt;

            anzSpalten = table.getColumnCount();
        } else if (tabellenObjekt instanceof Tree) {
            Tree tree = (Tree) tabellenObjekt;

            anzSpalten = tree.getColumnCount();
        }

        //
        if (tabellenObjekt != null) {
            final GridPrint tabellePrint = ermittleDruckAuftragTabelleAllgemein(tabellenObjekt, anzSpalten, true, FONT_TABELLE_TITEL, FONT_TABELLE, true, true);

            grid.add(tabellePrint);
        }

        // ----------------------------------------------------
        // Druckauftrag:
        if (anzSpalten <= MAX_SPALTEN_ANZAHL_PRO_SEITE) {
            return new PagePrint(grid);
        } else {
            return new PagePrint(new BigPrint(grid));
        }
    }

    /**
     * Ermittle den Druck-Auftrag mit dem Image.
     *
     * @param comp Composite, deren Inhalt im Image gespeichert wird.
     *
     * @return Druck-Auftrag mit dem Image.
     */
    public static PagePrint ermittleDruckAuftragImage(Composite comp) {
        if (comp != null) {

            // ------------------------------------------------------
            // Anpasse die Gre der Composite
            // ------------------------------------------------------
            // Gre der Composite wird angepasst, nur falls OS != Windows 7
            // (wegen bug: Methode comp.print() hat Probleme mit Aero-Modus).
            boolean groesseAnpassen = (!System.getProperty("os.name").equals("Windows 7"));

            //
            Point compSizeAlt = comp.getSize();
            Point compSize    = comp.computeSize(SWT.DEFAULT, SWT.DEFAULT);

            // Falls Gre nicht gendert wird:
            if (!groesseAnpassen) {
                compSize = compSizeAlt;
            }

            comp.setSize(compSize);

            // ------------------------------------------------------
            // Definiere Image mit Composite-Zeichnung
            // ------------------------------------------------------
            final Image image = new Image(comp.getDisplay(), compSize.x, compSize.y);
            GC          gc    = new GC(image);

            comp.print(gc);
            gc.dispose();

            // ------------------------------------------------------
            // Setze die Gre der Composite zurck
            // ------------------------------------------------------
            comp.setSize(compSizeAlt);
            comp.layout();

            // ------------------------------------------------------
            // Berechne DPI und definiere PagePrint
            // ------------------------------------------------------
            // DPI - Punkte pro Inch
            // 1 Inch = 25.4 mm
            //
            // Definiere so DPI, dass die Zeichnung die ganze Breite
            // auf der Druck-Seite nimmt:
            // dpi = (Breite der Zeichnung / Breite der Druck-Seite) * 25.4
            Printer    pr  = new Printer(Printer.getDefaultPrinterData());
            int        dpi = (int) Math.round(25.4 * compSize.x / pr.getBounds().width);
            ImagePrint ip  = new ImagePrint(image.getImageData(), new Point(dpi, dpi));

            //
            image.dispose();

            //
            ScalePrint sp = new ScalePrint(ip);

            return new PagePrint(sp);
        }

        return null;
    }

    /**
     * Ermittle den Druckauftrag in der tabellarischer Form.
     *
     * @param tabellenObjekt SWT-Tabellenobjekt oder null. Kann von folgenden Typen sein:
     * {@link Tree},
     * {@link Table},
     * {@link List<Control>} - falls die Tabelle mit Titel erstellt werden soll (s.Parameter mitTitel),
     * stellen die ersten Elemente den Titel dar.
     * @param anzSpalten Anzahl der Spalten in der Tabelle.
     * @param mitTitel Stellt fest, ob die Tabelle die Titel-Zeile enthlt.
     * @param fontTabelleTitel Schrift des Titels der Tabelle oder null.
     * @param fontTabelle Schrift der Tabelle.
     * @param mitFarbe Stellt fest, ob die Tabelle farblich dargestellt werden soll.
     * @param mitRahmen Stellt fest, ob die Tabelle mit Rahmen dargestellt werden soll.
     *
     * @return Druckauftrag in der tabellarischer Form.
     */
    public static GridPrint ermittleDruckAuftragTabelleAllgemein(final Object tabellenObjekt, final int anzSpalten, final boolean mitTitel, final FontData fontTabelleTitel, final FontData fontTabelle, boolean mitFarbe, boolean mitRahmen) {

        // ----------------------------------------------------
        // Definiere den Typ der Tabellendaten:
        Table         table        = null;
        Tree          tree         = null;
        List<Control> listeControl = null;

        // Table:
        if (tabellenObjekt instanceof Table) {
            table = (Table) tabellenObjekt;
        }

        // Tree:
        else if (tabellenObjekt instanceof Tree) {
            tree = (Tree) tabellenObjekt;
        }

        // List<Control>:
        else if (tabellenObjekt instanceof List) {
            try {
                listeControl = (List<Control>) tabellenObjekt;
            }
            catch (Exception ignored) {}
        }

        // ----------------------------------------------------
        // Erstelle die Tabelle:
        final GridPrint       tabelleDruck = new GridPrint();
        final DefaultGridLook look         = new DefaultGridLook();

        look.setHeaderBackground(RGB_GRAU);

        if (mitRahmen) {
            look.setCellBorder(new LineBorder(RGB_SCHWARZ));
        }

        // ----------------------------------------------------
        // Farbe die Tabellenzeilen:
        if (mitFarbe) {
            final Table         tableF        = table;
            final Tree          treeF         = tree;
            final List<Control> listeControlF = listeControl;

            look.setBodyBackgroundProvider(new CellBackgroundProvider() {
                @Override
                public RGB getCellBackground(int zeile, int spalte, int colspan) {

                    // Table:
                    if (tableF != null) {
                        return tableF.getItem(zeile).getBackground().getRGB();
                    }

                    // Tree:
                    else if (treeF != null) {

                        // TODO Zeile des Baums mit Bercksichtigung der Kinder ermitteln...
                        // System.out.println("--- " + zeile);
                    }

                    // List<Control>:
                    else if (listeControlF != null) {
                        try {
                            int zeileN = zeile;

                            if (mitTitel) {
                                zeileN++;
                            }

                            return ermittleHintergrund(listeControlF.get(zeileN * anzSpalten));
                        }
                        catch (Exception e) {
                            return null;
                        }
                    }

                    // sonst:
                    return null;
                }
            });
        }

        //
        tabelleDruck.setLook(look);

        // ----------------------------------------------------
        // Erstelle die Spalten:
        final TextStyle textStyle          = new TextStyle();
        final TextStyle tabellenTitelStyle = textStyle.font(fontTabelleTitel);

        for (int iSpalte = 0; iSpalte < anzSpalten; iSpalte++) {
            final GridColumn gridColumn = new GridColumn(SWT.LEFT, GridColumn.DEFAULT_SIZE, 0);

            tabelleDruck.addColumn(gridColumn);

            // Titel der Spalte:
            if (mitTitel) {
                String txt = null;

                // Table:
                if (table != null) {
                    txt = table.getColumn(iSpalte).getText();
                }

                // Tree:
                else if (tree != null) {
                    txt = tree.getColumn(iSpalte).getText();
                }

                // List<Control>:
                else if (listeControl != null) {
                    txt = ermittleText(listeControl.get(iSpalte)).trim() + "  ";
                }

                //
                if (txt != null) {
                    tabelleDruck.addHeader(tabellenTitelStyle.create(txt));
                }
            }
        }

        // ----------------------------------------------------
        // Tabelleninhalt:
        // ----------------------------------------------------
        // Anzahl der Zeilen:
        int anzZeilen = 0;

        // Table:
        if (table != null) {
            anzZeilen = table.getItemCount();
        }

        // Tree:
        else if (tree != null) {
            anzZeilen = tree.getItemCount();
        }

        // List<Control>:
        else if (listeControl != null) {
            anzZeilen = listeControl.size() / anzSpalten;

            if (mitTitel) {
                anzZeilen--;
            }
        }

        // ----------------------------------------------------
        // Tabelleninhalt:
        final TextStyle tabellenStyle = textStyle.font(fontTabelle);
        Print           zelle;
        Image           image;
        String          txt;

        for (int iZeile = 0; iZeile < anzZeilen; iZeile++) {
            for (int iSpalte = 0; iSpalte < anzSpalten; iSpalte++) {
                image = null;
                txt   = null;

                // Table:
                if (table != null) {
                    image = table.getItem(iZeile).getImage(iSpalte);
                    txt   = table.getItem(iZeile).getText(iSpalte);
                }

                // Tree:
                else if (tree != null) {
                    image = tree.getItem(iZeile).getImage(iSpalte);
                    txt   = tree.getItem(iZeile).getText(iSpalte);
                }

                // List<Control>:
                else if (listeControl != null) {
                    int zeileN = iZeile;

                    if (mitTitel) {
                        zeileN++;
                    }

                    image = ermittleImage(listeControl.get(zeileN * anzSpalten + iSpalte));
                    txt   = ermittleText(listeControl.get(zeileN * anzSpalten + iSpalte)).trim() + "  ";
                }

                //
                if (image != null) {
                    zelle = new ImagePrint(image.getImageData());
                } else if (txt != null) {
                    zelle = tabellenStyle.create(txt);
                } else {
                    zelle = tabellenStyle.create("---");
                }

                tabelleDruck.add(zelle);
            }

            // ----------------------------------------------------
            // Tree - alle Kinder rekursiv ermitteln:
            if (tree != null) {
                if (tree.getItem(iZeile).getItemCount() > 0) {
                    List<TreeItem> listeKinder = ermittleBaumAlleKinder(tree.getItem(iZeile));
                    TreeItem       parent      = tree.getItem(iZeile);
                    String         abstand     = "--- ";

                    for (TreeItem kind : listeKinder) {
                        for (int iSpalte = 0; iSpalte < anzSpalten; iSpalte++) {

                            //
                            if (kind.getParentItem() != parent) {
                                parent  = kind.getParentItem();
                                abstand = String.format("%s%s", abstand, abstand);
                            }

                            //
                            image = kind.getImage(iSpalte);

                            if (iSpalte == 0) {
                                txt = String.format("%s%s", abstand, kind.getText(iSpalte));
                            } else {
                                txt = kind.getText(iSpalte);
                            }

                            if (image != null) {
                                zelle = new ImagePrint(image.getImageData());
                            } else if (txt != null) {
                                zelle = tabellenStyle.create(txt);
                            } else {
                                zelle = tabellenStyle.create("---");
                            }

                            tabelleDruck.add(zelle);
                        }
                    }
                }
            }
        }

        return tabelleDruck;
    }

    /**
     * Ermittle den Hintergrund vom GUI-Element.
     *
     * @param ctrl SWT-Control-Element.
     *
     * @return Hintergrund vom GUI-Element oder null, falls nicht ermittelbar.
     */
    public static RGB ermittleHintergrund(Control ctrl) {
        RGB rgb = null;

        if (ctrl instanceof Label) {
            Label lb = (Label) ctrl;

            rgb = lb.getBackground().getRGB();
        } else if (ctrl instanceof Combo) {
            Combo combo = (Combo) ctrl;

            rgb = combo.getBackground().getRGB();
        } else if (ctrl instanceof Text) {
            Text txt = (Text) ctrl;

            rgb = txt.getBackground().getRGB();
        } else if (ctrl instanceof Button) {
            Button btn = (Button) ctrl;

            rgb = btn.getBackground().getRGB();
        } else if (ctrl instanceof Spinner) {
            Spinner spinner = (Spinner) ctrl;

            rgb = spinner.getBackground().getRGB();
        } else if (ctrl instanceof DatumZeitOhneSekunden) {
            DatumZeitOhneSekunden dz = (DatumZeitOhneSekunden) ctrl;

            rgb = dz.getBackground().getRGB();
        } else if (ctrl instanceof ZeitOhneSekunden) {
            ZeitOhneSekunden zeit = (ZeitOhneSekunden) ctrl;

            rgb = zeit.getBackground().getRGB();
        } else if (ctrl instanceof Group) {
            Group grp = (Group) ctrl;

            rgb = grp.getBackground().getRGB();
        } else if (ctrl instanceof Section) {
            Section sct = (Section) ctrl;

            rgb = sct.getBackground().getRGB();
        }

        return rgb;
    }

    /**
     * Ermittle das Image vom GUI-Element.
     *
     * @param ctrl SWT-Control-Element.
     *
     * @return Image vom GUI-Element oder null, falls nicht ermittelbar.
     */
    public static Image ermittleImage(Control ctrl) {
        Image image = null;

        if (ctrl instanceof Label) {
            Label lb = (Label) ctrl;

            image = lb.getImage();
        } else if (ctrl instanceof Combo) {}
        else if (ctrl instanceof Text) {}
        else if (ctrl instanceof Button) {
            Button btn = (Button) ctrl;

            image = btn.getImage();
        } else if (ctrl instanceof Spinner) {}
        else if (ctrl instanceof DatumZeitOhneSekunden) {}
        else if (ctrl instanceof ZeitOhneSekunden) {}
        else if (ctrl instanceof Group) {}
        else if (ctrl instanceof Section) {}

        return image;
    }

    /**
     * Ermittle den Text vom GUI-Element.
     *
     * @param ctrl SWT-Control-Element.
     *
     * @return Text vom GUI-Element oder leeres String, falls nicht ermittelbar.
     */
    public static String ermittleText(Control ctrl) {
        String txt;

        if (ctrl instanceof Label) {
            Label lb = (Label) ctrl;

            txt = lb.getText();
        } else if (ctrl instanceof Combo) {
            Combo combo = (Combo) ctrl;

            txt = combo.getText();
        } else if (ctrl instanceof Text) {
            Text text = (Text) ctrl;

            txt = text.getText();
        } else if (ctrl instanceof Button) {
            Button btn = (Button) ctrl;

            txt = btn.getText();
        } else if (ctrl instanceof Spinner) {
            Spinner spinner = (Spinner) ctrl;

            txt = String.valueOf(spinner.getSelection());
        } else if (ctrl instanceof DatumZeitOhneSekunden) {
            DatumZeitOhneSekunden dz = (DatumZeitOhneSekunden) ctrl;

            try {
                txt = DatumFunktionen.getDatumZeitOhneSekundenAlsString(dz.ermittleDatumZeit());
            }
            catch (Exception e) {
                txt = "---";
            }
        } else if (ctrl instanceof ZeitOhneSekunden) {
            ZeitOhneSekunden zeit = (ZeitOhneSekunden) ctrl;

            try {
                txt = DatumFunktionen.getUhrzeitAlsStringOhneSekunden(zeit.ermittleDatumZeit());
            }
            catch (Exception e) {
                txt = "---";
            }
        } else if (ctrl instanceof Group) {
            Group grp = (Group) ctrl;

            txt = grp.getText();
        } else if (ctrl instanceof Section) {
            Section sct = (Section) ctrl;

            txt = sct.getText();
        } else {
            txt = "";
        }

        return txt;
    }

    /**
     * Erstelle eine leere Zeile in GridPrint.
     *
     * @param grid GridPrint.
     * @param font Font.
     */
    public static void erstelleLeereZeile(final GridPrint grid, FontData font) {
        grid.add(new LineBreakPrint(font));
    }

    /**
     * Erstelle den Titel in GridPrint.
     *
     * @param grid GridPrint.
     * @param titel Titel.
     */
    public static void erstelleTitel(final GridPrint grid, String titel) {
        grid.addColumn(new GridColumn(GridColumn.DEFAULT_ALIGN, GridColumn.DEFAULT_SIZE, GridColumn.DEFAULT_WEIGHT));
        grid.addHeader(new TextPrint(titel, FONT_TITEL));
        erstelleLeereZeile(grid, FONT_TITEL);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 24.09.13
