/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.funktionen.math;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.gui.pruefung.PruefungDialogFelder;

import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse enthlt statische Methoden fr die Arbeit mit den Zahlen.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class ZahlFunktionen {

    /**
     * Prfe die angegebene Zahl.
     *
     * @param listeZulaessigeWerte Liste der zulssigen Werte als Texte. Jedes Element der Liste enthlt zwei zulssige Grenzwerte.
     * @param zahl                 Der zu prfende Zahl.
     *
     * @return true, falls die angegebene Zahl zulssig ist, false - sonst.
     */
    public static boolean istZahlZulaessig(List<String[]> listeZulaessigeWerte, double zahl) {
        double z1;
        double z2;

        for (String[] ar : listeZulaessigeWerte) {
            z1 = PruefungDialogFelder.konvertiereTextInDouble(ar[0]);
            z2 = PruefungDialogFelder.konvertiereTextInDouble(ar[1]);

            if (liegtZahlImIntervall(zahl, z1, z2)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Stelle fest, ob die angegebene Zahl im angegebenen Intervall liegt.
     * Das Intervall kann auch absteigend sein.
     *
     * @param zahl Die zu prfende Zahl.
     * @param start Start des Intervalls.
     * @param ende Ende des Intervalls.
     *
     * @return <code>true</code>, die angegebene Zahl im angegebenen Intervall liegt,
     * <code>falls</code> - sonst.
     */
    public static boolean liegtZahlImIntervall(double zahl, double start, double ende) {

        // Falls das Intervall ist absteigend:
        if ((start > ende)) {
            double z;

            z     = start;
            start = ende;
            ende  = z;
        }

        if ((zahl >= start) && (zahl <= ende)) {
            return true;
        }

        return false;
    }

    /**
     * Runde die Double-Zahl bis 2 Nachkommastellen.
     *
     * @param zahl Die zu gerundete Zahl.
     *
     * @return Double-Zahl bis 2 Nachkommastellen gerundet.
     */
    public static double roundBis2(double zahl) {
        return Math.round(zahl * 100.) / 100.;
    }

    /**
     * Vergleiche zwei Double bis der angegebenen im Parameter nachkommaZahl Genauigkeit.
     *
     * @param d1 Erste zu vergleichende Zahl.
     * @param d2 Zweite zu vergleichende Zahl.
     * @param nachkommaZahl Legt fest die RundungGenaugikeit.
     *
     * @return true, falls die bis angegebener Genauigkeit gerundeten Double gleich sind,
     * false sonst.
     */
    public static boolean sindDoubleGleich(double d1, double d2, int nachkommaZahl) {
        if (d1 == d2) {
            return true;
        }

        // Runde die Zahl:
        // |d1-d2| * (10 hoch nachkommaZahl)
        // und vergleiche die mit 0:
        int genauigkeit = 1;

        for (int i = 0; i < nachkommaZahl; i++) {
            genauigkeit = genauigkeit * 10;
        }

        return (Math.round(Math.abs(d1 - d2) * genauigkeit) == 0);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 28.06.10
