/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.gui;

//~ NICHT JDK IMPORTE =========================================================

import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.PlatformUI;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;

//~ KLASSEN ===================================================================

/**
 * Klasse enthlt die Methode fr die Verwaltung des GUI.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class GuiFunktionen {

    /**
     * Gibt die ViewInstanz zurck.
     *
     * @param viewID ID der zu ermittelten Sicht.
     *
     * @return View.
     */
    public static IViewPart ermittleView(String viewID) {
        return PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().findView(viewID);
    }

    /**
     * Prfe ,ob alle GUI-Elemente aus dem Array belegt sind
     * (nicht null sind und getText().length()>0).
     *
     * @param arGui Array der GUI-Elemente, die berprfen werden sollen.
     * Knnen von Typ 'Combo' oder 'Text' sein.
     *
     * @return Liste der Nummern der GUI-Elemente im Array, die nicht belegt sind, oder leere Liste.
     */
    public static java.util.List<Integer> pruefeGuiElemente(Widget[] arGui) {
        java.util.List<Integer> listeFehler = new ArrayList<Integer>();

        if (arGui != null) {
            int i = 0;

            for (Widget el : arGui) {
                if ((el == null) ||                                                       // ((el == null) oder
                    ((el instanceof Combo) && (((Combo) el).getText().length() == 0)) ||  // (el ist Combo und Text="") oder
                    ((el instanceof Text) && (((Text) el).getText().length() == 0)))      // (el ist Text und Text="")
                    {
                    listeFehler.add(i);
                }

                i++;
            }
        }

        return listeFehler;
    }

    /**
     * Schliee das GUI.
     *
     * @param parent Parent-Composite vom GUI (falls vom Typ Shell ist, ist das GUI ein Dialog).
     * @param viewID ID der Sicht oder leeres String, falls GUI keine Sicht ist.
     */
    public static void schliesseGUI(Composite parent, String viewID) {

        // falls GUI als Dialog geffnet wurde:
        if (parent instanceof Shell) {
            ((Shell) parent).close();
        }

        // falls GUI als Sicht geffnet wurde:
        else {
            schliesseSicht(viewID);
        }
    }

    /**
     * Schliee die Sicht.
     *
     * @param viewID ID der Sicht, die geschlossen werden soll.
     */
    public static void schliesseSicht(String viewID) {
        IViewPart aktView = ermittleView(viewID);

        if (aktView != null) {
            PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(aktView);
        }
    }

    /**
     * Schliee die Multi-Sicht.
     *
     * @param viewID ID der Sicht, die geschlossen werden soll.
     * @param viewIdSecond Second-ID der Sicht, die geschlossen werden soll.
     */
    public static void schliesseSichtMulti(String viewID, String viewIdSecond) {
        try {

            // Multi Sicht:
            IViewReference aktViewMulti = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().findViewReference(viewID, viewIdSecond);

            PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(aktViewMulti);

            // Falls die Sicht ber Eclipse (show View as a fast view) geffnet wird:
            schliesseSicht(viewID);
        }
        catch (Exception ignored) {}
    }

    /**
     * Ermittelt ,ob alle GUI-Elemente aus dem Array belegt sind
     * (nicht null sind und getText().length()>0).
     *
     * @param arGui Array der GUI-Elemente, die berprfen werden sollen.
     * Knnen von Typ 'Combo' oder 'Text' sein.
     *
     * @return true, falls alle GUI-Elemente aus dem Array belegt sind,
     * false sonst.
     */
    public static boolean sindGuiElementeBelegt(Widget[] arGui) {
        String text;

        for (Widget gui : arGui) {
            if (gui == null) {
                return false;
            } else if (gui instanceof Combo) {
                Combo combo = (Combo) gui;

                text = combo.getText();
            } else if (gui instanceof Text) {
                Text txt = (Text) gui;

                text = txt.getText();
            } else {
                return false;
            }

            if (text.length() == 0) {
                return false;
            }
        }

        return true;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 09.10.13
