/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.gui.widget;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;

import org.eclipse.nebula.widgets.cdatetime.CDateTime;
import org.eclipse.nebula.widgets.cdatetime.CDT;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

//~ JDK IMPORTE ===============================================================

import java.util.Date;

//~ KLASSEN ===================================================================

/**
 * Eingabefeld fr Datum + Zeit ohne Sekunden (Sekunden und Millisekunden werden auf 0 gesetzt)
 * mit einstellbaren Minutenschritt.
 *
 * Auf der Basis der Klasse {@link de.bsvrz.buv.rw.basislib.kalender.DatumZeit} gemacht.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 15 $ / $Date: 2013-09-24 14:21:48 +0200 (Di, 24 Sep 2013) $ / ($Author: LG $)
 */
public class DatumZeitOhneSekunden extends Composite {

    /**
     * CDateTime widget
     */
    private CDateTime _cdt;

    /**
     * VorgabeDatum
     */
    private Date _vorgabeDatum;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse DatumZeitOhneSekunden.
     *
     * @param parent Parent-Composite.
     * @param style SWT-Style.
     */
    public DatumZeitOhneSekunden(Composite parent, int style) {
        super(parent, style);

        //
        init();
    }

    /**
     * Erzeugt eine Instanz der Klasse DatumZeitOhneSekunden.
     *
     * @param parent Parent-Composite.
     * @param style SWT-Style.
     * @param vorgabeDatum
     *          Vorgabe Datum/Uhrzeit als Instanz der Klasse {@link Date}. Wenn
     *          <code>null</code>, dann wird vom aktuellen Datum ausgegangen.
     */
    public DatumZeitOhneSekunden(Composite parent, int style, Date vorgabeDatum) {
        super(parent, style);

        //
        _vorgabeDatum = vorgabeDatum;
        init();
    }

    //~ METHODEN ==============================================================

    @Override
    public void addFocusListener(FocusListener listener) {
        _cdt.addFocusListener(listener);
        super.addFocusListener(listener);
    }

    /**
     * Add SelectionListener.
     *
     * @param listener SelectionListener.
     */
    public void addSelectionListener(SelectionListener listener) {
        _cdt.addSelectionListener(listener);
    }

    /**
     * Ermittle den selektierten Zeitpunkt ohne Sekunden und Millisekunden.
     *
     * @return Den selektierten Zeitpunkt ohne Sekunden und Millisekunden.
     */
    public long ermittleDatumZeit() {
        return DatumFunktionen.getDatumZeitOhneSekunden(_cdt.getSelection().getTime());
    }

    /**
     * Die Methode vergrert das Widget CDatumZeit um die bergebenen Parameter x und y.
     * @param sizeX die Gre um die das aktuelle widget CDatumZeit vergrert wird (Lnge)
     * @param sizeY die Gre um die das aktuelle widget CDatumZeit vergrert wird (Hhe).
     */
    public void erweitereGroesse(int sizeX, int sizeY) {
        GridData gdDatum = new GridData();

        gdDatum.widthHint  = _cdt.getSize().x + sizeX;
        gdDatum.heightHint = _cdt.getSize().y + sizeY;
        _cdt.setLayoutData(gdDatum);
        _cdt.setLayoutData(gdDatum);
    }

    /**
     * Erzeuge die Darstellung.
     */
    private void erzeugeDarstellung() {

        // Erzeuge Datum-Uhr:
        _cdt = new CDateTime(this, CDT.SPINNER | CDT.DROP_DOWN | CDT.DATE_MEDIUM | CDT.TIME_SHORT | CDT.CLOCK_24_HOUR);
        _cdt.setPattern("dd.MM.yyyy HH:mm");

        // GUI:
        GridData gridData = new GridData();

        gridData.horizontalAlignment       = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        gridData.verticalAlignment         = GridData.CENTER;
        _cdt.setLayoutData(gridData);

        GridLayout gridLayout = new GridLayout();

        gridLayout.numColumns   = 2;
        gridLayout.marginWidth  = 0;
        gridLayout.marginHeight = 0;

        // gridLayout.horizontalSpacing = 0;
        this.setLayout(gridLayout);
        this.pack(true);

        //
        int breite = _cdt.getSize().x;
        int hoehe  = _cdt.getSize().y;

        _cdt.setSize(breite + 8, hoehe);

        //
        this.update();
    }

    /**
     * Initialisiere das GUI.
     */
    private void init() {
        erzeugeDarstellung();

        if (_vorgabeDatum != null) {

            // Setze Sekunden und Millisekunden auf 0:
            Date datum = new Date(DatumFunktionen.getDatumZeitOhneSekunden(_vorgabeDatum.getTime()));

            _cdt.setSelection(datum);
        }
    }

    //~ SET METHODEN ==========================================================

    @Override
    public void setEnabled(boolean enabled) {
        _cdt.setEnabled(enabled);
        super.setEnabled(enabled);
    }

    //~ METHODEN ==============================================================

    /**
     * Setze den Zeitpunkt mit dem neuen Wert.
     * Dabei werden Sekunden und Millisekunden auf 0 gesetzt.
     *
     * @param dz Neuen Wert fr den Zeitpunkt.
     */
    public void setzeDatumZeit(long dz) {
        _cdt.setSelection(new Date(DatumFunktionen.getDatumZeitOhneSekunden(dz)));
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 07.11.13
