/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.internet.mail;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.Activator;
import de.inovat.buv.inovat.lib.debug.Log;

import org.jdesktop.jdic.desktop.DesktopException;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.mail.*;
import javax.mail.internet.*;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.*;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Die Klasse verwaltet Mailfunktionen.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class MailFunktionen {

    /**
     * Ersetzt Leerzeichen durch '%20', '-' durch '%32'.
     *
     * @param text Text, der gendert werden soll.
     * @return Genderter Text.
     */
    private static String ersetzeZeichen(String text) {
        text = text.replaceAll(" ", "%20");
        text = text.replaceAll("-", "%32");

        return text;
    }

    /**
     * ffnet das Standard MailProgramm.
     *
     * Mit Hilfe java.awt.Desktop implementiert
     *
     * @param subject Subjekt der EMail.
     * @param arTo Array der Empfnger.
     * @param text Text der EMail.
     * @param fileName Name der AnhangrDatei (wird nicht untersttzt).
     *
     * @throws Exception Exception.
     */
    public static void oeffneMail(String subject, String[] arTo, String text, String fileName) throws Exception {
        if (Desktop.isDesktopSupported()) {

            // 1. Erzeuge String mailTo.
            // Zeichen '?'als Trennzeichen kann nur ein mal vorkommen.
            // Weiter wird '&' benutzt.
            // MailAdresse To:
            String  mailAdress      = "";
            boolean istFrageZeichen = true;

            for (String adress : arTo) {
                if (istFrageZeichen) {
                    mailAdress      = mailAdress + adress + "?to=";
                    istFrageZeichen = false;
                } else {
                    mailAdress = mailAdress + adress + "&to=";
                }
            }

            mailAdress = mailAdress.substring(0, mailAdress.length() - 4);

            // Subjekt:
            String mailSubject;

            if (arTo.length > 1) {
                mailSubject = "&subject=" + ersetzeZeichen(subject);
            } else {
                mailSubject = "?subject=" + ersetzeZeichen(subject);
            }

            // Attachment:
            // File f=new File(fileName);
            // URI uriFile = f.toURI();
            String mailto = "mailto:" + mailAdress + mailSubject +

            // "&attach="+uriFile.toString()+
            // "&attachment="+uriFile.getPath()+
            "&body=" + ersetzeZeichen(text);
            URI uri = new URI(mailto);

            // 2. ffne das StandardMailprogramm.
            Desktop.getDesktop().mail(uri);
        } else {
            throw new Exception("Desktop wird nicht untersttzt");
        }
    }

    /**
     * ffnet das Standard MailProgramm.
     * Funktioniert nur unter Windows.
     *
     * Mit Hilfe org.jdesktop.jdic.desktop.* fr Windows implementiert
     *
     * Beim IOException im Aufruf von nachricht.setAttachments()
     * wird kein Exception weiter geleitet
     * und nur eine Meldung im Log-View erzeugt.
     *
     * @param subject Subjekt der EMail.
     * @param arTo Array der Empfnger.
     * @param text Text der EMail.
     * @param listeFileName Liste der Namen der AnhngerDatei.
     *
     * @throws Exception Exception.
     *
     * @return true, falls das Programm erfolgreich geffnet wird,
     * false - sonst.
     *
     *
     * !!! Methode benutzt nur die Bibliothek fr Windows !!!
     *
     * Info ber jdic:
     * https://jdic.dev.java.net/documentation/README.html#System%20Requirements
     *
     *  System Requirements
     *
     *     JDIC Packager:
     *
     *         * Supported platforms
     *               - WinME/NT/2000/XP/2003, Sun Java Desktop System (JDS) Release 1 or later, SuSE Linux 7.1 or later, RedHat Linux 8 or later, Solaris 8 or later.
     *         * Additional software
     *               - Windows Developers need to download and install windows MSI SDK 1.5 to generate MSI packages.
     *               - End user using Windows98/NT needs MSI service package to install MSI packages. Download the following executable and install.
     *                 Windows NT4 MSI Service Package, Windows 9x MSI Service Package
     *               - Linux: rpm-build package.
     *               - J2SE v1.5.0 or later.
     *
     *
     */
    public static boolean oeffneMail_Windows(String subject, String[] arTo, String text, List<String> listeFileName) throws Exception {

        // noinspection UnusedAssignment
        boolean      erfolgreich = false;
        final String emailTyp    = "SMTP:";

        /*
         * String os=System.getProperty("os.name");
         * if(os.toLowerCase().contains("windows")){
         *       // ...
         * }
         * else{
         *       throw new Exception("Das EMail-Programm kann nicht gestartet werden, " +
         *       "da das nur unter Windows funktioniert.");
         * }
         */
        List<String> empfaenger = Arrays.asList(arTo);
        List<String> attach     = new ArrayList<String>();

        for (String fileName : listeFileName) {
            attach.add(fileName);
        }

        org.jdesktop.jdic.desktop.Message nachricht = new org.jdesktop.jdic.desktop.Message();

        // nachricht.setToAddrs(empfaenger);
        List<String> listeEmpfaengerNeu = new ArrayList<String>();

        for (String einzelnerEmpfaenger : empfaenger) {
            einzelnerEmpfaenger = emailTyp + einzelnerEmpfaenger;
            listeEmpfaengerNeu.add(einzelnerEmpfaenger);
        }

        nachricht.setToAddrs(listeEmpfaengerNeu);
        nachricht.setSubject(subject);
        nachricht.setBody(text);

        try {
            nachricht.setAttachments(attach);
        }
        catch (IOException e) {

            // erfolgreich = false;
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Die Datei knnen nicht eingefgt werden!" + "Bitte fgen Sie die Anhang-Datei in die EMail manuell ein!", e));
        }

        try {
            org.jdesktop.jdic.desktop.Desktop.mail(nachricht);
            erfolgreich = true;
        }
        catch (DesktopException e) {
            erfolgreich = false;
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }

        return erfolgreich;
    }

    // ------------------------------------------------------------------------
    // STATISCHE METHODEN
    // ------------------------------------------------------------------------

    /**
     * Sende eine EMail. Falls das nicht geschafft ist, wird eine Exception geworfen.
     *
     * Mit Hilfe javax.mail implementiert.
     *
     * @param protocol Protocol.
     * @param provider Provider.
     * @param benutzer Benutzer.
     * @param kennwort Kennwort.
     * @param from Email from.
     * @param subject Subject.
     * @param arTo Array von to-Emails.
     * @param text Text der Email.
     * @param fileName FileName.
     *
     * @throws Exception Exception.
     */
    public static void sendeMail(String protocol, String provider, String benutzer, String kennwort, String from, String subject, String[] arTo, String text, String fileName) throws Exception {
        Properties mailProps = new Properties();

        mailProps.put("mail.transport.protocol", protocol);
        mailProps.put("mail.smtp.host", provider);
        mailProps.put("User", benutzer);
        mailProps.put("Password", kennwort);

        Session         session    = Session.getDefaultInstance(mailProps);
        Message         message    = new MimeMessage(session);
        InternetAddress adressFrom = new InternetAddress(from);

        message.setFrom(adressFrom);

        InternetAddress[] arAdresseTo = new InternetAddress[arTo.length];

        for (int i = 0; i < arTo.length; i++) {
            arAdresseTo[i] = new InternetAddress(arTo[i]);
        }

        message.setRecipients(Message.RecipientType.TO, arAdresseTo);
        message.setSubject(subject);

        MimeMultipart mimeMultipart = new MimeMultipart();
        MimeBodyPart  textP         = new MimeBodyPart();

        textP.setText(text);
        textP.setDisposition(MimeBodyPart.INLINE);

        File         file        = new File(fileName);
        MimeBodyPart attachement = new MimeBodyPart();

        attachement.setDataHandler(new DataHandler(new FileDataSource(file)));
        attachement.setFileName(file.getName());
        attachement.setDisposition(MimeBodyPart.ATTACHMENT);
        mimeMultipart.addBodyPart(textP);
        mimeMultipart.addBodyPart(attachement);
        message.setContent(mimeMultipart);
        message.saveChanges();
        Transport.send(message);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 12.05.10
