/**
 *
 */



package de.inovat.buv.plugin.gtm.bast.gui;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.plugin.gtm.bast.Activator;
import de.inovat.buv.projektlib.dav.DavArchivVew;
import de.inovat.buv.projektlib.dav.IDavArchiv;

import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

//~ KLASSEN ===================================================================

/**
 * Die Klasse beschreibt das GUI fr den Import.
 * Im Konstruktor wird die Instanz der ImportEinstellungen bergeben,
 * in der die Import-Methoden implementiert sind.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 394 $ / $Date: 2013-06-17 14:37:33 +0200 (Mo, 17 Jun 2013) $ / ($Author: LG $)
 */
public class ImportGUI extends Composite implements IParameterAenderung, IDavArchiv {
    private final FormToolkit formToolkit = new FormToolkit(Display.getDefault());

    /** Button Abbrechen. */
    private Button _btnAbbrechen;

    /** Button DateiAuswahl. */
    private Button _btnDateienAuswahl;

    /** Field description */
    private Button _btnImportieren;

    /** Button lschen der Importeinstellung. */
    private Button _btnloeschenImporteinstellung;

    /** Button Speichern. */
    private Button _btnSpeichern;
    private Button _cbtnPassivmodus;

    /** Combobox mit den gespeicherten Importeinstellungen. */
    private Combo _comboGespeicherteImporteinstellungen;

    /** Field description */
    private Composite _compositeAllgemein;

    /** Gruppe fr die Importeinstellungen. */
    private Group _groupImport;

    /** Instanz der Klasse ImportGUIVerwaltung. */
    private ImportGUIVerwaltung _guiVew;

    /** Label fr die Anzeige der Langzeitzhlstellenauswahl. */
    private Label _lbBAStBandDatenDateiAuswahl;

    /** Label Restzeit. */
    private Label _lbRestzeit;

    /** Field description */
    private ProgressBar _prgFortschrittsanzeige;

    /** Radiobutton BASt-Import 2004. */
    private Button _radioBASt2004;

    /** Radiobutton BASt-Import 2007. */
    private Button _radioBASt2007;
    private Button _rbtnFtp;
    private Button _rbtnSftp;

    /** Text Benutzer. */
    private Text _txtBenutzer;

    /** Textfeld Meldungen. */
    private Text _txtMeldungen;

    /** Textfeld Passwort fr den FTP-Serer. */
    private Text _txtPasswd;
    private Text _txtPort;

    /** Textfeld URL des FTP-Servers. */
    private Text _txtURL;

    /** ID der Sicht. */
    private String _viewID;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse ImportGUI.
     *
     * @param parent Parent-Composite.
     * @param viewID ID der Sicht.
     * @param pidKonfigurationsObjBastBand PID KonfigurationsObjBastBand.
     */
    public ImportGUI(Composite parent, String viewID, String pidKonfigurationsObjBastBand) {
        super(parent, 0);
        setLayout(new GridLayout());

        // Setze Variable:
        _viewID = viewID;

        // Initialisiere GUI:
        initGUI();

        // Initialisiere GUI-Werte:
        _guiVew = new ImportGUIVerwaltung(this, pidKonfigurationsObjBastBand);
        _guiVew.initialisiereGUI();
        _guiVew.initParameterMeldungUndFortschritt();
        _guiVew.initZeitbereich();
        _guiVew.erzeugeComboboxGespeicherteImporteinstellungen("");

        // Registriere die Klasse-Instanz als ParameteraenderungsListener:
        ParameterVerwaltung.addParameterListener(_viewID, this);

        // Lsche ParameterListener beim Schlieen vom GUI:
        final ImportGUI gui = this;

        parent.addDisposeListener(new DisposeListener() {
            public void widgetDisposed(final DisposeEvent e) {
                ParameterVerwaltung.removeParameterListener(_viewID, gui);
                DavArchivVew.getInstanz().removeArchivListener(gui);
            }
        });

        // Registrierung als Listener bzgl. der Archivverfgbarkeit.
        DavArchivVew.getInstanz().addArchivListener(gui);
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode abmeldenDav leitet die Abmeldung aus dem View an die Verwaltung weiter.
     * Beim Schlieen des GUI werden alle angemeldeten Datenverteilerverbindungen abgemeldet.
     */
    public void abmeldenDaV() {
        _guiVew.abmeldenDaVSenderImportImportSteuerung();
        _guiVew.abmeldenDaVEmpfaengerImportStatus();
    }

    @Override
    public void archivStatusGeaendert(final boolean archivVerfuegbar) {
        getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                try {
                    if (!archivVerfuegbar) {
                        _guiVew.aendereGUIAnzeige(1, "Das Archiv ist nicht verfgbar, daher kann kein Import durchgefhrt werden!", false);
                    } else {
                        _guiVew.aendereGUIAnzeige(1, "Das Archiv ist verfgbar...", false);
                        _guiVew.setzeImportSchaltflaeche();
                    }
                }
                catch (Exception e) {
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }
        });
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'btnAbbrechen'.
     *
     * @return Wert der Eigenschaft 'btnAbbrechen'.
     *
     * @see #_btnAbbrechen
     */
    public Button getBtnAbbrechen() {
        return _btnAbbrechen;
    }

    /**
     * Getter fr Eigenschaft 'btnDateienAuswahl'.
     *
     * @return Wert der Eigenschaft 'btnDateienAuswahl'.
     *
     * @see #_btnDateienAuswahl
     */
    public Button getBtnDateienAuswahl() {
        return _btnDateienAuswahl;
    }

    /**
     * Getter fr Eigenschaft 'btnImportieren'.
     *
     * @return Wert der Eigenschaft 'btnImportieren'.
     *
     * @see #_btnImportieren
     */
    public Button getBtnImportieren() {
        return _btnImportieren;
    }

    /**
     * Getter fr Eigenschaft 'btnloeschenImporteinstellung'.
     *
     * @return Wert der Eigenschaft 'btnloeschenImporteinstellung'.
     *
     * @see #_btnloeschenImporteinstellung
     */
    public Button getBtnloeschenImporteinstellung() {
        return _btnloeschenImporteinstellung;
    }

    /**
     *     Getter fr Eigenschaft 'cbtnPassivmodus'.
     *
     *     @return Wert der Eigenschaft 'cbtnPassivmodus'.
     *
     *     @see #_cbtnPassivmodus
     */
    public Button getCbtnPassivmodus() {
        return _cbtnPassivmodus;
    }

    /**
     * Getter fr Eigenschaft 'comboGespeicherteImporteinstellungen'.
     *
     * @return Wert der Eigenschaft 'comboGespeicherteImporteinstellungen'.
     *
     * @see #_comboGespeicherteImporteinstellungen
     */
    public Combo getComboGespeicherteImporteinstellungen() {
        return _comboGespeicherteImporteinstellungen;
    }

    /**
     * Getter fr Eigenschaft 'compositeAllgemein'.
     *
     * @return Wert der Eigenschaft 'compositeAllgemein'.
     *
     * @see #_compositeAllgemein
     */
    public Composite getCompositeAllgemein() {
        return _compositeAllgemein;
    }

    /**
     * Getter fr Eigenschaft 'formToolkit'.
     *
     * @return Wert der Eigenschaft 'formToolkit'.
     *
     * @see #formToolkit
     */
    public FormToolkit getFormToolkit() {
        return formToolkit;
    }

    /**
     * Getter fr Eigenschaft 'lbBAStBandDatenDateiAuswahl'.
     *
     * @return Wert der Eigenschaft 'lbBAStBandDatenDateiAuswahl'.
     *
     * @see #_lbBAStBandDatenDateiAuswahl
     */
    public Label getLbBAStBandDatenDateiAuswahl() {
        return _lbBAStBandDatenDateiAuswahl;
    }

    /**
     * Getter fr Eigenschaft 'lbRestzeit'.
     *
     * @return Wert der Eigenschaft 'lbRestzeit'.
     *
     * @see #_lbRestzeit
     */
    public Label getLbRestzeit() {
        return _lbRestzeit;
    }

    /**
     * Getter fr Eigenschaft 'prgFortschrittsanzeige'.
     *
     * @return Wert der Eigenschaft 'prgFortschrittsanzeige'.
     *
     */
    public ProgressBar getPrgFortschrittsanzeige() {
        return _prgFortschrittsanzeige;
    }

    /**
     * Getter fr Eigenschaft 'radioBASt2004'.
     *
     * @return Wert der Eigenschaft 'radioBASt2004'.
     *
     * @see #_radioBASt2004
     */
    public Button getRadioBASt2004() {
        return _radioBASt2004;
    }

    /**
     * Getter fr Eigenschaft 'radioBASt2009'.
     *
     * @return Wert der Eigenschaft 'radioBASt2009'.
     *
     * @see #_radioBASt2007
     */
    public Button getRadioBASt2007() {
        return _radioBASt2007;
    }

    /**
     * Getter fr Eigenschaft 'rbtnFtp'.
     *
     * @return Wert der Eigenschaft 'rbtnFtp'.
     *
     * @see #_rbtnFtp
     */
    public Button getRbtnFtp() {
        return _rbtnFtp;
    }

    /**
     * Getter fr Eigenschaft 'rbtnSftp'.
     *
     * @return Wert der Eigenschaft 'rbtnSftp'.
     *
     * @see #_rbtnSftp
     */
    public Button getRbtnSftp() {
        return _rbtnSftp;
    }

    /**
     * Getter fr Eigenschaft 'txtBenutzer'.
     *
     * @return Wert der Eigenschaft 'txtBenutzer'.
     *
     * @see #_txtBenutzer
     */
    public Text getTxtBenutzer() {
        return _txtBenutzer;
    }

    /**
     * Getter fr Eigenschaft 'txtMeldungen'.
     *
     * @return Wert der Eigenschaft 'txtMeldungen'.
     *
     * @see #_txtMeldungen
     */
    public Text getTxtMeldungen() {
        return _txtMeldungen;
    }

    /**
     * Getter fr Eigenschaft 'txtPasswd'.
     *
     * @return Wert der Eigenschaft 'txtPasswd'.
     *
     * @see #_txtPasswd
     */
    public Text getTxtPasswd() {
        return _txtPasswd;
    }

    /**
     * Getter fr Eigenschaft 'txtPort'.
     *
     * @return Wert der Eigenschaft 'txtPort'.
     *
     * @see #_txtPort
     */
    public Text getTxtPort() {
        return _txtPort;
    }

    /**
     * Getter fr Eigenschaft 'txtURL'.
     *
     * @return Wert der Eigenschaft 'txtURL'.
     *
     * @see #_txtURL
     */
    public Text getTxtURL() {
        return _txtURL;
    }

    /**
     * Getter fr Eigenschaft 'viewID'.
     *
     * @return Wert der Eigenschaft 'viewID'.
     *
     * @see #_viewID
     */
    public String getViewID() {
        return _viewID;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode initialisiert den Bereich Fortschrittsanzeige.
     */
    private void initBereichFortschrittsanzeige() {
        Group grpFortschritt = new Group(_groupImport, SWT.NONE);

        grpFortschritt.setText("Fortschrittsanzeige");
        grpFortschritt.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(grpFortschritt);
        formToolkit.paintBordersFor(grpFortschritt);
        grpFortschritt.setLayout(new GridLayout(1, false));
        _lbRestzeit = new Label(grpFortschritt, SWT.NONE);
        _lbRestzeit.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(_lbRestzeit, true, true);
        _prgFortschrittsanzeige = new ProgressBar(grpFortschritt, SWT.NONE);
        _prgFortschrittsanzeige.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        _prgFortschrittsanzeige.setSize(726, 21);
    }

    /**
     * Die Methode erzeugt den GUI Bereich Importeinstellungen.
     */
    private void initBereichImportEinstellungen() {
        _groupImport = new Group(_compositeAllgemein, SWT.NONE);
        _groupImport.setText("Import BASt-Band");
        _groupImport.setLayout(new GridLayout(1, false));
        _groupImport.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        formToolkit.adapt(_groupImport);
        formToolkit.paintBordersFor(_groupImport);

        Section sectionImportEinstellungen = formToolkit.createSection(_groupImport, Section.TWISTIE | Section.TITLE_BAR);

        sectionImportEinstellungen.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.paintBordersFor(sectionImportEinstellungen);
        sectionImportEinstellungen.setText("Importeinstellungen");

        Composite compEinstellungen = new Composite(sectionImportEinstellungen, SWT.NONE);

        formToolkit.adapt(compEinstellungen);
        formToolkit.paintBordersFor(compEinstellungen);
        sectionImportEinstellungen.setClient(compEinstellungen);
        compEinstellungen.setLayout(new GridLayout(2, false));

        Label lbGespeicherteImporteinstellungen = new Label(compEinstellungen, SWT.NONE);

        formToolkit.adapt(lbGespeicherteImporteinstellungen, true, true);
        lbGespeicherteImporteinstellungen.setText("Gespeicherte Importeinstellungen:");

        Composite compImportEinstellungen = new Composite(compEinstellungen, SWT.NONE);

        compImportEinstellungen.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
        formToolkit.adapt(compImportEinstellungen);
        formToolkit.paintBordersFor(compImportEinstellungen);
        compImportEinstellungen.setLayout(new GridLayout(3, false));
        _comboGespeicherteImporteinstellungen = new Combo(compImportEinstellungen, SWT.READ_ONLY);

        GridData gd_comboGespeicherteImporteinstellungen = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);

        gd_comboGespeicherteImporteinstellungen.widthHint = 259;
        _comboGespeicherteImporteinstellungen.setLayoutData(gd_comboGespeicherteImporteinstellungen);
        formToolkit.adapt(_comboGespeicherteImporteinstellungen);
        formToolkit.paintBordersFor(_comboGespeicherteImporteinstellungen);
        _comboGespeicherteImporteinstellungen.addFocusListener(new FocusListener() {
            @Override
            public void focusGained(FocusEvent e) {
                _guiVew.erzeugeComboboxGespeicherteImporteinstellungen("");
            }
            @Override
            public void focusLost(FocusEvent e) {}
        });
        _comboGespeicherteImporteinstellungen.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                if (_comboGespeicherteImporteinstellungen.getSelectionIndex() != -1) {
                    _btnloeschenImporteinstellung.setEnabled(true);
                    _guiVew.ladeGespeicherteImporteinstellungen(_comboGespeicherteImporteinstellungen.getItem(_comboGespeicherteImporteinstellungen.getSelectionIndex()));
                } else {
                    _btnloeschenImporteinstellung.setEnabled(false);
                }
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _btnSpeichern = new Button(compImportEinstellungen, SWT.NONE);

        StringBuilder stringToolTipTextSpeichern = new StringBuilder();

        stringToolTipTextSpeichern.append("Gespeichert werden alle Importeinstellungen bis auf die BASt-Band Dateiauswahl.");
        stringToolTipTextSpeichern.append(System.getProperty("line.separator"));
        _btnSpeichern.setToolTipText(stringToolTipTextSpeichern.toString());
        formToolkit.adapt(_btnSpeichern, true, true);
        _btnSpeichern.setText("Speichern");
        _btnSpeichern.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnSpeichern();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _btnloeschenImporteinstellung = new Button(compImportEinstellungen, SWT.NONE);
        _btnloeschenImporteinstellung.setEnabled(false);
        formToolkit.adapt(_btnloeschenImporteinstellung, true, true);
        _btnloeschenImporteinstellung.setText("Gespeicherte Importeinstellung l\u00F6schen");
        _btnloeschenImporteinstellung.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                if (_comboGespeicherteImporteinstellungen.getSelectionIndex() != -1) {
                    String ImporteinstellungsName = _comboGespeicherteImporteinstellungen.getItem(_comboGespeicherteImporteinstellungen.getSelectionIndex());

                    _guiVew.btnLoeschenImporteinstellung(ImporteinstellungsName);
                }
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });

        Label lbAuswahlBAStFormat = new Label(compEinstellungen, SWT.NONE);

        formToolkit.adapt(lbAuswahlBAStFormat, true, true);
        lbAuswahlBAStFormat.setText("Auswahl BASt-Band Import Format:");

        Composite compBastBandImportFormat = new Composite(compEinstellungen, SWT.NONE);

        formToolkit.adapt(compBastBandImportFormat);
        formToolkit.paintBordersFor(compBastBandImportFormat);
        compBastBandImportFormat.setLayout(new GridLayout(2, false));
        _radioBASt2004 = new Button(compBastBandImportFormat, SWT.RADIO);
        _radioBASt2004.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.bastFormatSelektiert(_radioBASt2004);
            }
        });
        _radioBASt2004.setSelection(true);
        formToolkit.adapt(_radioBASt2004, true, true);
        _radioBASt2004.setText("BASt2004");
        _radioBASt2007 = new Button(compBastBandImportFormat, SWT.RADIO);
        formToolkit.adapt(_radioBASt2007, true, true);
        _radioBASt2007.setText("BASt2007");
        _radioBASt2007.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.bastFormatSelektiert(_radioBASt2007);
            }
        });

        Label lbFTPServerZugangsdaten = new Label(compEinstellungen, SWT.NONE);

        formToolkit.adapt(lbFTPServerZugangsdaten, true, true);
        lbFTPServerZugangsdaten.setText("FTP Server Zugangsdaten:");

        Composite compFTPServerZugangsdaten = new Composite(compEinstellungen, SWT.NONE);

        formToolkit.adapt(compFTPServerZugangsdaten);
        formToolkit.paintBordersFor(compFTPServerZugangsdaten);
        compFTPServerZugangsdaten.setLayout(new GridLayout(2, false));

        Label lbServertyp = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbServertyp, true, true);
        lbServertyp.setText("Servertyp:");

        Group grpServertyp = new Group(compFTPServerZugangsdaten, SWT.NONE);

        grpServertyp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(grpServertyp);
        formToolkit.paintBordersFor(grpServertyp);

        GridLayout gl_grpServertyp = new GridLayout(2, false);

        gl_grpServertyp.marginHeight = 0;
        grpServertyp.setLayout(gl_grpServertyp);
        _rbtnFtp = new Button(grpServertyp, SWT.RADIO);
        _rbtnFtp.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnServertypGeaendert();
            }
        });
        formToolkit.adapt(_rbtnFtp, true, true);
        _rbtnFtp.setText("FTP");
        _rbtnSftp = new Button(grpServertyp, SWT.RADIO);
        _rbtnSftp.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnServertypGeaendert();
            }
        });
        formToolkit.adapt(_rbtnSftp, true, true);
        _rbtnSftp.setText("SFTP");

        Label lbURL = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbURL, true, true);
        lbURL.setText("URL:");
        _txtURL = new Text(compFTPServerZugangsdaten, SWT.BORDER);

        GridData gd_txtURL = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);

        gd_txtURL.widthHint = 213;
        _txtURL.setLayoutData(gd_txtURL);
        formToolkit.adapt(_txtURL, true, true);
        _txtURL.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                _guiVew.ermoeglicheDateiAuswahl();
                _guiVew.setzeImportSchaltflaeche();
            }
        });

        Label lbPort = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbPort, true, true);
        lbPort.setText("Port:");
        _txtPort = new Text(compFTPServerZugangsdaten, SWT.BORDER);
        _txtPort.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        _txtPort.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
                _guiVew.txtPortGeaendert();
            }
        });
        formToolkit.adapt(_txtPort, true, true);

        Label lbBenutzer = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbBenutzer, true, true);
        lbBenutzer.setText("Benutzer:");
        _txtBenutzer = new Text(compFTPServerZugangsdaten, SWT.BORDER);
        _txtBenutzer.setText("");
        _txtBenutzer.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(_txtBenutzer, true, true);
        _txtBenutzer.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                _guiVew.ermoeglicheDateiAuswahl();
                _guiVew.setzeImportSchaltflaeche();
            }
        });

        Label lbpasswd = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbpasswd, true, true);
        lbpasswd.setText("Passwort:");
        _txtPasswd = new Text(compFTPServerZugangsdaten, SWT.BORDER | SWT.PASSWORD);
        _txtPasswd.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        formToolkit.adapt(_txtPasswd, true, true);
        _txtPasswd.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                _guiVew.ermoeglicheDateiAuswahl();
                _guiVew.setzeImportSchaltflaeche();
            }
        });

        Label lbPassivmodus = new Label(compFTPServerZugangsdaten, SWT.NONE);

        formToolkit.adapt(lbPassivmodus, true, true);
        lbPassivmodus.setText("Passivmodus");
        _cbtnPassivmodus = new Button(compFTPServerZugangsdaten, SWT.CHECK);
        formToolkit.adapt(_cbtnPassivmodus, true, true);
        _cbtnPassivmodus.setText("");

        Label lbDateien = new Label(compEinstellungen, SWT.NONE);

        formToolkit.adapt(lbDateien, true, true);
        lbDateien.setText("Zu Importierende BASt-Band-Daten:");

        Composite compDateienAuswahl = new Composite(compEinstellungen, SWT.NONE);

        compDateienAuswahl.setLayout(new GridLayout(2, false));
        compDateienAuswahl.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
        formToolkit.adapt(compDateienAuswahl);
        formToolkit.paintBordersFor(compDateienAuswahl);
        _btnDateienAuswahl = new Button(compDateienAuswahl, SWT.NONE);
        _btnDateienAuswahl.setEnabled(false);
        _btnDateienAuswahl.setText("Dateiauswahl...");
        _btnDateienAuswahl.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                _guiVew.btnImportDateiauswahlSelektiert();
            }
        });
        _lbBAStBandDatenDateiAuswahl = new Label(compDateienAuswahl, SWT.NONE);
        _lbBAStBandDatenDateiAuswahl.setText("                                                                              ");
        formToolkit.adapt(_lbBAStBandDatenDateiAuswahl, true, true);
        sectionImportEinstellungen.setExpanded(true);
    }

    /**
     * Die Methode initialisiert den Meldungsbereich.
     */
    private void initBereichMeldungen() {
        Group grpMeldungsBereich = new Group(_groupImport, SWT.NONE);

        grpMeldungsBereich.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        grpMeldungsBereich.setText("Meldungen");
        formToolkit.adapt(grpMeldungsBereich);
        formToolkit.paintBordersFor(grpMeldungsBereich);
        grpMeldungsBereich.setLayout(new GridLayout(1, false));
        _txtMeldungen = new Text(grpMeldungsBereich, SWT.WRAP | SWT.H_SCROLL | SWT.V_SCROLL | SWT.CANCEL | SWT.MULTI);
        _txtMeldungen.setEditable(false);
        _txtMeldungen.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        formToolkit.adapt(_txtMeldungen, true, true);

        Composite compButtons = new Composite(_groupImport, SWT.NONE);

        compButtons.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        formToolkit.adapt(compButtons);
        formToolkit.paintBordersFor(compButtons);
        compButtons.setLayout(new GridLayout(3, false));
        _btnImportieren = new Button(compButtons, SWT.NONE);
        _btnImportieren.setText("Importieren");
        _btnImportieren.setEnabled(false);
        _btnImportieren.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                _guiVew.btnImportierenSelektiert();
            }
        });
        _btnAbbrechen = new Button(compButtons, SWT.NONE);
        _btnAbbrechen.setText("Abbrechen");
        _btnAbbrechen.setEnabled(false);
        _btnAbbrechen.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                _guiVew.btnAbbruchSelektiert();
            }
        });

        Button _btnSchliessen = new Button(compButtons, SWT.NONE);

        _btnSchliessen.setToolTipText("Die Bet\u00E4tigung der schlie\u00DFen Schaltfl\u00E4che schlie\u00DFt die aktuelle Sicht, bricht den laufenden Export aber nicht ab.");
        _btnSchliessen.setText("Sicht schlie\u00DFen");
        formToolkit.adapt(_btnSchliessen, true, true);
        _btnSchliessen.setText("Sicht schlie\u00DFen");
        _btnSchliessen.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent evt) {
                _guiVew.btnSchliessenSelektiert();
            }
        });
    }

    /**
     * Die Methode initialisiert den GUI Bereich.
     */
    private void initGUI() {
        final ScrolledComposite scrolledCompositeAllgemein = new ScrolledComposite(this, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);

        scrolledCompositeAllgemein.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        scrolledCompositeAllgemein.setExpandHorizontal(true);
        scrolledCompositeAllgemein.setExpandVertical(true);
        _compositeAllgemein = new Composite(scrolledCompositeAllgemein, SWT.NONE);
        _compositeAllgemein.setLocation(0, 0);
        _compositeAllgemein.setLayout(new GridLayout());
        initBereichImportEinstellungen();
        initBereichFortschrittsanzeige();
        initBereichMeldungen();
        scrolledCompositeAllgemein.setContent(_compositeAllgemein);
        scrolledCompositeAllgemein.setMinSize(_compositeAllgemein.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    }

    /*
     *  (non-Javadoc)
     * @see de.inovat.buv.plugin.bast.gui.IParameterAenderung#parameterGeaendert()
     */
    public void parameterGeaendert() {
        _guiVew.setzeGUIDatenMitAktParameter();
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 28.10.13
