/*
 * @(#)ImportPruefGUI.java   15.10.12
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.bast.importpruef;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;
import de.inovat.buv.inovat.lib.gui.widget.DatumOhneZeit;
import de.inovat.buv.inovat.lib.gui.widget.DatumOhneZeit.DATUM_MUSTER;
import de.inovat.buv.plugin.gtm.bast.Activator;
import de.inovat.buv.projektlib.dav.DavArchivVew;
import de.inovat.buv.projektlib.dav.IDavArchiv;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackAdapter;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;

//~ JDK IMPORTE ===============================================================

import java.util.Date;

//~ KLASSEN ===================================================================

/**
 * Die Klasse erzeugt das GUI "BASt-Band-Import Prfung".
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 184 $ / $Date: 2010-07-20 09:16:27 +0200 (Di, 20 Jul
 *          2010) $ / ($Author: LG $)
 */
public class ImportPruefGUI extends Composite implements IDavArchiv {

    /** Tabellen View fr die Tabelle. */
    private TableViewer _tableViewer = null;

    /** FormToolkit. */
    private final FormToolkit _toolkit = new FormToolkit(Display.getDefault());
    private Button            _btnExport;
    private Button            _btnFilterAlle;
    private Button            _btnFilterFehlende;
    private Button            _btnFilterSelektierte;
    private Button            _btnSelektionAlle;
    private Button            _btnSelektionFehlende;
    private Button            _btnSelektionKeine;
    private Button            _btnTablNeuLaden;
    private Button            _btnVollstaendigePruefung;
    private DatumOhneZeit     _datumBis;
    private DatumOhneZeit     _datumVon;

    /** Instanz GUI Verwaltung. */
    private ImportPruefGUIVew _guiVew;
    private Label             _lbLangZeitZaehlStellenAuswahl;
    private Button            _rbtnBast2004;
    private Button            _rbtnBast2007;
    private Text              _txtAsp;
    private Text              _txtAtg;
    private Text              _txtTyp;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     *
     * Erzeugt eine Instanz der Klasse ImportPruefGUI.
     *
     * @param parent
     *            Parent GUI.
     */
    public ImportPruefGUI(Composite parent) {
        super(parent, SWT.NONE);

        //
        _guiVew = new ImportPruefGUIVew(this);

        //
        _toolkit.adapt(this);
        _toolkit.paintBordersFor(this);

        //
        initGUI();
        _guiVew.initGui();

        // Lsche ParameterListener beim Schlieen vom GUI:
        final ImportPruefGUI gui = this;

        parent.addDisposeListener(new DisposeListener() {
            public void widgetDisposed(final DisposeEvent e) {
                DavArchivVew.getInstanz().removeArchivListener(gui);
            }
        });

        // Registrierung als Listener bzgl. der Archivverfgbarkeit.
        DavArchivVew.getInstanz().addArchivListener(gui);
    }

    //~ METHODEN ==============================================================

    @Override
    public void archivStatusGeaendert(final boolean archivVerfuegbar) {
        getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                try {
                    if (!_btnTablNeuLaden.isDisposed()) {
                        _btnTablNeuLaden.setEnabled(archivVerfuegbar);
                    }
                }
                catch (Exception e) {
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }
        });
    }

    //~ GET METHODEN ==========================================================

    /**
     *     Getter fr Eigenschaft 'btnExport'.
     *
     *     @return Wert der Eigenschaft 'btnExport'.
     *
     *     @see #_btnExport
     */
    public Button getBtnExport() {
        return _btnExport;
    }

    /**
     * Getter fr Eigenschaft 'btnFilterAlle'.
     *
     * @return Wert der Eigenschaft 'btnFilterAlle'.
     *
     * @see #_btnFilterAlle
     */
    public Button getBtnFilterAlle() {
        return _btnFilterAlle;
    }

    /**
     * Getter fr Eigenschaft 'btnFilterFehlende'.
     *
     * @return Wert der Eigenschaft 'btnFilterFehlende'.
     *
     * @see #_btnFilterFehlende
     */
    public Button getBtnFilterFehlende() {
        return _btnFilterFehlende;
    }

    /**
     * Getter fr Eigenschaft 'btnFilterSelektierte'.
     *
     * @return Wert der Eigenschaft 'btnFilterSelektierte'.
     *
     * @see #_btnFilterSelektierte
     */
    public Button getBtnFilterSelektierte() {
        return _btnFilterSelektierte;
    }

    /**
     *     Getter fr Eigenschaft 'btnSelektionAlle'.
     *
     *     @return Wert der Eigenschaft 'btnSelektionAlle'.
     *
     *     @see #_btnSelektionAlle
     */
    public Button getBtnSelektionAlle() {
        return _btnSelektionAlle;
    }

    /**
     * Getter fr Eigenschaft 'btnSelektionFehlende'.
     *
     * @return Wert der Eigenschaft 'btnSelektionFehlende'.
     *
     * @see #_btnSelektionFehlende
     */
    public Button getBtnSelektionFehlende() {
        return _btnSelektionFehlende;
    }

    /**
     * Getter fr Eigenschaft 'btnSelektionKeine'.
     *
     * @return Wert der Eigenschaft 'btnSelektionKeine'.
     *
     * @see #_btnSelektionKeine
     */
    public Button getBtnSelektionKeine() {
        return _btnSelektionKeine;
    }

    /**
     *     Getter fr Eigenschaft 'btnTablNeuLaden'.
     *
     *     @return Wert der Eigenschaft 'btnTablNeuLaden'.
     *
     *     @see #_btnTablNeuLaden
     */
    public Button getBtnTablNeuLaden() {
        return _btnTablNeuLaden;
    }

    /**
     * Getter fr Eigenschaft 'btnVollstaendigePruefung'.
     *
     * @return Wert der Eigenschaft 'btnVollstaendigePruefung'.
     *
     * @see #_btnVollstaendigePruefung
     */
    public Button getBtnVollstaendigePruefung() {
        return _btnVollstaendigePruefung;
    }

    /**
     * Getter fr Eigenschaft 'datumBis'.
     *
     * @return Wert der Eigenschaft 'datumBis'.
     *
     * @see #_datumBis
     */
    public DatumOhneZeit getDatumBis() {
        return _datumBis;
    }

    /**
     *     Getter fr Eigenschaft 'datumVon'.
     *
     *     @return Wert der Eigenschaft 'datumVon'.
     *
     *     @see #_datumVon
     */
    public DatumOhneZeit getDatumVon() {
        return _datumVon;
    }

    /**
     * Getter fr Eigenschaft '_toolkit'.
     *
     * @return Wert der Eigenschaft '_toolkit'.
     *
     * @see #_toolkit
     */
    public FormToolkit getFormToolkit() {
        return _toolkit;
    }

    /**
     *     Getter fr Eigenschaft 'lbLangZeitZaehlStellenAuswahl'.
     *
     *     @return Wert der Eigenschaft 'lbLangZeitZaehlStellenAuswahl'.
     *
     *     @see #_lbLangZeitZaehlStellenAuswahl
     */
    public Label getLbLangZeitZaehlStellenAuswahl() {
        return _lbLangZeitZaehlStellenAuswahl;
    }

    /**
     *     Getter fr Eigenschaft 'rbtnBast2004'.
     *
     *     @return Wert der Eigenschaft 'rbtnBast2004'.
     *
     *     @see #_rbtnBast2004
     */
    public Button getRbtnBast2004() {
        return _rbtnBast2004;
    }

    /**
     * Getter fr Eigenschaft 'rbtnBast2007'.
     *
     * @return Wert der Eigenschaft 'rbtnBast2007'.
     *
     * @see #_rbtnBast2007
     */
    public Button getRbtnBast2007() {
        return _rbtnBast2007;
    }

    /**
     *     @return the tableViewer
     */
    public TableViewer getTableViewer() {
        return _tableViewer;
    }

    /**
     *     Getter fr Eigenschaft 'txtAsp'.
     *
     *     @return Wert der Eigenschaft 'txtAsp'.
     *
     *     @see #_txtAsp
     */
    public Text getTxtAsp() {
        return _txtAsp;
    }

    /**
     * Getter fr Eigenschaft 'txtAtg'.
     *
     * @return Wert der Eigenschaft 'txtAtg'.
     *
     * @see #_txtAtg
     */
    public Text getTxtAtg() {
        return _txtAtg;
    }

    /**
     * Getter fr Eigenschaft 'txtTyp'.
     *
     * @return Wert der Eigenschaft 'txtTyp'.
     *
     * @see #_txtTyp
     */
    public Text getTxtTyp() {
        return _txtTyp;
    }

    //~ METHODEN ==============================================================

    /**
     *     Die Methode initialisiert das GUI.
     */
    private void initGUI() {
        setLayout(new GridLayout(1, false));

        //
        Composite compAll = new Composite(this, SWT.NONE);

        compAll.setLayout(new GridLayout(1, false));
        compAll.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

        ScrolledForm scrolledForm = _toolkit.createScrolledForm(compAll);

        scrolledForm.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        scrolledForm.getBody().setLayout(new GridLayout(1, false));

        //
        Section sectionAllgemeineAngaben = _toolkit.createSection(scrolledForm.getBody(), Section.TWISTIE | Section.TITLE_BAR);

        sectionAllgemeineAngaben.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        _toolkit.paintBordersFor(sectionAllgemeineAngaben);
        sectionAllgemeineAngaben.setText("Allgemeine Angaben");
        sectionAllgemeineAngaben.setExpanded(true);

        //
        Composite compAllgemeineAngaben = new Composite(sectionAllgemeineAngaben, SWT.NONE);

        _toolkit.adapt(compAllgemeineAngaben);
        _toolkit.paintBordersFor(compAllgemeineAngaben);
        sectionAllgemeineAngaben.setClient(compAllgemeineAngaben);
        compAllgemeineAngaben.setLayout(new GridLayout(3, false));

        // Zeitbereich:
        _toolkit.createLabel(compAllgemeineAngaben, "Zeitbereichsangabe:", SWT.NONE);

        Composite compZeitbereich = _toolkit.createComposite(compAllgemeineAngaben, SWT.NONE);

        compZeitbereich.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        _toolkit.paintBordersFor(compZeitbereich);

        GridLayout gl_compZeitbereich = new GridLayout(4, false);

        compZeitbereich.setLayout(gl_compZeitbereich);
        _toolkit.createLabel(compZeitbereich, "Von:", SWT.NONE);

        long dt = DatumFunktionen.getMonatAnfang(DatumFunktionen.getAktuelleDateTime());

        _datumVon = new DatumOhneZeit(compZeitbereich, SWT.BORDER, DATUM_MUSTER.MJ, new Date(dt));
        _datumVon.erweitereGroesse(4, 0);
        _toolkit.createLabel(compZeitbereich, "Bis:", SWT.NONE);
        _datumBis = new DatumOhneZeit(compZeitbereich, SWT.BORDER, DATUM_MUSTER.MJ, new Date(dt));
        _datumBis.erweitereGroesse(4, 0);

        // BASt-Fromat:
        _toolkit.createLabel(compAllgemeineAngaben, "BASt-Band Import Format:", SWT.NONE);

        Composite compBastFormat = _toolkit.createComposite(compAllgemeineAngaben, SWT.BORDER);

        compBastFormat.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        _toolkit.paintBordersFor(compBastFormat);

        GridLayout gl_compBastFormat = new GridLayout(2, false);

        gl_compBastFormat.marginHeight = 3;
        compBastFormat.setLayout(gl_compBastFormat);
        _rbtnBast2004 = _toolkit.createButton(compBastFormat, "BASt 2004", SWT.RADIO);
        _rbtnBast2004.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnBastFormatSelektiert();
            }
        });
        _rbtnBast2007 = _toolkit.createButton(compBastFormat, "BASt 2007", SWT.RADIO);
        _rbtnBast2007.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnBastFormatSelektiert();
            }
        });
        new Label(compAllgemeineAngaben, SWT.NONE);

        //
        Label lbTyp = new Label(compAllgemeineAngaben, SWT.NONE);

        _toolkit.adapt(lbTyp, true, true);
        lbTyp.setText("Typ:");
        _txtTyp = new Text(compAllgemeineAngaben, SWT.BORDER);
        _txtTyp.setEnabled(false);
        _txtTyp.setEditable(false);
        _txtTyp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        _toolkit.adapt(_txtTyp, true, true);
        new Label(compAllgemeineAngaben, SWT.NONE);

        Label lbAtt = new Label(compAllgemeineAngaben, SWT.NONE);

        _toolkit.adapt(lbAtt, true, true);
        lbAtt.setText("Attributgruppe:");
        _txtAtg = new Text(compAllgemeineAngaben, SWT.BORDER);
        _txtAtg.setEnabled(false);
        _txtAtg.setEditable(false);
        _txtAtg.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        _toolkit.adapt(_txtAtg, true, true);
        new Label(compAllgemeineAngaben, SWT.NONE);

        Label lbAsp = new Label(compAllgemeineAngaben, SWT.NONE);

        _toolkit.adapt(lbAsp, true, true);
        lbAsp.setText("Aspekt:");
        _txtAsp = new Text(compAllgemeineAngaben, SWT.BORDER);
        _txtAsp.setEditable(false);
        _txtAsp.setEnabled(false);
        _txtAsp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        _toolkit.adapt(_txtAsp, true, true);
        new Label(compAllgemeineAngaben, SWT.NONE);

        Label lbAuswahlLZZ = new Label(compAllgemeineAngaben, SWT.NONE);

        _toolkit.adapt(lbAuswahlLZZ, true, true);
        lbAuswahlLZZ.setText("Langzeitz\u00E4hlstellenauswahl:");

        Button btnZaehlStellenAuswahl = new Button(compAllgemeineAngaben, SWT.NONE);

        btnZaehlStellenAuswahl.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        btnZaehlStellenAuswahl.setAlignment(SWT.LEFT);
        _toolkit.adapt(btnZaehlStellenAuswahl, true, true);
        btnZaehlStellenAuswahl.setText("Langzeitzhlstellenauswahl...");
        btnZaehlStellenAuswahl.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnLangzeitzaehlstellenAuswahlSelektiert();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _lbLangZeitZaehlStellenAuswahl = new Label(compAllgemeineAngaben, SWT.NONE);
        _lbLangZeitZaehlStellenAuswahl.setText("selektion");
        _toolkit.adapt(_lbLangZeitZaehlStellenAuswahl, true, true);

        //
        Composite compSchaltfaechen = new Composite(scrolledForm.getBody(), SWT.NONE);

        compSchaltfaechen.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        compSchaltfaechen.setLayout(new GridLayout(12, false));
        _toolkit.adapt(compSchaltfaechen);
        _toolkit.paintBordersFor(compSchaltfaechen);

        //
        _btnTablNeuLaden = new Button(compSchaltfaechen, SWT.NONE);
        _btnTablNeuLaden.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnTabelleNeuLadenSelektiert();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _toolkit.adapt(_btnTablNeuLaden, true, true);
        _btnTablNeuLaden.setText("Tabelle neu laden");
        _btnTablNeuLaden.setToolTipText("In der Tabelle wird der aktuelle Zustand des BASt-Band-Imports fr die selektierten Zhlstellen und den angegeben Zeitbereich dargestellt");
        _btnVollstaendigePruefung = new Button(compSchaltfaechen, SWT.NONE);
        _toolkit.adapt(_btnVollstaendigePruefung, true, true);
        _btnVollstaendigePruefung.setText("Vollstndige Prfung");
        _btnVollstaendigePruefung.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnVollstaendigePruefungSelektiert();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _btnVollstaendigePruefung.setToolTipText("Der aktuelle Zustand des BASt-Band-Imports wird fr die selektierten Zeilen vollstdig geprft.");

        //
        _toolkit.createLabel(compSchaltfaechen, "Filter:", SWT.NONE);
        _btnFilterFehlende = _toolkit.createButton(compSchaltfaechen, "Fehlende", SWT.NONE);
        _btnFilterFehlende.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnFilterFehlendeSelektiert();
            }
        });
        _btnFilterAlle = _toolkit.createButton(compSchaltfaechen, "Alle", SWT.NONE);
        _btnFilterAlle.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnFilterAlleSelektiert();
            }
        });
        _btnFilterSelektierte = _toolkit.createButton(compSchaltfaechen, "Selektierte", SWT.NONE);
        _btnFilterSelektierte.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnFilterSelektierteSelektiert();
            }
        });
        _toolkit.createLabel(compSchaltfaechen, "Selektion:", SWT.NONE);
        _btnSelektionFehlende = _toolkit.createButton(compSchaltfaechen, "Fehlende", SWT.NONE);
        _btnSelektionFehlende.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnSelektionFehlendeSelektiert();
            }
        });
        _btnSelektionAlle = _toolkit.createButton(compSchaltfaechen, "Alle", SWT.NONE);
        _btnSelektionAlle.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnSelektionAlleSelektiert();
            }
        });
        _btnSelektionKeine = _toolkit.createButton(compSchaltfaechen, "Keine", SWT.NONE);
        _toolkit.createLabel(compSchaltfaechen, "Export:", SWT.NONE);
        _btnExport = _toolkit.createButton(compSchaltfaechen, "Export in csv...", SWT.NONE);
        _btnExport.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnExportSelektiert();
            }
        });
        _btnSelektionKeine.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnSelektionKeineSelektiert();
            }
        });

        Section sectionTabelle = _toolkit.createSection(scrolledForm.getBody(), Section.TWISTIE | Section.TITLE_BAR);

        sectionTabelle.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1));
        _toolkit.paintBordersFor(sectionTabelle);
        sectionTabelle.setText("Tabellendarstellung");
        sectionTabelle.setExpanded(true);

        Composite compTabelle = new Composite(sectionTabelle, SWT.NONE);

        _toolkit.adapt(compTabelle);
        _toolkit.paintBordersFor(compTabelle);
        sectionTabelle.setClient(compTabelle);
        compTabelle.setLayout(new GridLayout());

        Table table = new Table(compTabelle, SWT.BORDER | SWT.CHECK | SWT.FULL_SELECTION | SWT.MULTI);

        table.addMouseTrackListener(new MouseTrackAdapter() {
            @Override
            public void mouseHover(MouseEvent e) {
                _guiVew.mausUeberTabelle(e);
            }
        });
        table.setLinesVisible(true);
        table.setHeaderVisible(true);
        table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        _toolkit.adapt(table);
        _toolkit.paintBordersFor(table);

        //
        _tableViewer = new TableViewer(table);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 28.10.13
