/*
 * @(#)AggregationDtvGUIVew.java   17.09.10
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.lzzs.aggregation;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;

import de.inovat.buv.gtm.datvew.lib.DatenVew;
import de.inovat.buv.gtm.datvew.lib.SelektionVew;
import de.inovat.buv.gtm.datvew.lib.ZaehlStelle;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;
import de.inovat.buv.inovat.lib.gui.GuiFunktionen;
import de.inovat.buv.plugin.gtm.lzzs.Activator;
import de.inovat.buv.plugin.gtm.lzzs.ViewAggregationDtv;
import de.inovat.buv.plugin.gtm.navigation.lib.LangzeitzaehlstellenAuswahlFunktionen;
import de.inovat.buv.projektlib.dav.DavDatenVew;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Verwaltung des GUI 'Aggregation DTV'.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 174 $ / $Date: 2012-09-12 17:04:43 +0200 (Mi, 12 Sep 2012) $ / ($Author: LG $)
 */
public class AggregationDtvGUIVew {
    private static final String ASPEKT_FUER_STATUS    = "asp.aggrLzzsDtvStatus";
    private static final String ASPEKT_FUER_STEUERUNG = "asp.anfrage";
    private static final String ATG_AGGR_STATUS       = "atg.aggrLzzsDtvStatus";
    private static final String ATG_AGGR_STEUERUNG    = "atg.aggrLzzsDtvSteuerung";

    //~ FELDER ================================================================

    /** Liste der selektierten Zhlstellen. */
    List<ZaehlStelle> _listeZs = new ArrayList<ZaehlStelle>();

    /** Datenverteilerverbindung. */
    private ClientDavInterface _dav;

    /** Die Datenverteiler-Konfiguration. */
    private DataModel _davKonf;

    /** GUI-Instanz. */
    private AggregationDtvGUI _gui;

    /** AggregationVerwaltungsObjekt. */
    private SystemObject _verwaltungsObjektAggrLzzs;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse AggregationDtvGUIVew.
     *
     * @param gui GUI-Instanz.
     */
    protected AggregationDtvGUIVew(AggregationDtvGUI gui) {
        _gui = gui;

        // ------------------------------------------------------------------
        // Datenverteiler Zugriff holen:
        _dav     = DavDatenVew.getInstanz().getDav();
        _davKonf = _dav.getDataModel();
    }

    //~ METHODEN ==============================================================

    /**
     * Melde das AggregationVerwaltungsObjekt von DAV ab.
     */
    public void abmeldenDav() {
        DataDescription datenBeschreibung1 = new DataDescription(_davKonf.getAttributeGroup(ATG_AGGR_STEUERUNG), _davKonf.getAspect(ASPEKT_FUER_STEUERUNG));
        DataDescription datenBeschreibung2 = new DataDescription(_davKonf.getAttributeGroup(ATG_AGGR_STATUS), _davKonf.getAspect(ASPEKT_FUER_STATUS));

        try {
            _dav.unsubscribeSender(_gui, _verwaltungsObjektAggrLzzs, datenBeschreibung1);
            _dav.unsubscribeReceiver(_gui, _verwaltungsObjektAggrLzzs, datenBeschreibung2);
        }
        catch (Exception e) {
            Log.zeigeInterneMultiMeldung(Activator.PLUGIN_ID, "Probleme beim Abmelden der Systemobejkte vom Datenverteiler.", e);
            MessageDialog.openError(_gui.getShell(), "Fehler", "Probleme beim Abmelden der Systemobejkte vom Datenverteiler.");
        }
    }

    /**
     * Melde das GUI auf die nderungen von AggregationDtvStatus und
     * AggregationDtvSteuerung an.
     */
    protected void anmeldenAggregationDtvAufSendenUndEmfang() {

        // -----------------------------------------------------------------------------
        // Initialisiere das AggregationVerwaltungsObjekt ber Aufruf Parameter.
        String par          = "-pidVerwaltungsObjektAggrLzzsDtv";
        String pidVewObjekt = DavDatenVew.getInstanz().getAufrufParameterAlsString(par);

        if (pidVewObjekt == null) {
            pidVewObjekt = "aggrLzzsDtv.vewStandard1";
        }

        _verwaltungsObjektAggrLzzs = _davKonf.getObject(pidVewObjekt);

        // Prfe den Wert:
        if (_verwaltungsObjektAggrLzzs != null) {
            if (!_verwaltungsObjektAggrLzzs.isOfType("typ.aggrLzzsDtv")) {
                String meldung = String.format(                                                                                                                                                                                       //
                    "VerwaltungsObjekt fr Aggregation Langzeitzhlstellen [%s=%s] ist nicht vom Typ [typ.aggrLzzsDtv] sondern vom Typ [%s].%nVerwaltungsobjekt wird nicht weiter verwendet. Die Sicht ist damit funktionsunfhig.",  //
                    par, pidVewObjekt, _verwaltungsObjektAggrLzzs.getType().getPid());

                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung));
                MessageDialog.openError(_gui.getShell(), "Fehler", meldung);
                _verwaltungsObjektAggrLzzs = null;
            }
        } else {
            String meldung = String.format(                                                                                                                                                                                           //
                "VerwaltungsObjekt fr Aggregation Langzeitzhlstellen [%s=%s] konnte nicht ermittelt werden.%nVerwaltungsobjekt wird nicht weiter verwendet. Die Sicht ist damit funktionsunfhig.",                                 //
                par, pidVewObjekt);

            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung));
            MessageDialog.openError(_gui.getShell(), "Fehler", meldung);
        }

        // -----------------------------------------------------------------------------
        if (_verwaltungsObjektAggrLzzs != null) {

            // -----------------------------------------------------------------------------
            // Anmeldung des Verwaltungsobjekts und der ATG zum Senden.
            DataDescription datenBeschreibung;

            // Anmeldung auf das Versenden von externen Vorgaben ber DAV:
            datenBeschreibung = new DataDescription(_davKonf.getAttributeGroup(ATG_AGGR_STEUERUNG), _davKonf.getAspect(ASPEKT_FUER_STEUERUNG));

            try {
                _dav.subscribeSender(_gui, _verwaltungsObjektAggrLzzs, datenBeschreibung, SenderRole.sender());
            }
            catch (Exception e) {
                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Es ist ein Fehler bei der Anmeldung als Sender aufgetretten.", e));
                MessageDialog.openError(_gui.getShell(), "Fehler", "Es ist ein Fehler bei der Anmeldung als Sender aufgetretten (s.Meldung unten im ErrorLog).");
            }

            // -----------------------------------------------------------------------------
            // Anmeldung des Verwaltungsobjekts und der ATG zum Empfangen.
            // Anmeldung auf Empfang von Statusinformationen / Fortschrittsanzeige:
            datenBeschreibung = new DataDescription(_davKonf.getAttributeGroup(ATG_AGGR_STATUS), _davKonf.getAspect(ASPEKT_FUER_STATUS));
            _dav.subscribeReceiver(_gui, _verwaltungsObjektAggrLzzs, datenBeschreibung, ReceiveOptions.normal(), ReceiverRole.receiver());
        }
    }

    /**
     * Schaltflche 'Aggregation' wird selektiert.
     */
    protected void btnAggregationSelektiert() {

        // -----------------------------------------------------------------
        // Sende den AggregationAuftrag:
        // Lese die Daten aus dem GUI aus:
        long         von                = _gui.getDzVon().getDatum().getTime();
        long         bis                = _gui.getDzBis().getDatum().getTime();
        boolean      berechneDTVTag     = _gui.getCbtnDtvTag().getSelection();
        boolean      berechneDTVMonat   = _gui.getCbtnDtvMonat().getSelection();
        boolean      berechneDTVQuartal = _gui.getCbtnDtvQuartal().getSelection();
        boolean      berechneDTVJahr    = _gui.getCbtnDtvJahr().getSelection();
        boolean      berechneMSVJahr    = _gui.getCbtnMsvJahr().getSelection();
        List<String> listeLzzs          = new ArrayList<String>();

        for (ZaehlStelle zs : _listeZs) {
            listeLzzs.add(zs.getSystemObjekt().getPidOrId());
        }

        // Datensatz fr die Steuerung:
        Data daten = _dav.createData(_davKonf.getAttributeGroup(ATG_AGGR_STEUERUNG));

        daten.getTimeValue("von").setMillis(von);
        daten.getTimeValue("bis").setMillis(bis);
        daten.getTextValue("berechneDTVTag").setText(berechneDTVTag ? "Ja" : "Nein");
        daten.getTextValue("berechneDTVMonat").setText(berechneDTVMonat ? "Ja" : "Nein");
        daten.getTextValue("berechneDTVQuartal").setText(berechneDTVQuartal ? "Ja" : "Nein");
        daten.getTextValue("berechneDTVJahr").setText(berechneDTVJahr ? "Ja" : "Nein");
        daten.getTextValue("berechneMSVJahr").setText(berechneMSVJahr ? "Ja" : "Nein");

        Array liste = daten.getArray("Lzzs");

        liste.setLength(listeLzzs.size());

        for (int i = 0; i < listeLzzs.size(); i++) {
            liste.getItem(i).asTextValue().setText(listeLzzs.get(i));
        }

        // Sende die Daten:
        DataDescription datenBeschreibung = new DataDescription(_davKonf.getAttributeGroup(ATG_AGGR_STEUERUNG), _davKonf.getAspect(ASPEKT_FUER_STEUERUNG));
        boolean         erfolgreich       = true;

        try {
            _dav.sendData(new ResultData(_verwaltungsObjektAggrLzzs, datenBeschreibung, System.currentTimeMillis(), daten));
        }
        catch (SendSubscriptionNotConfirmed e) {
            erfolgreich = false;
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Versuch des Versands von Steurunginformationen ohne Sendefreigabe", e));
        }

        // -----------------------------------------------------------------
        // Meldung:
        if (erfolgreich) {
            MessageDialog.openInformation(_gui.getShell(), "Information", "Der Auftrag wurde versandt." + NEWLINE + NEWLINE + "Die Fortschrittanzeige und Meldung werden eventuell nicht angezeigt, da die Auftragsabarbeitung eventuell erst zu einem spteren Zeitpunkt stattfindet.");

            // ------------
            // Blockiere Aggregation-Button fr wenige Sekunden:
            _gui.getBtnSenden().setEnabled(false);

            // Die Pause wird im parallelen thread angelegt
            // und dann wird den EnableWert fr Button gesetzt:
            Thread tBlockieren = new Thread() {
                @Override
                public void run() {
                    try {
                        Thread.sleep(4000);
                        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                            @Override
                            public void run() {
                                if (!_gui.isDisposed()) {
                                    setzeEnabledWertFuerAggregationButton();
                                }
                            }
                        });
                    }
                    catch (InterruptedException e) {
                        Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                    }
                }
            };

            tBlockieren.start();
        }

        // FehlerMeldung:
        else {
            MessageDialog.openError(_gui.getShell(), "Fehler", "Der Auftrag wurde nicht versandt (siehe Meldung unten im ErrorLog).");
        }
    }

    /**
     * Die Methode schliet das aktuelle gui.
     */
    protected void btnSchliessenSelektiert() {
        GuiFunktionen.schliesseGUI(_gui.getParent(), ViewAggregationDtv.ID);
    }

    /**
     * Schaltflche 'Alle Zhlstellen auswhlen' wird selektiert.
     */
    protected void btnZaehlstellenAuswahlAlleSelektiert() {

        // Selektiere alle:
        _listeZs = new ArrayList<ZaehlStelle>();
        _listeZs.addAll(DatenVew.getInstanz().getListeZaehlStellen());

        // Zeige die Anzahl der selektierten Langzeitzaehlstellen:
        zeigeAnzahlSelektierterZaehlstellen();

        // Setze EnabledWert fr AggregationButton:
        setzeEnabledWertFuerAggregationButton();
    }

    /**
     * Schaltflche 'ZhlstellenAuswahl' wird selektiert.
     */
    protected void btnZaehlstellenAuswahlSelektiert() {

        // Zeige den Dialog:
        _listeZs = LangzeitzaehlstellenAuswahlFunktionen.langzeitZaehlstellenAuswahlDialogOeffnen(_gui.getShell(), _listeZs, null);

        // Zeige die Anzahl der selektierten Langzeitzaehlstellen:
        zeigeAnzahlSelektierterZaehlstellen();

        // Setze EnabledWert fr AggregationButton:
        setzeEnabledWertFuerAggregationButton();
    }

    /**
     * Checkbox fr einen der Aggregation-Parameter wird selektiert.
     */
    protected void cbtnAggrParSelektiert() {
        setzeEnabledWertFuerAggregationButton();
    }

    /**
     * Datum wird Selektiert.
     */
    protected void dzDatumSelektiert() {
        setzeEnabledWertFuerAggregationButton();
    }

    /**
     * Initialisiere das GUI.
     */
    protected void initialisiereGui() {

        // Initialisiere die Liste der Zhlstellen mit den
        // zuvor im NavigationBaum selektierten Objekten:
        _listeZs = SelektionVew.getInstanz().ermittleListeSelektierterZaehlStellen();
        zeigeAnzahlSelektierterZaehlstellen();

        // Initialisiere das Zeitintervall (von Anfang des letzten Monats bis Ende des letzten Monats):
        long letztenMonat = DatumFunktionen.getVerschobenenDateTime(DatumFunktionen.getInitDateTime(), -1, 0);

        _gui.getDzVon().setDatum(new Date(DatumFunktionen.getMonatAnfang(letztenMonat)));
        _gui.getDzBis().setDatum(new Date(DatumFunktionen.getMonatEnde(letztenMonat)));

        // Initialisiere AggregationParameter:
        _gui.getCbtnDtvTag().setSelection(true);
        _gui.getCbtnDtvMonat().setSelection(true);

        // Setze EnabledWert fr AggregationButton:
        setzeEnabledWertFuerAggregationButton();
    }

    /**
     * Setze EnabledWert fr AggregationButton.
     */
    private void setzeEnabledWertFuerAggregationButton() {

        // mindestens ein Parameter selektiert:
        boolean istPar = _gui.getCbtnDtvJahr().getSelection() || _gui.getCbtnDtvMonat().getSelection() || _gui.getCbtnDtvQuartal().getSelection() || _gui.getCbtnDtvTag().getSelection() || _gui.getCbtnMsvJahr().getSelection();

        // mindestens eine Zhlselle selektiert:
        boolean istZs = (_listeZs.size() > 0);

        // vonDatum <= bisDatum:
        boolean istDatum = (_gui.getDzVon().getDatum().getTime() <= _gui.getDzBis().getDatum().getTime());

        // AggregationProzess ist erlaubt, wenn alle Eingaben korrekt sind:
        _gui.getBtnSenden().setEnabled(istZs && istDatum && istPar);
    }

    /**
     * Reagiere auf die nderungen von Statusinformationen / Fortschrittsanzeige.
     *
     * @param results Neue Werte.
     */
    protected void update(ResultData[] results) {
        for (ResultData ergebnis : results) {
            if (ergebnis.hasData()) {

                // Lese die Daten:
                final int    fortschrittProzent = ergebnis.getData().getUnscaledValue("FortschrittProzent").intValue();
                final String statusMeldung      = ergebnis.getData().getTextValue("StatusMeldung").getText();

                // Aktualisiere das GUI:
                PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        if (!_gui.isDisposed()) {

                            // Fortschrittanzeige:
                            _gui.getPrgFortschrittsanzeige().setSelection(fortschrittProzent);

                            // Meldung:
                            StringBuilder meldung = new StringBuilder(_gui.getTxtMeldungen().getText());

                            meldung.append(statusMeldung);
                            meldung.append(NEWLINE).append("-------------------").append(NEWLINE);
                            _gui.getTxtMeldungen().setText(meldung.toString());
                            _gui.getTxtMeldungen().setSelection(_gui.getTxtMeldungen().getCharCount());
                        }
                    }
                });
            }
        }
    }

    /**
     * Zeige die Anzahl der selektierten Langzeitzaehlstellen.
     */
    private void zeigeAnzahlSelektierterZaehlstellen() {
        String text;

        // alle selektiert:
        if (_listeZs.size() == DatenVew.getInstanz().getListeZaehlStellen().size()) {
            text = "Es sind alle Langzeitzhlstellen selektiert.";
        }

        // keine selektiert:
        else if (_listeZs.size() == 0) {
            text = "Es sind keine Langzeitzhlstellen selektiert.";
        }

        // einige selektiert:
        else {
            text = "Anzahl der selektierten Langzeitzhlstellen: " + _listeZs.size();
        }

        _gui.getLbZaehlstellenAnzahl().setText(text);

        //
        _gui.getSectionSteuerung().setExpanded(true);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 22.10.13
