/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.plugin.gtm.hilfe.HilfeVew;
import de.inovat.buv.plugin.gtm.navigation.gui.NavigationGUI;

import org.eclipse.jface.action.Action;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.PlatformUI;

//~ KLASSEN ===================================================================

/**
 * Die Sicht 'Navigation'.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class ViewNavigation extends ViewPart {

    /** ID der Sicht 'Projektnavigation'. */
    public static final String ID = "de.inovat.buv.plugin.gtm.navigation.view";

    //~ FELDER ================================================================

    /** NavigationGUI-Instanz */
    private NavigationGUI _gui;

    //~ METHODEN ==============================================================

    @Override
    public void createPartControl(Composite parent) {

        // Hilfe
        PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, HilfeVew.getInstanz().ermittleIdFuerAnker(HilfeVew.ANKER_NAVIGATION));
        _gui = new NavigationGUI(parent, 0);

        // Alternative fr SelektionsTransfer:
        // Selektion im Baum wird fr andere Plugins ber den Eclipse-SelectionService erreichbar.
        // Z.B. getSite().getWorkbenchWindow().getSelectionService().getSelection()
        getSite().setSelectionProvider(_gui.getTreeViewer());

        // Untersttzung der COPY-Aktion (Ctrl-C) fr die Sicht
        // (erstes selektiertes Knoten wird in Buffer gespeichert):
        getViewSite().getActionBars().setGlobalActionHandler(ActionFactory.COPY.getId(), new Action() {
            @Override
            public void run() {
                try {
                    String       txt          = _gui.getTreeViewer().getTree().getSelection()[0].getText();
                    TextTransfer textTransfer = TextTransfer.getInstance();
                    Clipboard    cb           = new Clipboard(Display.getCurrent());

                    cb.setContents(new Object[] { txt }, new Transfer[] { textTransfer });
                    cb.dispose();
                }
                catch (Exception ignored) {}
            }
        });
    }

    //~ SET METHODEN ==========================================================

    @Override
    public void setFocus() {
        _gui.setFocus();
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 06.10.14
