/*
 * @(#)GTMDialogGUI.java   08.03.13
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.gtmdiag;

//~ NICHT JDK IMPORTE =========================================================

import com.swtdesigner.SWTResourceManager;

import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.dav.daf.main.ResultData;

import de.inovat.buv.gtm.datvew.aktionen.AktionenGTM;
import de.inovat.buv.gtm.datvew.lib.BaumHilfsFunktionen;
import de.inovat.buv.inovat.lib.gui.GuiFunktionen;
import de.inovat.buv.plugin.gtm.navigation.archiv.ArchivanfrageDaten;
import de.inovat.buv.plugin.gtm.navigation.archiv.ArchivanfrageDatenGUIVew;
import de.inovat.buv.plugin.gtm.navigation.tabellen.ZeilenObjekt;
import de.inovat.buv.plugin.gtm.navigation.tabellenverwaltung.AlleTabellenVerwaltungen;
import de.inovat.buv.plugin.gtm.navigation.tabellenverwaltung.TabellenVerwaltung;
import de.inovat.buv.plugin.gtm.navigation.tabellenverwaltung.TabellenVerwaltungEintrag;
import de.inovat.buv.plugin.gtm.navigation.ViewFilter;
import de.inovat.buv.plugin.gtm.navigation.ViewSortierung;
import de.inovat.buv.plugin.gtm.navigation.ViewSpaltenauswahl;
import de.inovat.buv.projektlib.rechtevew.aktionen.AktionenVew;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.PlatformUI;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Iterator;

//~ KLASSEN ===================================================================

/**
 *  Die Klasse erzeugt das GUI der Tabellendarstellung.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class GTMDialogGUI extends Composite implements ClientReceiverInterface, Runnable {

    /** Konstante Defaulteinstellung Maximalanzahl Zeilen. */
    protected static final int DEFAULT_MAX_ZEILEN = 1000;

    /** Konstante Defaulteinstellungen Variable Anzahl Attribute. */
    protected static final int DEFAULT_VAR_ATT = 0;

    /** Konstante Defaulteinstellung Increment Spinner Max Zeilen. */
    private static final int INCREMENT_MAX_ZEILEN = 10;

    /** Konstante Defaulteinstellungen Increment Spinner Variable Anzahl Attribute. */
    private static final int INCREMENT_VAR_ATT = 1;

    /** Konstante Maximalzahl der Zeilen in der Tabelle. */
    private static final int MAXIMUM_MAX_ZEILEN = 10000;

    /** Konstante Maximalzahl Variable Anzahl Attribute. */
    private static final int MAXIMUM_VAR_ATT = 255;

    //~ FELDER ================================================================

    /** ArrayList die die noch nicht abgearbeiteteten Resultdata enthlt. */
    private ArrayList<ResultData[]> _arListResultData = new ArrayList<ResultData[]>();

    /** Tabellen View fr die Tabelle. */
    private TableViewer _tableViewer = null;

    /** FormToolkit. */
    private final FormToolkit _toolkit = new FormToolkit(Display.getCurrent());

    /** Flag. Zeigt an, ob die Tabelle schon mal gefllt wurde. */
    private boolean _tabelleSchonMalGefuellt = false;

    /**
     * Attribut, das den Zustand des Start/Stopp buttons reprsentiert.
     * _start == false, wenn zur Zeit keine Daten beim Datenverteiler angemeldet sind.
     * _start == true, wenn die aktuelle Auswahl der Objekte beim Datenverteiler zum Empfang angemeldet sind.
     */
    protected boolean _start = false;

    /** Ein Thread wird unterbrochen dadurch dass der _blinker = null gesetzt wird. */
    @SuppressWarnings("unused")
    private volatile Thread _blinker;

    /** Schaltflche nderungsdarstellung. */
    private Button _btnAenderungsDarstellung;

    /** Button Drucken. */
    private Button _btnDrucken;

    /** Button Export Csv. */
    private Button _btnExportInCsv;

    /** Button Export Html. */
    private Button _btnExportInHtml;

    /** Button Speichern. */
    private Button _btnSpeichern;

    /** Button Speichern unter. */
    private Button _btnSpeichernUnter;

    /** Button StartStopp. */
    private Button _btnStartStopp;

    /** Button VerwaltungDialog ffnen. */
    private Button _btnVerwOeffnen;

    /** Checkbox fr die Archivanfrageoption. */
    private Button _checkArchiv;

    /** Checkbox Konfigurationsdaten. */
    private Button _checkKonfDaten;

    /**
     * Combobox Aspekt.
     */
    private Combo _comboAsp;

    /** Combobox Attributgruppe. */
    private Combo _comboAtg;

    /** Combobox Darstellungsname. */
    private Combo _comboDarstellungsName;

    /** Combobox Filter. */
    private Combo _comboFilter;

    /** Combobox Sortierrung. */
    private Combo _comboSortierung;

    /** Combobox Spalten. */
    private Combo _comboSpalten;

    /** Composite fr die Benutzerangaben. */
    private Composite _compBenutzerAngaben;

    /** Instanz der GUI Verwaltung. */
    private GTMDialogGUIVerwaltung   _guiVew;
    private ArchivanfrageDatenGUIVew _instanzArchivVew;

    /** Objekttyp. */
    private Object _objTyp;

    /** Scrolled Composite fr das gesamte GUI. */
    private ScrolledComposite _scrolledComposite;

    /** Bereich fr die archivspezifische Dateneingabe. */
    private Section _sectionArchiv;
    private Section _sectionTabelle;

    /** Spinner fr die maximale Anzahl der variablen Attribute. */
    private Spinner _spinnerAnzahlVarAtt;

    /** Spinner fr die maximale Anzahl der Tabellenzeilen. */
    private Spinner _spinnerMaxZeilen;

    /** Tabellendatenstruktur. */
    private TabellenDatenStruktur _tabellenDatenStruktur;

    /** Attribut das angibt, ob der Import aktiv ist. */
    private boolean _threadImportAktiv;

    /**
     * Thread resultdataberwachung. Soblad resultdaten eintreffen werden diese zwischengespeichert,
     * um die update Methode nicht fr andere Data Objekte zu blocken. Um dies zu realisieren wird
     * ein Thread bentigt, der die update Methode beobachtet, und sobald Daten eintreffen den Prozess
     * in Gang setzt, der diese zwischenspeichert.
     */
    private Thread _threadResultDataUeberwachung;

    /** Textfeld fr die Auswahl der aktuell gewhlten Knoten. */
    private Text _txtGewaehlteKnoten;

    /** Textfeld Objekttyp. */
    private Text _txtObjTyp;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor initialisiert.
     * @param parent aufrufende Composite.
     */
    public GTMDialogGUI(Composite parent) {
        super(parent, SWT.NONE);

        //
        _guiVew           = new GTMDialogGUIVerwaltung(this);
        _instanzArchivVew = new ArchivanfrageDatenGUIVew(_guiVew);
        _guiVew.setInstanzArchivVew(_instanzArchivVew);

        SystemObjectType typ = BaumHilfsFunktionen.bestimmeTypDerListe(BaumHilfsFunktionen.getListeObjekte());

        if (typ == null) {
            _objTyp = "";
        } else {
            _objTyp = typ.getPidOrId();
        }

        //
        setLayout(new GridLayout(1, false));
        _toolkit.adapt(this);
        _toolkit.paintBordersFor(this);
        initGUI();

        //
        _instanzArchivVew.setzeGUIbereich(new ArchivanfrageDaten());

        //
        _guiVew.keineDarstellungGewaehltObjFuellen();

        // DarstellungsVorauswahl:
        String vorauswahl = BaumHilfsFunktionen.getTABELLEN_DARSTELLUNGS_VORAUSWAHL();

        if (vorauswahl != null) {

            // Ermittle die TabellenVerwaltung fr die Vorauswahl:
            TabellenVerwaltung tVew = AlleTabellenVerwaltungen.getTabellenVerwaltungInstanz().getTabellenVerwaltungsObjekt(vorauswahl);

            if (tVew != null) {

                // Prfe die Einstellungen fr Archiv und Konfiguruierende Daten:
                boolean istArchiv = false;
                boolean istKonf   = false;

                for (TabellenVerwaltungEintrag te : tVew.getTabellenVerwaltungsEintrag()) {
                    istArchiv = te.getDatenidentifikation().isArchivDaten();
                    istKonf   = te.getDatenidentifikation().isKonfigurierendeDaten();
                }

                if (istArchiv) {
                    _checkArchiv.setSelection(true);
                    _guiVew.checkArchivDaten();
                } else if (istKonf) {
                    _checkKonfDaten.setSelection(true);
                    _guiVew.checkKonfDaten();
                }

                // Selektiere Darstellung:
                int index = -1;
                int i     = 0;

                for (String name : _comboDarstellungsName.getItems()) {
                    if (name.equals(vorauswahl)) {
                        index = i;

                        break;
                    }

                    i++;
                }

                if (index > -1) {
                    _comboDarstellungsName.select(index);
                    comboDarstellungsNameSelektiert();
                }
            }
        }

        //
        starteThread();
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode abmeldenDav leitet die Abmeldung aus dem View an die Verwaltung weiter.
     * Desweiteren wird der Thread fr die Abarbeitung der ResultData beendet.
     */
    public void abmeldenDavBeendenThread() {
        _guiVew.abmeldenDav();
        stoppeThread();
    }

    /**
     * Die Methode setzt - bei Fokuserhalt - den Darstellungsnamen auf Benutzerspezfisich, wenn sich der Inhalt des GUI's im Vergleich
     * zum gespeicherten Darstellungsnamen gendert hat. Zudem werden die letzten nderungen
     * der Tabellenverwaltung auf die Tabellendarstellung angewendet.
     */
    public void aktualisiereDarstellungBeiFocuserhalt() {
        boolean aenderung = _guiVew.aenderungenEinerGespeichertenDarstellung();

        if (aenderung) {
            if (_btnSpeichern.isEnabled()) {
                for (int i = 0; i < _comboDarstellungsName.getItemCount(); i++) {
                    if (_comboDarstellungsName.getItem(i).equals(GTMDialogGUIVerwaltung.BENUTZERSPEZIFISCH)) {
                        _comboDarstellungsName.select(i);
                        _btnSpeichern.setEnabled(false);
                    }
                }
            } else {
                _guiVew.aktualisiereDarstellungBeiFocuserhalt();
            }
        }
    }

    /**
     * Die Methode anzeigenArchivBereich sorgt dafr, dass die Section Archivbereich
     * je nach Flag ein oder ausgeblendet wird.
     * @param anzeigen boolesche Variable die angibt ob die Section fr den Archivbereich ein oder
     * ausgeblendet werden soll.
     */
    protected void anzeigenArchivBereich(boolean anzeigen) {
        _sectionArchiv.setExpanded(anzeigen);
        _sectionArchiv.setEnabled(anzeigen);
    }

    /**
     * Combo 'DarstellungsName' wird selektiert.
     */
    private void comboDarstellungsNameSelektiert() {
        int selektierterIndex = _comboDarstellungsName.getSelectionIndex();

        if (_comboDarstellungsName.getText().equals(GTMDialogGUIVerwaltung.KEINE_DARSTELLUNG)) {
            initialisiereGUIAufDefaultWerte();
        } else {
            if (_start) {
                _guiVew.btnStartStopp();
            }

            if (selektierterIndex != -1) {
                _comboDarstellungsName.select(selektierterIndex);
                ladeDarstellungsOptionen();
                GewaehlteDarstellung.getInstanz().setGewaehlteDarstellung(_comboDarstellungsName.getText());

                String selektierteAsp = _comboAsp.getText();

                if ((selektierteAsp == null) || (selektierteAsp.isEmpty())) {
                    _btnStartStopp.setEnabled(false);
                } else {
                    _guiVew.aktionTabelleFuellen();
                    _instanzArchivVew.setzeEnabledWertFuerAlleBtnArchivanfrage(true);

                    if (!_checkKonfDaten.getSelection() & !_checkArchiv.getSelection()) {
                        _btnStartStopp.setEnabled(true);
                    }
                }
            }
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'anzahlVarAtt'.
     *
     * @return Wert der Eigenschaft 'anzahlVarAtt'.
     */
    public int getAnzahlVarAtt() {
        return _spinnerAnzahlVarAtt.getSelection();
    }

    /**
     *     Getter fr Eigenschaft 'btnAenderungsDarstellung'.
     *
     *     @return Wert der Eigenschaft 'btnAenderungsDarstellung'.
     *
     *     @see #_btnAenderungsDarstellung
     */
    public Button getBtnAenderungsDarstellung() {
        return _btnAenderungsDarstellung;
    }

    /**
     * Getter fr Eigenschaft 'btnDrucken'.
     *
     * @return Wert der Eigenschaft 'btnDrucken'.
     *
     * @see #_btnDrucken
     */
    public Button getBtnDrucken() {
        return _btnDrucken;
    }

    /**
     * Getter fr Eigenschaft 'btnExportInCsv'.
     *
     * @return Wert der Eigenschaft 'btnExportInCsv'.
     *
     * @see #_btnExportInCsv
     */
    public Button getBtnExportInCsv() {
        return _btnExportInCsv;
    }

    /**
     *     Getter fr Eigenschaft 'btnExportInHtml'.
     *
     *     @return Wert der Eigenschaft 'btnExportInHtml'.
     *
     *     @see #_btnExportInHtml
     */
    public Button getBtnExportInHtml() {
        return _btnExportInHtml;
    }

    /**
     * Getter fr Eigenschaft 'btnSpeichern'.
     *
     * @return Wert der Eigenschaft 'btnSpeichern'.
     *
     * @see #_btnSpeichern
     */
    public Button getBtnSpeichern() {
        return _btnSpeichern;
    }

    /**
     *     Getter fr Eigenschaft 'btnStartStopp'.
     *
     *     @return Wert der Eigenschaft 'btnStartStopp'.
     *
     *     @see #_btnStartStopp
     */
    public Button getBtnStartStopp() {
        return _btnStartStopp;
    }

    /**
     *     Getter fr Eigenschaft 'checkArchiv'.
     *
     *     @return Wert der Eigenschaft 'checkArchiv'.
     *
     *     @see #_checkArchiv
     */
    public Button getCheckArchiv() {
        return _checkArchiv;
    }

    /**
     *     Getter fr Eigenschaft 'checkKonfDaten'.
     *
     *     @return Wert der Eigenschaft 'checkKonfDaten'.
     *
     *     @see #_checkKonfDaten
     */
    public Button getCheckKonfDaten() {
        return _checkKonfDaten;
    }

    /**
     *     Getter fr Eigenschaft 'comboAsp'.
     *
     *     @return Wert der Eigenschaft 'comboAsp'.
     *
     *     @see #_comboAsp
     */
    public Combo getComboAsp() {
        return _comboAsp;
    }

    /**
     * Getter fr Eigenschaft 'comboAtg'.
     *
     * @return Wert der Eigenschaft 'comboAtg'.
     *
     * @see #_comboAtg
     */
    public Combo getComboAtg() {
        return _comboAtg;
    }

    /**
     *     Getter fr Eigenschaft 'comboDarstellungsName'.
     *
     *     @return Wert der Eigenschaft 'comboDarstellungsName'.
     *
     *     @see #_comboDarstellungsName
     */
    public Combo getComboDarstellungsName() {
        return _comboDarstellungsName;
    }

    /**
     * Getter fr Eigenschaft 'comboFilter'.
     *
     * @return Wert der Eigenschaft 'comboFilter'.
     *
     * @see #_comboFilter
     */
    public Combo getComboFilter() {
        return _comboFilter;
    }

    /**
     * Getter fr Eigenschaft 'comboSortierung'.
     *
     * @return Wert der Eigenschaft 'comboSortierung'.
     *
     * @see #_comboSortierung
     */
    public Combo getComboSortierung() {
        return _comboSortierung;
    }

    /**
     *     Getter fr Eigenschaft 'comboSpalten'.
     *
     *     @return Wert der Eigenschaft 'comboSpalten'.
     *
     *     @see #_comboSpalten
     */
    public Combo getComboSpalten() {
        return _comboSpalten;
    }

    /**
     *     Getter fr Eigenschaft 'objTyp'.
     *
     *     @return Wert der Eigenschaft 'objTyp'.
     *
     *     @see #_objTyp
     */
    public String getObjTyp() {
        return _objTyp.toString();
    }

    /**
     *     Getter fr Eigenschaft 'scrolledComposite'.
     *
     *     @return Wert der Eigenschaft 'scrolledComposite'.
     *
     *     @see #_scrolledComposite
     */
    public ScrolledComposite getScrolledComposite() {
        return _scrolledComposite;
    }

    /**
     *     Getter fr Eigenschaft 'sectionTabelle'.
     *
     *     @return Wert der Eigenschaft 'sectionTabelle'.
     *
     *     @see #_sectionTabelle
     */
    public Section getSectionTabelle() {
        return _sectionTabelle;
    }

    /**
     * Getter fr Eigenschaft 'selektierteSpalte'.
     *
     * @return Wert der Eigenschaft 'selektierteSpalte'.
     */
    public String getSelektierteSpalte() {
        return _comboSpalten.getItem(_comboSpalten.getSelectionIndex());
    }

    /**
     * GUI Tabellenverwaltungseintragsobjekt, das die aktuell im GUI vom Benutzer gewhlten
     * Eingaben reprsentiert. Dieses Objekt wird mit der gespeicherten Darstellung verglichen.
     */

    // private TabellenVerwaltungEintrag _guiTabellenVerwaltungEintragVergleichsObj;

    /**
     *     Getter fr Eigenschaft 'spinnerAnzahlVarAtt'.
     *
     *     @return Wert der Eigenschaft 'spinnerAnzahlVarAtt'.
     *
     *     @see #_spinnerAnzahlVarAtt
     */
    public Spinner getSpinnerAnzahlVarAtt() {
        return _spinnerAnzahlVarAtt;
    }

    /**
     * Getter fr Eigenschaft 'spinnerMaxZeilen'.
     *
     * @return Wert der Eigenschaft 'spinnerMaxZeilen'.
     *
     * @see #_spinnerMaxZeilen
     */
    public Spinner getSpinnerMaxZeilen() {
        return _spinnerMaxZeilen;
    }

    /**
     * Getter fr Eigenschaft 'tableViewer'.
     *
     * @return Wert der Eigenschaft 'tableViewer'.
     *
     * @see #_tableViewer
     */
    public TableViewer getTableViewer() {
        return _tableViewer;
    }

    /**
     *     Getter fr Eigenschaft 'toolkit'.
     *
     *     @return Wert der Eigenschaft 'toolkit'.
     *
     *     @see #_toolkit
     */
    public FormToolkit getToolkit() {
        return _toolkit;
    }

    /**
     *     Getter fr Eigenschaft 'txtGewaehlteKnoten'.
     *
     *     @return Wert der Eigenschaft 'txtGewaehlteKnoten'.
     *
     *     @see #_txtGewaehlteKnoten
     */
    public Text getTxtGewaehlteKnoten() {
        return _txtGewaehlteKnoten;
    }

    /**
     *     Getter fr Eigenschaft 'txtObjTyp'.
     *
     *     @return Wert der Eigenschaft 'txtObjTyp'.
     *
     *     @see #_txtObjTyp
     */
    public Text getTxtObjTyp() {
        return _txtObjTyp;
    }

    //~ METHODEN ==============================================================

    /**
     * Mittels der initGUI Methode wird das gesamte GUI aufgebaut.
     * Der Aufruf der Methoden entspricht dem Aufbau des GUI's, d.h. wenn
     * die Reihenfolge der Bereiche im GUI vertauscht werden soll, mssen hier einfach
     * die Methoden in der Aufrufreihenfolge dementsprechend angepasst werden.
     */
    private void initGUI() {
        _scrolledComposite = new ScrolledComposite(this, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        _scrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        _scrolledComposite.setExpandHorizontal(true);
        _scrolledComposite.setExpandVertical(true);

        //
        initGUI_SectionDarstellungsAuswahl();
        initGUI_SectionDetailinformaionen();

        // -------------------------------------------------
        // Archiv:
        _sectionArchiv = _toolkit.createSection(_compBenutzerAngaben, Section.TWISTIE | Section.TITLE_BAR);
        _sectionArchiv.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        _toolkit.paintBordersFor(_sectionArchiv);
        _sectionArchiv.setText("Archivoptionen");
        _sectionArchiv.setEnabled(false);

        //
        _instanzArchivVew.initGUI(_sectionArchiv, _toolkit);

        //
        initGUI_SectionDarstellungsoptionen();
        initGUI_SektionAnzeigenDerGewaehltenKnotenImBaum();
        initGUI_SektionTabelle();

        //
        _scrolledComposite.setContent(_compBenutzerAngaben);
        _scrolledComposite.setMinSize(_compBenutzerAngaben.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    }

    /**
     * Die Methode erzeugt den GUI Bereich fr die Darstellungsauswahl.
     */
    private void initGUI_SectionDarstellungsAuswahl() {
        _compBenutzerAngaben = new Composite(_scrolledComposite, SWT.NONE);
        _compBenutzerAngaben.setLayout(new GridLayout(1, false));

        Section sectionDarstellun = _toolkit.createSection(_compBenutzerAngaben, Section.TWISTIE | Section.TITLE_BAR);

        sectionDarstellun.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        _toolkit.paintBordersFor(sectionDarstellun);
        sectionDarstellun.setText("Darstellungsauswahl");

        Composite _compAuswahl = new Composite(sectionDarstellun, SWT.NONE);

        sectionDarstellun.setClient(_compAuswahl);
        _toolkit.adapt(_compAuswahl);
        _toolkit.paintBordersFor(_compAuswahl);
        _compAuswahl.setLayout(new GridLayout(6, false));
        _checkArchiv = new Button(_compAuswahl, SWT.CHECK);
        _checkArchiv.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.checkArchivDaten();
            }
        });
        _checkArchiv.setToolTipText("Wenn Sie dieses Flag setzten, erhalten Sie zus\u00E4tzliche Optionen f\u00FCr die Archivdatenauswahl.");
        _toolkit.adapt(_checkArchiv, true, true);
        _checkArchiv.setText("Archiv");

        Label _lbDarstellungsName = new Label(_compAuswahl, SWT.NONE);

        _toolkit.adapt(_lbDarstellungsName, true, true);
        _lbDarstellungsName.setText("Darstellungs Name:");
        _comboDarstellungsName = new Combo(_compAuswahl, SWT.READ_ONLY);
        _comboDarstellungsName.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {}
            @Override
            public void focusGained(FocusEvent e) {
                int selektierterIndex = _comboDarstellungsName.getSelectionIndex();

                _comboDarstellungsName.removeAll();
                _guiVew.erzeugeComboboxDarstellungsName();

                if (selektierterIndex != -1) {
                    _comboDarstellungsName.select(selektierterIndex);
                }
            }
        });
        _comboDarstellungsName.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                comboDarstellungsNameSelektiert();
            }
        });

        GridData gd__comboDarstellungsName = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);

        gd__comboDarstellungsName.widthHint = 298;
        _comboDarstellungsName.setLayoutData(gd__comboDarstellungsName);
        _toolkit.adapt(_comboDarstellungsName);
        _toolkit.paintBordersFor(_comboDarstellungsName);
        _guiVew.erzeugeComboboxDarstellungsName();
        _btnSpeichernUnter = new Button(_compAuswahl, SWT.NONE);
        _btnSpeichernUnter.setToolTipText("Speichert die aktuell erzeugte Darstellung unter einem neuen Namen. ");
        _btnSpeichernUnter.setEnabled(true);
        _toolkit.adapt(_btnSpeichernUnter, true, true);
        _btnSpeichernUnter.setText("Speichern unter...");
        _btnSpeichernUnter.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnSpeichernUnter();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _btnSpeichern = new Button(_compAuswahl, SWT.NONE);

        StringBuilder stringToolTipTextBtnSpeichernUnter = new StringBuilder();

        stringToolTipTextBtnSpeichernUnter.append("Die Modifizierte Darstellungsoption wird \u00FCbernommen, so dass unter dem gleichen");
        stringToolTipTextBtnSpeichernUnter.append(System.getProperty("line.separator"));
        stringToolTipTextBtnSpeichernUnter.append("Darstellungsnamen die neue Darstellung abgespeichert wird.");
        _btnSpeichern.setToolTipText(stringToolTipTextBtnSpeichernUnter.toString());
        _btnSpeichern.setEnabled(false);
        _toolkit.adapt(_btnSpeichern, true, true);
        _btnSpeichern.setText("Speichern...");
        _btnSpeichern.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnSpeichern();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _btnVerwOeffnen = new Button(_compAuswahl, SWT.NONE);
        _btnVerwOeffnen.setToolTipText("Es \u00F6ffnet sich der Verwaltungsdialog.");
        _btnVerwOeffnen.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnVerwOeffnen();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _toolkit.adapt(_btnVerwOeffnen, true, true);
        _btnVerwOeffnen.setText("Verwaltungsdialog \u00F6ffnen");
        sectionDarstellun.setExpanded(true);
    }

    /**
     * Erzeugt den Bereich fr die allgemeinen Einstellungen, d.h. Schaltflchen mittels denen
     * Filter erzeugt werden knnen, oder Sortierungen fr die Tabelle eingestellt werden knnen.
     */
    private void initGUI_SectionDarstellungsoptionen() {
        Section sectionDarstellungsoptionen = _toolkit.createSection(_compBenutzerAngaben, Section.TWISTIE | Section.TITLE_BAR);

        sectionDarstellungsoptionen.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        _toolkit.paintBordersFor(sectionDarstellungsoptionen);
        sectionDarstellungsoptionen.setText("Darstellungsoptionen");

        Composite compDartellungsOptionen = new Composite(sectionDarstellungsoptionen, SWT.NONE);

        sectionDarstellungsoptionen.setClient(compDartellungsOptionen);
        _toolkit.adapt(compDartellungsOptionen);
        _toolkit.paintBordersFor(compDartellungsOptionen);
        compDartellungsOptionen.setLayout(new GridLayout(11, false));

        Label lblStartstopp = new Label(compDartellungsOptionen, SWT.WRAP);

        _toolkit.adapt(lblStartstopp, true, true);
        lblStartstopp.setText("Start\r\nStopp:");

        Label lblnderungsdarstellung = new Label(compDartellungsOptionen, SWT.WRAP);

        _toolkit.adapt(lblnderungsdarstellung, true, true);
        lblnderungsdarstellung.setText("\u00C4nderungs-\r\ndarstellung:");

        Label _lbMaxAtt = new Label(compDartellungsOptionen, SWT.WRAP);

        _toolkit.adapt(_lbMaxAtt, true, true);
        _lbMaxAtt.setText("Variable \r\nAttribute:");

        Label _lbTabellenZeilenZahl = new Label(compDartellungsOptionen, SWT.WRAP);

        _toolkit.adapt(_lbTabellenZeilenZahl, true, true);
        _lbTabellenZeilenZahl.setText("Tabellen-\r\nzeilen:");

        Label lblFilter = new Label(compDartellungsOptionen, SWT.NONE);

        lblFilter.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseDoubleClick(MouseEvent e) {

                // Aufruf des GUI fr die Filtererstellung.
                if (_comboFilter.getSelectionIndex() != -1) {
                    GewaehlteDarstellung.getInstanz().setGewaehlterFilter(_comboFilter.getItem(_comboFilter.getSelectionIndex()));
                } else {
                    GewaehlteDarstellung.getInstanz().setGewaehlterFilter("");
                }

                if (PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().findView(ViewFilter.ID) != null) {
                    GuiFunktionen.schliesseSicht(ViewFilter.ID);
                }

                if (AktionenVew.getInstanz().ausfuehrenAktion(AktionenGTM.COMMAND_ID_FILTER) != null) {
                    MessageDialog.openWarning(getShell(), "Achtung", "Sie haben kein Recht mit dem GUI 'Filter' zu arbeiten!");
                }
            }
        });

        /*
        StringBuilder stringToolTipTextlblFilter = new StringBuilder();

        stringToolTipTextlblFilter.append("Die Filter sind abh\u00E4ngig von der gew\u00E4hlten Attributgruppe,");
        stringToolTipTextlblFilter.append(System.getProperty("line.separator"));
        stringToolTipTextlblFilter.append("d.h. es werden nur die Filter zur Auswahl angeboten,");
        stringToolTipTextlblFilter.append(System.getProperty("line.separator"));
        stringToolTipTextlblFilter.append("die die gleiche Attributgruppe haben.");
        */
        lblFilter.setToolTipText("Durch Doppelklick auf das Label \"Filter\" k\u00F6nnen Sie direkt den Pflegeview Filter \u00F6ffnen.");
        _toolkit.adapt(lblFilter, true, true);
        lblFilter.setText("Filter:");

        Label lbSortierung = new Label(compDartellungsOptionen, SWT.NONE);

        lbSortierung.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseDoubleClick(MouseEvent e) {

                // Aufruf des GUI fr die Sortiererstellung.
                if (!(_comboSortierung.getSelectionIndex() == -1)) {
                    GewaehlteDarstellung.getInstanz().setGewaehlteSortierung(_comboSortierung.getItem(_comboSortierung.getSelectionIndex()));
                } else {
                    GewaehlteDarstellung.getInstanz().setGewaehlteSortierung("");
                }

                if (PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().findView(ViewSortierung.ID) != null) {
                    GuiFunktionen.schliesseSicht(ViewSortierung.ID);
                }

                if (AktionenVew.getInstanz().ausfuehrenAktion(AktionenGTM.COMMAND_ID_SORTIRUNG) != null) {
                    MessageDialog.openWarning(getShell(), "Achtung", "Sie haben kein Recht mit dem GUI 'Sortierauswahl' zu arbeiten!");
                }
            }
        });

        /*
        StringBuilder stringToolTipTextLbSortierung = new StringBuilder();

        stringToolTipTextLbSortierung.append("Die Sortierungen sind abh\u00E4ngig von der gew\u00E4hlten Attributgruppe,");
        stringToolTipTextLbSortierung.append(System.getProperty("line.separator"));
        stringToolTipTextLbSortierung.append("d.h. es werden nur die Sortierungen zur Auswahl angeboten, die die gleiche Attributgruppe haben.");
        stringToolTipTextLbSortierung.append(System.getProperty("line.separator"));
        stringToolTipTextLbSortierung.append("");
        */
        lbSortierung.setToolTipText("Durch Doppelklick auf das Label \"Sortierung\", k\u00F6nnen Sie direkt den Pflegeview f\u00FCr die Sortierungen \u00F6ffnen.");
        _toolkit.adapt(lbSortierung, true, true);
        lbSortierung.setText("Sortierung:");

        Label lbSpalten = new Label(compDartellungsOptionen, SWT.NONE);

        lbSpalten.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseDoubleClick(MouseEvent e) {

                // Aufruf des GUI fr die Spaltenauswahlerstellung.
                if (!(_comboSpalten.getSelectionIndex() == -1)) {
                    GewaehlteDarstellung.getInstanz().setGewaehlteSpalten(_comboSpalten.getItem(_comboSpalten.getSelectionIndex()));
                } else {
                    GewaehlteDarstellung.getInstanz().setGewaehlteSpalten("");
                }

                if (PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().findView(ViewSpaltenauswahl.ID) != null) {
                    GuiFunktionen.schliesseSicht(ViewSpaltenauswahl.ID);
                }

                if (AktionenVew.getInstanz().ausfuehrenAktion(AktionenGTM.COMMAND_ID_SPALTENAUSWAHL) != null) {
                    MessageDialog.openWarning(getShell(), "Achtung", "Sie haben kein Recht mit dem GUI 'Spaltenauswahl' zu arbeiten!");
                }
            }
        });

        /*
        StringBuilder stringToolTipTextLbSpalten = new StringBuilder();

        stringToolTipTextLbSpalten.append("Die Spaltenauswahlen sind abh\u00E4ngig von der gew\u00E4hlten Attributgruppe,");
        stringToolTipTextLbSpalten.append(System.getProperty("line.separator"));
        stringToolTipTextLbSpalten.append("d.h. es werden nur die Spaltenauswahlen angeboten, die die gleiche Attributgruppe haben.");
        stringToolTipTextLbSpalten.append(System.getProperty("line.separator"));
        stringToolTipTextLbSpalten.append("");
        */
        lbSpalten.setToolTipText("Durch Doppelklick auf das Label \"Spalten\", k\u00F6nnen Sie direkt den Pflegeview f\u00FCr die Spaltenauswahl \u00F6ffnen. ");
        _toolkit.adapt(lbSpalten, true, true);
        lbSpalten.setText("Spalten:");
        new Label(compDartellungsOptionen, SWT.NONE);
        new Label(compDartellungsOptionen, SWT.NONE);
        new Label(compDartellungsOptionen, SWT.NONE);
        new Label(compDartellungsOptionen, SWT.NONE);
        _btnStartStopp = new Button(compDartellungsOptionen, SWT.NONE);

        StringBuilder stringToolTipTextBtnStartStopp = new StringBuilder();

        stringToolTipTextBtnStartStopp.append("Drucken sowie auch die Konvertierung in csv oder html ist nur m\u00F6glich, wenn die Tabelle statisch ist.");
        stringToolTipTextBtnStartStopp.append(System.getProperty("line.separator"));
        stringToolTipTextBtnStartStopp.append("Bet\u00E4tigen Sie das Auswahlfeld Stopp, um die Verbindung zum Datenverteiler zu unterbrechen,");
        stringToolTipTextBtnStartStopp.append(System.getProperty("line.separator"));
        stringToolTipTextBtnStartStopp.append("so dass Sie die Optionen w\u00E4hlen k\u00F6nnen.");
        _btnStartStopp.setToolTipText(stringToolTipTextBtnStartStopp.toString());

        GridData gd__btnStartStopp = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);

        gd__btnStartStopp.widthHint = 65;
        _btnStartStopp.setLayoutData(gd__btnStartStopp);
        _toolkit.adapt(_btnStartStopp, true, true);
        _btnStartStopp.setText("Start");
        _btnStartStopp.setBackground(SWTResourceManager.getColor(SWT.COLOR_GREEN));
        _btnStartStopp.setEnabled(false);
        _btnStartStopp.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {}
        });
        _btnStartStopp.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnStartStopp();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _btnAenderungsDarstellung = new Button(compDartellungsOptionen, SWT.CHECK);
        _btnAenderungsDarstellung.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _tabellenDatenStruktur = _guiVew.getTabellenDatenStruktur();
                _tabellenDatenStruktur.setAenderungsDarstellung(_btnAenderungsDarstellung.getSelection());
                _guiVew.setzeTabellenInputErneut();
                _guiVew.aenderungErfolgt();
            }
        });
        _btnAenderungsDarstellung.setToolTipText("Wenn dieses Flag gesetzt ist, werden nur die Datens\u00E4tze erneuert, bei denen es eine \u00C4nderung gegeben hat. Wenn das Flag nicht gesetzt ist, wird in einem xy intervall der gesamte Datenbestand neu eingelesen und dargestellt.  ");
        _toolkit.adapt(_btnAenderungsDarstellung, true, true);
        _spinnerAnzahlVarAtt = new Spinner(compDartellungsOptionen, SWT.BORDER);
        _spinnerAnzahlVarAtt.setMaximum(MAXIMUM_VAR_ATT);
        _spinnerAnzahlVarAtt.setIncrement(INCREMENT_VAR_ATT);
        _spinnerAnzahlVarAtt.setSelection(DEFAULT_VAR_ATT);
        _spinnerAnzahlVarAtt.setToolTipText("Mittels des Spinners ist es Ihnen M\u00F6glich die Anzahl der anzuzeigenden Attribute fest zu legen. Der Maximalwert betr\u00E4gt 100. Bei der Einstellung 0 werden alle Datens\u00E4tze des Feldes in einer Liste dargestellt.");
        _spinnerAnzahlVarAtt.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                if (_comboAsp.getSelectionIndex() != -1) {
                    _guiVew.aktionTabelleMitRingBufferAufbauen();
                }

                _guiVew.aenderungErfolgt();
            }
        });
        _toolkit.adapt(_spinnerAnzahlVarAtt);
        _toolkit.paintBordersFor(_spinnerAnzahlVarAtt);
        _spinnerMaxZeilen = new Spinner(compDartellungsOptionen, SWT.BORDER);
        _spinnerMaxZeilen.setToolTipText("Mittels des Spinners ist es Ihnen m\u00F6glich die maximale Zeilenzahl der Tabelle festzulegen. Sobald die Zeilenzahl erreicht ist, werden f\u00FCr neu hinzuzuf\u00FCgende Zeilen die \u00C4ltesten gel\u00F6scht.");
        _spinnerMaxZeilen.setIncrement(INCREMENT_MAX_ZEILEN);
        _spinnerMaxZeilen.setMaximum(MAXIMUM_MAX_ZEILEN);
        _spinnerMaxZeilen.setSelection(DEFAULT_MAX_ZEILEN);
        _spinnerMaxZeilen.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                if (!_checkArchiv.getSelection()) {
                    _guiVew.loescheWennMaxAnzahlTabellenzeilenUeberschritten(GTMDialogGUIVerwaltung.BENUTZERDEFINIERT);
                }

                _guiVew.aenderungErfolgt();
            }
        });
        _toolkit.paintBordersFor(_spinnerMaxZeilen);

        // Combo Filter
        _comboFilter = new Combo(compDartellungsOptionen, SWT.READ_ONLY);
        _comboFilter.setToolTipText("Filterauswahl. Beachten Sie, dass die Filterauswahl im Verwaltungsdialog erstellt wird. Im Filterdialog k\u00F6nne Sie die Filter anlegen. ");

        GridData gd_comboFilter = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);

        gd_comboFilter.widthHint = 150;
        _comboFilter.setLayoutData(gd_comboFilter);
        _comboFilter.removeAll();
        _comboFilter.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.aktualisiereDarstellungBzglGewaehlterFilter();
                _guiVew.aenderungErfolgt();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _comboFilter.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {}
            @Override
            public void focusGained(FocusEvent e) {
                String selektion = _comboFilter.getText();

                _comboFilter.removeAll();
                _guiVew.fuelleCombo(_comboFilter, selektion);
            }
        });
        _toolkit.adapt(_comboFilter);
        _toolkit.paintBordersFor(_comboFilter);
        _guiVew.fuelleCombo(_comboFilter, "");
        _guiVew.aktualisiereDarstellungBzglGewaehlterFilter();

        // Combo Sortierung
        _comboSortierung = new Combo(compDartellungsOptionen, SWT.READ_ONLY);
        _comboSortierung.setToolTipText("Sortierungsauswahl. Beachten Sie, dass die Sortierauswahl im Verwaltungsdialog erstellt wird. Im Sortierdialog k\u00F6nnen Sie die Sortierungen anlegen.");

        GridData gd_comboSortierung = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);

        gd_comboSortierung.widthHint = 150;
        _comboSortierung.setLayoutData(gd_comboSortierung);
        _comboSortierung.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.aktualisiereDarstellungBzglGewaehlterSortierung();
                _guiVew.aenderungErfolgt();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _comboSortierung.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {}
            @Override
            public void focusGained(FocusEvent e) {
                String selektion = _comboSortierung.getText();

                _comboSortierung.removeAll();
                _guiVew.fuelleCombo(_comboSortierung, selektion);
            }
        });
        _guiVew.fuelleCombo(_comboSortierung, "");
        _toolkit.adapt(_comboSortierung);
        _toolkit.paintBordersFor(_comboSortierung);
        _guiVew.aktualisiereDarstellungBzglGewaehlterSortierung();

        // Combo Spalten
        _comboSpalten = new Combo(compDartellungsOptionen, SWT.READ_ONLY);
        _comboSpalten.setToolTipText("Spaltenauswahl. Beachten Sie, dass die Spaltenauswahl im Verwaltungsdialog erstellt wird. Im Spaltenauswahldialog k\u00F6nnen Sie die Spaltenauwahlen anlegen. \r\n\r\n");

        GridData gd_comboSpalten = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);

        gd_comboSpalten.widthHint = 150;
        _comboSpalten.setLayoutData(gd_comboSpalten);
        _comboSpalten.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.aktualisiereDarstellungBzglGewaehlterSpalten();
                _guiVew.aenderungErfolgt();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _comboSpalten.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {}
            @Override
            public void focusGained(FocusEvent e) {
                String selektion = _comboSpalten.getText();

                _comboSpalten.removeAll();
                _guiVew.fuelleCombo(_comboSpalten, selektion);
            }
        });
        _toolkit.adapt(_comboSpalten);
        _toolkit.paintBordersFor(_comboSpalten);
        _guiVew.fuelleCombo(_comboSpalten, "");
        _guiVew.aktualisiereDarstellungBzglGewaehlterSpalten();

        //
        _btnDrucken = new Button(compDartellungsOptionen, SWT.NONE);
        _btnDrucken.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {}
        });
        _toolkit.adapt(_btnDrucken, true, true);
        _btnDrucken.setText("Drucken...");
        _btnDrucken.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnDrucken();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _btnDrucken.setEnabled(false);
        _btnExportInHtml = new Button(compDartellungsOptionen, SWT.NONE);
        _btnExportInHtml.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnExportInHtml();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        _btnExportInHtml.setEnabled(false);
        _toolkit.adapt(_btnExportInHtml, true, true);
        _btnExportInHtml.setText("Export in html...");
        _btnExportInCsv = new Button(compDartellungsOptionen, SWT.NONE);
        _btnExportInCsv.setEnabled(false);
        _toolkit.adapt(_btnExportInCsv, true, true);
        _btnExportInCsv.setText("Export in csv...");
        new Label(compDartellungsOptionen, SWT.NONE);
        _btnExportInCsv.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnExportInCsv();
            }
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
        });
        sectionDarstellungsoptionen.setExpanded(true);
    }

    /**
     * Erzeugt den Bereich fr die Detailinformationen, d.h. den Bereich fr die
     * Attributgruppen wie auch Aspektauswahl.
     */
    private void initGUI_SectionDetailinformaionen() {
        Section sectionDatenidentifikation = _toolkit.createSection(_compBenutzerAngaben, Section.TWISTIE | Section.TITLE_BAR);

        sectionDatenidentifikation.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        _toolkit.paintBordersFor(sectionDatenidentifikation);
        sectionDatenidentifikation.setText("Datenidentifikation");

        Composite _compAtgAsp = new Composite(sectionDatenidentifikation, SWT.NONE);

        sectionDatenidentifikation.setClient(_compAtgAsp);
        _compAtgAsp.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
        _compAtgAsp.setLayout(new GridLayout(2, false));
        _checkKonfDaten = new Button(_compAtgAsp, SWT.CHECK);
        _checkKonfDaten.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.checkKonfDaten();
            }
        });
        _checkKonfDaten.setToolTipText("Wenn Sie diese Checkbox ausw\u00E4hlen, werden die angezeigten Attributgruppen auf Konfigurierende Attributgruppen eingeschr\u00E4nkt.");
        _checkKonfDaten.setText("Konfigurierende Daten");
        _toolkit.adapt(_checkKonfDaten, true, true);
        new Label(_compAtgAsp, SWT.NONE);

        Label _lbTyp = new Label(_compAtgAsp, SWT.NONE);

        _toolkit.adapt(_lbTyp, true, true);
        _lbTyp.setText("Typ:");
        _txtObjTyp = new Text(_compAtgAsp, SWT.BORDER);
        _txtObjTyp.setToolTipText("Typ des gew\u00E4hlten Objektes.");
        _txtObjTyp.setEnabled(false);
        _txtObjTyp.setEditable(false);
        _txtObjTyp.setText(getObjTyp());

        GridData gd_text = new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1);

        gd_text.widthHint = 483;
        _txtObjTyp.setLayoutData(gd_text);
        _toolkit.adapt(_txtObjTyp, true, true);

        Label _lbAttGr = new Label(_compAtgAsp, SWT.NONE);

        _lbAttGr.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
        _lbAttGr.setText("Attributgruppe:");
        _comboAtg = new Combo(_compAtgAsp, SWT.READ_ONLY);

        GridData gd_comboAtg = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);

        gd_comboAtg.widthHint = 470;
        _comboAtg.setLayoutData(gd_comboAtg);
        _comboAtg.setToolTipText("Bitte w\u00E4hlen Sie die Attributgruppe aus, zu der Sie die Informationen angezeigt bekommen m\u00F6chten.");
        _comboAtg.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {}
            @Override
            public void focusGained(FocusEvent e) {
                String selektion = _comboAtg.getText();

                _comboAtg.removeAll();
                _guiVew.fuelleCombo(_comboAtg, selektion);
            }
        });
        _comboAtg.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
            @Override
            public void widgetSelected(SelectionEvent e) {

                // _guiVew.abmeldenDav();
                _btnStartStopp.setEnabled(false);
                _instanzArchivVew.setzeEnabledWertFuerAlleBtnArchivanfrage(false);
                initialisiereAbhaengigeCombosDarstellungsOpTabellenDarstellung(false);
                _guiVew.aenderungErfolgt();
            }
        });
        _guiVew.fuelleCombo(_comboAtg, "");

        Label _lbAsp = new Label(_compAtgAsp, SWT.NONE);

        _lbAsp.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
        _lbAsp.setText("Aspekt:");
        _comboAsp = new Combo(_compAtgAsp, SWT.READ_ONLY);

        GridData gd_comboAsp = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);

        gd_comboAsp.widthHint = 470;
        _comboAsp.setLayoutData(gd_comboAsp);
        _comboAsp.setToolTipText("Bitte w\u00E4hlen Sie den Aspekt aus, zu dem Sie die Informationen angezeigt bekommen m\u00F6chten.");
        _comboAsp.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {}
            @Override
            public void focusGained(FocusEvent e) {
                String selektion = _comboAsp.getText();

                _comboAsp.removeAll();
                _guiVew.fuelleCombo(_comboAsp, selektion);
            }
        });
        _comboAsp.addSelectionListener(new SelectionListener() {
            @Override
            public void widgetDefaultSelected(SelectionEvent e) {}
            @Override
            public void widgetSelected(SelectionEvent e) {
                _tabelleSchonMalGefuellt = true;
                initialisiereAbhaengigeCombosDarstellungsOpTabellenDarstellung(true);
                _guiVew.loescheTabellenZeilen(GTMDialogGUIVerwaltung.ALLE);
                _guiVew.aktionTabelleFuellen();

                if (!_checkKonfDaten.getSelection() && (!_checkArchiv.getSelection())) {
                    _btnStartStopp.setEnabled(true);
                } else if (_checkArchiv.getSelection()) {
                    _instanzArchivVew.setzeEnabledWertFuerAlleBtnArchivanfrage(true);
                }

                _guiVew.aenderungErfolgt();
            }
        });
        _guiVew.fuelleCombo(_comboAsp, "");
        sectionDatenidentifikation.setExpanded(true);
    }

    /**
     * Die Methode erzeugt die Sektion fr den Bereich der Anzeige der ausgewhlten Knoten.
     * Da es sich um eine Section handelt, kann diese bei belieben einfach zugeklappt werden.
     */
    private void initGUI_SektionAnzeigenDerGewaehltenKnotenImBaum() {
        Section sectionGewaehlteKnotenImBaum = _toolkit.createSection(_compBenutzerAngaben, Section.TWISTIE | Section.TITLE_BAR);

        sectionGewaehlteKnotenImBaum.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
        sectionGewaehlteKnotenImBaum.setText("Gewhlte Knoten im Baum");

        Composite compGewaehlteKnotenBaum = new Composite(sectionGewaehlteKnotenImBaum, SWT.NONE);

        sectionGewaehlteKnotenImBaum.setClient(compGewaehlteKnotenBaum);
        compGewaehlteKnotenBaum.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
        _toolkit.adapt(compGewaehlteKnotenBaum);
        _toolkit.paintBordersFor(compGewaehlteKnotenBaum);
        compGewaehlteKnotenBaum.setLayout(new GridLayout(1, false));
        _txtGewaehlteKnoten = new Text(compGewaehlteKnotenBaum, SWT.READ_ONLY | SWT.MULTI | SWT.WRAP | SWT.H_SCROLL | SWT.BORDER | SWT.V_SCROLL);

        GridData gd_txtGewaehlteKnoten = new GridData(SWT.LEFT, SWT.FILL, false, false, 1, 1);

        gd_txtGewaehlteKnoten.widthHint  = 590;
        gd_txtGewaehlteKnoten.heightHint = 150;
        _txtGewaehlteKnoten.setLayoutData(gd_txtGewaehlteKnoten);
        _txtGewaehlteKnoten.setEditable(false);
        _guiVew.fuelleTextFeldGewaehlteKnotenImBaum(BaumHilfsFunktionen.getPidSelektierterObj(BaumHilfsFunktionen.getListeObjekte()));
    }

    /**
     * Erzeugt den Bereich fr die Tabelle. Zustzlich existiert in dem Bereich ein Filter der auf die aktuelle Tabelle
     * angewendet werden kann.
     */
    private void initGUI_SektionTabelle() {
        _sectionTabelle = _toolkit.createSection(_compBenutzerAngaben, Section.EXPANDED | Section.TWISTIE | Section.TITLE_BAR);
        _sectionTabelle.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        _toolkit.paintBordersFor(_sectionTabelle);
        _sectionTabelle.setText("Tabelle");

        Composite compTabelle = new Composite(_sectionTabelle, SWT.NONE);

        _toolkit.adapt(compTabelle);
        _toolkit.paintBordersFor(compTabelle);
        _sectionTabelle.setClient(compTabelle);

        StringBuilder stringToolTipTextTabelle = new StringBuilder();

        stringToolTipTextTabelle.append("Kopieren der markierten Tabellenzeilen mittels der Tastenkombination \"Strg+c\"");
        stringToolTipTextTabelle.append(System.getProperty("line.separator"));
        stringToolTipTextTabelle.append("Danach k\u00F6nnen Sie die kopierten Zeilen in einen beliebigen Editor mittels der Tastenkombination \"Strg+v\" einf\u00FCgen.");
        _sectionTabelle.setToolTipText(stringToolTipTextTabelle.toString());
        compTabelle.setLayout(new GridLayout());

        final Table table = new Table(compTabelle, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI);

        table.setLinesVisible(true);
        table.setHeaderVisible(true);
        table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        _toolkit.adapt(table);
        _toolkit.paintBordersFor(table);
        _tableViewer = new TableViewer(table);
        _tableViewer.addSelectionChangedListener(new ISelectionChangedListener() {
            @SuppressWarnings("unchecked")
            @Override
            public void selectionChanged(SelectionChangedEvent event) {
                ISelection    selektion                   = _tableViewer.getSelection();
                StringBuilder stringBufferZeilen          = new StringBuilder();
                Clipboard     clipBoardMitKopiertenZeilen = new Clipboard(Display.getDefault());

                if ((selektion != null) && (selektion instanceof IStructuredSelection)) {
                    IStructuredSelection selektierteZeilen = (IStructuredSelection) selektion;

                    for (Iterator<ZeilenObjekt> zeilenIterator = selektierteZeilen.iterator(); zeilenIterator.hasNext(); ) {
                        ZeilenObjekt zeilenObj = zeilenIterator.next();

                        for (int i = 0; i < table.getColumnCount(); i++) {
                            stringBufferZeilen.append(zeilenObj.getAttribut(i));
                            stringBufferZeilen.append("\t");
                        }

                        stringBufferZeilen.append(System.getProperty("line.separator"));
                    }

                    TextTransfer textTransfer = TextTransfer.getInstance();

                    try {
                        clipBoardMitKopiertenZeilen.setContents(new Object[] { stringBufferZeilen.toString() }, new Transfer[] { textTransfer });
                    }
                    catch (Exception ignored) {}
                }
            }
        });
    }

    /**
     * Die Methode initialisiereAbhaengigeCombosDarstellungsOpTabellenDarstellung initialisiert:
     * 1. alle abhngigen Comboboxen, wobei der flag isAsp angibt, ob auch die Aspektcombobox initialisiert werden soll. Dies ist
     * nicht der Fall, wenn diese Methode durch den Selektionlistener der Aspektcombobox aufgerufen wird.
     * 2. Initialisierung der Darstellungsoptionen, d.h. die Combobox wird vollstndig gelscht.
     * 3. Lschen der Tabellenkopfzeile, so dass eine neue Tabelle aufgebaut werden kann.
     * 4. Die Combobox Darstellungsname wird erzeugt.
     *
     * @param isAsp
     *          Parameter der angibt, ob die Methode durch den Selektionlistener der Aspektcombobox aufgerufen wurde.
     */
    private void initialisiereAbhaengigeCombosDarstellungsOpTabellenDarstellung(boolean isAsp) {
        if (_start) {
            _guiVew.btnStartStopp();
        }

        initialisiereAlleAbhaengigenCombos(isAsp);
        _guiVew.loescheKopfzeile();
    }

    /**
     * Die Methode initialisiereAlleAbhaengigenCombos initialisiert alle
     * abhngigen Comboboxen. Folgende Comboboxen sind von der Attributgruppe abhngig:
     * 1. Aspekt - wenn die Methode nicht durch die Aspektcombobox aufgerufen wird.
     * 2. Filter
     * 3. Sortierung
     * 4. Spalten
     * @param isAsp
     *      Parameter der angibt, ob die Methode durch den Selektionlistener der Combobox Aspekte aufgerufen wurde.
     */
    protected void initialisiereAlleAbhaengigenCombos(boolean isAsp) {
        if (!isAsp) {
            _guiVew.fuelleCombo(_comboAsp, "");
        }

        _spinnerAnzahlVarAtt.setSelection(0);
        _spinnerMaxZeilen.setSelection(1000);
        _guiVew.fuelleCombo(_comboFilter, "");
        _guiVew.fuelleCombo(_comboSortierung, "");
        _guiVew.fuelleCombo(_comboSpalten, "");
    }

    /**
     * Das GUI wird auf default zurck gesetzt.
     *
     */
    private void initialisiereGUIAufDefaultWerte() {
        _guiVew.erzeugeComboboxDarstellungsName();
        _comboDarstellungsName.select(0);
        _checkArchiv.setSelection(false);
        anzeigenArchivBereich(false);
        _checkKonfDaten.setSelection(false);
        _comboAtg.removeAll();
        _comboAsp.removeAll();

        // Archiv:
        _instanzArchivVew.setzeGUIbereich(new ArchivanfrageDaten());

        //
        _btnAenderungsDarstellung.setSelection(false);
        _spinnerAnzahlVarAtt.setSelection(DEFAULT_VAR_ATT);
        _spinnerMaxZeilen.setSelection(DEFAULT_MAX_ZEILEN);
        _guiVew.fuelleCombo(_comboFilter, "");
        _guiVew.fuelleCombo(_comboSortierung, "");
        _guiVew.fuelleCombo(_comboSpalten, "");

        if (_start) {
            _guiVew.btnStartStopp();
        }

        _guiVew.loescheKopfzeile();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'start'.
     *
     * @return Wert der Eigenschaft 'start'.
     *
     * @see #_start
     */
    public boolean isStart() {
        return _start;
    }

    /**
     * Getter fr Eigenschaft 'tabelleSchonMalGefuellt'.
     *
     * @return Wert der Eigenschaft 'tabelleSchonMalGefuellt'.
     *
     * @see #_tabelleSchonMalGefuellt
     */
    public boolean isTabelleSchonMalGefuellt() {
        return _tabelleSchonMalGefuellt;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode ladeDarstellungsOptionen ldt alle Informationen aus dem ausgewhlten
     * DarstellungsNamen in die aktuelle Tabellensicht.
     */
    protected void ladeDarstellungsOptionen() {
        _guiVew.initialisiereDarstellungsOptionen();
        _guiVew.fuelleGUIElemente(true);
    }

    @Override
    public void run() {
        while (_threadImportAktiv) {
            try {
                Thread.sleep(2);
            }
            catch (InterruptedException e) {
                e.printStackTrace();
            }

            PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    while (!_arListResultData.isEmpty()) {
                        if (!isDisposed()) {
                            _guiVew.verarbeiteDavDaten(_arListResultData.get(0));
                            _guiVew.loescheWennMaxAnzahlTabellenzeilenUeberschritten(GTMDialogGUIVerwaltung.BENUTZERDEFINIERT);
                            _arListResultData.remove(0);
                        }
                    }
                }
            });
        }
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'start'.
     *
     * @param start Wert zum Setzen der Eigenschaft 'start'.
     *
     * @see #_start
     */
    public void setStart(boolean start) {
        _start = start;
    }

    /**
     * Setter fr Eigenschaft 'tabelleSchonMalGefuellt'.
     *
     * @param tabelleSchonMalGefuellt Wert zum Setzen der Eigenschaft 'tabelleSchonMalGefuellt'.
     *
     * @see #_tabelleSchonMalGefuellt
     */
    public void setTabelleSchonMalGefuellt(boolean tabelleSchonMalGefuellt) {
        _tabelleSchonMalGefuellt = tabelleSchonMalGefuellt;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode startet den Thread, sobald der View gnzlich aufgebaut ist.
     * Der Thread ist verantwortlich fr die Abarbeitung der ResultData die
     * in das Array geschrieben werden.
     */
    private void starteThread() {
        if (!_threadImportAktiv) {
            _threadResultDataUeberwachung = new Thread(this);
            _threadResultDataUeberwachung.setName("ResultDataUeberwachung");
            _threadImportAktiv = true;
            _threadResultDataUeberwachung.start();
        }
    }

    /**
     * Die Methode stoppt den Thread vor dem Schlieen der Sicht.
     */
    private void stoppeThread() {
        if (_threadImportAktiv) {

            // noinspection AssignmentToNull
            _blinker           = null;
            _threadImportAktiv = false;
        }
    }

    @Override
    public void update(ResultData[] results) {
        if (!isDisposed()) {
            _arListResultData.add(results);
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 07.03.14
