/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.gui;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.gtm.datvew.lib.BaumHilfsFunktionen;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.plugin.gtm.navigation.Activator;
import de.inovat.buv.plugin.gtm.navigation.gui.lib.DarstellungenVew;
import de.inovat.buv.plugin.gtm.navigation.gui.lib.IDarstellung;
import de.inovat.buv.plugin.gtm.navigation.gui.lib.IDatenModell;

import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MenuDetectEvent;
import org.eclipse.swt.events.MenuDetectListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.FilteredTree;
import org.eclipse.ui.dialogs.PatternFilter;

//~ KLASSEN ===================================================================

/**
 * Klasse stellt das GUI fr den Navigationbaum dar.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class NavigationGUI extends Composite implements IDatenModell {

    /** Navigationsbaum. */
    private static TreeViewer _treeViewer;

    //~ FELDER ================================================================

    private Button _btnAktualisieren;
    private Button _cbtnAktualisieren;
    private Combo  _comboSicht;

    /** GUI Verwaltungs-Instanz. */
    private NavigationGUIVew _guiVew;

    /** Label mit dem Icon fr auf/zuklappen. */
    private Label  _lbAufZuKlappen;
    private Button _rbtnFilter;
    private Button _rbtSuche;

    /** Text-Feld fr die Suche bzw. den Filter. */
    private Text _txtSucheFilter;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse NavigationGUI.
     *
     * @param parent
     *            Parent-Composite.
     * @param style
     *            SWT-Style.
     */
    public NavigationGUI(Composite parent, int style) {
        super(parent, style);

        //
        _guiVew = new NavigationGUIVew(this);
        initGUI();
        _guiVew.initGui();

        // Listener auf DatenModell nderungen:
        DarstellungenVew.addModellListener(this);

        // Lsche Listener:
        final IDatenModell klasse = this;

        parent.addDisposeListener(new DisposeListener() {
            @Override
            public void widgetDisposed(DisposeEvent arg0) {
                DarstellungenVew.removeModellListener(klasse);
            }
        });
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'btnAktualisieren'.
     *
     * @return Wert der Eigenschaft 'btnAktualisieren'.
     *
     * @see #_btnAktualisieren
     */
    public Button getBtnAktualisieren() {
        return _btnAktualisieren;
    }

    /**
     *     Getter fr Eigenschaft 'cbtnAktualisieren'.
     *
     *     @return Wert der Eigenschaft 'cbtnAktualisieren'.
     *
     *     @see #_cbtnAktualisieren
     */
    public Button getCbtnAktualisieren() {
        return _cbtnAktualisieren;
    }

    /**
     * Getter fr Eigenschaft 'comboSicht'.
     *
     * @return Wert der Eigenschaft 'comboSicht'.
     *
     * @see #_comboSicht
     */
    public Combo getComboSicht() {
        return _comboSicht;
    }

    /**
     * Getter fr Eigenschaft 'lbAufZuKlappen'.
     *
     * @return Wert der Eigenschaft 'lbAufZuKlappen'.
     *
     * @see #_lbAufZuKlappen
     */
    public Label getLbAufZuKlappen() {
        return _lbAufZuKlappen;
    }

    /**
     * Getter fr Eigenschaft 'rbtnFilter'.
     *
     * @return Wert der Eigenschaft 'rbtnFilter'.
     *
     * @see #_rbtnFilter
     */
    public Button getRbtnFilter() {
        return _rbtnFilter;
    }

    /**
     * Getter fr Eigenschaft 'rbtSuche'.
     *
     * @return Wert der Eigenschaft 'rbtSuche'.
     *
     * @see #_rbtSuche
     */
    public Button getRbtSuche() {
        return _rbtSuche;
    }

    /**
     * Getter fr Eigenschaft '_treeViewer'.
     *
     * @return Wert der Eigenschaft '_treeViewer'.
     *
     * @see #_treeViewer
     */
    public TreeViewer getTreeViewer() {
        return _treeViewer;
    }

    /**
     * Getter fr Eigenschaft 'txtSucheFilter'.
     *
     * @return Wert der Eigenschaft 'txtSucheFilter'.
     *
     * @see #_txtSucheFilter
     */
    public Text getTxtSucheFilter() {
        return _txtSucheFilter;
    }

    //~ METHODEN ==============================================================

    /** Initialisiere das GUI. */
    private void initGUI() {
        try {
            GridLayout thisLayout = new GridLayout();

            thisLayout.makeColumnsEqualWidth = true;
            this.setLayout(thisLayout);

            ScrolledComposite scompNavigation = new ScrolledComposite(this, SWT.H_SCROLL | SWT.V_SCROLL);

            scompNavigation.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
            scompNavigation.setExpandHorizontal(true);
            scompNavigation.setExpandVertical(true);

            Composite compNavigation = new Composite(scompNavigation, SWT.NONE);

            compNavigation.setLayout(new GridLayout(1, false));

            final Composite compositeEinstellungen = new Composite(compNavigation, SWT.NONE);

            compositeEinstellungen.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

            final GridLayout gridLayout = new GridLayout();

            gridLayout.marginHeight = 0;
            gridLayout.numColumns   = 4;
            compositeEinstellungen.setLayout(gridLayout);

            //
            _comboSicht = new Combo(compositeEinstellungen, SWT.READ_ONLY);
            _comboSicht.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 4, 1));
            _comboSicht.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    _guiVew.comboSichtSelektiert();
                }
            });

            //
            final Group grpTools = new Group(compNavigation, SWT.NONE);

            grpTools.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

            final GridLayout gl_grpTools = new GridLayout();

            gl_grpTools.marginHeight = 0;
            gl_grpTools.numColumns   = 7;
            grpTools.setLayout(gl_grpTools);

            //
            _lbAufZuKlappen = new Label(grpTools, SWT.NONE);
            _lbAufZuKlappen.setText("auf");
            _lbAufZuKlappen.addMouseListener(new MouseAdapter() {
                public void mouseDown(final MouseEvent e) {
                    _guiVew.baumAufZuKlappen();
                }
            });
            _guiVew.initLbAufZuKlappen();

            //
            int      sepH      = 5 + _lbAufZuKlappen.computeSize(SWT.DEFAULT, SWT.DEFAULT).y;
            int      sepW      = 10;
            Label    lbSep1    = new Label(grpTools, SWT.SEPARATOR);
            GridData gd_lbSep1 = new GridData(SWT.CENTER, SWT.CENTER, false, false);

            gd_lbSep1.heightHint = sepH;
            gd_lbSep1.widthHint  = sepW;
            lbSep1.setLayoutData(gd_lbSep1);

            //
            _rbtSuche = new Button(grpTools, SWT.RADIO);
            _rbtSuche.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    _guiVew.rbtnSucheFilterSelektiert();
                }
            });
            _rbtSuche.setText("Suche");
            _rbtnFilter = new Button(grpTools, SWT.RADIO);
            _rbtnFilter.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    _guiVew.rbtnSucheFilterSelektiert();
                }
            });
            _rbtnFilter.setText("Filter");

            //
            Label    lbSep2    = new Label(grpTools, SWT.SEPARATOR);
            GridData gd_lbSep2 = new GridData(SWT.CENTER, SWT.CENTER, false, false);

            gd_lbSep2.heightHint = sepH;
            gd_lbSep2.widthHint  = sepW;
            lbSep2.setLayoutData(gd_lbSep2);

            //
            _cbtnAktualisieren = new Button(grpTools, SWT.CHECK);
            _cbtnAktualisieren.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    _guiVew.cbtnAktualisierenSelektiert();
                }
            });
            _cbtnAktualisieren.setToolTipText("Sicht automatisch aktualisieren");

            //
            _btnAktualisieren = new Button(grpTools, SWT.NONE);
            _btnAktualisieren.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    _guiVew.btnAktualisierenSelektiert();
                }
            });
            _btnAktualisieren.setToolTipText("Baum aktualisieren");
            _btnAktualisieren.setText("wird automatisch aktualisiert");

            // ----------------------------------------
            // $hide<<$
            final FilteredTree filterTree = new FilteredTree(compNavigation, SWT.MULTI | SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL, new PatternFilter(), true) {
                @Override
                protected String getFilterString() {

                    // Unterschied zwischen Suche und Filter:
                    if (istSucheEingeschaltet()) {
                        return "";
                    } else {
                        return super.getFilterString();
                    }
                }
            };

            // $hide>>$
            //
            _treeViewer = filterTree.getViewer();

            GridData treeViewer1LData = new GridData(SWT.FILL, SWT.FILL, true, true);

            _treeViewer.getControl().setLayoutData(treeViewer1LData);

            // Selektion-Listener:
            _treeViewer.addSelectionChangedListener(new ISelectionChangedListener() {
                @Override
                public void selectionChanged(SelectionChangedEvent event) {
                    _guiVew.treeViewerSelektionGeaendert();
                }
            });

            // Kontexmen-Listener (wie Selektion-Listener):
            //
            // Die Selektion im Baum kann durch die Selektion der Objekte in einer anderen Sicht,
            // die SelektionsTransfer untersttzt, gelscht werden.
            // In diesem Fall, falls man im Baum sofort ohne neue Selektion das Kontexmen aufruft,
            // wird SelektionTransfeer leer Liste liefern.
            //
            // Um die Situation zu verhindern, wird die Selektion beim Kontexmen-Listener
            // noch mal gesetzt.
            _treeViewer.getTree().addMenuDetectListener(new MenuDetectListener() {
                public void menuDetected(MenuDetectEvent event) {
                    _guiVew.treeViewerSelektionGeaendert();
                }
            });

            // DoubleClickListener:
            _treeViewer.addDoubleClickListener(new IDoubleClickListener() {
                @Override
                public void doubleClick(DoubleClickEvent event) {

                    // ffne die TabellenDarstellung:
                    BaumHilfsFunktionen.oeffneTabellendarstellung();
                }
            });

            // Listener fr die Suche:
            _txtSucheFilter = filterTree.getFilterControl();
            _txtSucheFilter.addModifyListener(new ModifyListener() {
                @Override
                public void modifyText(ModifyEvent event) {
                    _guiVew.sucheImBaum(true);
                }
            });

            //
            _txtSucheFilter.addKeyListener(new KeyListener() {
                @Override
                public void keyPressed(KeyEvent event) {
                    _guiVew.keyListener(event.keyCode, true);
                }
                @Override
                public void keyReleased(KeyEvent event) {}
            });

            // Listener fr die Suche:
            filterTree.getViewer().getTree().addKeyListener(new KeyListener() {
                @Override
                public void keyPressed(KeyEvent event) {
                    _guiVew.keyListener(event.keyCode, false);
                }
                @Override
                public void keyReleased(KeyEvent event) {}
            });

            //
            this.layout();

            //
            scompNavigation.setContent(compNavigation);
            scompNavigation.setMinSize(compNavigation.computeSize(SWT.DEFAULT, SWT.DEFAULT));
        }
        catch (Exception e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
    }

    /**
     * Stellt fest, ob den Such-Modus eingeschaltet ist.
     *
     * @return true, falls den Such-Modus eingeschaltet ist,
     * false - Filter-Modus.
     */
    protected boolean istSucheEingeschaltet() {
        return _rbtSuche.getSelection();
    }

    @Override
    public void modellGeaendert(IDarstellung sicht) {
        _guiVew.modellGeaendert(sicht);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 06.10.14
