/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.sortierung;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.gtm.datvew.konstanten.KonstantenGTM;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.plugin.gtm.navigation.Activator;
import de.inovat.buv.projektlib.speicher.GrundEinstellungen;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

//~ JDK IMPORTE ===============================================================

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import java.util.Arrays;
import java.util.Hashtable;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet die Einstellungen fr die Sortierungen, die in XML-Format gespeichert werden.
 * Diese Klasse ist als Singelton realisiert.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public final class AlleSortierungen {
    private static final int MODUS_SORTIERUNG = 0;
    private static final int MODUS_SPALTEN    = 1;

    /**
     * Stellt fest, ob die Initialisierung der Instanz abgeschlossen ist (s.{@link #AlleSortierungen(int)}).
     *
     * WICHTIG:
     *
     * Die Methode {@link #getSpaltenAuswahlInstanz()} MUSS auf das Ende der Initialisierung warten.
     */
    private static boolean _istInitialisierungAbgeschlossenSpalten = false;

    /**
     * Stellt fest, ob die Initialisierung der Instanz abgeschlossen ist (s.{@link #AlleSortierungen(int)}).
     *
     * WICHTIG:
     *
     * Die Methode {@link #getSortierInstanz()} MUSS auf das Ende der Initialisierung warten.
     */
    private static boolean _istInitialisierungAbgeschlossenSort = false;

    /**
     * Instanz der Klasse fr Spaltenauswahl.
     */
    private static AlleSortierungen _alleSpaltenAuswahlInstanz = new AlleSortierungen(MODUS_SPALTEN);

    /**
     * Instanz der Klasse fr die Sortierungen.
     */
    private static AlleSortierungen _alleSortierungenInstanz = new AlleSortierungen(MODUS_SORTIERUNG);
    public static boolean           _klasseLaden;

    //~ FELDER ================================================================

    /**
     * Gibt an ob es sich um eine Sortierung (=0) oder Spaltenauswahl (=1) handelt.
     */
    private int _modus;

    /**
     * Map der definierten Sortierngen.
     * Key: Name der Sortierung.
     */
    private Map<String, Sortierung> _sortierungMap;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse AlleSortierungen.
     * @param modus, gibt an ob es sich um eine Sortierung oder Spaltenauswahl handelt.
     */
    private AlleSortierungen(int modus) {
        _modus = modus;

        // -------------------------------------------------------------------
        // WICHTIG
        // Die Initialisierung der Tabellendarstellungen luft langsam ===>
        // wird in JOB ausgelagert.
        // ===>
        // Die Methode {@link #getTabellenVerwaltungInstanz()} MUSS auf
        // das Ende der Initialisierung warten.
        String jobName = "";

        if (_modus == MODUS_SORTIERUNG) {
            jobName = "Initialisiere alle Sortierungen...";
        } else if (_modus == MODUS_SPALTEN) {
            jobName = "Initialisiere alle Spalteneinstellungen...";
        }

        Job job = new Job(jobName) {
            @Override
            protected IStatus run(IProgressMonitor monitor) {
                ladeAlleSortierungen();

                //
                if (_modus == MODUS_SORTIERUNG) {
                    _istInitialisierungAbgeschlossenSort = true;
                } else if (_modus == MODUS_SPALTEN) {
                    _istInitialisierungAbgeschlossenSpalten = true;
                }

                return Status.OK_STATUS;
            }
        };

        job.schedule();
    }

    //~ METHODEN ==============================================================

    /**
     * Fge neue Sortierung in sortierungMap.
     *
     * @param sortierung - SortierungObjekt.
     */
    public void addSortierung(Sortierung sortierung) {
        _sortierungMap.put(sortierung.getSortierungName(), sortierung);
    }

    //~ GET METHODEN ==========================================================

    /**
     * Gibt die Instanz der Klasse fr den AlleSortierungen zurck.
     *
     * @return Instanz der Klasse fr den AlleSortierungen mit Modus 0,
     * d.h. Sortierinstanz..
     */
    public static AlleSortierungen getSortierInstanz() {
        pauseBisEndeDerInitialisierung(MODUS_SORTIERUNG);

        return _alleSortierungenInstanz;
    }

    /**
     * Gibt das sortierte Array der Namen von definierten Sortierungen zurck,
     * oder leeres StringArray, falls keine Sortierungen definiert sind.
     *
     * @return Sortiertes Array der Namen von definierten Sortierungen,
     * oder leeres StringArray, falls keine Sortierungen definiert sind.
     */
    public String[] getSortierungArray() {
        String[] ar = _sortierungMap.keySet().toArray(new String[_sortierungMap.keySet().size()]);

        Arrays.sort(ar);

        return ar;
    }

    /**
     * Gibt das SortierungObjekt mit angegebenem Namen zurck
     * oder null, falls solche nicht existiert.
     *
     * @param sortierungName - der Name der Sortierung.
     *
     * @return SortierungObjekt mit angegebenem Namen oder null.
     */
    public Sortierung getSortierungObjekt(String sortierungName) {
        return _sortierungMap.get(sortierungName);
    }

    /**
     * Gibt die Instanz der Klasse fr den AlleSortierungen zurck.
     *
     * @return Instanz der Klasse fr den AlleSortierungen, mit Modus 1, d.h.
     * Spaltenauswahlinstanz.
     */
    public static AlleSortierungen getSpaltenAuswahlInstanz() {
        pauseBisEndeDerInitialisierung(MODUS_SPALTEN);

        return _alleSpaltenAuswahlInstanz;
    }

    //~ METHODEN ==============================================================

    /**
     * Lade die XMLDatei ins sortierungMap.
     */
    @SuppressWarnings("unchecked")
    private void ladeAlleSortierungen() {
        _sortierungMap = new Hashtable<String, Sortierung>();

        String parameterID;

        if (_modus == MODUS_SORTIERUNG) {
            parameterID = KonstantenGTM.ALLE_SORTIERUNGEN;
        } else if (_modus == MODUS_SPALTEN) {
            parameterID = KonstantenGTM.ALLE_SPALTENAUSWAHL;
        } else {
            return;
        }

        // Ermittle die String-Representation des Maps aus dem Speicher:
        String mapString = GrundEinstellungen.ermittleParameterWertFuerAlleBenutzer(parameterID);

        if (mapString != null) {

            // Dekodiere den Parameter:
            try {
                ByteArrayInputStream input     = new ByteArrayInputStream(mapString.getBytes());
                XMLDecoder           xml       = new XMLDecoder(input);
                Object               mapObjekt = xml.readObject();

                xml.close();

                if (mapObjekt instanceof Map) {
                    try {
                        _sortierungMap = (Hashtable<String, Sortierung>) mapObjekt;
                    }
                    catch (Exception e) {
                        Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Sortierungen knnen nicht geladen werden!", e));
                    }
                }
            }
            catch (Exception e) {
                Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Sortierungen knnen nicht geladen werden!", e));
            }
        }
    }

    /**
     * Lsche das SortierungObjekt mit dem Namen sortierungName aus dem sortierungMap.
     *
     * @param sortierungName Name der zu lschenden Sortierung.
     */
    public void loescheSortierung(String sortierungName) {
        _sortierungMap.remove(sortierungName);
    }

    /**
     * Pause bis dem Ende der Initialisierung
     * s.{@link #_istInitialisierungAbgeschlossenSort} und
     * s.{@link #_istInitialisierungAbgeschlossenSpalten}.
     *
     * @param modus Gibt an ob es sich um eine Sortierung (=0) oder Spaltenauswahl (=1) handelt.
     */
    private static void pauseBisEndeDerInitialisierung(int modus) {
        if (modus == MODUS_SORTIERUNG) {
            while (!_istInitialisierungAbgeschlossenSort) {
                try {
                    Thread.sleep(100);
                }
                catch (Exception e) {
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }
        } else if (modus == MODUS_SPALTEN) {
            while (!_istInitialisierungAbgeschlossenSpalten) {
                try {
                    Thread.sleep(100);
                }
                catch (Exception e) {
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }
        }
    }

    /**
     * Speichere den Inhalt von _sortierungMap in XMLDatei.
     */
    public void speichereAlleSortierungen() {
        String parameterID;
        String ursache;

        if (_modus == MODUS_SORTIERUNG) {
            parameterID = KonstantenGTM.ALLE_SORTIERUNGEN;
            ursache     = "Speichern von Sortierungen";
        } else if (_modus == MODUS_SPALTEN) {
            parameterID = KonstantenGTM.ALLE_SPALTENAUSWAHL;
            ursache     = "Speichern von Spalten";
        } else {
            MessageDialog.openError(PlatformUI.getWorkbench().getDisplay().getActiveShell(), "Fehler beim Speichern", "Es ist ein Fehler beim Speichern aufgetretten. Modus kann nur 0 oder 1 sein.");

            return;
        }

        // Kodiere _sortierungMap ber XMLEncoder:
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        XMLEncoder            xml = new XMLEncoder(out);

        xml.writeObject(_sortierungMap);
        xml.close();

        boolean gespeichert = GrundEinstellungen.speichereParameterFuerAlleBenutzer(parameterID, out.toString(), ursache);

        if (!gespeichert) {
            MessageDialog.openError(PlatformUI.getWorkbench().getDisplay().getActiveShell(), "Fehler beim Speichern", "Es ist ein Fehler beim Speichern der Sortierungen aufgetretten. Details siehe unten im Meldungsfenster.");
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 05.11.13
