/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.sortierung;

//~ KLASSEN ===================================================================

/**
 * Ein Sortierobjekt besteht aus einem String, welcher den Spaltennamen reprsentiert und einer
 * booleschen Variable, die angibt ob die Sortierung aufsteigend oder absteigend ist.
 * True steht fr aufsteigend, false fr absteigend.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class SortierObjekt {
    private String  _sortierWahl = "";
    private boolean _aufsteigend;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Wichtig! Wird vom Datenverteiler fr die Speicherung bentigt.
     * Erzeugt eine Instanz der Klasse SortierObjekt.
     *
     */
    public SortierObjekt() {}

    /**
     *
     * Erzeugt eine Instanz der Klasse SortierObjekt.
     *
     * @param sortierWahl Sortierauswahl.
     * @param aufsteigend true, wenn aufsteigend sortiert, false sonst.
     */
    public SortierObjekt(String sortierWahl, boolean aufsteigend) {
        _sortierWahl = sortierWahl;
        _aufsteigend = aufsteigend;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (!(obj instanceof SortierObjekt)) {
            return false;
        }

        SortierObjekt other = (SortierObjekt) obj;

        if (_aufsteigend != other._aufsteigend) {
            return false;
        }

        if (_sortierWahl == null) {
            if (other._sortierWahl != null) {
                return false;
            }
        } else if (!_sortierWahl.equals(other._sortierWahl)) {
            return false;
        }

        return true;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'sortierWahl'.
     *
     * @return Wert der Eigenschaft 'sortierWahl'.
     *
     * @see #_sortierWahl
     */
    public String getSortierWahl() {
        return _sortierWahl;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime  = 31;
        int       result = 1;

        result = prime * result + (_aufsteigend ? 1231 : 1237);
        result = prime * result + ((_sortierWahl == null) ? 0 : _sortierWahl.hashCode());

        return result;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'aufsteigend'.
     *
     * @return Wert der Eigenschaft 'aufsteigend'.
     *
     * @see #_aufsteigend
     */
    public boolean isAufsteigend() {
        return _aufsteigend;
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'aufsteigend'.
     *
     * @param aufsteigend Wert zum Setzen der Eigenschaft 'aufsteigend'.
     *
     * @see #_aufsteigend
     */
    public void setAufsteigend(boolean aufsteigend) {
        _aufsteigend = aufsteigend;
    }

    /**
     * Setter fr Eigenschaft 'sortierWahl'.
     *
     * @param sortierWahl Wert zum Setzen der Eigenschaft 'sortierWahl'.
     * Wichtig! Wird vom Datenverteiler fr die Speicherung bentigt.
     * @see #_sortierWahl
     */
    public void setSortierWahl(String sortierWahl) {
        _sortierWahl = sortierWahl;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 03.09.10
