/*
 * @(#)GanglinienGUIVew.java   17.09.10
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.visualisierung.ganglinien;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.kalender.DatumZeit;
import de.bsvrz.dav.daf.main.archive.ArchiveData;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.archive.ArchiveDataQueryResult;
import de.bsvrz.dav.daf.main.archive.ArchiveDataSpecification;
import de.bsvrz.dav.daf.main.archive.ArchiveDataStream;
import de.bsvrz.dav.daf.main.archive.ArchiveOrder;
import de.bsvrz.dav.daf.main.archive.ArchiveQueryPriority;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestOption;
import de.bsvrz.dav.daf.main.archive.ArchiveTimeSpecification;
import de.bsvrz.dav.daf.main.archive.TimingType;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;

import de.inovat.buv.gtm.datvew.lib.Fahrbahnen;
import de.inovat.buv.gtm.datvew.lib.Messquerschnitt;
import de.inovat.buv.gtm.datvew.lib.SelektionVew;
import de.inovat.buv.gtm.datvew.lib.ZaehlStelle;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;
import de.inovat.buv.inovat.lib.konstanten.Konstanten;
import de.inovat.buv.plugin.gtm.navigation.lib.LangzeitzaehlstellenAuswahlFunktionen;
import de.inovat.buv.plugin.gtm.visualisierung.Activator;
import de.inovat.buv.plugin.gtm.visualisierung.KonstantenVisualisierung;
import de.inovat.buv.projektlib.dav.DavDatenVew;
import de.inovat.buv.projektlib.speicher.GrundEinstellungen;
import de.inovat.buv.projektlib.speicher.OrdnerEinstellungen;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.TreeItem;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;

//~ JDK IMPORTE ===============================================================

import java.io.IOException;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

//~ KLASSEN ===================================================================

/**
 * Verwaltung des GUI 'Ganglinien'.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 408 $ / $Date: 2013-08-01 14:05:39 +0200 (Do, 01 Aug 2013) $ / ($Author: LG $)
 */
public class GanglinienGUIVew {

    /** Anzahl der zu selektierten Tage, MQs oer Zhlstellen. */
    protected static final int ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS = 7;

    //~ FELDER ================================================================

    /** Fahrstreife, fr die Ganglinien-Berechnungen durchgefhrt wurden. */
    private Fahrbahnen _fahrstreifeBerechnet = null;

    /**
     * Array der Zeiten, fr die Ganglinien-Berechnungen durchgefhrt wurden.
     * Kann null-Elemente am Ende enthalten.
     */
    private Long[] _arDatumBerechnet = new Long[ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS];

    /** GanglinienArt, fr die die Ganglinien berechnet wurden. */
    private GanglinienArt _ganglinienArtBerechnet = GanglinienArt.Undefiniert;

    /** Die im GUI selektierte GanglinienArt. */
    private GanglinienArt _ganglinienArtSelektiert = GanglinienArt.Undefiniert;

    /**
     * Liste der ZeitPunkte fr die X-Achse.
     *
     * Falls GanglinienArt 'Tage' ist, werden nur die ZeitAngaben relevant (von 00 bis 24 Uhr).
     * Falls GanglinienArt 'Monat' ist, wird alle Tage des angegebenen Monats mit der Zeit 00 enthalten von (1 bis 28-31).
     * Falls GanglinienArt 'Jahr' ist, wird alle Tage des angegebenen Jahres mit der Zeit 00 enthalten.
     */
    private List<Long> _listeZeitPunkte = new ArrayList<Long>();

    /**
     * Liste der QSgv-Werte fr die selektierte Fahrbahn und jedes Datum aus dem Array (GanglinienArt='Tage')
     * oder fr selektiertes Datum und jeden MQ aus dem Array (GanglinienArt='Monat'oder'Jahr').
     *
     * Jedes Element der Liste ist eine Liste der QSgv-Werte oder null.
     */
    private List<List<Integer>> _listenQSgv = new ArrayList<List<Integer>>();

    /**
     * Liste der QKfz-Werte fr die selektierte Fahrbahn und jedes Datum aus dem Array (GanglinienArt='Tage')
     * oder fr selektiertes Datum und jeden MQ aus dem Array (GanglinienArt='Monat'oder'Jahr').
     *
     * Jedes Element der Liste ist eine Liste der QKFZ-Werte oder null.
     */
    private List<List<Integer>> _listenQKfz = new ArrayList<List<Integer>>();

    /**
     * Array der ZaehlStelle, fr die Ganglinien-Berechnungen durchgefhrt wurden.
     * Kann null-Elemente am Ende enthalten.
     */
    private ZaehlStelle[] _arZsBerechnet = new ZaehlStelle[ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS];

    /**
     * Array der MQs, fr die Ganglinien-Berechnungen durchgefhrt wurden.
     * Kann null-Elemente am Ende enthalten.
     */
    private Messquerschnitt[] _arMqBerechnet = new Messquerschnitt[ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS];

    /** Liste der ausgewhlten Zhlstellen. */
    private List<ZaehlStelle> _listeZs = new ArrayList<ZaehlStelle>();

    /** Datenverteilerverbindung. */
    private ClientDavInterface _dav;

    /** GUI-Instanz. */
    private GanglinienGUI _gui;
    private boolean       _istArchivVerfuegbar;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse GanglinienGUIVew.
     *
     * @param gui GUI-Instanz.
     */
    protected GanglinienGUIVew(GanglinienGUI gui) {
        _gui = gui;

        // ------------------------------------------------------------------
        // Datenverteiler Zugriff holen:
        _dav = DavDatenVew.getInstanz().getDav();
    }

    //~ AUFZAEHLUNGSTYPEN =====================================================

    /**
     * GanglinienArt.
     */
    protected static enum GanglinienArt {
        TAG_TLS, TAG_BAST, MONAT_MQ, JAHR_MQ, MONAT_ZS, JAHR_ZS, Undefiniert;

        /**
         * Stelle fest, ob die Instanz ein Jahr darstellt.
         *
         * @return true, falls die Instanz ein Jahr darstellt,
         * false - sonst.
         */
        protected boolean istJahr() {
            return (this == JAHR_MQ) || (this == JAHR_ZS);
        }

        /**
         * Stelle fest, ob die Instanz einen Monat darstellt.
         *
         * @return true, falls die Instanz einen Monat darstellt,
         * false - sonst.
         */
        protected boolean istMonat() {
            return (this == MONAT_MQ) || (this == MONAT_ZS);
        }

        /**
         * Stelle fest, ob die Instanz einen Intervall fr MQ darstellt.
         *
         * @return true, falls die Instanz einen Intervall fr MQ darstellt,
         * false - sonst.
         */
        protected boolean istMq() {
            return (this == MONAT_MQ) || (this == JAHR_MQ);
        }

        /**
         * Stelle fest, ob die Instanz einen Tag darstellt.
         *
         * @return true, falls die Instanz einen Tag darstellt,
         * false - sonst.
         */
        protected boolean istTag() {
            return (this == TAG_BAST) || (this == TAG_TLS);
        }

        /**
         * Stelle fest, ob die Instanz einen Intervall fr Zhlstelle darstellt.
         *
         * @return true, falls die Instanz einen Intervall fr Zhlstelle darstellt,
         * false - sonst.
         */
        protected boolean istZs() {
            return (this == MONAT_ZS) || (this == JAHR_ZS);
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Erlaube die Aktion 'Aktualisieren',
     * falls die EinstellungsDaten sich gendert haben und korrekt sind.
     */
    private void aktiviereBtnAktualisieren() {

        //
        Fahrbahnen        fs                = ermittleFahrstreifenSelektiert();
        Messquerschnitt[] arMqSelektiert    = ermittleArMqSelektiert();
        ZaehlStelle[]     arZsSelektiert    = ermittleArZsSelektiert();
        Long[]            arDatumSelektiert = ermittleArrayDatumSelektiert();

        // -------------------------------------------------------------------
        // Prfe, ob die selektierten Daten korrekt sind:
        boolean sindSelektierteDatenKorrekt = true;

        // Tag:
        if (_ganglinienArtSelektiert.istTag()) {

            // --- Datum:
            int anzNull = 0;

            for (Long datum : arDatumSelektiert) {
                if (datum == null) {
                    anzNull++;
                }
            }

            if (anzNull == ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS) {
                sindSelektierteDatenKorrekt = false;
            }

            // --- Im Baum selektierte Elemente:
            else {
                if (fs == null) {
                    sindSelektierteDatenKorrekt = false;
                }
            }
        }

        // Monat /Jahr:
        else {

            // --- Datum:
            if (arDatumSelektiert[0] == null) {
                sindSelektierteDatenKorrekt = false;
            }

            // --- Im Baum selektierte Elemente:
            else {
                int anzNull = 0;

                // MQ:
                if (_ganglinienArtSelektiert.istMq()) {
                    for (Messquerschnitt mq : arMqSelektiert) {
                        if (mq == null) {
                            anzNull++;
                        }
                    }
                }

                // ZS:
                else if (_ganglinienArtSelektiert.istZs()) {
                    for (ZaehlStelle zs : arZsSelektiert) {
                        if (zs == null) {
                            anzNull++;
                        }
                    }
                }

                if (anzNull == ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS) {
                    sindSelektierteDatenKorrekt = false;
                }
            }
        }

        // -------------------------------------------------------------------
        // Prfe, ob die zu berechnete Daten sich gendert haben:
        boolean neuBerechnen = false;

        if (sindSelektierteDatenKorrekt) {

            // --- GanglinienArt:
            if (_ganglinienArtBerechnet != _ganglinienArtSelektiert) {
                neuBerechnen = true;
            } else {

                // --- Datum:
                // Tag:
                if (_ganglinienArtSelektiert.istTag()) {
                    boolean gefunden;

                    for (Long lSel : arDatumSelektiert) {
                        gefunden = false;

                        for (Long lBer : _arDatumBerechnet) {
                            if ((lSel != null) && (lSel.equals(lBer))) {
                                gefunden = true;

                                break;
                            }
                        }

                        if (!gefunden) {
                            neuBerechnen = true;

                            break;
                        }
                    }
                }

                // Monat:
                else if (_ganglinienArtSelektiert.istMonat()) {
                    if (DatumFunktionen.getMonatAnfang(_arDatumBerechnet[0]) != DatumFunktionen.getMonatAnfang(arDatumSelektiert[0])) {
                        neuBerechnen = true;
                    }
                }

                // Jahr:
                else if (_ganglinienArtSelektiert.istJahr()) {
                    if (DatumFunktionen.getJahr(_arDatumBerechnet[0]) != DatumFunktionen.getJahr(arDatumSelektiert[0])) {
                        neuBerechnen = true;
                    }
                }

                // --- Im Baum selektierte Elemente:
                if (!neuBerechnen) {

                    // Tag (FS):
                    if (_ganglinienArtSelektiert.istTag()) {
                        if (fs != _fahrstreifeBerechnet) {
                            neuBerechnen = true;
                        }
                    }

                    // MQ:
                    else if (_ganglinienArtSelektiert.istMq()) {
                        List<Messquerschnitt> liste = Arrays.asList(_arMqBerechnet);

                        for (Messquerschnitt mq : arMqSelektiert) {
                            if (!liste.contains(mq)) {
                                neuBerechnen = true;

                                break;
                            }
                        }
                    }

                    // ZS:
                    else if (_ganglinienArtSelektiert.istZs()) {
                        List<ZaehlStelle> liste = Arrays.asList(_arZsBerechnet);

                        for (ZaehlStelle zs : arZsSelektiert) {
                            if (!liste.contains(zs)) {
                                neuBerechnen = true;

                                break;
                            }
                        }
                    }
                }
            }
        }

        // -------------------------------------------------------------------
        // ToolTip fr den Baum:
        String text = "";

        // Tag (FS):
        if (_ganglinienArtSelektiert.istTag()) {
            text = "Selektierter Fahrstreifen:";

            if (fs != null) {
                text = String.format("%s %s (%s)", text, fs.getSystemObjekt().getName(), fs.getSystemObjekt().getPidOrId());
            }
        }

        // MQ:
        else if (_ganglinienArtSelektiert.istMq()) {
            text = "Selektierte MQs (nur erste " + ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS + "):" + NEWLINE + NEWLINE;

            for (Messquerschnitt mq : arMqSelektiert) {
                if (mq != null) {
                    text = String.format("%s%s (%s)%s", text, mq.getSystemObjekt().getName(), mq.getSystemObjekt().getPidOrId(), NEWLINE);
                }
            }
        }

        // ZS:
        else if (_ganglinienArtSelektiert.istZs()) {
            text = "Selektierte ZS (nur erste " + ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS + "):" + NEWLINE + NEWLINE;

            for (ZaehlStelle zs : arZsSelektiert) {
                if (zs != null) {
                    text = String.format("%s%s (%s)%s", text, zs.getSystemObjekt().getName(), zs.getSystemObjekt().getPidOrId(), NEWLINE);
                }
            }
        }

        _gui.getTreeViewerZs().getTree().setToolTipText(text);

        // -------------------------------------------------------------------
        // Aktiviere das Button:
        _gui.getBtnAktualisieren().setEnabled(neuBerechnen);
    }

    /**
     * Aktualisiere die Grafik.
     */
    private void aktualisiereGrafik() {

        // Baue Grafik neu:
        if (_listeZeitPunkte.size() > 0) {
            _gui.getGrafik().build();
            _gui.getGrafik().redraw();
        }
    }

    /**
     * Ermittle die Q-Werte aus dem Archiv fr das angegebene Objekt und alle ZeitPunkte.
     *
     * @param objekt Objekt, fr das die Werte abgefragt werden soll.
     * Kann von Typ Fahrbahnen, Messquerschnitt oder ZaehlStelle sein.
     * @param listeZeitPunkte Liste der ZeitPunkte.
     *
     * @return Q-Werte aus dem Archiv fr das angegebene Objekt und alle ZeitPunkte.
     * Geliefert werden zwei Listen: erste - fr die QKfz-Werte und die zweite - fr die Q-Sgv.
     * Q-Wert kann null sein.
     */
    private List<List<Integer>> anfragenArchivDaten(Object objekt, List<Long> listeZeitPunkte) {

        // --------------------------------------------------------------
        int          objektN  = 0;     // 1 - Fahrstreife, 2 - Messquerschnitt.
        String       aspekt   = "";
        String       atg      = "";
        SystemObject so       = null;
        String[]     arQKfz   = new String[] { "", "", "", "", "", "", "", "", "" };
        String[]     arQSgv   = new String[] { "" };
        String       attInAtl = null;  // falls QKfz und QSgv ein AttributListe darstellen, wird mit dem AttributWert initialisiert.

        // Fahrstreife:
        if (objekt instanceof Fahrbahnen) {
            Fahrbahnen fb = (Fahrbahnen) objekt;

            so      = fb.getSystemObjekt();
            objektN = 1;

            if (_ganglinienArtBerechnet == GanglinienArt.TAG_TLS) {
                atg    = "atg.verkehrsDatenLangZeitIntervallGeschwindigkeitVerteilungFs";
                aspekt = "asp.externeErfassung";
                arQKfz = new String[] { "qKfzNk", "qKrad", "qPkw", "qLfw", "qPkwA", "qLkw", "qLkwA", "qSattelKfz", "qBus" };
                arQSgv = new String[] { "qSGV" };
            } else if (_ganglinienArtBerechnet == GanglinienArt.TAG_BAST) {
                atg      = "atg.verkehrsDatenLangZeitIntervall";
                aspekt   = "asp.messWertErsetzung";
                arQKfz   = new String[] { "qKfz" };
                arQSgv   = new String[] { "qLkw", "qLkwA", "qSattelKfz" };
                attInAtl = "Wert";
            }
        }

        // Messquerschnitt:
        else if (objekt instanceof Messquerschnitt) {
            Messquerschnitt mq = (Messquerschnitt) objekt;

            so      = mq.getSystemObjekt();
            objektN = 2;
            aspekt  = "asp.agregationDtvTag";
            atg     = "atg.verkehrsDatenLangZeitIntervallGeschwindigkeitVerteilungDtvMq";
            arQKfz  = new String[] { "QKfzNk", "QKrad", "QPkw", "QLfw", "QPkwA", "QLkw", "QLkwA", "QSattelKfz", "QBus" };
            arQSgv  = new String[] { "QSGV" };
        }

        // ZaehlStelle:
        else if (objekt instanceof ZaehlStelle) {
            ZaehlStelle zs = (ZaehlStelle) objekt;

            so      = zs.getSystemObjekt();
            objektN = 2;
            aspekt  = "asp.agregationDtvTag";
            atg     = "atg.verkehrsDatenLangZeitIntervallGeschwindigkeitVerteilungDtvMq";
            arQKfz  = new String[] { "QKfzNk", "QKrad", "QPkw", "QLfw", "QPkwA", "QLkw", "QLkwA", "QSattelKfz", "QBus" };
            arQSgv  = new String[] { "QSGV" };
        }

        List<ArchiveData> listeArchivDaten = anfragenArchivDaten(aspekt, atg, so, listeZeitPunkte);

        // --------------------------------------------------------------
        List<Integer> listeQKfz = new ArrayList<Integer>();
        List<Integer> listeQSgv = new ArrayList<Integer>();
        ArchiveData   arch;
        ArchiveData   archPassendes;
        long          maxDatumArch;

        // Die Archivliste kann Elemente enthalten, deren Zeitpunkt frher als abgefragtes Intervall liegt.
        // Durchlaufe die Liste bis der erste Eintrag gefunden wird,
        // der an Anfang des Intervalls liegt (mit dem wird weiter die Suche anfangen):
        int indexArchiv = listeArchivDaten.size();

        for (int i = 0; i < listeArchivDaten.size(); i++) {
            arch = listeArchivDaten.get(i);

            if (arch.getDataTime() >= listeZeitPunkte.get(0)) {
                indexArchiv = i;

                break;
            }
        }

        System.out.println("---------------------------");

        // Ermittle fr jeden ZeitPunkt die Q-Werte:
        for (long datum : listeZeitPunkte) {
            System.out.println(String.format(" --- %s", DatumFunktionen.getDatumZeitAlsString(datum)));

            // --------------------------------------------------------------------------------
            // Lese ArchivDaten und betrachte alle Eintrge, die dem ZeitPunkt entsprechen,
            // d.h. fr die StundenWerte werden alle ArchivDaten mit den gleichen Datum und
            // Stunde ohne die Bercksichtigung der Minuten selektiert und fr die TagesWerte
            // - alle ArchivDaten mit dem gleichen Datum ohne die Bercksichtigung der Zeit.
            //
            // Ausgewhlt wird der Eintrag mit spterem Datum der Archivierung.
            // --------------------------------------------------------------------------------
            maxDatumArch  = -1;
            archPassendes = null;

            // Suche in Archivdaten passende Datenstzte:
            for (int i = indexArchiv; i < listeArchivDaten.size(); i++) {
                arch = listeArchivDaten.get(i);

                // Prfe, ob das ArchivDataDatum zum angegebenen ZeitPunkt passt:
                // Fahrstreife:
                if (objektN == 1) {

                    // Falls ArchivDataDatum schon in der nchsten Stunde liegt -> Abbruch:
                    if (arch.getDataTime() >= datum + 60 * 60 * 1000) {
                        indexArchiv = i;

                        break;
                    }
                }

                // Messquerschnitt:
                else if (objektN == 2) {

                    // Falls ArchivDataDatum schon in dem nchsten Tag liegt -> Abbruch:
                    if (arch.getDataTime() >= datum + DatumFunktionen.ANZAHL_MILLISEKUNDEN_PRO_TAG) {
                        indexArchiv = i;

                        break;
                    }
                }

                System.out.println(String.format(" --- --- %s -- %s", DatumFunktionen.getDatumZeitAlsString(arch.getDataTime()), DatumFunktionen.getDatumZeitAlsString(arch.getArchiveTime())));

                // Falls das ArchivDataDatum zum angegebenen ZeitPunkt passt,
                // vergleiche das Datum der Archivierung:
                if (arch.getArchiveTime() > maxDatumArch) {
                    maxDatumArch  = arch.getArchiveTime();
                    archPassendes = arch;
                }

                //
                if (i == listeArchivDaten.size() - 1) {
                    indexArchiv = listeArchivDaten.size();
                }
            }

            // Falls ein passender ArchivEintrag gefunden wurde, lese die Q-Werte:
            if (archPassendes != null) {
                System.out.println(String.format(" --- --- --- %s -- %s", DatumFunktionen.getDatumZeitAlsString(archPassendes.getDataTime()), DatumFunktionen.getDatumZeitAlsString(archPassendes.getArchiveTime())));

                // qKfz:
                int     summeQKfz = 0;
                int     q;
                boolean nichtNull = false;

                for (String qKfz : arQKfz) {
                    try {

                        // qKfz ist Attribut:
                        if (attInAtl == null) {
                            q = archPassendes.getData().getScaledValue(qKfz).intValue();
                        }

                        // qKfz istAttributListe:
                        else {
                            q = archPassendes.getData().getItem(qKfz).getScaledValue(attInAtl).intValue();
                        }

                        summeQKfz = summeQKfz + q;
                        nichtNull = true;
                        System.out.println(String.format(" --- --- --- %s: %s", qKfz, q));
                    }
                    catch (Exception e) {
                        System.out.println(String.format(" --- --- --- %s: null", qKfz));
                    }
                }

                if (nichtNull) {
                    listeQKfz.add(summeQKfz);
                } else {
                    listeQKfz.add(null);
                }

                // QSgv:
                int summeQSgv = 0;

                nichtNull = false;

                for (String qSgv : arQSgv) {
                    try {

                        // qKfz ist Attribut:
                        if (attInAtl == null) {
                            q = archPassendes.getData().getScaledValue(qSgv).intValue();
                        }

                        // qKfz istAttributListe:
                        else {
                            q = archPassendes.getData().getItem(qSgv).getScaledValue(attInAtl).intValue();
                        }

                        summeQSgv = summeQSgv + q;
                        nichtNull = true;
                        System.out.println(String.format(" --- --- --- QSgv: %s", q));
                    }
                    catch (Exception e) {
                        System.out.println(" --- --- --- QSgv: null");
                    }
                }

                if (nichtNull) {
                    listeQSgv.add(summeQSgv);
                } else {
                    listeQSgv.add(null);
                }
            }

            //
            else {
                listeQKfz.add(null);
                listeQSgv.add(null);
            }
        }

        List<List<Integer>> listen = new ArrayList<List<Integer>>();

        listen.add(listeQKfz);
        listen.add(listeQSgv);

        return listen;
    }

    /**
     * Archiv-Anfrage fr die angegebene Parameter.
     *
     * @param aspekt Aspekt.
     * @param atg AtributGruppe.
     * @param so SystemObjekt.
     * @param listeZeitPunkte Liste der ZeitPunkte.
     *
     * @return Liste der ArchivDaten fr angegebene Parameter.
     * Liste ist nach Archivdatum sortiert.
     */
    private List<ArchiveData> anfragenArchivDaten(String aspekt, String atg, SystemObject so, List<Long> listeZeitPunkte) {

        // Aspekt fr die Archivanfrage.
        Aspect aspExterneErfassung = _dav.getDataModel().getAspect(aspekt);

        // Array Archivdatastream.
        ArchiveDataStream[] archiveDataStream;

        // Archivanfrageergebnis.
        ArchiveDataQueryResult queryResult;
        ArchiveRequestManager  archiveRequestManager = _dav.getArchive();

        // Es werden nur die Daten vom Typ ArchiveDataKind.ONLINE angefordert.
        ArchiveDataKindCombination archiveDataKindCombination = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE);

        // Liste der Archivspezifikation fr die Anfrage.
        List<ArchiveDataSpecification> archiveDataSpecifications = new LinkedList<ArchiveDataSpecification>();

        // Datenbeschreibung der Archivanfrage.
        DataDescription dataDescription = new DataDescription(_dav.getDataModel().getAttributeGroup(atg), aspExterneErfassung);

        // Startzeitpunkt fr die Archivanfrage.
        long startZeit = listeZeitPunkte.get(0);

        // Endzeitpunkt fr die Archivanfrage.
        long endZeit = listeZeitPunkte.get(listeZeitPunkte.size() - 1);

        // Erzeugung ArchiveTimeSpecification mit aktuellen Start und Endzeitpunkten.
        ArchiveTimeSpecification archiveTimeSpecification = new ArchiveTimeSpecification(TimingType.DATA_TIME, false, startZeit, endZeit);

        archiveDataSpecifications.add(new ArchiveDataSpecification(archiveTimeSpecification, archiveDataKindCombination, ArchiveOrder.BY_INDEX, ArchiveRequestOption.NORMAL, dataDescription, so));

        // Alle Daten einsortieren nach Datenzeitstempel. Dadurch werden bei doppelten Datenstzen immer nur die neusten bercksichtigt.
        Map<Long, ArchiveData> mapArchivIntervallDaten = new TreeMap<Long, ArchiveData>();

        // Auswertung der Importierten Daten zuzglich der Archivanfrage Daten.
        if (archiveRequestManager.isArchiveAvailable()) {
            try {

                // Archivanfrageergebnis.
                queryResult = archiveRequestManager.request(ArchiveQueryPriority.MEDIUM, archiveDataSpecifications);

                if (queryResult.isRequestSuccessful()) {
                    archiveDataStream = queryResult.getStreams();

                    for (ArchiveDataStream anArchiveDataStream : archiveDataStream) {
                        ArchiveData archivDaten = anArchiveDataStream.take();

                        while (archivDaten != null) {
                            if ((archivDaten.getData() != null) && (archivDaten.getDataType().equals(DataState.DATA))) {
                                mapArchivIntervallDaten.put(archivDaten.getDataTime(), archivDaten);
                            }

                            archivDaten = anArchiveDataStream.take();
                        }
                    }
                }
            }
            catch (IllegalStateException e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
            catch (InterruptedException e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
            catch (IOException e) {
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
        } else {
            Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, "Das Archiv ist nicht verfgbar."));
            _istArchivVerfuegbar = false;
        }

        // Erstelle die nach Archivdatum sortierte Liste:
        List<ArchiveData> listeArchivDaten = new ArrayList<ArchiveData>();

        for (long eitPunkt : mapArchivIntervallDaten.keySet()) {
            listeArchivDaten.add(mapArchivIntervallDaten.get(eitPunkt));
        }

        return listeArchivDaten;
    }

    /**
     * Berechne alle Werte fr die Grafik
     */
    private void berechneGrafikWerte() {

        // -------------------------------------------------------
        // bernehme selektierte Werte:
        // --- GanglinienArt:
        _ganglinienArtBerechnet = _ganglinienArtSelektiert;

        // --- Datum:
        _arDatumBerechnet = ermittleArrayDatumSelektiert();

        // --- Im Baum selektierte Elemente:
        _fahrstreifeBerechnet = ermittleFahrstreifenSelektiert();
        _arMqBerechnet        = ermittleArMqSelektiert();
        _arZsBerechnet        = ermittleArZsSelektiert();

        // -------------------------------------------------------
        // Verwalte die darzustellenden Werte:
        String  text;
        boolean istGanglinienArtTage = _ganglinienArtBerechnet.istTag();

        // Setze den GruppenName und LabelInfo:
        if (istGanglinienArtTage) {
            _gui.getGrpDarstellungsWerte().setText("Darzustellende Tage");
            _gui.getLbDarstellungInfo().setText(_fahrstreifeBerechnet.getSystemObjekt().getNameOrPidOrId());
        } else {
            if (_ganglinienArtBerechnet.istMq()) {
                _gui.getGrpDarstellungsWerte().setText("Darzustellende MQs");
            } else if (_ganglinienArtBerechnet.istZs()) {
                _gui.getGrpDarstellungsWerte().setText("Darzustellende ZS");
            }

            int jahr = DatumFunktionen.getJahr(_arDatumBerechnet[0]);

            if (_ganglinienArtBerechnet.istMonat()) {
                String monat = DatumFunktionen.getMonatNamen(DatumFunktionen.getMonat(_arDatumBerechnet[0]));

                _gui.getLbDarstellungInfo().setText(monat + " " + jahr);
            } else if (_ganglinienArtBerechnet.istJahr()) {
                _gui.getLbDarstellungInfo().setText("Jahr " + jahr);
            }
        }

        for (int i = 0; i < ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
            text = "";

            if (istGanglinienArtTage) {
                if (_arDatumBerechnet[i] != null) {
                    text = DatumFunktionen.getDatumAlsString(_arDatumBerechnet[i]);
                }
            } else if (_ganglinienArtBerechnet.istMq()) {
                if (_arMqBerechnet[i] != null) {
                    text = _arMqBerechnet[i].getSystemObjekt().getNameOrPidOrId();
                }
            } else if (_ganglinienArtBerechnet.istZs()) {
                if (_arZsBerechnet[i] != null) {
                    text = _arZsBerechnet[i].getSystemObjekt().getNameOrPidOrId();
                }
            }

            _gui.getArCbtnDarzustellendeWerte()[i].setText(text);
            _gui.getArCbtnDarzustellendeWerte()[i].setSelection(text.length() > 0);
            _gui.getArCbtnDarzustellendeWerte()[i].setVisible(text.length() > 0);
        }

        _gui.getGrpDarstellungsWerte().layout();
        _gui.getGrpDarstellungsWerte().getParent().layout();

        // -------------------------------------------------------
        erstelleListeZeitPunkte();

        // -------------------------------------------------------
        erstelleListenQ();
    }

    /**
     * Schaltflche 'Aktualisieren' wird selektiert.
     */
    protected void btnAktualisierenSelektiert() {

        //
        berechneGrafikWerte();

        // Aktualisiere die Grafik:
        aktualisiereGrafik();

        //
        _gui.getBtnAktualisieren().setEnabled(false);
    }

    /**
     * Schaltflche 'Speichern' wird selektiert.
     *
     * Exportiere die Grafik in die png-Datei.
     */
    protected void btnSpeichernSelektiert() {
        if (_gui.getGrafik().getImage() != null) {

            // DateiAuswahlDialog:
            FileDialog fileDialog = new FileDialog(_gui.getShell(), SWT.SAVE);

            fileDialog.setText("Exportdatei auswhlen");
            fileDialog.setFilterExtensions(new String[] { "*.png" });

            String ordner = ermittleGanglinienOrdner();

            fileDialog.setFilterPath(ordner);

            String file = _gui.getLbDarstellungInfo().getText();

            if (_ganglinienArtBerechnet == GanglinienArt.TAG_TLS) {
                file = file + "(TLS)";
            } else if (_ganglinienArtBerechnet == GanglinienArt.TAG_BAST) {
                file = file + "(BASt)";
            } else if (_ganglinienArtBerechnet.istMq()) {
                file = file + "(MQ)";
            } else if (_ganglinienArtBerechnet.istZs()) {
                file = file + "(ZS)";
            }

            fileDialog.setFileName(file + ".png");

            if (fileDialog.open() != null) {
                file = fileDialog.getFilterPath() + Konstanten.FILESEPARATOR + fileDialog.getFileName();

                // Exportiere die Grafik in die Datei:
                ImageData   id     = _gui.getGrafik().getImage().getImageData();
                ImageLoader loader = new ImageLoader();

                loader.data = new ImageData[] { id };

                try {
                    loader.save(file, SWT.IMAGE_PNG);

                    // Speichere den ausgewhlten Ordner:
                    GrundEinstellungen.speichereParameterFuerBenutzer(KonstantenVisualisierung.PARAMETER_GANGLINIEN_ORDNER, fileDialog.getFilterPath(), "Ganglinien Ordner");
                }
                catch (Exception e) {
                    MessageDialog.openError(_gui.getShell(), "Fehler", "Es ist ein Fehler beim Speichern passiert." + NEWLINE + NEWLINE + "(siehe Meldungen im ErrorLog)");
                }
            }
        } else {
            MessageDialog.openError(_gui.getShell(), "Fehler", "Die Grafik ist noch nicht erstellt !" + NEWLINE + NEWLINE + "Bitte, zuerst <Aktualisieren> drcken !");
        }
    }

    /**
     * Schaltflche 'ZhlstellenAuswahl' wird selektiert.
     */
    protected void btnZsAuswahlSelektiert() {

        // ffne den AuswahlDialog:
        _listeZs = LangzeitzaehlstellenAuswahlFunktionen.langzeitZaehlstellenAuswahlDialogOeffnen(_gui.getShell(), _listeZs, null);

        // Falls die ZhlStellen im Dialog selektiert wurden:
        if (_listeZs.size() > 0) {

            // Initialisiere den Baum:
            initialisiereTreeViewerZs();
        }
    }

    /**
     * Check-Button 'DarstellungsWert' (Tag / MQ) wird selektiert.
     */
    protected void cbtnDarstellungsWertSelektiert() {
        aktualisiereGrafik();
    }

    /**
     * CheckButton 'QKfz' wird selektiert.
     */
    protected void cbtnQKfzSelektiert() {
        aktualisiereGrafik();
    }

    /**
     * CheckButton 'QSgv' wird selektiert.
     */
    protected void cbtnQSgvSelektiert() {
        aktualisiereGrafik();
    }

    /**
     * Check-Button 'QWerteManuell' wird selektiert.
     */
    protected void cbtnQWerteManuellSelektiert() {
        boolean istSel = _gui.getCbtnQWerteManuell().getSelection();

        _gui.getLbQMax().setEnabled(istSel);
        _gui.getLbQMin().setEnabled(istSel);
        _gui.getSpinnerQMax().setEnabled(istSel);
        _gui.getSpinnerQMin().setEnabled(istSel);

        //
        aktualisiereGrafik();
    }

    /**
     * Datum wird selektiert.
     */
    protected void dzDatumSelektiert() {

        // Aktiviere das Button:
        aktiviereBtnAktualisieren();
    }

    /**
     * Ermittle das Array der im Baum selektierten Messquerschnitte.
     *
     * @return Array der im Baum selektierten Messquerschnitte.
     * Das Array hat die Gre ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS und kann null-Elemente enthalten.
     */
    private Messquerschnitt[] ermittleArMqSelektiert() {
        Messquerschnitt[] arMqSelektiert = new Messquerschnitt[ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS];

        if (_ganglinienArtSelektiert.istMq()) {
            Object[] arElementeSelektiert = _gui.getTreeViewerZs().getCheckedElements();
            int      anzMq                = 0;

            for (Object element : arElementeSelektiert) {
                if (element instanceof Messquerschnitt) {
                    Messquerschnitt mg = (Messquerschnitt) element;

                    arMqSelektiert[anzMq] = mg;
                    anzMq++;
                }

                if (anzMq == ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS) {
                    break;
                }
            }
        }

        return arMqSelektiert;
    }

    /**
     * Ermittle das Array der im GUI angegebenen Zeitpunkten.
     *
     * @return Array der im GUI angegebenen Zeitpunkten.
     * Das Array hat die Gre ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS und kann null-Elemente enthalten.
     */
    private Long[] ermittleArrayDatumSelektiert() {
        Long[] arDatumSelektiert = new Long[ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS];

        if (_ganglinienArtSelektiert.istTag()) {
            int i = 0;

            for (DatumZeit dz : _gui.getArDzDatum()) {
                if (dz.getDatum() != null) {
                    arDatumSelektiert[i] = dz.getDatum().getTime();
                    i++;
                }
            }

            Arrays.sort(arDatumSelektiert, 0, i);

            for (int j = i; j < ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; j++) {
                arDatumSelektiert[j] = null;
            }
        } else {
            if (_gui.getArDzDatum()[0].getDatum() != null) {
                arDatumSelektiert[0] = _gui.getArDzDatum()[0].getDatum().getTime();
            } else {
                arDatumSelektiert[0] = null;
            }

            for (int i = 1; i < ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
                arDatumSelektiert[i] = null;
            }
        }

        return arDatumSelektiert;
    }

    /**
     * Ermittle das Array der im Baum selektierten Zhlstellen.
     *
     * @return Array der im Baum selektierten Zhlstellen.
     * Das Array hat die Gre ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS und kann null-Elemente enthalten.
     */
    private ZaehlStelle[] ermittleArZsSelektiert() {
        ZaehlStelle[] arZsSelektiert = new ZaehlStelle[ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS];

        if (_ganglinienArtSelektiert.istZs()) {
            Object[] arElementeSelektiert = _gui.getTreeViewerZs().getCheckedElements();
            int      anzZs                = 0;

            for (Object element : arElementeSelektiert) {
                if (element instanceof ZaehlStelle) {
                    ZaehlStelle zs = (ZaehlStelle) element;

                    arZsSelektiert[anzZs] = zs;
                    anzZs++;
                }

                if (anzZs == ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS) {
                    break;
                }
            }
        }

        return arZsSelektiert;
    }

    /**
     * Ermittle den im Baum selektierte Fahrstreifen.
     *
     * @return Der im Baum selektierte Fahrstreifen
     * oder null.
     */
    private Fahrbahnen ermittleFahrstreifenSelektiert() {
        if (_ganglinienArtSelektiert.istTag()) {
            Object[] arElementeSelektiert = _gui.getTreeViewerZs().getCheckedElements();
            int      l                    = arElementeSelektiert.length;

            if (l > 0) {
                if (arElementeSelektiert[l - 1] instanceof Fahrbahnen) {
                    return (Fahrbahnen) arElementeSelektiert[l - 1];
                }
            }
        }

        return null;
    }

    /**
     * Ermittelt den Ordner, wo Ganglinien-Dateien gespeichert werden.
     *
     * @return Ordner, wo Ganglinien-Dateien gespeichert werden.
     */
    private static String ermittleGanglinienOrdner() {

        // Ermittle den gespeicherten Ordner:
        String ordner = GrundEinstellungen.ermittleParameterWertFuerBenutzer(KonstantenVisualisierung.PARAMETER_GANGLINIEN_ORDNER);

        // Ermittle DefaultOrdener:
        if (ordner == null) {
            ordner = OrdnerEinstellungen.ermittleProjektAusgabenUnterOrdner(KonstantenVisualisierung.ORDNER_GANGLINIEN);
        }

        return ordner;
    }

    /**
     * Ermittle den Legende-Text fr den darzustellenden Wert (Datum/MQ) mit angegebenen Index.
     *
     * @param i Index des Wertes im Array.
     *
     * @return Legende-Text fr den darzustellenden Wert (Datum/MQ) mit angegebenen Index.
     */
    protected String ermittleLegende(int i) {
        String text;

        if (_ganglinienArtBerechnet.istTag()) {
            text = "am ";
        } else {
            text = "  ";
        }

        return text + _gui.getArCbtnDarzustellendeWerte()[i].getText();
    }

    /**
     * Ermittle den Titel fr die Legende.
     *
     * @return Titel fr die Legende.
     */
    protected String ermittleLegendenTitel() {
        String tage;

        if (_ganglinienArtBerechnet == GanglinienArt.TAG_TLS) {
            tage = NEWLINE + "(Erfasste Daten)";
        } else if (_ganglinienArtBerechnet == GanglinienArt.TAG_BAST) {
            tage = NEWLINE + "(Messwertersetzte Daten)";
        } else {
            tage = NEWLINE + "DTV Tag";
        }

        return "Legende" + NEWLINE + _gui.getLbDarstellungInfo().getText() + tage;
    }

    /**
     * Ermittle den Wert QMax.
     *
     * @return Im Spinner eingestellter Wert QMax
     * oder -1, falls automatisch berechnet soll.
     */
    protected int ermittleQMax() {
        if (_gui.getCbtnQWerteManuell().getSelection()) {
            return _gui.getSpinnerQMax().getSelection();
        }

        return -1;
    }

    /**
     * Ermittle den Wert QMin.
     *
     * @return Im Spinner eingestellter Wert QMin
     * oder -1, falls automatisch berechnet soll.
     */
    protected int ermittleQMin() {
        if (_gui.getCbtnQWerteManuell().getSelection()) {
            return _gui.getSpinnerQMin().getSelection();
        }

        return -1;
    }

    /**
     * Ermittle die Anzahl der Hauptlinien (MajorGrids) Schritte fr die X-Skala.
     *
     * @return Anzahl der Hauptlinien-Schritte fr die X-Skala.
     */
    protected int ermittleSchrittHauptlinienXSkala() {
        if (_ganglinienArtSelektiert.istJahr()) {
            return 15;
        } else {
            return 1;
        }
    }

    /**
     * Ermittle die Anzahl der Schritte fr die X-Skala.
     *
     * @return Anzahl der Schritte fr die X-Skala.
     */
    protected int ermittleSchrittXSkala() {
        int schritt;

        switch (_ganglinienArtSelektiert) {
        case TAG_BAST :
            schritt = 24;

            break;

        case TAG_TLS :
            schritt = 24;

            break;

        case MONAT_MQ :
            schritt = _listeZeitPunkte.size() - 1;

            break;

        case JAHR_MQ :
            schritt = 24;

            break;

        case MONAT_ZS :
            schritt = _listeZeitPunkte.size() - 1;

            break;

        case JAHR_ZS :
            schritt = 24;

            break;

        case Undefiniert :
            schritt = 0;

            break;

        default :
            schritt = 0;

            break;
        }

        return schritt;
    }

    /**
     * Erstelle die Listen der QKfz- und QSgv-Werte.
     */
    private void erstelleListenQ() {
        _listenQKfz = new ArrayList<List<Integer>>();
        _listenQSgv = new ArrayList<List<Integer>>();

        //
        _istArchivVerfuegbar = true;

        // Tag:
        if (_ganglinienArtBerechnet.istTag()) {
            Long       datum;
            List<Long> listeZeitPunkte;

            for (int i = 0; i < ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
                datum = _arDatumBerechnet[i];

                //
                if (datum == null) {
                    _listenQKfz.add(null);
                    _listenQSgv.add(null);
                } else {

                    // Falls Datum[i]==Datum[i-1] ==> keine Archivanfrage:
                    if ((i > 0) && (_arDatumBerechnet[i].equals(_arDatumBerechnet[i - 1]))) {
                        _listenQKfz.add(_listenQKfz.get(i - 1));
                        _listenQSgv.add(_listenQSgv.get(i - 1));
                    } else {
                        listeZeitPunkte = new ArrayList<Long>();
                        datum           = DatumFunktionen.getDatumMitInitZeit(datum);

                        for (int j = 0; j < _listeZeitPunkte.size(); j++) {
                            if (j == _listeZeitPunkte.size() - 1) {
                                listeZeitPunkte.add(datum + DatumFunktionen.getUhrzeitInMillis(_listeZeitPunkte.get(j)) + DatumFunktionen.ANZAHL_MILLISEKUNDEN_PRO_TAG);
                            } else {
                                listeZeitPunkte.add(datum + DatumFunktionen.getUhrzeitInMillis(_listeZeitPunkte.get(j)));
                            }
                        }

                        // Archivanfrage:
                        List<List<Integer>> listen = anfragenArchivDaten(_fahrstreifeBerechnet, listeZeitPunkte);

                        _listenQKfz.add(listen.get(0));
                        _listenQSgv.add(listen.get(1));
                    }
                }
            }
        }

        // Monat/Jahr MQ:
        else if (_ganglinienArtBerechnet.istMq()) {
            Messquerschnitt mq;

            for (int i = 0; i < ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
                mq = _arMqBerechnet[i];

                if (mq == null) {
                    _listenQKfz.add(null);
                    _listenQSgv.add(null);
                } else {
                    List<List<Integer>> listen = anfragenArchivDaten(mq, _listeZeitPunkte);

                    _listenQKfz.add(listen.get(0));
                    _listenQSgv.add(listen.get(1));
                }
            }
        }

        // Monat/Jahr ZS:
        else if (_ganglinienArtBerechnet.istZs()) {
            ZaehlStelle zs;

            for (int i = 0; i < ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
                zs = _arZsBerechnet[i];

                if (zs == null) {
                    _listenQKfz.add(null);
                    _listenQSgv.add(null);
                } else {
                    List<List<Integer>> listen = anfragenArchivDaten(zs, _listeZeitPunkte);

                    _listenQKfz.add(listen.get(0));
                    _listenQSgv.add(listen.get(1));
                }
            }
        }

        // FehlerMeldung:
        if (!_istArchivVerfuegbar) {
            MessageDialog.openWarning(_gui.getShell(), "Warnung", "Das Archiv ist nicht verfgbar!");
        } else {

            // Prfe die Listen:
            List<List<Integer>> listenQ;
            boolean             istFehler = false;

            for (int q = 0; q < 2; q++) {
                if (q == 0) {
                    listenQ = _listenQKfz;
                } else {
                    listenQ = _listenQSgv;
                }

                for (List<Integer> liste : listenQ) {
                    if (liste != null) {
                        for (Integer qWert : liste) {
                            if (qWert == null) {
                                istFehler = true;

                                break;
                            }
                        }
                    }
                }

                if (istFehler) {
                    MessageDialog.openWarning(_gui.getShell(), "Warnung", "Es sind nicht alle notwendigen Daten in Archiv vorhanden!");

                    break;
                }
            }
        }
    }

    /**
     * Erstelle die Liste der ZeitPunkte.
     */
    private void erstelleListeZeitPunkte() {
        _listeZeitPunkte = new ArrayList<Long>();

        long datum;
        int  anzPunkte;
        long intervall;

        // Tag (24 Stunden):
        if (_ganglinienArtBerechnet.istTag()) {
            datum     = DatumFunktionen.getJahrAnfang(System.currentTimeMillis());
            anzPunkte = 24 + 1;
            intervall = 60 * 60 * 1000;

            for (int i = 0; i < anzPunkte; i++) {
                _listeZeitPunkte.add(datum + i * intervall);
            }
        }

        // Monat (28-31 Tage):
        else if (_ganglinienArtBerechnet.istMonat()) {
            datum     = DatumFunktionen.getMonatAnfang(_arDatumBerechnet[0]);
            anzPunkte = DatumFunktionen.getMaximalenTagImMonat(datum) + 1;

            for (int i = 0; i < anzPunkte; i++) {
                _listeZeitPunkte.add(datum);
                datum = DatumFunktionen.getNaechstenTag(datum);
            }
        }

        // Jahr (365... Tage):
        else if (_ganglinienArtBerechnet.istJahr()) {
            datum = DatumFunktionen.getJahrAnfang(_arDatumBerechnet[0]);

            for (int i = 0; i < 12; i++) {
                int anzTage = DatumFunktionen.getMaximalenTagImMonat(datum);

                for (int j = 0; j < anzTage; j++) {
                    _listeZeitPunkte.add(datum);
                    datum = DatumFunktionen.getNaechstenTag(datum);
                }
            }

            _listeZeitPunkte.add(datum);
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     *     Getter fr Eigenschaft 'ganglinienArtBerechnet'.
     *
     *     @return Wert der Eigenschaft 'ganglinienArtBerechnet'.
     *
     *     @see #_ganglinienArtBerechnet
     */
    public GanglinienArt getGanglinienArtBerechnet() {
        return _ganglinienArtBerechnet;
    }

    /**
     *     Getter fr Eigenschaft 'listenQKfz'.
     *
     *     @return Wert der Eigenschaft 'listenQKfz'.
     *
     *     @see #_listenQKfz
     */
    public List<List<Integer>> getListenQKfz() {
        return _listenQKfz;
    }

    /**
     *     Getter fr Eigenschaft 'listenQSgv'.
     *
     *     @return Wert der Eigenschaft 'listenQSgv'.
     *
     *     @see #_listenQSgv
     */
    public List<List<Integer>> getListenQSgv() {
        return _listenQSgv;
    }

    /**
     * Getter fr Eigenschaft 'listeZeitPunkte'.
     *
     * @return Wert der Eigenschaft 'listeZeitPunkte'.
     *
     * @see #_listeZeitPunkte
     */
    public List<Long> getListeZeitPunkte() {
        return _listeZeitPunkte;
    }

    //~ METHODEN ==============================================================

    /**
     * Gruppe 'Zeitangabe' MouseDoubleClick.
     */
    protected void grpZeitangabeMouseDoubleClick() {

        // Setze alle Zeitangaben mit Tag-Unterschied:
        long datum = _gui.getArDzDatum()[0].getDatum().getTime();

        for (int i = 1; i < ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
            datum = DatumFunktionen.getNaechstenTag(datum);
            _gui.getArDzDatum()[i].getDatum().setTime(datum);

            // Update:
            _gui.getArDzDatum()[i].setFocus();
        }
    }

    /**
     * Initialisiere das GUI.
     */
    protected void initialisiereGui() {

        // ---------------------------------------------------------
        // Initialisiere das Input fr den Baum mit den
        // zuvor im NavigationBaum selektierten Objekten:
        List<ZaehlStelle> listeZs = SelektionVew.getInstanz().ermittleListeSelektierterZaehlStellen();

        _listeZs.addAll(listeZs);

        // ---------------------------------------------------------
        // Initialisiere die GanglinienArt:
        // --- RadioButtons:
        _gui.getRbtnTagTls().setSelection(true);
        _gui.getRbtnTagBast().setSelection(false);
        _gui.getRbtnMonatMq().setSelection(false);
        _gui.getRbtnJahrMq().setSelection(false);
        _gui.getRbtnMonatZs().setSelection(false);
        _gui.getRbtnJahrZs().setSelection(false);

        //
        rbtnGanglinienArtSelektiert();

        // ---------------------------------------------------------
        // Initialisiere die darzustellenden Werte:
        for (int i = 0; i < ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
            _gui.getArCbtnDarzustellendeWerte()[i].setVisible(false);
        }

        // ---------------------------------------------------------
        // Initialisiere die Q-Werte:
        _gui.getCbtnQKfz().setSelection(true);
        _gui.getCbtnQSgv().setSelection(true);
        _gui.getCbtnQWerteManuell().setSelection(false);
        cbtnQWerteManuellSelektiert();
    }

    /**
     * Initialisiere TreeViewerZs.
     */
    private void initialisiereTreeViewerZs() {

        // Setze die BaumElemente:
        _gui.getTreeViewerZs().setInput(_listeZs);

        //
        _gui.getTreeViewerZs().expandAll();

        // Setze grau-Elemente:
        for (ZaehlStelle zs : _listeZs) {
            _gui.getTreeViewerZs().setGrayed(zs, !_ganglinienArtSelektiert.istZs());

            for (Messquerschnitt mq : zs.ermittleArMq()) {
                _gui.getTreeViewerZs().setGrayed(mq, (_ganglinienArtSelektiert.istTag()));
            }
        }

        // Selektiere BaumElemente:
        for (ZaehlStelle zs : _listeZs) {
            _gui.getTreeViewerZs().setChecked(zs, false);
            _gui.getTreeViewerZs().setSubtreeChecked(zs, false);
        }

        // --- Tag (FS):
        if (_ganglinienArtSelektiert.istTag()) {
            if (_fahrstreifeBerechnet != null) {
                _gui.getTreeViewerZs().setChecked(_fahrstreifeBerechnet, true);
            }
        }

        // --- MQ:
        else if (_ganglinienArtSelektiert.istMq()) {
            for (Messquerschnitt mq : _arMqBerechnet) {
                if (mq != null) {
                    _gui.getTreeViewerZs().setChecked(mq, true);
                }
            }
        }

        // --- ZS:
        else if (_ganglinienArtSelektiert.istZs()) {
            for (ZaehlStelle zs : _arZsBerechnet) {
                if (zs != null) {
                    _gui.getTreeViewerZs().setChecked(zs, true);
                }
            }
        }
    }

    /**
     * Stellt fest, ob der darzustellende Wert (Datum/MQ) mit angegebenen Index
     * in der Grafik angezeigt werden soll.
     *
     * @param i Index des Wertes im Array.
     *
     * @return true, falls der darzustellende Wert (Datum/MQ) mit angegebenen Index
     * in der Grafik angezeigt werden soll, false - sonst.
     */
    protected boolean istDarzustellenderWertSelektiert(int i) {
        return _gui.getArCbtnDarzustellendeWerte()[i].getSelection();
    }

    /**
     * Stellt fest, ob die Werte Q-KFZ in der Grafik angezeigt werden sollen.
     *
     * @return true, falls die Werte Q-KFZ in der Grafik angezeigt werden sollen,
     * false - sonst.
     */
    protected boolean istQKfzSelektiert() {
        return _gui.getCbtnQKfz().getSelection();
    }

    /**
     * Stellt fest, ob die Werte Q-SGV in der Grafik angezeigt werden sollen.
     *
     * @return true, falls die Werte Q-SGV in der Grafik angezeigt werden sollen,
     * false - sonst.
     */
    protected boolean istQSgvSelektiert() {
        return _gui.getCbtnQSgv().getSelection();
    }

    /**
     * RadioButton 'GanglinienArt' wird selektiert.
     */
    protected void rbtnGanglinienArtSelektiert() {

        // -------------------------------------------------------
        // Ermittle die Ganglinienart und reagiere auf die nderung:
        GanglinienArt ganglinienArtAlt = _ganglinienArtSelektiert;

        if (_gui.getRbtnTagBast().getSelection()) {
            _ganglinienArtSelektiert = GanglinienArt.TAG_BAST;
        } else if (_gui.getRbtnTagTls().getSelection()) {
            _ganglinienArtSelektiert = GanglinienArt.TAG_TLS;
        } else if (_gui.getRbtnMonatMq().getSelection()) {
            _ganglinienArtSelektiert = GanglinienArt.MONAT_MQ;
        } else if (_gui.getRbtnJahrMq().getSelection()) {
            _ganglinienArtSelektiert = GanglinienArt.JAHR_MQ;
        } else if (_gui.getRbtnMonatZs().getSelection()) {
            _ganglinienArtSelektiert = GanglinienArt.MONAT_ZS;
        } else if (_gui.getRbtnJahrZs().getSelection()) {
            _ganglinienArtSelektiert = GanglinienArt.JAHR_ZS;
        }

        //
        if (_ganglinienArtSelektiert != ganglinienArtAlt) {
            boolean istGanglinienArtTage = _ganglinienArtSelektiert.istTag();

            // Falls die Art Tag / MQ / ZS umgeschaltet wird:
            if ((_ganglinienArtSelektiert.istTag() != ganglinienArtAlt.istTag()) || (_ganglinienArtSelektiert.istMq() != ganglinienArtAlt.istMq()) || (_ganglinienArtSelektiert.istZs() != ganglinienArtAlt.istZs())) {

                // ---------------------------------------------------------
                // Initialisiere den Baum:
                initialisiereTreeViewerZs();

                // -------------------------------------------------------
                // Stelle die Sichtbarkeit der GUI-Zeit-Elemente ein:
                for (int i = 1; i < ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
                    _gui.getArDzDatum()[i].setVisible(istGanglinienArtTage);
                }

                // -------------------------------------------------------
                // Setze den GruppenName:
                if (istGanglinienArtTage) {
                    _gui.getGrpZs().setText("Auswahl der Fahrstreifen (max. 1 FS)");
                } else if (_ganglinienArtSelektiert.istMq()) {
                    _gui.getGrpZs().setText("Auswahl der Messquerschnitte (max. 7 MQ)");
                } else {
                    _gui.getGrpZs().setText("Auswahl der Zhlstellen (max. 7 ZS)");
                }
            }

            // Aktiviere das Button:
            aktiviereBtnAktualisieren();
        }
    }

    /**
     * Spinner 'QMax' wird selektiert.
     */
    protected void spinnerQMaxSelektiert() {
        aktualisiereGrafik();
    }

    /**
     * Spinner 'QMin' wird selektiert.
     */
    protected void spinnerQMinSelektiert() {
        aktualisiereGrafik();
    }

    /**
     * Checkbutton in TreeViewer wird selektiert.
     *
     * @param event CheckStateChangedEvent.
     */
    protected void treeViewerZsSelektiert(CheckStateChangedEvent event) {

        // Tag (FS):
        if (_ganglinienArtSelektiert.istTag()) {
            boolean selektiert = event.getChecked();

            // Ganze Selektion ausschalten:
            for (ZaehlStelle zs : _listeZs) {
                _gui.getTreeViewerZs().setChecked(zs, false);
                _gui.getTreeViewerZs().setSubtreeChecked(zs, false);
            }

            // Selektiere das Element und alle erste Kinder:
            if (selektiert) {
                List<Object> liste = new ArrayList<Object>();

                if (event.getElement() instanceof ZaehlStelle) {
                    ZaehlStelle zs = (ZaehlStelle) event.getElement();

                    liste.add(zs);

                    for (TreeItem ti : _gui.getTreeViewerZs().getTree().getItems()) {
                        if (ti.getData() == event.getElement()) {
                            if (ti.getItemCount() > 0) {
                                liste.add(ti.getItem(0).getData());

                                if (ti.getItem(0).getItemCount() > 0) {
                                    liste.add(ti.getItem(0).getItem(0).getData());
                                }
                            }

                            break;
                        }
                    }
                } else if (event.getElement() instanceof Messquerschnitt) {
                    Messquerschnitt mq = (Messquerschnitt) event.getElement();

                    liste.add(mq);

                    for (TreeItem ti : _gui.getTreeViewerZs().getTree().getItems()) {
                        for (TreeItem ti2 : ti.getItems()) {
                            if (ti2.getData() == event.getElement()) {
                                if (ti2.getItemCount() > 0) {
                                    liste.add(ti2.getItem(0).getData());
                                }

                                break;
                            }
                        }
                    }
                } else {
                    liste.add(event.getElement());
                }

                _gui.getTreeViewerZs().setCheckedElements(liste.toArray(new Object[liste.size()]));
            }
        }

        // MQ:
        else if (_ganglinienArtSelektiert.istMq()) {

            // Setze die gleiche Selektion fr die Kinder:
            _gui.getTreeViewerZs().setSubtreeChecked(event.getElement(), event.getChecked());
        }

        // Aktiviere das Button:
        aktiviereBtnAktualisieren();
    }

    //~ INNERE KLASSEN ========================================================

    /**
     * Die Klasse beschreibt das DatenModel fr den ZhlstellenBaum.
     */
    protected class ZaehlstellenBaumContentProvider implements ITreeContentProvider {
        @Override
        public void dispose() {}

        //~ GET METHODEN ======================================================

        @Override
        public Object[] getChildren(Object parentElement) {
            if (!_ganglinienArtSelektiert.istZs()) {
                if (parentElement instanceof ZaehlStelle) {
                    ZaehlStelle zs = (ZaehlStelle) parentElement;

                    return zs.ermittleArMq();
                } else if (parentElement instanceof Messquerschnitt) {
                    if (_ganglinienArtSelektiert.istTag()) {
                        Messquerschnitt mq = (Messquerschnitt) parentElement;

                        return mq.getArFahrbahnen();
                    }
                }
            }

            return null;
        }

        @Override
        public Object[] getElements(Object inputElement) {

            // Falls inputElement vom Typ (Collection) ist:
            if (inputElement instanceof Collection) {
                ZaehlStelle[] arZs;
                Collection<?> inputCollection = (Collection<?>) inputElement;

                arZs = new ZaehlStelle[inputCollection.size()];

                int i = 0;

                for (Object element : inputCollection) {
                    arZs[i] = (ZaehlStelle) element;
                    i++;
                }

                return arZs;
            }

            return null;
        }

        @Override
        public Object getParent(Object element) {
            if (element instanceof Messquerschnitt) {
                Messquerschnitt mq = (Messquerschnitt) element;

                return mq.getParentAuswerteQuerschnittZuMQ().getParentZaehlStelleZuAuswerteQuerschnitt();
            } else if (element instanceof Fahrbahnen) {
                Fahrbahnen fb = (Fahrbahnen) element;

                return fb.getParentMQZuEinerFahrbahn();
            }

            return null;
        }

        @Override
        public boolean hasChildren(Object element) {
            if (!_ganglinienArtSelektiert.istZs()) {
                if (element instanceof ZaehlStelle) {
                    return true;
                } else if ((element instanceof Messquerschnitt) && (_ganglinienArtSelektiert.istTag())) {
                    return true;
                }
            }

            return false;
        }

        //~ METHODEN ==========================================================

        @Override
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
    }


    /**
     * Die Klasse beschreibt die Beschriftung fr den ZhlstellenBaum.
     */
    protected class ZaehlstellenBaumLabelProvider extends LabelProvider {
        @Override
        public String getText(Object element) {
            if (element instanceof ZaehlStelle) {
                ZaehlStelle zs = (ZaehlStelle) element;

                return zs.getSystemObjekt().getNameOrPidOrId();
            } else if (element instanceof Messquerschnitt) {
                Messquerschnitt mq = (Messquerschnitt) element;

                return mq.getSystemObjekt().getNameOrPidOrId();
            } else if (element instanceof Fahrbahnen) {
                Fahrbahnen fb = (Fahrbahnen) element;

                return fb.getSystemObjekt().getNameOrPidOrId();
            }

            return element.toString();
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 30.10.13
