/*
 * @(#)Activator.java   21.04.10
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib;

//~ NICHT JDK IMPORTE =========================================================

import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.internal.util.BundleUtility;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;

//~ JDK IMPORTE ===============================================================

import java.net.MalformedURLException;
import java.net.URL;

//~ KLASSEN ===================================================================

/**
 * Die Klasse Activator hat die Kontrolle ber den Lebenszyklus des Plugins
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 24 $ / $Date: 2013-09-25 15:38:14 +0200 (Mi, 25 Sep 2013) $ / ($Author: LG $)
 */
public class Activator extends AbstractUIPlugin {

    // The plug-in ID
    public static final String PLUGIN_ID = "de.inovat.buv.projektlib";

    // The shared instance
    private static Activator plugin;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * The constructor
     */
    public Activator() {}

    //~ GET METHODEN ==========================================================

    /**
     * Returns the shared instance
     *
     * @return the shared instance
     */
    public static Activator getDefault() {
        return plugin;
    }

    /**
     * Returns an image descriptor for the image file at the given plug-in
     * relative path
     *
     * @param path
     *            the path
     * @return the image descriptor
     */
    public static ImageDescriptor getImageDescriptorInovat(String path) {
        return unserImageDescriptorFromPlugin(PLUGIN_ID, path);
    }

    //~ METHODEN ==============================================================

    @Override
    public void start(BundleContext context) throws Exception {
        super.start(context);
        plugin = this;
    }

    @Override
    public void stop(BundleContext context) throws Exception {
        plugin = null;
        super.stop(context);
    }

    private static ImageDescriptor unserImageDescriptorFromPlugin(String pluginId, String imageFilePath) {
        if ((pluginId == null) || (imageFilePath == null)) {
            throw new IllegalArgumentException();
        }

        // if the bundle is not ready then there is no image
        Bundle bundle = Platform.getBundle(pluginId);

        if (!BundleUtility.isReady(bundle)) {
            return null;
        }

        // look for the image (this will check both the plugin and fragment
        // folders
        URL fullPathString = BundleUtility.find(bundle, imageFilePath);

        if (fullPathString == null) {
            try {
                fullPathString = new URL(imageFilePath);
            }
            catch (MalformedURLException e) {
                return null;
            }
        }

        if (fullPathString == null) {
            return null;
        }

        return ImageDescriptor.createFromURL(fullPathString);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 25.09.13
