/*
 * @(#)HilfeInovatVew.java   18.02.11
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.hilfe;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.projektlib.Activator;

import java.util.*;

//~ JDK IMPORTE ===============================================================

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet alle Hilfe-Resource fr das inovatProjekt.
 *
 * Hier sind alle Anker fr die HilfeTexte als Konstanten definiert.
 * Im ARRAY_HILFE_INHALT ist den HilfeInhalt bestimmt.
 *
 * Als SingleTon realisiert.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 22 $ / $Date: 2013-09-24 15:51:26 +0200 (Di, 24 Sep 2013) $ / ($Author: LG $)
 */
public class HilfeInovatVew {
    private static final String ANKER_MENUPUNKT_ERRORLOG                  = "MP_ERRORLOG";
    private static final String ANKER_MENUPUNKT_FEHLER_MELDEN             = "MP_FEHLER_MELDEN";
    private static final String ANKER_MENUPUNKT_HILFE                     = "MP_HILFE";
    private static final String ANKER_MENUPUNKT_PERSPEKTIVE_ZURUECKSETZEN = "MP_PERSPEKTIVE_ZURUECKSETZEN";
    private static final String ANKER_MENUPUNKT_PRAEFERENZEN_OEFFNEN      = "MP_PRAEFERENZEN_OEFFNEN";

    //
    private static final String ANKER_MENUPUNKT_PROJEKT_STARTEN = "MP_PROJEKT_STARTEN";
    private static final String ANKER_MENUPUNKT_UEBER           = "MP_UEBER";

    /** Index vom ID im HilfeMap-Eintrag. */
    private static final int INDEX_MAP_ID = 0;

    /** Index vom Name im HilfeMap-Eintrag. */
    private static final int INDEX_MAP_NAME = 1;

    /**
     * Liste des gesamten HilfeInhalts: Anker, Name.
     * In dieser Reihenfolge wird der HilfeInhalt angezeigt.
     */
    private static String[][] ARRAY_HILFE_INHALT;

    /** Einzige Instanz der Klasse. */
    private static HilfeInovatVew hilfeInstanz=new HilfeInovatVew();

    //~ FELDER ================================================================

    /** Lokale der Hilfe. */
    private String _lokale = "de_DE";

    /** HilfeDatei, die vom gestarteten Bundesland abhngig ist. */
    private String _dateiHilfe;

    /**
     * Menge der Informationen zur allen Hilfeeintrgen.
     *
     * Key: Anker
     * Value: Information: ID, Name.
     */
    private Map<String, String[]> _mapHilfeInfo;

    /** Titel des HilfeInhalts. */
    private String _titelHilfeInhalt;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt einzige Instanz der Klasse HilfeInovatVew.
     */
    private HilfeInovatVew() {

        //
        _dateiHilfe = "hilfe/texte/Benutzerhandbuch_inovat.htm";

        //
        _titelHilfeInhalt = "Hilfe inovat";

        //
        initialisiereHilfeInhalt();

        //
        _mapHilfeInfo = new HashMap<String, String[]>();

        String anker;
        String name;
        String id;

        for (String[] info : ARRAY_HILFE_INHALT) {
            anker = info[0];
            name  = info[1];
            id    = Activator.PLUGIN_ID + "." + anker;
            _mapHilfeInfo.put(anker, new String[] { id, name });
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Ermittle den Anker fr angegebene Hilfe-Id.
     *
     * @param id Hilfe-ID.
     *
     * @return Anker fr angegebene Hilfe-Id.
     */
    protected String ermittleAnkerFuerId(String id) {
        try {
            return id.substring(Activator.PLUGIN_ID.length() + 1);
        }
        catch (Exception e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Anker kann nicht fr <" + id + "> definiert werden.", e));

            return id;
        }
    }

    /**
     * Ermittle die ganze Referenzangabe fr den Anker.
     *
     * @param anker Anker.
     *
     * @return Ganze Referenzangabe fr den Anker.
     */
    protected String ermittleHrefGanz(String anker) {
        return Activator.PLUGIN_ID + "/" + ermittleHrefOhnePluginId(anker);
    }

    /**
     * Ermittle die Referenzangabe fr den Anker ohne die PluginID am Anfang.
     *
     * @param anker Anker.
     *
     * @return Referenzangabe fr den Anker ohne die PluginID am Anfang.
     */
    protected String ermittleHrefOhnePluginId(String anker) {
        return _dateiHilfe + "#" + anker;
    }

    /**
     * Ermittle Hilfe-Id fr angegebenen Anker.
     *
     * @param anker Anker.
     *
     * @return Hilfe-Id fr angegebenen Anker oder null, falls nicht gefunden wird.
     */
    public String ermittleIdFuerAnker(String anker) {
        try {
            return _mapHilfeInfo.get(anker)[INDEX_MAP_ID];
        }
        catch (Exception e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Hilfe-ID kann nicht fr <" + anker + "> definiert werden.", e));

            return null;
        }
    }

    /**
     * Ermittle die Liste aller definierten Anker.
     *
     * @return Liste aller definierten Anker.
     */
    protected List<String> ermittleListeAnker() {
        List<String> liste = new ArrayList<String>();

        for (String[] ar : ARRAY_HILFE_INHALT) {
            liste.add(ar[0]);
        }

        return liste;
    }

    /**
     * Ermittle den Name fr den angegebenen Anker.
     *
     * @param anker Anker.
     *
     * @return Name fr den angegebenen Anker oder null, falls nicht gefunden wird.
     */
    protected String ermittleNameFuerAnker(String anker) {
        try {
            return _mapHilfeInfo.get(anker)[INDEX_MAP_NAME];
        }
        catch (Exception e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Name kann nicht fr <" + anker + "> definiert werden.", e));

            return null;
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     * Ermittelt einzige Instanz der Klasse HilfeInovatVew.
     *
     * @return Einzige Instanz der Klasse HilfeInovatVew
     */
    public static HilfeInovatVew getInstanz() {
        return hilfeInstanz;
    }

    /**
     * Getter fr Eigenschaft 'lokale'.
     *
     * @return Wert der Eigenschaft 'lokale'.
     *
     * @see #_lokale
     */
    protected String getLokale() {
        return _lokale;
    }

    /**
     * Getter fr Eigenschaft 'titelHilfeInhalt'.
     *
     * @return Wert der Eigenschaft 'titelHilfeInhalt'.
     *
     * @see #_titelHilfeInhalt
     */
    protected String getTitelHilfeInhalt() {
        return _titelHilfeInhalt;
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiere die Liste des gesamten HilfeInhalts: Anker, Name.
     * In dieser Reihenfolge wird der HilfeInhalt angezeigt.
     */
    private void initialisiereHilfeInhalt() {
        ARRAY_HILFE_INHALT = new String[][] {};
        ARRAY_HILFE_INHALT = new String[][] {

            //
            { ANKER_MENUPUNKT_PROJEKT_STARTEN, "Projekt Starten" },                     //
            { ANKER_MENUPUNKT_FEHLER_MELDEN, "Fehler an inovat melden" },               //
            { ANKER_MENUPUNKT_PERSPEKTIVE_ZURUECKSETZEN, "Perspektive zurcksetzen" },  //
            { ANKER_MENUPUNKT_ERRORLOG, "ErrorLog" },                                   //
            { ANKER_MENUPUNKT_PRAEFERENZEN_OEFFNEN, "Prferenzen ffnen" },             //
            { ANKER_MENUPUNKT_HILFE, "Hilfe" },                                         //
            { ANKER_MENUPUNKT_UEBER, "ber" }                                           //
        };
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 04.03.11
