/*
 * @(#)AktionenVew.java   04.03.10
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.rechtevew.aktionen;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.inovat.lib.gui.GuiFunktionen;
import de.inovat.buv.projektlib.Activator;
import de.inovat.buv.projektlib.rechtevew.lib.RechtePropertyTester;
import de.inovat.buv.projektlib.rechtevew.lib.RechteVew;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

//~ KLASSEN ===================================================================

/**
 * Singleton-Klasse stellt die Methoden zum Ausfhren der Aktionen zu Verfgung.
 * </p>
 * Aktionen fr das Rahmenwerk-Men sind als Extensions org.eclipse.ui.commands in PlugIn
 * eingetragen und somit sind sie fr die Menerstellung im Rahmenwerk verfgbar.
 * Handler zu Commands sind ber Extensions org.eclipse.ui.handlers registriert.
 * Handler knnen eine Prfung (enabledWhen) enthalten. Oft wird als Prfung
 * die RechtePrfung angegeben (s. Klasse {@link RechtePropertyTester}).
 * </p>
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 88 $ / $Date: 2013-11-05 14:19:14 +0100 (Di, 05 Nov 2013) $ / ($Author: LG $)
 */
public class AktionenVew {

    /** Einzige Instanz der Klasse. */
    private static AktionenVew instanz = new AktionenVew();

    //~ FELDER ================================================================

    /** Zhler fr die Anzahl der Instanzen pro mehrfach geffneter Ansicht. */
    private int _anzahlSichte = 0;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt einzige Instanz der Klasse AktionenVew.
     */
    private AktionenVew() {}

    //~ METHODEN ==============================================================

    /**
     * Fhre die angegebene Aktion aus.
     * Die Aktion stellt dabei ein Command aus Extensions org.eclipse.ui.commands dar.
     * Falls in Handler zum Command eine Prfung (enabledWhen) gesetzt ist (z.B. Rechte-Prfung),
     * wird die Prfung durchgefhrt.
     *
     * @param aktionId Die ID der Aktion (ID vom Command aus Extensions org.eclipse.ui.commands).
     *
     * @return Exception, falls die Aktion nicht ausgefhrt werden kann,
     * null - falls die Aktion erfolgreich ausgefhrt wurde.
     */
    public Exception ausfuehrenAktion(String aktionId) {
        try {
            IHandlerService handlerService = (IHandlerService) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getService(IHandlerService.class);

            handlerService.executeCommand(aktionId, null);
        }
        catch (Exception e) {
            return e;
        }

        return null;
    }

    /**
     * Fhre die angegebene Aktion aus (s. {@link #ausfuehrenAktion(String)}).
     * Falls die Aktion nicht ausgefhrt werden kann (ein Exception auftritt),
     * wird der Fehler im Debug ausgegeben.
     *
     * @param aktionId Die ID der Aktion.
     *
     * @return true, falls die Aktion erfolgreich ausgefhrt wurde,
     * false - sonst.
     */
    public boolean ausfuehrenAktionMitDebugausgabe(String aktionId) {
        Exception exception = ausfuehrenAktion(aktionId);

        if (exception == null) {
            return true;
        } else {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Aktion kann nicht ausgefhrt werden", exception));

            return false;
        }
    }

    /**
     * Erstelle das leere GUI in der Composite.
     *
     * @param parent Parent-Composite.
     * @param text Der angezeigte Text.
     */
    public void erstelleLeeresGui(Composite parent, String text) {
        parent.setLayout(new GridLayout());
        new Label(parent, 0);
        new Label(parent, 0);

        Label lb = new Label(parent, 0);

        lb.setText(text);

        // Update:
        parent.layout();
        parent.getParent().layout();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Gibt die Instanz der AktionenVew zurck. <p/>
     * Die AktionenVew ist als Singleton realisiert und kann nicht direkt instanziert werden.
     * Der Zugriff auf die Methoden erfolgt ausschlielich ber die Instanz.
     *
     * @return Instanz der Klasse AktionenVew.
     */
    public static AktionenVew getInstanz() {
        return instanz;
    }

    //~ METHODEN ==============================================================

    /**
     * ffne die angegebene Perspektive.
     *
     * @param perspektiveId ID der Perspektive.
     * @param perspektiveZuruecksetzen true, falls die Perspektive zurckgesetzt soll,
     * false - sonst.
     *
     * @return true, falls die Aktion erfolgreich ausgefhrt wurde,
     * false - sonst.
     */
    public boolean perspektiveOeffnen(String perspektiveId, boolean perspektiveZuruecksetzen) {
        try {

            // Projekt-Perspective ffnen:
            IPerspectiveDescriptor pers = PlatformUI.getWorkbench().getPerspectiveRegistry().findPerspectiveWithId(perspektiveId);

            if (pers != null) {
                PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().setPerspective(pers);

                // Perspective zurcksetzen:
                if (perspektiveZuruecksetzen) {
                    PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().resetPerspective();
                }
            } else {
                return false;
            }

            return true;
        }
        catch (Exception e) {

            // Exception kann auftreten, falls PlatformUI.getWorkbench().getActiveWorkbenchWindow() null zurck liefert.
            System.out.println(perspektiveId + " Perspective kann nicht angezeigt werden!");
            e.printStackTrace();

            return false;
        }
    }

    /**
     * Setze die angegebene Perspektive zurck.
     *
     * @param perspektiveId ID der Perspektive.
     */
    public void perspektiveZuruecksetzen(String perspektiveId) {
        IWorkbench workbench = PlatformUI.getWorkbench();

        // Falls die Perspektive aktuell ist:
        if (workbench.getActiveWorkbenchWindow().getActivePage().getPerspective().getId().equals(perspektiveId)) {
            workbench.getActiveWorkbenchWindow().getActivePage().resetPerspective();
        }

        // sonst
        else {

            // Finde die Perspektive:
            IPerspectiveDescriptor pers = workbench.getPerspectiveRegistry().findPerspectiveWithId(perspektiveId);

            // Setze die Perspektive zurck:
            if (pers != null) {

                // TODO revertPerspective() funktioniert nicht immer???
                workbench.getPerspectiveRegistry().revertPerspective(pers);

                // workbench.getActiveWorkbenchWindow().getActivePage().resetPerspective();
            }
        }
    }

    /**
     * Prfe das Recht fr den angemeldeten Benutzer auf die Sicht und,
     * falls das Recht abgezogen wird, zeige das leer GUI.
     *
     * @param funktion ID der BerechigungsFunktion, die die Sicht zeigt.
     * @param parent Parent-Composite der Sicht.
     * @param parentReinigen Stellt fest, ob die Parent-Composite zuerst bereinigt werden soll.
     * <li>
     * Falls das GUI noch nicht erstellt wurde, braucht man das nicht zu reinigen
     * /parentReinigen=false/.
     * <li>
     * Falls das GUI schon geffnet wurde und dann die Rechte abgezogen wurden,
     * sollen zuerst die GUI-Elemente gelscht werden
     * /parentReinigen=true/.
     * @param viewTitel Titel der Sicht.
     *
     * @return true, falls der angemeldete Benutzer das Recht auf die Sicht hat,
     * false - sonst.
     */
    public boolean pruefeRechtFuerSicht(String funktion, final Composite parent, final boolean parentReinigen, final String viewTitel) {

        // Falls das Recht fr die Sicht abgezogen wurde, wird das GUI disposed
        // und das leere GUI erstellt:
        if (!RechteVew.getInstanz().istFunktionFuerAngemeldetenBenutzerErlaubt(funktion)) {
            PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    Composite parentNeu;

                    if (parentReinigen) {

                        // Reinige das GUI:
                        Composite viewComposite = parent.getParent();

                        parent.dispose();
                        parentNeu = new Composite(viewComposite, SWT.NONE);
                    } else {
                        parentNeu = parent;
                    }

                    //
                    erstelleLeeresGui(parentNeu, String.format(                                                                                                                                                                                                       //
                        "ACHTUNG:%n%nSie haben fr diese Sicht <%s> nicht die notwendigen Benutzerrechte! %n%nndern Sie gegebenenfalls die Rechte und schlieen Sie die Sicht! %nGenderte Rechte werden erst nach dem Schlieen und Neuffnen der Sicht wirksam.",  //
                        viewTitel));
                }
            });

            return false;
        } else {
            return true;
        }
    }

    /**
     * Prfe das Recht fr den angemeldeten Benutzer auf die Sicht und,
     * falls das Recht abgezogen wird, schliee das GUI (nur in aktueller Perspektive).
     *
     * @param funktion ID der BerechigungsFunktion, die die Sicht zeigt.
     * @param viewId ID der Sicht.
     * @param viewIdSecond Second-ID der Sicht oder null.
     * @param viewTitel Titel der Sicht.
     * @param meldungZeigen true, falls die Meldung mit der Warnung angezeigt werden soll,
     * false - sonst.
     */
    public void schliesseSichtBeiRechtSperrung(String funktion, final String viewId, final String viewIdSecond, final String viewTitel, final boolean meldungZeigen) {

        // Falls das Recht fr die Sicht abgezogen wurde, wird das GUI geschlossen.
        if (!RechteVew.getInstanz().istFunktionFuerAngemeldetenBenutzerErlaubt(funktion)) {
            PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {

                    // Wird nur in der aktuellen Perspektive geschlossen:
                    if (viewIdSecond != null) {
                        GuiFunktionen.schliesseSichtMulti(viewId, viewIdSecond);
                    } else {
                        GuiFunktionen.schliesseSicht(viewId);
                    }

                    if (meldungZeigen) {
                        MessageDialog.openWarning(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), "Achtung",            //
                                                  String.format(                                                                         //
                                                  "Sie haben kein Recht mit dem GUI <%s> zu arbeiten! %n%nDie Sicht wird geschlossen!",  //
                                                  viewTitel));
                    }
                }
            });
        }
    }

    /**
     * Zeige die Sicht.
     *
     * @param sichtId ID der Sicht.
     *
     * @return true, falls die Aktion erfolgreich war, false - sonst.
     */
    public boolean zeigeSicht(String sichtId) {
        try {
            IWorkbenchPage workbenchPage = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

            workbenchPage.showView(sichtId);
        }
        catch (Exception e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);

            return false;
        }

        return true;
    }

    /**
     * Zeige die Multi-Sicht.
     * Second-ID der Sicht wird dabei zentral verwaltet.
     *
     * @param sichtId ID der Sicht.
     *
     * @return true, falls die Aktion erfolgreich war, false - sonst.
     */
    public boolean zeigeSichtMulti(String sichtId) {
        _anzahlSichte++;

        return zeigeSichtMulti(sichtId, sichtId + _anzahlSichte);
    }

    /**
     * Zeige die Multi-Sicht.
     *
     * @param sichtId ID der Sicht.
     * @param sichtSecondId Second-ID der Sicht
     *
     * @return true, falls die Aktion erfolgreich war, false - sonst.
     */
    public boolean zeigeSichtMulti(String sichtId, String sichtSecondId) {
        try {
            IWorkbenchPage workbenchPage = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();

            workbenchPage.showView(sichtId, sichtSecondId, IWorkbenchPage.VIEW_ACTIVATE);
        }
        catch (Exception e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);

            return false;
        }

        return true;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 05.11.13
