/*
 * Copyright 2008 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kni, Kln
 * <p>
 * This file is part of de.inovat.pat.datkat2html
 * <p>
 * de.inovat.pat.datkat2html is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <p>
 * de.inovat.pat.datkat2html is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.datkat2html; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * <p>
 * Contact Information:<br>
 * inovat, Dipl.-Ing. H. C. Kni<br>
 * Klner Strae 30<br>
 * 50859 Kln<br>
 * Phone: +49 2234 4301 800<br>
 * mailto: info@inovat.de
 */
package de.inovat.pat.datkat2html.ausgabe.beschreibung;

import java.util.ArrayList;
import java.util.List;

/**
 * Klasse verwaltet dtd-Element mengenDefinition.
 * 
 * <!ELEMENT mengenDefinition (info)>
 * <!ATTLIST mengenDefinition
 *      pid ID #REQUIRED
 *      elemente NMTOKENS #REQUIRED
 * 		aenderbar (ja|nein) "nein"
 * 		mindestens CDATA #IMPLIED
 * 		hoechstens CDATA #IMPLIED
 * 		referenzierungsart (assoziation|aggregation|komposition) #IMPLIED
 * >
 * 
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 408 $ / $Date: 2010-04-06 08:14:07 +0200 (Di, 06. Apr 2010) $ / ($Author: HCK $)
 */

public class MengenDefinitionBeschreibung {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Private ----------------------------------------------------------------
	/** bersicht der MengenDefinitionBeschreibung. */
	private Uebersicht _uebersicht;

	/** Elemente der Menge. */
	private List<String> _elemente=new ArrayList<String>();

	/** bersicht der Mengenelementen. */
	private List<Uebersicht> _elementeUebersicht=new ArrayList<Uebersicht>();

	/** Ist die Menge nderbar (ja|nein)? */
	private String _aenderbar;

	/** Mindestanzahl der Elementen. */
	private String _mindestens;

	/** Hchste Anzahl der Elementen. */
	private String _hoechstens;

	/** Referenzierungsart (assoziation|aggregation|komposition). */
	private String _referenzierungsart;

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Erzeugt eine Instanz der Klasse MengenDefinitionBeschreibung.
	 *
	 * @param uebersicht
	 * @param elemente
	 * @param elementeUebersicht
	 * @param aenderbar
	 * @param mindestens
	 * @param hoechstens
	 * @param referenzierungsart
	 */
	public MengenDefinitionBeschreibung(Uebersicht uebersicht,
			List<String> elemente, List<Uebersicht> elementeUebersicht,
			String aenderbar, String mindestens, String hoechstens,
			String referenzierungsart) {
		_uebersicht = uebersicht;
		if(elemente!=null){
			_elemente = elemente;
		}
		if(elementeUebersicht!=null){
			_elementeUebersicht = elementeUebersicht;
		}
		_aenderbar = aenderbar;
		_mindestens = mindestens;
		_hoechstens = hoechstens;
		_referenzierungsart = referenzierungsart;
	}

// ------------------------------------------------------------------------
// GETTER / SETTER METHODEN
// ------------------------------------------------------------------------

	/**
	 * Getter fr Eigenschaft 'aenderbar'.
	 *
	 * @return Wert der Eigenschaft 'aenderbar'.
	 *
	 * @see  {#link _aenderbar}
	 */
	public String getAenderbar() {
		return _aenderbar;
	}

	/**
	 * Getter fr Eigenschaft 'elemente'.
	 *
	 * @return Wert der Eigenschaft 'elemente'.
	 *
	 * @see  {#link _elemente}
	 */
	public List<String> getElemente() {
		return _elemente;
	}

	/**
	 * Getter fr Eigenschaft 'elementeUebersicht'.
	 *
	 * @return Wert der Eigenschaft 'elementeUebersicht'.
	 *
	 * @see  {#link _elementeUebersicht}
	 */
	public List<Uebersicht> getElementeUebersicht() {
		return _elementeUebersicht;
	}

	/**
	 * Getter fr Eigenschaft 'hoechstens'.
	 *
	 * @return Wert der Eigenschaft 'hoechstens'.
	 *
	 * @see  {#link _hoechstens}
	 */
	public String getHoechstens() {
		return _hoechstens;
	}

	/**
	 * Getter fr Eigenschaft 'mindestens'.
	 *
	 * @return Wert der Eigenschaft 'mindestens'.
	 *
	 * @see  {#link _mindestens}
	 */
	public String getMindestens() {
		return _mindestens;
	}

	/**
	 * Getter fr Eigenschaft 'referenzierungsart'.
	 *
	 * @return Wert der Eigenschaft 'referenzierungsart'.
	 *
	 * @see  {#link _referenzierungsart}
	 */
	public String getReferenzierungsart() {
		return _referenzierungsart;
	}

	/**
	 * Getter fr Eigenschaft 'uebersicht'.
	 *
	 * @return Wert der Eigenschaft 'uebersicht'.
	 *
	 * @see  {#link _uebersicht}
	 */
	public Uebersicht getUebersicht() {
		return _uebersicht;
	}
}
