/*

 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.gtm.datvew.lib;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DoubleAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.ReferenceAttributeType;
import de.bsvrz.dav.daf.main.config.StringAttributeType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.dav.daf.main.config.TimeAttributeType;
import de.bsvrz.dav.daf.main.Data;

import de.inovat.buv.projektlib.dav.DavDatenVew;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;
import static de.inovat.buv.gtm.datvew.konstanten.KonstantenGTMGUIResource.*;
//~ KLASSEN ===================================================================

/**
 * Verwaltet smtliche Zhlstellen des Systems und ermglicht den
 * zentralen Zugriff auf Zhlstellen sowie auf bergreifenden
 * allgemeine Eigenschaften.
 *
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public final class BaumVerwaltung {

    /** Instanz BaumVerwaltung. */
    private static BaumVerwaltung instanz = new BaumVerwaltung();

    /** Die Datenverteiler-Konfiguration. */
    private static DataModel _davKonf;

    //~ FELDER ================================================================

    /** Attribut das angibt um welche Importversion es sich handelt. */
    private boolean _isVersion2004 = true;

    /** Liste aller Importdateien BASt-Band Importverzeichnis Verkehr_Menge_2004. */
    private List<String> _listeAllerImportDateienFTPServer2004 = new ArrayList<String>();

    /** Liste aller Importdateien BASt-Band Importverzeichnis Verkehr_Geschwindigkeit_2007. */
    private List<String> _listeAllerImportDateienFTPServer2007 = new ArrayList<String>();

    // Sonstige ---------------------------------------------------------------

    /**
     * Liste der LangZeitZhlstellen bei denen es sich um eine Achslastmessstelle handelt.
     * Dies ist der Fall, wenn die Fahrstreifen der LZZ auf ein de vom Typ achsLastMessStelle verweisen.
     */
    private List<ZaehlStelle> _listeAxl = new ArrayList<ZaehlStelle>();

    /** Datenverteilerverbindung. */
    private ClientDavInterface _dav;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Privater Konstuktor zur Erzeugung der einzigen Instanz der Klasse
     * BaumVerwaltung.
     * <p/>
     */
    private BaumVerwaltung() {
        _dav     = DavDatenVew.getInstanz().getDav();
        _davKonf = _dav.getDataModel();
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode bestimmt aus allen Langzeitzhlstellen die Achslastmessstelle,
     */
    private void bestimmeAchslastZaehlStellen() {
        List<SystemObject> list                     = null;
        SystemObjectType   typeDeAxl                = _davKonf.getType(TYP_AXL);
        SystemObjectType   soTypeAchslastMessStelle = _davKonf.getType("typ.achsLastMessStelle");
        AttributeGroup     atgAxlLastMessStelle     = _davKonf.getAttributeGroup("atg.achsLastMessStelle");
        SystemObjectType   typeFahrstreifen         = _davKonf.getType(TYP_FS);

        // Fllen der Liste aller existierender Achslastmessstellen.
        if (soTypeAchslastMessStelle != null) {
            list = soTypeAchslastMessStelle.getElements();
        }

        if ((list != null) && (list.size() > 0)) {
            Data[] dataArray = _davKonf.getConfigurationData(list, atgAxlLastMessStelle);

            for (int i = 0; i < list.size(); i++) {
                Data         oData       = dataArray[i];
                SystemObject soAxlQuelle = oData.getReferenceValue("AchsLastMessStellenQuelle").getSystemObject();

                if ((soAxlQuelle != null) && (soAxlQuelle.isOfType(typeDeAxl))) {
                    SystemObject soFS = oData.getReferenceValue("FahrStreifen").getSystemObject();

                    if ((soFS != null) && (soFS.isOfType(typeFahrstreifen))) {
                        for (ZaehlStelle lzz : DatenVew.getInstanz().getListeZaehlStellen()) {
                            List<SystemObject> liste = lzz.ermittleAlleFsAlsSystemObject();

                            if (liste.contains(soFS)) {
                                if (!_listeAxl.contains(lzz)) {
                                    _listeAxl.add(lzz);

                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    /**
     * Bestimmt die Attributgruppe zu einer Pid einer Attributgruppe.
     * @param atgPid Pid der Attributgruppe.
     * @return AttributGruppe.
     */
    public AttributeGroup bestimmeAttributGruppeZuPidTabellenstruktur(String atgPid) {
        AttributeGroup atgPID;

        atgPID = _davKonf.getAttributeGroup(atgPid);

        return atgPID;
    }

    /**
     * Die Methode bestimmt die AttributListDefinitionen zu dem bergebenen Parameter.
     * @param att Attribut zu dem die AttributeListDefinition bestimmt wird.
     * @return AttributListDefinition.
     */
    public AttributeListDefinition bestimmeAttributListe(Attribute att) {
        try {
            return (AttributeListDefinition) att.getAttributeType();
        }
        catch (Exception e) {

            // e.printStackTrace();
        }

        return null;
    }

    /**
     * Die Methode bestimmt eine Liste aller SystemObjekte.
     * @return Liste der SystemObjekte.
     */
    public List<SystemObject> bestimmeSystemObjektTypen() {
        List<SystemObject> listeSystemObjects;

        try {
            listeSystemObjects = _davKonf.getTypeTypeObject().getElements();
        }
        catch (Exception e) {
            listeSystemObjects = new ArrayList<SystemObject>();
        }

        return listeSystemObjects;
    }

    /**
     * Die Methode erzeugt den Prfix fr die Spaltennamen.
     * @param pref der Prefix der hinzugefgt werden soll. Handelt es sich um
     * den initialen Aufruf der Methode fr den Spaltennamen ist der String pref leer,
     * ist dies der FAll wird der Name oder die Pid oder die Id des SystemObjektes als
     * Prfix retuniert. Ist der Prefix nicht leer, wird dieser samt "." vor den Namen oder
     * die Pid oder Id als erzeugter Prefix retuniert.
     * @param nameOrPidOrId fr das Attribut, fr das ein Prfix erzeugt wird.
     * @return Prfix als String.
     */
    public String erzeugePrefixFuerSpaltenNamen(String pref, String nameOrPidOrId) {
        if (!(pref.equals(""))) {
            StringBuilder stringBuffer = new StringBuilder();

            stringBuffer.append(pref);
            stringBuffer.append(".");
            stringBuffer.append(nameOrPidOrId);

            return stringBuffer.toString();
        } else {
            return nameOrPidOrId;
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     * Die Methode retuniert die Liste der Achslastmessstellen.
     * @return Liste der Achslastzhlstellen.
     */
    public List<ZaehlStelle> getAchslastZaehlStellen() {
        if (_listeAxl.isEmpty()) {
            bestimmeAchslastZaehlStellen();
        }

        return _listeAxl;
    }
    
    /**
     * Retuniert eine Instanz der BaumVerwaltung.
     * @return Instanz der BaumVerwaltung.
     */
    public static BaumVerwaltung getInstanz() {
        return instanz;
    }

    /**
     * Getter fr Eigenschaft 'listeAllerImportDateienFTPServer2007'.
     *
     * @return Wert der Eigenschaft 'listeAllerImportDateienFTPServer2007'.
     *
     * @see #_listeAllerImportDateienFTPServer2007
     */
    public List<String> getListeAllerImportDateienFTPServer() {
        if (_isVersion2004) {
            return _listeAllerImportDateienFTPServer2004;
        } else {
            return _listeAllerImportDateienFTPServer2007;
        }
    }

    /**
     * Die Methode ermittelt den Spaltentext zu einem Attribut.
     * Bei dem Spaltentext handelt es sich um den Namen, die Pid oder Id des
     * bergebenen Attributes.
     * @param att Attribut fr das der Spaltentext bestimmt wird.
     * @return Spaltentext als String.
     */
    public String getSpaltenText(Attribute att) {
        return att.getNameOrPidOrId();
    }

    /**
     * Methode prft ob das Attribut ein Array reprsentiert.
     * @param att das bergebene Attribut.
     * @return true, wenn es ein Attribut ist, das ein Array reprsentiert, false sonst.
     */
    public boolean isArray(Attribute att) {
        return (att.isArray());
    }

    /**
     * Methode prft das Attribut ob es sich um eine AttributListeDefinition handelt.
     * @param att das bergebene Attribut.
     * @return true, wenn es ein Attribut vom Typ AttributListeDefinition ist, false sonst.
     */
    public boolean isAttributeListeDefinition(Attribute att) {
        return (att.getAttributeType() instanceof AttributeListDefinition);
    }

    /**
     * Die Methode bestimmt ob es sich bei dem bergebenem Attribut um einen Basistyp handelt.
     * @param att
     *                      bergebenes Attribut.
     * @return true wenn es sich um einen Basistyp handelt, d.h. wenn das Attribut eine
     * Instanz des StringAttributeTyps, IntegerAttributeTyps, DoubleAttributeTyps oder
     * TimeAttributeTyps ist, false sonst.
     */
    public boolean isBasisType(Attribute att) {
        return ((att.getAttributeType() instanceof StringAttributeType) || (att.getAttributeType() instanceof IntegerAttributeType) || (att.getAttributeType() instanceof DoubleAttributeType) || (att.getAttributeType() instanceof TimeAttributeType));
    }

    /**
     * Methode bestimmt ob es sich bei dem Attribut um ein ReferenceAttribute handelt.
     * @param att Das Attribut.
     * @return true wenn es ein ReferenceAttribute ist, false sonst.
     */
    public boolean isRefrenceAttributeType(Attribute att) {
        return (att.getAttributeType() instanceof ReferenceAttributeType);
    }

    /**
     * Die Methode isStringAttributeType ermittelt zu einem Attribut
     * ob es sich um ein String Attribut handelt. Dies ist notwendig,
     * um die Tabelle bei String-werten linksbndig im Gegensatz zu
     * Zahlenwerten rechtsbndig zu fllen.
     * @param att das Attribut fr das der Typ ermittelt wird.
     * @return true wenn es sich um ein StringAttributeType handelt, false sonst.
     */
    public boolean isStringAttributeType(Attribute att) {
        return (att.getAttributeType() instanceof StringAttributeType);
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'listeAllerImportDateienFTPServer'.
     *
     * @param listeAllerImportDateienFTPServer Wert zum Setzen der Eigenschaft 'listeAllerImportDateienFTPServer'.
     * @param isVersion2004
     *
     * @see #_listeAllerImportDateienFTPServer2004
     */
    public void setListeAllerImportDateienFTPServer2004(List<String> listeAllerImportDateienFTPServer, boolean isVersion2004) {
        _isVersion2004                        = isVersion2004;
        _listeAllerImportDateienFTPServer2004 = listeAllerImportDateienFTPServer;
    }

    /**
     * Setter fr Eigenschaft 'listeAllerImportDateienFTPServer2007'.
     *
     * @param listeAllerImportDateienFTPServer2007 Wert zum Setzen der Eigenschaft 'listeAllerImportDateienFTPServer2007'.
     *
     * @see #_listeAllerImportDateienFTPServer2007
     */
    public void setListeAllerImportDateienFTPServer2007(List<String> listeAllerImportDateienFTPServer2007, boolean isVersion2004) {
        _isVersion2004                        = isVersion2004;
        _listeAllerImportDateienFTPServer2007 = listeAllerImportDateienFTPServer2007;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 08.03.12
