/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.gtm.datvew.lib;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.Data;

import de.inovat.buv.gtm.datvew.Activator;
import de.inovat.buv.gtm.datvew.konstanten.KonstantenGTMGUIResource;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.projektlib.dav.DavDatenVew;

import org.eclipse.swt.graphics.Image;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Die Klasse reprstentiert eine Zhlstelle.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class ZaehlStelle implements ISystemObjekt {

    /** Array der Auswertequerschnitte (in/gegen Richtung). */
    private AuswerteQuerschnitt[] _arAWQ = new AuswerteQuerschnitt[2];

    /** Pid oder ID der Zhlstelle. */
    private String _id;

    /** Eltern Systemobjekt Strasse. */
    private SystemObject _parentSOstrasse;

    /** SystemObjekt der Zhlstelle.. */
    private SystemObject _soLangZeitZaehlStelle;

    /** xWGS84-Koordinate der Zhlstelle. */
    private double _xWgs84;

    /** yWGS84-Koordinate der Zhlstelle. */
    private double _yWgs84;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Konstruktor .
     * @param parentSOStrasse  Eltern Systemobjekt Strasse.
     * @param langZeitZaehlStelle LangzeitZhlstelle.
     */
    public ZaehlStelle(SystemObject parentSOStrasse, SystemObject langZeitZaehlStelle) {
        _parentSOstrasse       = parentSOStrasse;
        _soLangZeitZaehlStelle = langZeitZaehlStelle;

        //
        _id = langZeitZaehlStelle.getPidOrId();

        try {
            DataModel           davKonf       = DavDatenVew.getInstanz().getDav().getDataModel();
            ConfigurationObject coZaehlStelle = (ConfigurationObject) langZeitZaehlStelle;
            AttributeGroup      atg           = davKonf.getAttributeGroup("atg.langZeitZhlstelle");
            Data                oData         = coZaehlStelle.getConfigurationData(atg);
            Data                data;
            AuswerteQuerschnitt auswertequerschnitt;
            boolean             istInRichtung;

            // AuswerteQuerschnitte (InRichtung, GegenRichtung):
            for (String richtung : new String[] { "InRichtung", "GegenRichtung" }) {
                istInRichtung       = (richtung.equals("InRichtung"));
                data                = oData.getItem(richtung);
                auswertequerschnitt = new AuswerteQuerschnitt((ConfigurationObject) data.getReferenceValue("Auswertequerschnitt").getSystemObject(), this);

                // Messquerschnitt:
                if (istInRichtung) {
                    _arAWQ[0] = auswertequerschnitt;
                } else {
                    _arAWQ[1] = auswertequerschnitt;
                }
            }

            // Koordinaten (suche zuerst in der Konfigurationdaten von ZaehlStelle,
            // dann in MQ-InRichtung, MQ-GegenRichtung):
            atg   = davKonf.getAttributeGroup("atg.punktKoordinaten");
            oData = coZaehlStelle.getConfigurationData(atg);

            if (oData == null) {

                // Koordinaten aus dem Messquerschnitt InRichtung:
                oData = _arAWQ[0].getMessquerschnitt().getSystemObjekt().getConfigurationData(atg);

                if (oData == null) {

                    // Koordinaten aus dem Messquerschnitt GegenRichtung:
                    oData = _arAWQ[1].getMessquerschnitt().getSystemObjekt().getConfigurationData(atg);
                }
            }

            if (oData == null) {
                throw new Exception("Die Zhlstellen-Koordinaten knnen nicht ermittelt werden.");
            } else {
                _xWgs84 = oData.getScaledValue("x").doubleValue();
                _yWgs84 = oData.getScaledValue("y").doubleValue();
            }
        }
        catch (Exception e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, String.format(  //
                "Es ist ein Fehler bei der Erstellung der Zhlstelle <%s> aufgetreten", _id), e));
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Ermittle die Liste aller zur ZaehlStelle zugehrige Fahrstreifen als SystemObjecte.
     *
     * @return Liste aller Fahrstreifen als SystemObjecte.
     */
    public List<SystemObject> ermittleAlleFsAlsSystemObject() {
        List<SystemObject> listeFs = new ArrayList<SystemObject>();

        for (AuswerteQuerschnitt awq : _arAWQ) {
            for (Fahrbahnen fb : awq.getMessquerschnitt().getArFahrbahnen()) {
                listeFs.add(fb.getSystemObjekt());
            }
        }

        return listeFs;
    }

    /**
     * Ermittle Array der Fahrbahnen gegen Richtung.
     *
     * @return Array der Fahrbahnen gegen Richtung oder leeres Array.
     */
    public Fahrbahnen[] ermittleArFsGegenRichtung() {
        try {
            return ermittleArMq()[1].getArFahrbahnen();
        }
        catch (Exception e) {
            return new Fahrbahnen[] {};
        }
    }

    /**
     * Ermittle Array der Fahrbahnen in Richtung.
     *
     * @return Array der Fahrbahnen in Richtung oder leeres Array.
     */
    public Fahrbahnen[] ermittleArFsInRichtung() {
        try {
            return ermittleArMq()[0].getArFahrbahnen();
        }
        catch (Exception e) {
            return new Fahrbahnen[] {};
        }
    }

    /**
     * Ermittle Array der Messquerschnitte.
     *
     * @return Array der Messquerschnitte ((in/gegen Richtung))
     * oder leeres Array.
     */
    public Messquerschnitt[] ermittleArMq() {
        try {
            return new Messquerschnitt[] { _arAWQ[0].getMessquerschnitt(), _arAWQ[1].getMessquerschnitt(), };
        }
        catch (Exception e) {
            return new Messquerschnitt[] {};
        }
    }

    /**
     * Ermittle die HTML-Beschreibung der Zhlstelle.
     *
     * @return HTML-Beschreibung der Zhlstelle als Text.
     */
    public String ermittleHtmlBeschreibung() {
        StringBuilder text = new StringBuilder();

        //
        DataModel           davKonf       = DavDatenVew.getInstanz().getDav().getDataModel();
        ConfigurationObject coZaehlStelle = (ConfigurationObject) _soLangZeitZaehlStelle;
        AttributeGroup      atg           = davKonf.getAttributeGroup("atg.langZeitZhlstelle");
        Data                oData         = coZaehlStelle.getConfigurationData(atg);

        //
        String name = oData.getTextValue("ZhlstellenName").getText();

        text.append(String.format("<font size=2><nobr><b>&lt%s&gt &nbsp %s</b></nobr></font></br>", _soLangZeitZaehlStelle.getName(), name));

        //
        String vonNK   = oData.getTextValue("vonNK").getText();
        String nachNK  = oData.getTextValue("nachNK").getText();
        double st      = oData.getScaledValue("Station").doubleValue();
        String station = String.format("%06.3f km", st);

        text.append(String.format("<font size=2><nobr>von NK: %s &nbsp nach NK: %s &nbsp Station: %s</nobr></font></br>", vonNK, nachNK, station));

        String klass = oData.getTextValue("KlassifizierungsgruppeBAST").getText();

        text.append(String.format("<font size=2><nobr>Klassifizierungsgruppe BAST: %s  </nobr></font>", klass));

        return text.toString();
    }

    /**
     * Ermittle die Nah- und FernZiele fr beide Richtungen  der Zhlstelle.
     *
     * @return Nah- und FernZiele fr beide Richtungen  der Zhlstelle
     * als 4-Dimensionales Array:
     * 0.Element=Nahziel in Richtung,
     * 1.Element=Fernziel in Richtung,
     * 2.Element=Nahziel gegen Richtung,
     * 3.Element=Fernziel gegen Richtung,
     *
     * Das Array kann leere Elemente enthalten.
     */
    public String[] ermittleNahFernZiele() {
        String[]            arZiele       = new String[] { "", "", "", "" };
        DataModel           davKonf       = DavDatenVew.getInstanz().getDav().getDataModel();
        ConfigurationObject coZaehlStelle = (ConfigurationObject) _soLangZeitZaehlStelle;
        AttributeGroup      atg           = davKonf.getAttributeGroup("atg.langZeitZhlstelle");
        Data                oData         = coZaehlStelle.getConfigurationData(atg);

        //
        arZiele[0] = oData.getItem("InRichtung").getTextValue("Nahziel").getText();
        arZiele[1] = oData.getItem("InRichtung").getTextValue("Fernziel").getText();
        arZiele[2] = oData.getItem("GegenRichtung").getTextValue("Nahziel").getText();
        arZiele[3] = oData.getItem("GegenRichtung").getTextValue("Fernziel").getText();

        return arZiele;
    }

    //~ GET METHODEN ==========================================================

    /**
     *     Getter fr Eigenschaft 'arAWQ'.
     *
     *     @return Wert der Eigenschaft 'arAWQ'.
     *
     *     @see #_arAWQ
     */
    public AuswerteQuerschnitt[] getArAWQ() {
        return _arAWQ;
    }

    /**
     * Getter fr Eigenschaft 'id'.
     *
     * @return Wert der Eigenschaft 'id'.
     *
     * @see #_id
     */
    public String getId() {
        return _id;
    }

    @Override
    public Image getImage() {
        return KonstantenGTMGUIResource.ICON_GTM_LZZ;
    }

    /**
     * Getter Strasse zu der Zhlstelle.
     * @return   Strasse.
     */
    public Strasse getParentStrasseZuZaehlStelle() {
        return DatenVew.getInstanz().ermittleStrasse(_parentSOstrasse.getPidOrId());
    }

    @Override
    public SystemObject getSystemObjekt() {
        return _soLangZeitZaehlStelle;
    }

    //~ METHODEN ==============================================================

    /**
     * Getter fr Eigenschaft 'xWgs84'.
     *
     * @return Wert der Eigenschaft 'xWgs84'.
     *
     * @see #_xWgs84
     */
    public double getxWgs84() {
        return _xWgs84;
    }

    /**
     * Getter fr Eigenschaft 'yWgs84'.
     *
     * @return Wert der Eigenschaft 'yWgs84'.
     *
     * @see #_yWgs84
     */
    public double getyWgs84() {
        return _yWgs84;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 17.09.12
