/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.bast.gui;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.listendarstellung.DataTableObjectErweitert;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;

import de.inovat.buv.gtm.datvew.lib.SelektionVew;
import de.inovat.buv.gtm.datvew.lib.ZaehlStelle;
import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;
import de.inovat.buv.inovat.lib.gui.GuiFunktionen;
import de.inovat.buv.plugin.gtm.bast.KonstantenGTMBast;
import de.inovat.buv.plugin.gtm.navigation.lib.LangzeitzaehlstellenAuswahlFunktionen;
import de.inovat.buv.projektlib.dav.DavArchivVew;
import de.inovat.buv.projektlib.dav.DavDatenVew;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;

//~ JDK IMPORTE ===============================================================

import java.text.SimpleDateFormat;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

//~ KLASSEN ===================================================================

/**
 * Die Klasse verwaltet das GUI und bearbeitet die SchaltflaecheAktionen.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 395 $ / $Date: 2013-06-19 14:41:05 +0200 (Mi, 19 Jun 2013) $ / ($Author: LG $)
 */
public class ExportGUIVerwaltung implements ClientReceiverInterface, ClientSenderInterface {

    /** Aspekt Anfrage. */
    private static final String ASP_ANFRAGE = "asp.anfrage";

    /** Aspekt Zustand. */
    private static final String ASP_EXPORT_STATUS = "asp.bastExportStatus";

    /** Attributgruppe fr den Empfang des Exportstatus. */
    private static final String ATG_BAST_STATUS = "atg.bastStatus";

    /** Attributgruppe zur Steuerung des BASt-Band Exports: */
    private static final String ATG_EXPORT_STEUERUNG = "atg.bastExportSteuerung";

    //~ FELDER ================================================================

    /** Die im GUI angezeigte Meldung. */
    private String _meldung = "";

    /** Die selektierten Langzeitzaehlstellen. */
    private List<ZaehlStelle> _listeLangzeitzaehlstellen = new ArrayList<ZaehlStelle>();

    /** Shell */
    private Shell _shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

    /** Map mit allen ExportBenutzereinstellungen, Schlssel ist der Name der Exportbenutzereinstellung, Wert eine ExportBenutzereinstellung. */
    private Map<String, ExportBenutzerEinstellungen> _mapExportBenutzerEinstellungen = new HashMap<String, ExportBenutzerEinstellungen>();

    /** Datenbeschreibungsobjekt Export Status. */
    private DataDescription _datenbeschreibungExportStatus;

    /** Datenbeschreibungsobjekt Export Steuerung. */
    private DataDescription _datenbeschreibungExportSteuerung;

    /** ClientDavInteface. */
    private ClientDavInterface _dav;

    /** Instanz der Klasse IExportEinstellungen. */
    private IExportEinstellungen _einstellungen;

    /** ExportBenutzerEinstellungenVerwaltungsinstanz. */
    private ExportBenutzerEinstellungenVerwaltung _exportVerwaltung;

    /** Instanz der Klasse ExportGUI, wo die Felder definiert sind. */
    private ExportGUI _gui;

    /**
     * Prozent der schon bearbeiteten Langzeitzhlstellen.
     */
    private int _prozentWert;

    /**
     * Zeitpunkt, wann der Export gestartet wurde.
     * Dient als Startpunkt fr die Berechnung der Restzeit.
     */
    private long _startZeit;

    /** Systemobjekt fr den BASt-Band Export. */
    private SystemObject _systemObjectBAStExportImportSteuerung;

    /**
     * Typ des Systemobjekte fr den BASt-Band Export.
     */
    private String _typBAStExport;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse ExportGUIVerwaltung.
     *
     * @param einstellungen Instanz der Klasse IExportEinstellungen.
     * @param guiInstanz Instanz der GUI-Klasse.
     * @param pidKonfigurationsObjBastBand bastBandObj.
     */
    protected ExportGUIVerwaltung(IExportEinstellungen einstellungen, ExportGUI guiInstanz, String pidKonfigurationsObjBastBand) {
        _typBAStExport                  = pidKonfigurationsObjBastBand;
        _einstellungen                  = einstellungen;
        _gui                            = guiInstanz;
        _exportVerwaltung               = new ExportBenutzerEinstellungenVerwaltung(_mapExportBenutzerEinstellungen);
        _mapExportBenutzerEinstellungen = _exportVerwaltung.getMapExportBenutzerEinstellungen();

        if (_listeLangzeitzaehlstellen.size() == 0) {
            _listeLangzeitzaehlstellen = SelektionVew.getInstanz().ermittleListeSelektierterZaehlStellen();
        }

        zeigeAnzahlSelLangzeitzaehlstellen();
        anmeldenDaVEmpfaengerBAStExportStatusMeldung();
        anmeldenDaVSenderBAStExport();
    }

    //~ METHODEN ==============================================================

    /**
     * Abmelden des Empfngers. Diese Abmelderoutine wird beim erfolgreichen Ende des Exportvorganges, wie auch
     * beim Schlieen des GUI aufgerufen.
     */
    protected void abmeldenDaVEmpfaengerExportStatus() {
        if (_systemObjectBAStExportImportSteuerung != null) {
            _dav.unsubscribeReceiver(this, _systemObjectBAStExportImportSteuerung, _datenbeschreibungExportStatus);
        }
    }

    /**
     * Abmelden des Senders. Auch wenn der Export gescheitert ist, muss die Abmeldung vollzogen werden.
     */
    protected void abmeldenDaVSenderExportImportSteuerung() {
        if (_systemObjectBAStExportImportSteuerung != null) {
            _dav.unsubscribeSender(this, _systemObjectBAStExportImportSteuerung, _datenbeschreibungExportSteuerung);
        }
    }

    /**
     * aendere die Werte fr die Fortschrittsanzeigen und Meldung.
     * Die Meldung wird zum alten Text hinzugefgt.
     *
     * @param prozent Der neue Wert der allgemeinen Fortschrittsanzeige oder
     *  -1, falls der Wert nicht geaendert werden soll.
     * @param meldung Neue Meldung oder null, falls der Wert nicht geaendert werden soll.
     * @param ausUpdateMethode Meldung.
     */
    public void aendereGUIAnzeige(final int prozent, final String meldung, final boolean ausUpdateMethode) {

        // Zeitstempel
        long             zeitstempel          = System.currentTimeMillis();
        SimpleDateFormat sdf                  = new SimpleDateFormat("dd.MM.yyyy hh:mm:ss");
        String           time                 = sdf.format(zeitstempel);
        final String     meldungMitZeitangabe = time + ": " + meldung + NEWLINE;

        if (!_gui.isDisposed()) {
            if (prozent >= 0) {
                _gui.getPrgFortschrittsanzeige().setSelection(prozent);

                // berechneRestZeit(prozent);
            }

            if (meldung != null) {
                if (ausUpdateMethode) {
                    _meldung = _meldung + meldung;
                } else {
                    _meldung = _meldung + meldungMitZeitangabe;
                }

                _gui.getTxtMeldungen().setText(_meldung);

                // Scrolle die Meldungen zum Ende:
                _gui.getTxtMeldungen().setSelection(_gui.getTxtMeldungen().getCharCount());
            }

            _gui.getCompositeAllgemein().redraw();
            _gui.getCompositeAllgemein().update();
        }
    }

    /**
     * Die Methode meldet sich zum Empfang des Status BASt-Band Exportes an.
     * Diese Anmeldung wird genau dann vollzogen, wenn die Anmeldung zum Senden
     * der Daten zum BASt-Band export geklappt haben.
     */
    public void anmeldenDaVEmpfaengerBAStExportStatusMeldung() {
        AttributeGroup attributgruppeStatus;
        Aspect         aspektZustand;
        ReceiveOptions anmeldeoptionen = ReceiveOptions.normal();
        ReceiverRole   empfaengerrolle = ReceiverRole.drain();

        _dav                                   = DavDatenVew.getInstanz().getDav();
        _systemObjectBAStExportImportSteuerung = _dav.getDataModel().getObject(_typBAStExport);
        attributgruppeStatus                   = _dav.getDataModel().getAttributeGroup(ATG_BAST_STATUS);
        aspektZustand                          = _dav.getDataModel().getAspect(ASP_EXPORT_STATUS);
        _datenbeschreibungExportStatus         = new DataDescription(attributgruppeStatus, aspektZustand);

        // Anmelden zum Empfang der Daten
        String benutzerInfo = subscribe(_systemObjectBAStExportImportSteuerung, _datenbeschreibungExportStatus, anmeldeoptionen, empfaengerrolle);
        long   startZeit    = DatumFunktionen.getAktuelleDateTime();
        String zeit         = DatumFunktionen.getDatumZeitAlsString(startZeit);

        aendereGUIAnzeige(1, zeit + benutzerInfo, false);
    }

    /**
     * Anmeldung beim Datenverteiler fr den BASt-Band Daten export.
     */
    public void anmeldenDaVSenderBAStExport() {
        AttributeGroup attributgruppeExportSteuerung;
        Aspect         aspektAnfrage;
        ReceiveOptions anmeldeoptionen = ReceiveOptions.normal();
        SenderRole     senderRolle     = SenderRole.sender();
        short          simulationsvariante;

        _dav                                   = DavDatenVew.getInstanz().getDav();
        _systemObjectBAStExportImportSteuerung = _dav.getDataModel().getObject(_typBAStExport);
        attributgruppeExportSteuerung          = _dav.getDataModel().getAttributeGroup(ATG_EXPORT_STEUERUNG);
        aspektAnfrage                          = _dav.getDataModel().getAspect(ASP_ANFRAGE);
        simulationsvariante                    = 0;
        _datenbeschreibungExportSteuerung      = new DataDescription(attributgruppeExportSteuerung, aspektAnfrage, simulationsvariante);

        // Anmelden zum Senden der Daten
        String benutzerInfo = subscribe(_systemObjectBAStExportImportSteuerung, _datenbeschreibungExportSteuerung, anmeldeoptionen, senderRolle);
        long   startZeit    = DatumFunktionen.getAktuelleDateTime();
        String zeit         = DatumFunktionen.getDatumZeitAlsString(startZeit);

        aendereGUIAnzeige(1, zeit + benutzerInfo, false);
    }

    /**
     * Die Methode bearbeitet die Aktionen zu den Radiobuttons. Wenn ein Radiobutton selektiert wurde,
     * mssen die anderen Radiobuttons unselektiert sein.
     * @param radioBASt der selektierte Radiobutton.
     */
    public void bastFormatSelektiert(Button radioBASt) {
        if (radioBASt.equals(_gui.getRadioBASt2004())) {
            _gui.getRadioBASt2007().setSelection(false);
            _gui.getRadioAXL2009().setSelection(false);
            _gui.getRadioBASt2004().setSelection(true);
        } else if (radioBASt.equals(_gui.getRadioBASt2007())) {
            _gui.getRadioAXL2009().setSelection(false);
            _gui.getRadioBASt2004().setSelection(false);
            _gui.getRadioBASt2007().setSelection(true);
        } else {
            _gui.getRadioBASt2004().setSelection(false);
            _gui.getRadioBASt2007().setSelection(false);
            _gui.getRadioAXL2009().setSelection(true);
        }
    }

    /**
     * Berechne die Restzeit fr den Exportvorgang
     * und zeige die im GUI.
     *
     * @param prozentGemacht Wie viel Prozent des Exports schon gemacht wurde.
     *
     * private void berechneRestZeit(int prozentGemacht) {
     *   String rest = "";
     *
     *   if (prozentGemacht > 1 && prozentGemacht < 100) {
     *
     *       // Wie viel Zeit wurde verbraucht fr das gemachte Prozent des Exports:
     *       long zeit = DatumFunktionen.getAktuelleDateTime() - _startZeit;
     *
     *       // Wie viel Zeit ist geblieben fr den Rest der Arbeit (entspricht 100-prozentGemacht Prozenten).
     *       long restZeit = zeit / prozentGemacht * (100 - prozentGemacht);
     *
     *       rest = "geschaetzte Restzeit   " + DatumFunktionen.getZeitdauerAlsString(restZeit);
     *   }else {
     *       rest = "Die bentigete Zeit kann erst nach dem Start des Exportforganges bestimmt werden.";
     *   }
     *
     *   // Zeige den Restzeit im GUI:
     *   _gui.getLbRestzeit().setText(rest);
     * }
     */

    /**
     * Die Taste "Abbruch"  wird selektiert.
     *
     * <p>
     * 1. Der aktuelle Export wird unterbrochen, dafr wird eine Meldung an die SWE
     * BASt-Band geschickt, die den Abbruch kennzeichnet.
     *
     * 2. Das GUI wird geschlossen.
     * </p>
     */
    protected void btnAbbruchSelektiert() {

        // Zeitstempel
        long zeitstempel = System.currentTimeMillis();

        // Datensatz erstellen
        AttributeGroup atg = _dav.getDataModel().getAttributeGroup(ATG_EXPORT_STEUERUNG);

        if (atg != null) {
            Data datensatzExportSteuerung = _dav.createData(atg);

            // vonMonat
            TextValue vonMonat       = datensatzExportSteuerung.getTextValue("vonMonat");
            long      datumZeitVon   = _gui.getDatumVon().ermittleDatumZeit();
            int       monatVonAlsInt = DatumFunktionen.getMonat(datumZeitVon);
            String    monatVon       = DatumFunktionen.getMonatNamen(monatVonAlsInt);

            vonMonat.setText(monatVon);

            // vonJahr
            TextValue vonJahr       = datensatzExportSteuerung.getTextValue("vonJahr");
            int       jahrVon       = DatumFunktionen.getJahr(datumZeitVon);
            String    jahrVonString = Integer.toString(jahrVon);

            vonJahr.setText(jahrVonString);

            // bisMonat
            TextValue bisMonat       = datensatzExportSteuerung.getTextValue("bisMonat");
            long      datumZeitBis   = _gui.getDatumBis().ermittleDatumZeit();
            int       monatBisAlsInt = DatumFunktionen.getMonat(datumZeitBis);
            String    monatBis       = DatumFunktionen.getMonatNamen(monatBisAlsInt);

            bisMonat.setText(monatBis);

            // bisJahr
            TextValue bisJahr       = datensatzExportSteuerung.getTextValue("bisJahr");
            int       jahrBis       = DatumFunktionen.getJahr(datumZeitBis);
            String    jahrBisString = Integer.toString(jahrBis);

            bisJahr.setText(jahrBisString);

            // Ausgabeformat je nach Wahl im View.
            TextValue bastBandAusgabeFormat = datensatzExportSteuerung.getTextValue("BastBandAusgabeFormat");
            String    ausgabeFormat;

            if (_gui.getRadioBASt2004().getSelection()) {
                ausgabeFormat = "V2004";
            } else if (_gui.getRadioBASt2007().getSelection()) {
                ausgabeFormat = "V2007";
            } else {
                ausgabeFormat = "AXL2009";
            }

            bastBandAusgabeFormat.setText(ausgabeFormat);

            String ueberschreiben = "Nein";

            // Angabe ob schon existierende BASt-Band Dateien zur gleichen Selektion berschrieben werden sollen
            TextValue bastBandUeberschreiben = datensatzExportSteuerung.getTextValue("BAStBandUeberschreiben");

            // name ja, wert 1, name nein, wert 0
            if (_gui.getCheckDateiErsetzen().getSelection()) {
                ueberschreiben = "Ja";
            }

            bastBandUeberschreiben.setText(ueberschreiben);

            String    konfigurationsAenderung = "Nein";
            TextValue konfAenderung           = datensatzExportSteuerung.getTextValue("AenderungDatenstruktur");

            konfAenderung.setText(konfigurationsAenderung);

            TextValue     ftpServer             = datensatzExportSteuerung.getTextValue("FTP-Server");
            StringBuilder ftpZugangsInformation = new StringBuilder();

            ftpZugangsInformation.append("EXPORT-ABBRUCH");
            ftpServer.setText(ftpZugangsInformation.toString());

            // Telegramm erstellen
            ResultData datenExportBAStBandResultData = new ResultData(_systemObjectBAStExportImportSteuerung, _datenbeschreibungExportSteuerung, zeitstempel, datensatzExportSteuerung);

            // Versenden
            try {
                _dav.sendData(datenExportBAStBandResultData);
            }
            catch (DataNotSubscribedException e) {
                System.err.println(getClass().getName() + "::");
                e.printStackTrace();
            }
            catch (SendSubscriptionNotConfirmed e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Die Taste "Exportieren"  wird selektiert.
     * <p>
     * Speichere die GUI-Werte, exportiere die selektierten Langzeitzaehlstellen in die Datei,
     * leere das Set der selektierten Langzeitzaehlstellenn und erlaube weitere Funktionen mit der erzeugten Datei.
     * </p>
     */
    protected void btnExportierenSelektiert() {

        // Fortschrittanzeige und Meldungen initialisieren:
        initParameterMeldungUndFortschritt();
        _gui.getBtnExportieren().setEnabled(false);

        // Starte:
        _startZeit = DatumFunktionen.getAktuelleDateTime();

        // Die Zeitdauer knstlich um 2 sek. verlaengern (fr die Berechnung der Restzeit):
        _startZeit = _startZeit - 2000;
        sendeExportDaten();
    }

    /**
     * Die Taste "LangzeitzaehlstellenAuswahl"  wird selektiert.
     * <p>
     * Die Menge der selektierten Langzeitzaehlstellen wird gesetzt.
     * Wenn keine Auswahl im Baum getroffen wurde, dann werden alle Langzeitzhlstellen in der Liste angezeigt.
     * </p>
     * <p>
     * Die Anzahl der selektierten Langzeitzhlstellen wird im GUI angezeigt.
     * </p>
     * <p>
     * Die Exportschaltflche wird entsprechend der Bedingungen gesetzt, d.h. nur wenn alle
     * Pflichtfelder ausgefllt sind und mindestens eine Langzeitzhlstelle selektiert wurde,
     * wird die Exportierenschaltflche eingabebereit.
     * </p>
     */
    protected void btnLangzeitzaehlstellenAuswahlSelektiert() {
        oeffneLangzeitzaehlstellenAuswahlDialogMitVorselektion(_listeLangzeitzaehlstellen);

        // Zeige im GUI, wie viel Langzeitzaehlstellen sind selektiert:
        zeigeAnzahlSelLangzeitzaehlstellen();

        // Setzte den Exportbutton entsprechend der Bedingung, dass alle Pflichteingabefelder gesetzt sind und
        // mindestens eine LZZ ausgewhlt worden ist.
        _gui.getBtnExportieren().setEnabled(istStartExportOK());
    }

    /**
     * Die Methode lscht den exporteinstellungsNamen aus der Map aller Exporteinstellungen.
     * @param exporteinstellungsName der zu lschende Eintrag in der Map aller Exporteinstellungen.
     */
    public void btnLoeschenExporteinstellung(String exporteinstellungsName) {
        _mapExportBenutzerEinstellungen.remove(exporteinstellungsName);
        _exportVerwaltung.setMapExportBenutzerEinstellungen(_mapExportBenutzerEinstellungen);
        _exportVerwaltung.speichereExportVerwaltungen(KonstantenGTMBast.ALLE_EXPORT_EINSTELLUNGEN);
        erzeugeComboboxGespeicherteExporteinstellungen("");
        initialisiereGUI();
    }

    /**
     * Die Methode schliet das aktuelle gui.
     */
    public void btnSchliessenSelektiert() {
        GuiFunktionen.schliesseGUI(_gui.getParent(), _gui.getViewID());
    }

    /**
     * Die Methode sorgt dafr, dass die gettigten Eingaben gespeichert werden.
     */
    public void btnSpeichern() {

        // Definiere den Validator:
        IInputValidator validator = new IInputValidator() {
            @Override
            public String isValid(String text) {
                text = text.trim();

                if (text.length() == 0) {
                    return "";
                } else if (exportEinstellungsNameExistiertSchon(text)) {
                    return "Der Name existiert schon !";
                }

                return null;
            }
        };

        // Definiere den Dialog:
        InputDialog inputDialog = new InputDialog(_gui.getShell(), "Speichern", "Bitte geben Sie den Exporteinstellungsnamen ein", "", validator);

        // ffne den Dialog:
        if (inputDialog.open() == Dialog.OK) {

            // Merke den eingegebenen Namen:
            String speicherName = inputDialog.getValue().trim();

            // Speichere den Namen:
            speicherAktuelleExportDarstellungsOptionen(speicherName);

            // Passe das Combo an:
            erzeugeComboboxGespeicherteExporteinstellungen(speicherName);
        }
    }

    @Override
    public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {}

    /**
     * Ermittle Instanz der Klasse ParameterVerwaltung fr bestimmten Export
     * (wird eindeutig ber View.ID definiert)..
     *
     * @return Instanz der Klasse ParameterVerwaltung.
     */
    private ParameterVerwaltung ermittleParVewInstanz() {
        return ParameterVerwaltung.getInstanz(_gui.getViewID(), _einstellungen.ermittleParameterID(), _einstellungen.ermittleDefaultExportDatei());
    }

    /**
     * Die Methode fllt die Combobox mit den gespeicherten Exporteinstellungsnamen neu und
     * belegt sie.
     * @param auswahl Auswahl.
     */
    public void erzeugeComboboxGespeicherteExporteinstellungen(String auswahl) {
        _gui.getComboGespeicherteExporteinstellungen().removeAll();

        for (String exportBenutzerEinstellung : _mapExportBenutzerEinstellungen.keySet()) {
            _gui.getComboGespeicherteExporteinstellungen().add(exportBenutzerEinstellung);
        }

        if ((_gui.getComboGespeicherteExporteinstellungen().getItemCount() > 1) &&!auswahl.equals("")) {
            for (int i = 0; i < _gui.getComboGespeicherteExporteinstellungen().getItemCount(); i++) {
                if (_gui.getComboGespeicherteExporteinstellungen().getItem(i).equals(auswahl)) {
                    _gui.getComboGespeicherteExporteinstellungen().select(i);
                    _gui.getBtnloeschenExporteinstellung().setEnabled(true);
                }
            }
        } else if (_gui.getComboGespeicherteExporteinstellungen().getItemCount() == 0) {
            _gui.getComboGespeicherteExporteinstellungen().add("", 0);
            _gui.getComboGespeicherteExporteinstellungen().select(0);
            _gui.getBtnloeschenExporteinstellung().setEnabled(false);
        } else {
            _gui.getComboGespeicherteExporteinstellungen().add("", 0);
            _gui.getComboGespeicherteExporteinstellungen().select(0);
            _gui.getBtnloeschenExporteinstellung().setEnabled(false);
        }
    }

    /**
     * Die Methode berprft ob der Name schon existiert.
     * @param exportEinstellungsName der geprfte Name.
     * @return true wenn der Exporteinstellungsname schon existiert, false sonst.
     */
    private boolean exportEinstellungsNameExistiertSchon(String exportEinstellungsName) {
        return _mapExportBenutzerEinstellungen.containsKey(exportEinstellungsName);
    }

    /**
     * Die Methode initialisiert das GUI.
     */
    public void initialisiereGUI() {
        _gui.getRadioBASt2004().setSelection(true);
        _gui.getRadioBASt2007().setSelection(false);
        _gui.getRadioAXL2009().setSelection(false);

        //
        _gui.getRbtnFtp().setSelection(true);
        _gui.getRbtnSftp().setSelection(false);
        rbtnServertypGeaendert();
        _gui.getTxtURL().setText("");
        _gui.getTxtBenutzer().setText("");
        _gui.getTxtPasswd().setText("");
        _gui.getCbtnPassivmodus().setSelection(false);
        _gui.getCheckDateiErsetzen().setSelection(false);
    }

    /**
     * Initialisiere den Meldungstext und die Fortschrittsanzeige.
     */
    protected void initParameterMeldungUndFortschritt() {
        _gui.getPrgFortschrittsanzeige().setSelection(0);
        _meldung = "";
        _gui.getTxtMeldungen().setText("");
    }

    /**
     * Initialisiert die Zeitbereichsangabe. Dabei wird das Von-Datum auf den
     * ersten Tag des letzten Monats gesetzt, das Bis-Datum auf den letzten Tag
     * des letzten Monats.
     */
    protected void initZeitbereich() {
        long vormonat       = DatumFunktionen.getVerschobenenDateTime(DatumFunktionen.getAktuelleDateTime(), -1, 0);
        long vormonatAnfang = DatumFunktionen.getMonatAnfang(vormonat);
        long vormonatEnde   = DatumFunktionen.getMonatEnde(vormonat);

        _gui.getDatumVon().setzeDatumZeit(vormonatAnfang);
        _gui.getDatumBis().setzeDatumZeit(vormonatEnde);
        _gui.getDatumVon().erweitereGroesse(5, 0);
        _gui.getDatumBis().erweitereGroesse(5, 0);
        _gui.getCompositeAllgemein().pack();
        _gui.getCompositeAllgemein().redraw();
        _gui.getCompositeAllgemein().update();
    }

    //~ GET METHODEN ==========================================================

    @Override
    public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
        return false;
    }

    //~ METHODEN ==============================================================

    /**
     * Die Methode prft die Eingabefelder bzgl. der FTP Server Zugangsdaten vollstndig eingegeben sind,
     * und ob ein Export derzeit mglich ist.
     * @return true wenn alle FTP Server Zugangsdaten Felder nicht leer sind, und ein
     * Export durchgefhrt werden kann, false sonst.
     */
    private boolean istStartExportOK() {
        boolean startExportOK;

        if ((DavArchivVew.getInstanz().istArchivVerfuegbar()) && (!_listeLangzeitzaehlstellen.isEmpty()) &&  //
            (_prozentWert == 100) &&                                                                         //
            (_gui.getRbtnFtp().getSelection() == !_gui.getRbtnSftp().getSelection()) &&                      //
            (!_gui.getTxtURL().getText().isEmpty()) &&                                                       //
            (!_gui.getTxtPort().getText().isEmpty()) &&                                                      //
            (!_gui.getTxtBenutzer().getText().isEmpty()) &&                                                  //
            (!_gui.getTxtPasswd().getText().isEmpty())) {

            //
            startExportOK = true;
        } else {
            startExportOK = false;
        }

        return startExportOK;
    }

    /**
     * Die Methode ldt die gespeicherten Exporteinstellungen der ausgewhlten Exporteinstellung.
     * @param einstellungsName Exporteinstellung.
     */
    public void ladeGespeicherteExporteinstellungen(String einstellungsName) {
        ExportBenutzerEinstellungen einstellung = _mapExportBenutzerEinstellungen.get(einstellungsName);

        if (einstellung != null) {
            _gui.getRadioBASt2004().setSelection(einstellung.isBast2004());
            _gui.getRadioBASt2007().setSelection(einstellung.isBast2007());
            _gui.getRadioAXL2009().setSelection(einstellung.isAxl2009());

            //
            _gui.getTxtURL().setText(einstellung.getFtpServerURL());
            _gui.getTxtPort().setText(einstellung.getFtpServerPort());
            _gui.getTxtBenutzer().setText(einstellung.getFtpServerBenutzer());
            _gui.getTxtPasswd().setText(einstellung.getFtpServerPasswort());
            _gui.getCbtnPassivmodus().setSelection(einstellung.isPassivmodus());
            _gui.getRbtnFtp().setSelection(einstellung.isIstFtp());
            _gui.getRbtnSftp().setSelection(!einstellung.isIstFtp());
            _gui.getCheckDateiErsetzen().setSelection(einstellung.isVorhandeneDateiErsetzten());
        }
    }

    /**
     * ffne den LangzeitzaehlstellenAuswahlDialog mit der Vorselektion.
     * <p>
     * Die Menge der selektierten Langzeitzaehlstellen wird gesetzt.
     * </p>
     *
     * @param listeVorselektierteLangzeitzaehlstellen Liste der vorselektierten Langzeitzaehlstellen oder
     * null, falls keine Langzeitzaehlstellen vorselektiert werden sollen.
     */
    public void oeffneLangzeitzaehlstellenAuswahlDialogMitVorselektion(List<ZaehlStelle> listeVorselektierteLangzeitzaehlstellen) {
        _listeLangzeitzaehlstellen = LangzeitzaehlstellenAuswahlFunktionen.langzeitZaehlstellenAuswahlDialogOeffnen(_shell, listeVorselektierteLangzeitzaehlstellen, null);
    }

    /**
     * Servertyp wurde gendert
     */
    protected void rbtnServertypGeaendert() {

        // Defaultwert fr FTP:
        if (_gui.getRbtnFtp().getSelection()) {
            _gui.getTxtPort().setText("21");
        }

        // Defaultwert fr SFTP:
        else {
            _gui.getTxtPort().setText("22");
        }
    }

    /**
     * Anlegen eines leeren Datensatzes zur Attributgruppe atg.bastExportSteuerung.<p>
     * Die werte des Datensatzes werden mit den Werten aus dem Export-View belegt.
     * Senden der Daten als Sender.
     * <p>
     * DataNotSubscribeException: Ausnahme, die beim Senden von Datenstzen ohne entsprechende Sendeanmeldungen
     * generiert wird.
     * SendSubscriptionNotConfirmed: Ausnahme, die beim Senden von Daten als einfacher Sender generiert wird,
     * wenn noch keine positive Sendesteuerung vom Datenverteiler fr die zu versendenden Daten vorliegt
     */
    public void sendeExportDaten() {

        // Zeitstempel
        long zeitstempel = System.currentTimeMillis();

        // Datensatz erstellen
        AttributeGroup atg = _dav.getDataModel().getAttributeGroup(ATG_EXPORT_STEUERUNG);

        if (atg != null) {
            Data datensatzExportSteuerung = _dav.createData(atg);

            // Prfe das Zeitintervall:
            long datumZeitVon = _gui.getDatumVon().ermittleDatumZeit();
            long datumZeitBis = _gui.getDatumBis().ermittleDatumZeit();

            if (datumZeitVon > datumZeitBis) {
                MessageDialog.openError(_shell, "Fehler", "Fehler in der Zeitbereichsangabe: von-Datum darf nicht nach dem bis-Datum liegen!");
                setzeExportSchaltflaeche();

                return;
            }

            // vonTag
            NumberValue vonTag       = datensatzExportSteuerung.getScaledValue("vonTag");
            int         tagVonAlsInt = DatumFunktionen.getTag(datumZeitVon);

            vonTag.set(tagVonAlsInt);

            // vonMonat
            TextValue vonMonat       = datensatzExportSteuerung.getTextValue("vonMonat");
            int       monatVonAlsInt = DatumFunktionen.getMonat(datumZeitVon);
            String    monatVon       = DatumFunktionen.getMonatNamen(monatVonAlsInt);

            vonMonat.setText(monatVon);

            // vonJahr
            TextValue vonJahr       = datensatzExportSteuerung.getTextValue("vonJahr");
            int       jahrVon       = DatumFunktionen.getJahr(datumZeitVon);
            String    jahrVonString = Integer.toString(jahrVon);

            vonJahr.setText(jahrVonString);

            // bisTag
            NumberValue bisTag       = datensatzExportSteuerung.getScaledValue("bisTag");
            int         tagBisAlsInt = DatumFunktionen.getTag(datumZeitBis);

            bisTag.set(tagBisAlsInt);

            // bisMonat
            TextValue bisMonat       = datensatzExportSteuerung.getTextValue("bisMonat");
            int       monatBisAlsInt = DatumFunktionen.getMonat(datumZeitBis);
            String    monatBis       = DatumFunktionen.getMonatNamen(monatBisAlsInt);

            bisMonat.setText(monatBis);

            // bisJahr
            TextValue bisJahr       = datensatzExportSteuerung.getTextValue("bisJahr");
            int       jahrBis       = DatumFunktionen.getJahr(datumZeitBis);
            String    jahrBisString = Integer.toString(jahrBis);

            bisJahr.setText(jahrBisString);

            // Ausgabeformat je nach Wahl im View.
            TextValue bastBandAusgabeFormat = datensatzExportSteuerung.getTextValue("BastBandAusgabeFormat");
            String    ausgabeFormat;

            if (_gui.getRadioBASt2004().getSelection()) {
                ausgabeFormat = "V2004";
            } else if (_gui.getRadioBASt2007().getSelection()) {
                ausgabeFormat = "V2007";
            } else {
                ausgabeFormat = "AXL2009";
            }

            bastBandAusgabeFormat.setText(ausgabeFormat);

            String ueberschreiben = "Nein";

            // Angabe ob schon existierende BASt-Band Dateien zur gleichen Selektion berschrieben werden sollen
            TextValue bastBandUeberschreiben = datensatzExportSteuerung.getTextValue("BAStBandUeberschreiben");

            // name ja, wert 1, name nein, wert 0
            if (_gui.getCheckDateiErsetzen().getSelection()) {
                ueberschreiben = "Ja";
            }

            bastBandUeberschreiben.setText(ueberschreiben);

            String    konfigurationsAenderung = "Nein";
            TextValue konfAenderung           = datensatzExportSteuerung.getTextValue("AenderungDatenstruktur");

            if (_gui.getCheckAenderungDatenstruktur().getSelection()) {
                konfigurationsAenderung = "Ja";
            }

            konfAenderung.setText(konfigurationsAenderung);

            // DaV-Datenformat: "-Benutzer:%s-Passwort:%s-URL:%s-Port:%s-Passivmodus:%s-Servertyp:%s"
            //
            // alte Variante (ohne Port und FTP/SFTP):
            // "Benutzer:%sPasswort:%sURL:%sPassivmodus:%s"
            TextValue     ftpServer             = datensatzExportSteuerung.getTextValue("FTP-Server");
            StringBuilder ftpZugangsInformation = new StringBuilder();

            ftpZugangsInformation.append("-Benutzer:");
            ftpZugangsInformation.append(_gui.getTxtBenutzer().getText());
            ftpZugangsInformation.append("-Passwort:");
            ftpZugangsInformation.append(_gui.getTxtPasswd().getText());
            ftpZugangsInformation.append("-URL:");
            ftpZugangsInformation.append(_gui.getTxtURL().getText());
            ftpZugangsInformation.append("-Port:");
            ftpZugangsInformation.append(_gui.getTxtPort().getText());
            ftpZugangsInformation.append("-Passivmodus:");
            ftpZugangsInformation.append(_gui.getCbtnPassivmodus().getSelection() ? "ja" : "nein");
            ftpZugangsInformation.append("-Servertyp:");
            ftpZugangsInformation.append(_gui.getRbtnFtp().getSelection() ? "FTP" : "SFTP");
            ftpServer.setText(ftpZugangsInformation.toString());

            Data.ReferenceArray refAr_Lzz = datensatzExportSteuerung.getItem("LangZeitZhlstellen").asReferenceArray();

            refAr_Lzz.setLength(_listeLangzeitzaehlstellen.size());

            Data.ReferenceValue[] refValueAr_Lzz = refAr_Lzz.getReferenceValues();

            for (int i = 0; i < _listeLangzeitzaehlstellen.size(); i++) {
                refValueAr_Lzz[i].setSystemObject(_listeLangzeitzaehlstellen.get(i).getSystemObjekt());
            }

            // Telegramm erstellen
            ResultData datenExportBAStBandResultData = new ResultData(_systemObjectBAStExportImportSteuerung, _datenbeschreibungExportSteuerung, zeitstempel, datensatzExportSteuerung);

            // Versenden
            try {
                _dav.sendData(datenExportBAStBandResultData);
                _gui.getBtnAbbrechen().setEnabled(true);
            }
            catch (DataNotSubscribedException e) {
                System.err.println(getClass().getName() + "::");
                e.printStackTrace();
            }
            catch (SendSubscriptionNotConfirmed e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * Die Methode berprft ob eines der Pflichtfelder
     * URL, Benutzer oder Passwort leer ist. Ist ein
     * Pflichtfeld nicht gefllt ist Exportieren nicht mglich.
     */
    public void setzeExportSchaltflaeche() {
        if (istStartExportOK()) {
            _gui.getBtnExportieren().setEnabled(true);
        } else {
            _gui.getBtnExportieren().setEnabled(false);
        }
    }

    /**
     * Setze die GUI Daten mit den Werten aus der aktuellen Parameter-Instanz.
     */
    protected void setzeGUIDatenMitAktParameter() {

        // hole den aktuellen Parameter:
        Parameter par = ermittleParVewInstanz().getParameter();

        // setze die Werte im GUI:
        _gui.getPrgFortschrittsanzeige().setSelection(par.getFortschrittsanzeige());
        _gui.getTxtMeldungen().setText(par.getMeldungen());

        // Scrolle die Meldungen zum Ende:
        _gui.getTxtMeldungen().setSelection(_gui.getTxtMeldungen().getCharCount());

        //
        zeigeAnzahlSelLangzeitzaehlstellen();
        _gui.getCompositeAllgemein().redraw();
        _gui.getCompositeAllgemein().update();
    }

    /**
     * Speichern der aktuellen Exporteinstellung.
     * @param einstellungsName Name unter dem die Exportdarstellung gespeichert wird.
     */
    private void speicherAktuelleExportDarstellungsOptionen(String einstellungsName) {
        ExportBenutzerEinstellungen einstellung =                                                            //
            new ExportBenutzerEinstellungen(_listeLangzeitzaehlstellen,                                      //
                                            _gui.getRadioBASt2004().getSelection(),                          //
                                            _gui.getRadioBASt2007().getSelection(),                          //
                                            _gui.getRadioAXL2009().getSelection(),                           //
                                            _gui.getTxtBenutzer().getText(), _gui.getTxtPasswd().getText(),  //
                                            _gui.getTxtURL().getText(), _gui.getTxtPort().getText(),         //
                                            _gui.getCbtnPassivmodus().getSelection(),                        //
                                            _gui.getRbtnFtp().getSelection(),                                //
                                            _gui.getCheckDateiErsetzen().getSelection());

        _mapExportBenutzerEinstellungen.put(einstellungsName, einstellung);
        _exportVerwaltung.setMapExportBenutzerEinstellungen(_mapExportBenutzerEinstellungen);
        _exportVerwaltung.speichereExportVerwaltungen(KonstantenGTMBast.ALLE_EXPORT_EINSTELLUNGEN);
    }

    /**
     * Anemldung am Datenverteiler.
     *
     * @param systemObjectBAStExportImportSteuerung
     *            das SystemObjekt das angemeldet wird.
     * @param datenbeschreibungExportSteuerung
     *            die Datenbeschreibung.
     * @param anmeldeoptionen
     *            die Anmeldeoption.
     * @param rolle
     *            dabei handelt es sich entweder um einen Sender oder einen Empfnger.
     * @return BenutzerInfo.
     */
    public String subscribe(SystemObject systemObjectBAStExportImportSteuerung, DataDescription datenbeschreibungExportSteuerung, ReceiveOptions anmeldeoptionen, Object rolle) {
        Shell        shell;
        ReceiverRole empfaengerrolle = ReceiverRole.receiver();
        SenderRole   senderRolle     = SenderRole.sender();
        String       benutzerInfo    = "";

        if (systemObjectBAStExportImportSteuerung == null) {
            shell        = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
            benutzerInfo = "Das SystemObjekt fr den BASt-Band Export existiert nicht!";
            MessageDialog.openError(shell, "Fehler", benutzerInfo);
        } else {
            try {
                if (rolle instanceof ReceiverRole) {
                    _dav.subscribeReceiver(ExportGUIVerwaltung.this, systemObjectBAStExportImportSteuerung, datenbeschreibungExportSteuerung, anmeldeoptionen, empfaengerrolle);
                    benutzerInfo = "Als Empfnger fr die Statusinformation angemeldet.";
                } else if (rolle instanceof SenderRole) {
                    _dav.subscribeSender(ExportGUIVerwaltung.this, systemObjectBAStExportImportSteuerung, datenbeschreibungExportSteuerung, senderRolle);
                    benutzerInfo = "Als Sender fr die Exportinformationen angemeldet.";
                }
            }
            catch (Exception e) {
                shell        = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
                benutzerInfo = "Die Objekte konnten nicht beim Datenverteiler angemeldet werden! ";
                MessageDialog.openError(shell, "Fehler", benutzerInfo);
            }
        }

        return benutzerInfo;
    }

    /**
     * Text 'Port' wurde gendert.
     */
    protected void txtPortGeaendert() {

        // Prfe, ob eine Zahl ist:
        try {
            Integer.parseInt(_gui.getTxtPort().getText());
        }
        catch (Exception e) {
            MessageDialog.openError(_shell, "Fehler", "Port-Angabe muss eine Zahl sein !!!");
        }
    }

    @Override
    public void update(ResultData[] results) {
        final Display display = Display.getDefault();

        if (!_gui.isDisposed()) {
            for (ResultData result : results) {
                if (result.hasData()) {
                    Data data = result.getData();

                    try {

                        // Der bermittelte Prozentwert als Integervalue, so dass dieser direkt fr die Fortschrittsanzeige verwendet werden kann.
                        _prozentWert = data.getScaledValue("BAStBandFortschrittProzent").intValue();
                    }
                    catch (Exception e) {
                        Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

                        MessageDialog.openError(shell, "Fehler", String.format("Die Daten entsprechen nicht den Konventionen. %s", e));
                    }

                    if (_prozentWert > 0) {
                        try {

                            // Der Statusmeldungstext.
                            final String statusMeldung = data.getItem("StatusMeldung").valueToString();

                            display.syncExec(new Runnable() {
                                @Override
                                public void run() {
                                    aendereGUIAnzeige(_prozentWert, statusMeldung, true);
                                }
                            });
                        }
                        catch (Exception e) {
                            Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

                            MessageDialog.openError(shell, "Fehler", "Es sind Probleme bei der Aktualisierung aufgetreten!.");
                        }

                        if (_prozentWert != 100) {
                            try {
                                display.syncExec(new Runnable() {
                                    @Override
                                    public void run() {
                                        _gui.getBtnExportieren().setEnabled(false);
                                    }
                                });
                            }
                            catch (Exception e) {
                                Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

                                MessageDialog.openError(shell, "Fehler", "Es sind Probleme bei der Aktualisierung aufgetreten!.");
                            }
                        } else if (_prozentWert == 100) {
                            try {
                                display.syncExec(new Runnable() {
                                    @Override
                                    public void run() {
                                        _gui.getBtnAbbrechen().setEnabled(false);

                                        if (istStartExportOK()) {
                                            _gui.getBtnExportieren().setEnabled(true);
                                        }
                                    }
                                });
                            }
                            catch (Exception e) {
                                Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

                                MessageDialog.openError(shell, "Fehler", "Es sind Probleme bei der Aktualisierung aufgetreten!.");
                            }
                        }
                    }
                } else {
                    DataTableObjectErweitert dto             = new DataTableObjectErweitert(result);
                    DataState                resultDataState = dto.getDataState();

                    try {
                        display.syncExec(new Runnable() {
                            @Override
                            public void run() {
                                _gui.getBtnAbbrechen().setEnabled(false);
                            }
                        });
                    }
                    catch (Exception e) {
                        Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

                        MessageDialog.openError(shell, "Fehler", "Es sind Probleme bei der Aktualisierung aufgetreten!.");
                    }

                    if (resultDataState == DataState.NO_SOURCE) {
                        try {
                            display.syncExec(new Runnable() {
                                @Override
                                public void run() {
                                    _gui.getBtnExportieren().setEnabled(false);
                                    _gui.getBtnAbbrechen().setEnabled(false);
                                    aendereGUIAnzeige(1, "Der Export kann nicht durchgefhrt werden, da der Prozess BASt-Band nicht luft." + NEWLINE, true);
                                }
                            });
                        }
                        catch (Exception e) {
                            Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();

                            MessageDialog.openError(shell, "Fehler", "Es sind Probleme bei der Aktualisierung aufgetreten!.");
                        }
                    }
                }
            }
        }
    }

    /**
     * Zeige im GUI, wie viele Langzeitzaehlstellen werden exportiert.
     */
    protected void zeigeAnzahlSelLangzeitzaehlstellen() {
        String text;

        if ((_listeLangzeitzaehlstellen == null) || (_listeLangzeitzaehlstellen.size() == 0)) {
            text = "Die aktuelle Auswahl exportiert keine Langzeitzhlstellendaten.";
        } else if (_listeLangzeitzaehlstellen.size() == 1) {
            text = "Es werden die Daten zu einer Langzeitzaehlstelle exportiert.";
        } else {
            text = "Es werden " + _listeLangzeitzaehlstellen.size() + " Langzeitzaehlstellendaten exportiert.";
        }

        //
        _gui.getLbLangZeitZaehlStellenAuswahl().setText(text);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 28.10.13
