/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.datenidentifikation;

//~ KLASSEN ===================================================================

/**
 * Die Klasse Datenidentifikation beschreibt eine Datenidentifikation (Objekttyp, Attributgruppe, Aspekt, Konfigurierend ja oder nein).
 *
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class Datenidentifikation {

    /** Attribut das angibt, ob es sich um Archivdaten handelt. */
    private boolean _archivDaten;

    /** Aspekt. */
    private String _aspect;

    /** Attributgruppe. */
    private String _atgGroup;

    /** Booleschhe Variable die angibt ob es sich um konfigurierende Daten handelt oder nicht. */
    private boolean _konfigurierendeDaten;

    /** Objekttyp. */
    private String _objTyp;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    public Datenidentifikation() {}

    public Datenidentifikation(String objTyp, String atgGroup, boolean archivDaten, boolean konfigurierendeDaten) {
        _objTyp               = objTyp;
        _atgGroup             = atgGroup;
        _archivDaten          = archivDaten;
        _konfigurierendeDaten = konfigurierendeDaten;
    }

    public Datenidentifikation(String objTyp, String atgGroup, String aspect, boolean archivDaten, boolean konfigurierendeDaten) {
        _objTyp               = objTyp;
        _atgGroup             = atgGroup;
        _aspect               = aspect;
        _archivDaten          = archivDaten;
        _konfigurierendeDaten = konfigurierendeDaten;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (!(obj instanceof Datenidentifikation)) {
            return false;
        }

        Datenidentifikation other = (Datenidentifikation) obj;

        if (_archivDaten != other._archivDaten) {
            return false;
        }

        if (_aspect == null) {
            if (other._aspect != null) {
                return false;
            }
        } else if (!_aspect.equals(other._aspect)) {
            return false;
        }

        if (_atgGroup == null) {
            if (other._atgGroup != null) {
                return false;
            }
        } else if (!_atgGroup.equals(other._atgGroup)) {
            return false;
        }

        if (_konfigurierendeDaten != other._konfigurierendeDaten) {
            return false;
        }

        if (_objTyp == null) {
            if (other._objTyp != null) {
                return false;
            }
        } else if (!_objTyp.equals(other._objTyp)) {
            return false;
        }

        return true;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'aspect'.
     *
     * @return Wert der Eigenschaft 'aspect'.
     *
     * @see #_aspect
     */
    public String getAspect() {
        return _aspect;
    }

    /**
     * Getter fr Eigenschaft 'atgGroup'.
     *
     * @return Wert der Eigenschaft 'atgGroup'.
     *
     * @see #_atgGroup
     */
    public String getAtgGroup() {
        return _atgGroup;
    }

    /**
     * Getter fr Eigenschaft 'objTyp'.
     *
     * @return Wert der Eigenschaft 'objTyp'.
     *
     * @see #_objTyp
     */
    public String getObjTyp() {
        return _objTyp;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     *     @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime  = 31;
        int       result = 1;

        result = prime * result + (_archivDaten ? 1231 : 1237);
        result = prime * result + ((_aspect == null) ? 0 : _aspect.hashCode());
        result = prime * result + ((_atgGroup == null) ? 0 : _atgGroup.hashCode());
        result = prime * result + (_konfigurierendeDaten ? 1231 : 1237);
        result = prime * result + ((_objTyp == null) ? 0 : _objTyp.hashCode());

        return result;
    }

    //~ GET METHODEN ==========================================================

    /**
     *     Getter fr Eigenschaft 'archivDaten'.
     *
     *     @return Wert der Eigenschaft 'archivDaten'.
     *
     *     @see #_archivDaten
     */
    public boolean isArchivDaten() {
        return _archivDaten;
    }

    /**
     * Getter fr Eigenschaft 'konfigurierendeDaten'.
     *
     * @return Wert der Eigenschaft 'konfigurierendeDaten'.
     *
     * @see #_konfigurierendeDaten
     */
    public boolean isKonfigurierendeDaten() {
        return _konfigurierendeDaten;
    }

    //~ SET METHODEN ==========================================================

    /**
     * Setter fr Eigenschaft 'archivDaten'.
     *
     * @param archivDaten Wert zum Setzen der Eigenschaft 'archivDaten'.
     *
     * @see #_archivDaten
     */
    public void setArchivDaten(boolean archivDaten) {
        _archivDaten = archivDaten;
    }

    /**
     * Setter fr Eigenschaft 'aspect'.
     *
     * @param aspect Wert zum Setzen der Eigenschaft 'aspect'.
     * WICHTIG! Verwendung fr die Speicherung im XML.
     *
     * @see #_aspect
     */
    public void setAspect(String aspect) {
        _aspect = aspect;
    }

    /**
     * Setter fr Eigenschaft 'atgGroup'.
     *
     * @param atgGroup Wert zum Setzen der Eigenschaft 'atgGroup'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_atgGroup
     */
    public void setAtgGroup(String atgGroup) {
        _atgGroup = atgGroup;
    }

    /**
     * Setter fr Eigenschaft 'konfigurierendeDaten'.
     *
     * @param konfigurierendeDaten Wert zum Setzen der Eigenschaft 'konfigurierendeDaten'.
     * WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_konfigurierendeDaten
     */
    public void setKonfigurierendeDaten(boolean konfigurierendeDaten) {
        _konfigurierendeDaten = konfigurierendeDaten;
    }

    /**
     * Setter fr Eigenschaft 'objTyp'.
     *
     * @param objTyp Wert zum Setzen der Eigenschaft 'objTyp'.
     *  WICHTIG! Verwendung fr die Speicherung im XML.
     * @see #_objTyp
     */
    public void setObjTyp(String objTyp) {
        _objTyp = objTyp;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 03.09.10
