/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.lib;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.gtm.datvew.lib.BaumVerwaltung;
import de.inovat.buv.gtm.datvew.lib.DatenVew;
import de.inovat.buv.gtm.datvew.lib.ZaehlStelle;
import de.inovat.buv.projektlib.konstanten.KonstantenGUIResource;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.internal.misc.StringMatcher;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Die Klasse Verwaltet den Auswahldialog Import und Export.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class AuswahlDialogGUI extends Dialog {

    /** Liste der Langzeitzhlstellen. */
    private List<ZaehlStelle> _listeLangZeitZaehlStelle = new ArrayList<ZaehlStelle>();

    /** Liste der ausgewhlten Langzeitzhlstellen. */
    private List<ZaehlStelle> _listeLangZeitZaehlStelleAuswahl = new ArrayList<ZaehlStelle>();

    /** Liste der Dateien. */
    private List<String> _listeDateien = new ArrayList<String>();

    /** Liste der Dateiauswahl. */
    private List<String> _listeDateiAuswahl = new ArrayList<String>();

    /** Liste aller Langzeitzhlstellen fr den angegebenen Modus. */
    private List<ZaehlStelle> _listeAllerLangZeitZaehlStellenFuerModus = new ArrayList<ZaehlStelle>();

    /** Liste aller Dateien. */
    private List<String> _listeAllerDateien = new ArrayList<String>();

    /** Attribut ob es sich um den Dateiauswahldialog handelt, ist true, wenn dem so ist. */
    private boolean _isDateiAuswahlDialog;

    /** ListViewer. */
    private ListViewer _listViewer;

    /** ListViewer der ausgewhlten Objekte. */
    private ListViewer _listViewerAuswahl;
    private Text       _txtFilterLinks;

    //~ KONSTANTE AUFZAEHLUNGSTYPEN ===========================================

    /**
     * Klasse definiert Zhlstellen-Modus: Langzeitzhlstellen oder Achslastzhlstellen.
     *
     * null-Objekt representiert alle Zhlstellen.
     */
    public static enum ZS_MODUS { LZZ, AXL }

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * TODO AuswahlDialogGUI...
     *
     * Erzeugt eine Instanz der Klasse AuswahlDialogGUI.
     *
     * @param parent Shell.
     * @param isDateiAuswahlDialog ist Datei-Auswahl.
     * @param modus Modus (null - Alle Langzeitzhlstellen).
     */
    public AuswahlDialogGUI(Shell parent, boolean isDateiAuswahlDialog, ZS_MODUS modus) {
        super(parent);
        _isDateiAuswahlDialog = isDateiAuswahlDialog;

        if (_isDateiAuswahlDialog) {
            initialisiereListeDateien();
        } else {
            initialisiereListeLzz(modus);
        }
    }

    //~ METHODEN ==============================================================

    /**
     * Reaktion auf die Selektion der Schaltflche 'Pfeile nach links'.
     */
    protected void btnPfeilLinksWidgetSelected() {

        // Definiere die Liste von selektierten Objekten:
        IStructuredSelection selection = (IStructuredSelection) _listViewerAuswahl.getSelection();
        Iterator<?>          it        = selection.iterator();

        if (_isDateiAuswahlDialog) {
            List<String> selektierteAuswahl = new ArrayList<String>();

            while (it.hasNext()) {
                selektierteAuswahl.add((String) (it.next()));
            }

            if (selektierteAuswahl.size() > 0) {

                // Entferne die selektierten Objekte aus der GUI Liste:
                _listeDateiAuswahl.removeAll(selektierteAuswahl);

                // Fge die selektierten Objekte in die andere GUI Liste:
                definiereLinkeInputListe();
                zeichneListen();
            }
        } else {
            List<ZaehlStelle> selektierteAuswahl = new ArrayList<ZaehlStelle>();

            while (it.hasNext()) {
                selektierteAuswahl.add((ZaehlStelle) (it.next()));
            }

            if (selektierteAuswahl.size() > 0) {

                // Entferne die selektierten Objekte aus der GUI Liste:
                _listeLangZeitZaehlStelleAuswahl.removeAll(selektierteAuswahl);

                // Fge die selektierten Objekte in die andere GUI Liste:
                definiereLinkeInputListe();
                zeichneListen();
            }
        }
    }

    /**
     * Die Methode verarbeitet die Aktionen nachdem der rechte Pfeil selektiert wurde.
     * D.h. die selektierten Objekte werden aus der einen Liste entfernt und zur anderen
     * Liste hinzugefgt. <p>
     * Bei den Objekten kann es sich um Langzeitzhlstellen oder Dateinamen handeln.
     */
    protected void btnPfeilRechtsWidgetSelected() {

        // Definiere die Liste von selektierten Objekten:
        IStructuredSelection selection = (IStructuredSelection) _listViewer.getSelection();
        Iterator<?>          it        = selection.iterator();

        if (_isDateiAuswahlDialog) {
            List<String> selektierteAuswahl = new ArrayList<String>();

            while (it.hasNext()) {
                selektierteAuswahl.add((String) (it.next()));
            }

            if (selektierteAuswahl.size() > 0) {

                // Entferne die selektierten Objekte aus der GUI Liste:
                _listeDateien.removeAll(selektierteAuswahl);

                // Fge die selektierten Objekte in die andere GUI Liste:
                _listeDateiAuswahl.addAll(selektierteAuswahl);
                zeichneListen();
            }
        } else {
            List<ZaehlStelle> selektierteAuswahl = new ArrayList<ZaehlStelle>();

            while (it.hasNext()) {
                selektierteAuswahl.add((ZaehlStelle) (it.next()));
            }

            if (selektierteAuswahl.size() > 0) {

                // Entferne die selektierten Objekte aus der GUI Liste:
                _listeLangZeitZaehlStelle.removeAll(selektierteAuswahl);

                // Fge die selektierten Objekte in die andere GUI Liste:
                _listeLangZeitZaehlStelleAuswahl.addAll(selektierteAuswahl);
                zeichneListen();
            }
        }
    }

    /**
     * Name der Shell wird je nach aufrufparameter (fr den Import oder fr den Export)
     * gesetzt. Beim Import handelt es sich um einen Dateiauswahldialog, beim Export um
     * eine Langzeitzhlstellen Auswahl.
     */
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);

        if (_isDateiAuswahlDialog) {
            newShell.setText("Dateiauswahldialog der Dateien auf dem FTP-Server.");
        } else {
            newShell.setText("Langzeitzhlstellen Auswahl");
        }
    }

    /*
     *  (non-Javadoc)
     * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected Control createDialogArea(Composite parent) {
        Composite  compositeAll       = (Composite) super.createDialogArea(parent);
        GridLayout compositeAllLayout = new GridLayout();

        compositeAllLayout.makeColumnsEqualWidth = true;
        compositeAll.setLayout(compositeAllLayout);
        compositeAll.layout();
        compositeAll.pack();
        {
            GridData compAllgemeinLData = new GridData();

            compAllgemeinLData.grabExcessHorizontalSpace = true;
            compAllgemeinLData.grabExcessVerticalSpace   = true;
            compAllgemeinLData.horizontalAlignment       = GridData.FILL;
            compAllgemeinLData.verticalAlignment         = GridData.FILL;

            Composite  compAllgemein       = new Composite(compositeAll, SWT.NONE);
            GridLayout compAllgemeinLayout = new GridLayout();

            compAllgemeinLayout.makeColumnsEqualWidth = true;
            compAllgemein.setLayout(compAllgemeinLayout);
            compAllgemein.setLayoutData(compAllgemeinLData);
            {
                Composite  compAuswahl       = new Composite(compAllgemein, SWT.NONE);
                GridLayout compAuswahlLayout = new GridLayout();

                compAuswahlLayout.numColumns = 3;

                GridData compAuswahlLData = new GridData();

                compAuswahlLData.grabExcessHorizontalSpace = true;
                compAuswahlLData.horizontalAlignment       = GridData.FILL;
                compAuswahlLData.grabExcessVerticalSpace   = true;
                compAuswahlLData.verticalAlignment         = GridData.FILL;
                compAuswahl.setLayoutData(compAuswahlLData);
                compAuswahl.setLayout(compAuswahlLayout);
                {}
                {
                    Label lbLangZeitZaehlStellenListe = new Label(compAuswahl, SWT.NONE);

                    lbLangZeitZaehlStellenListe.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false));

                    if (_isDateiAuswahlDialog) {
                        lbLangZeitZaehlStellenListe.setText("Dateien");
                    } else {
                        lbLangZeitZaehlStellenListe.setText("Langzeitzhlstellen");
                    }

                    lbLangZeitZaehlStellenListe.setAlignment(SWT.CENTER);
                }
                new Label(compAuswahl, SWT.NONE);
                {
                    Label lbLangZeitZaehlStellenAuswahl = new Label(compAuswahl, SWT.NONE);

                    lbLangZeitZaehlStellenAuswahl.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false));

                    if (_isDateiAuswahlDialog) {
                        lbLangZeitZaehlStellenAuswahl.setText("Ausgewhlte Dateien");
                    } else {
                        lbLangZeitZaehlStellenAuswahl.setText("Ausgewhlte Langzeitzhlstellen");
                    }

                    lbLangZeitZaehlStellenAuswahl.setAlignment(SWT.CENTER);
                }
                {
                    GridData listViewerProjekteLData = new GridData(SWT.FILL, SWT.FILL, true, true);

                    listViewerProjekteLData.widthHint = 800;
                    {
                        _txtFilterLinks = new Text(compAuswahl, SWT.BORDER);
                        _txtFilterLinks.addModifyListener(new ModifyListener() {
                            public void modifyText(ModifyEvent e) {
                                zeichneListen();
                            }
                        });
                        _txtFilterLinks.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
                    }
                    new Label(compAuswahl, SWT.NONE);
                    new Label(compAuswahl, SWT.NONE);
                    _listViewer = new ListViewer(compAuswahl, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL | SWT.MULTI);

                    org.eclipse.swt.widgets.List list = _listViewer.getList();

                    list.addMouseListener(new MouseAdapter() {
                        @Override
                        public void mouseDoubleClick(MouseEvent e) {
                            btnPfeilRechtsWidgetSelected();
                        }
                    });
                    _listViewer.getControl().setLayoutData(listViewerProjekteLData);
                    _listViewer.setContentProvider(new ArrayContentProvider());
                    _listViewer.setLabelProvider(new ListLabelProvider());
                }
                {
                    Composite compBtnPfeile = new Composite(compAuswahl, SWT.NONE);

                    compBtnPfeile.setLayout(new GridLayout(1, false));
                    {
                        Button btnPfeilRechts = new Button(compBtnPfeile, SWT.PUSH | SWT.CENTER);

                        btnPfeilRechts.setImage(KonstantenGUIResource.ICON_VEW_PFEIL_RECHTS);
                        btnPfeilRechts.addSelectionListener(new SelectionAdapter() {
                            public void widgetSelected(SelectionEvent evt) {
                                btnPfeilRechtsWidgetSelected();
                            }
                        });
                    }
                    {
                        Button btnPfeilLinks = new Button(compBtnPfeile, SWT.NONE);

                        btnPfeilLinks.setImage(KonstantenGUIResource.ICON_VEW_PFEIL_LINKS);
                        btnPfeilLinks.addSelectionListener(new SelectionAdapter() {
                            @Override
                            public void widgetSelected(SelectionEvent e) {
                                btnPfeilLinksWidgetSelected();
                            }
                        });
                    }
                }
                {
                    GridData listViewerLangZeitZaehlStellenAuswahlLData = new GridData(SWT.FILL, SWT.FILL, true, true);

                    listViewerLangZeitZaehlStellenAuswahlLData.widthHint = 800;
                    _listViewerAuswahl                                   = new ListViewer(compAuswahl, SWT.BORDER | SWT.V_SCROLL | SWT.H_SCROLL | SWT.MULTI);

                    org.eclipse.swt.widgets.List list = _listViewerAuswahl.getList();

                    list.addMouseListener(new MouseAdapter() {
                        @Override
                        public void mouseDoubleClick(MouseEvent e) {
                            btnPfeilLinksWidgetSelected();
                        }
                    });
                    _listViewerAuswahl.getControl().setLayoutData(listViewerLangZeitZaehlStellenAuswahlLData);
                    _listViewerAuswahl.setContentProvider(new ArrayContentProvider());
                    _listViewerAuswahl.setLabelProvider(new ListLabelProvider());
                }
            }
        }
        zeichneListen();

        return compositeAll;
    }

    /**
     * Definiert den Input fr die linke Liste =
     * Array von LangZeitZaehlStellen oder von Dateinamen.
     */
    private void definiereLinkeInputListe() {
        if (_isDateiAuswahlDialog) {
            _listeDateien.clear();
            _listeDateien.addAll(_listeAllerDateien);
            _listeDateien.removeAll(_listeDateiAuswahl);
        } else {
            _listeLangZeitZaehlStelle.clear();
            _listeLangZeitZaehlStelle.addAll(_listeAllerLangZeitZaehlStellenFuerModus);
            _listeLangZeitZaehlStelle.removeAll(_listeLangZeitZaehlStelleAuswahl);
        }
    }

    //~ GET METHODEN ==========================================================

    /**
     * Return the initial size of the dialog
     */
    @Override
    protected Point getInitialSize() {
        return new Point(1000, 800);
    }

    /**
     * Getter fr Eigenschaft 'listeDateiAuswahl'.
     *
     * @return Wert der Eigenschaft 'listeDateiAuswahl'.
     *
     * @see #_listeDateiAuswahl
     */
    public List<String> getListeDateiAuswahl() {
        return _listeDateiAuswahl;
    }

    /**
     * Getter fr Eigenschaft 'listeLangZeitZaehlStellenAuswahl'.
     * @return Liste der Zhlstellen.
     */
    public List<ZaehlStelle> getListeLangZeitZaehlStellenAuswahl() {
        return _listeLangZeitZaehlStelleAuswahl;
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisierung der Listen fr den Dateiauswahldialog.
     */
    private void initialisiereListeDateien() {
        _listeDateien.clear();
        _listeAllerDateien.addAll(BaumVerwaltung.getInstanz().getListeAllerImportDateienFTPServer());
        _listeDateien.addAll(_listeAllerDateien);
    }

    /**
     * Die Methode initialisiert die Liste der LangzeitzaehlStellen.
     * @param modus Modus.
     */
    private void initialisiereListeLzz(ZS_MODUS modus) {
        if (modus == null) {
            _listeLangZeitZaehlStelle.addAll(DatenVew.getInstanz().getListeZaehlStellen());
        } else {

            // Langzeitzhlstellen ohne Achslastzhlstellen.
            if (modus == ZS_MODUS.LZZ) {
                _listeLangZeitZaehlStelle.addAll(DatenVew.getInstanz().getListeZaehlStellen());
            } else if (modus == ZS_MODUS.AXL) {  // nur Achslastzhlstellen.
                _listeLangZeitZaehlStelle.addAll(BaumVerwaltung.getInstanz().getAchslastZaehlStellen());
            }
        }

        _listeAllerLangZeitZaehlStellenFuerModus.addAll(_listeLangZeitZaehlStelle);
    }

    /**
     * Selektion der LangZeitZaehlStellen.
     * @param listeVorselektierteLangZeitZaehlStellen LangZeitZaehlStellen.
     */
    public void selektiereLangZeitZaehlStellen(List<ZaehlStelle> listeVorselektierteLangZeitZaehlStellen) {
        if (listeVorselektierteLangZeitZaehlStellen.size() > 0) {
            _listeLangZeitZaehlStelleAuswahl.addAll(listeVorselektierteLangZeitZaehlStellen);

            for (int i = 0; i < listeVorselektierteLangZeitZaehlStellen.size(); i++) {
                _listeLangZeitZaehlStelle.remove(listeVorselektierteLangZeitZaehlStellen.get(i));
            }
        }
    }

    /**
     * Selektion der Dateien.
     * @param listeVorselektierterDateien Dateinamen.
     */
    public void selektierteDateien(List<String> listeVorselektierterDateien) {
        _listeDateiAuswahl.clear();

        if (listeVorselektierterDateien.size() > 0) {
            _listeDateiAuswahl.addAll(listeVorselektierterDateien);

            for (int i = 0; i < listeVorselektierterDateien.size(); i++) {
                _listeDateien.remove(listeVorselektierterDateien.get(i));
            }
        }
    }

    //~ SET METHODEN ==========================================================

    /**
     *     Setter fr Eigenschaft 'listeAllerDateien'.
     *
     *     @param listeAllerDateien Wert zum Setzen der Eigenschaft 'listeAllerDateien'.
     *
     *     @see #_listeAllerDateien
     */
    public void setListeAllerDateien(List<String> listeAllerDateien) {
        _listeAllerDateien = listeAllerDateien;
    }

    /**
     *     Setter fr Eigenschaft 'listeDateien'.
     *
     *     @param listeDateien Wert zum Setzen der Eigenschaft 'listeDateien'.
     *
     *     @see #_listeDateien
     */
    public void setListeDateien(List<String> listeDateien) {
        _listeDateien = listeDateien;
    }

    //~ METHODEN ==============================================================

    /**
     * Zeichnen der beiden Listen von LangZeitZaehlStellen oder von Dateinamen.
     */
    private void zeichneListen() {
        String filterText = _txtFilterLinks.getText();

        if (_isDateiAuswahlDialog) {

            // Falls Filter leer ist, setze alle Elemente:
            if (filterText.length() == 0) {
                _listViewer.setInput(_listeDateien);
            }

            // Filtere die Elemente:
            else {
                StringMatcher filter         = new StringMatcher("*" + filterText.replaceAll("\\.", "?").replaceAll("_", "?") + "*", true, false);
                List<String>  listeGefiltert = new ArrayList<String>();

                for (String d : _listeDateien) {
                    if (filter.match(d)) {
                        listeGefiltert.add(d);
                    }
                }

                // Setze die gefilterten Elemente:
                _listViewer.setInput(listeGefiltert);
            }

            //
            _listViewerAuswahl.setInput(_listeDateiAuswahl);
        } else {

            // Falls Filter leer ist, setze alle Elemente:
            if (filterText.length() == 0) {
                _listViewer.setInput(_listeLangZeitZaehlStelle);
            }

            // Filtere die Elemente:
            else {
                StringMatcher     filter         = new StringMatcher("*" + filterText.replaceAll("\\.", "?").replaceAll("_", "?") + "*", true, false);
                List<ZaehlStelle> listeGefiltert = new ArrayList<ZaehlStelle>();

                for (ZaehlStelle zs : _listeLangZeitZaehlStelle) {
                    if (filter.match(zs.getSystemObjekt().getName())) {
                        listeGefiltert.add(zs);
                    }
                }

                // Setze die gefilterten Elemente:
                _listViewer.setInput(listeGefiltert);
            }

            //
            _listViewerAuswahl.setInput(_listeLangZeitZaehlStelleAuswahl);
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 12.09.12
