/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.tabellen;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.tabellen.Sortierung;
import de.bsvrz.buv.rw.basislib.tabellen.SpaltenElement;
import de.bsvrz.buv.rw.basislib.tabellen.TabellenDatenStruktur;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Die Klasse beschreibt ein Tabellenobjekt.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class TabellenObjekt {

    /** Konstante die fr die Erzeugung der TabellenZeilen im Archivmodus bentigt wird. */
    private static final int ARCHIV_MODUS = 4;

    /** Konsante die fr die Erzeugung der Tabellenzeilen im normalmodus bentigt wird. */
    private static final int NORMAL_MODUS = 3;

    /** Konstante Spaltenbreite. */
    private static final int SPALTENBREITE = 100;

    /** Konstante Spaltenbreite Art. */
    private static final int SPALTENBREITE_ART = 33;

    //~ FELDER ================================================================

    private final String[] _archivSpaltenAuswahl = { "Art", "Datenzeit", "Archivzeit", "Objekt" };

    /** Anzahl der Filterspalten. */
    private int            _filterSpaltenAnzahl  = 3;
    private final String[] _normalSpaltenAuswahl = { "Art", "Datenzeit", "Objekt" };

    /** Menge der gewhlten Objekte. */
    private List<String> _gewaehlteObj = new ArrayList<String>();

    /** Liste aller Spaltennamen. */
    private List<String> _spaltenNamen = new ArrayList<String>();

    /** Liste der Typen der Spaltennamen. */
    private List<String> _spaltenNamenTyp = new ArrayList<String>();

    /** Aspekt. */
    private String _aspPid;

    /** Attributgruppe. */
    private String _atgPid;

    /** Die tatschliche Spaltenzahl der Tabelle */
    private int _spaltenZahl;

    /** Spaltenobjekt. */
    private SpaltenObjekt _spo;

    /** Tabellendatenstruktur. */
    private TabellenDatenStruktur _tds;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    public TabellenObjekt() {}

    /**
     * Speichert die Objekte zur Reprsentation von SystmeObjekten.
     * @param atgPid
     *          Pid Attributgruppe.
     * @param aspPid
     *          Pid Aspekt.
     * @param listePidNrGewaehlterKnotenImBaum
     *          Menge der Objekte eines Tabellenobjektes.
     */
    public TabellenObjekt(String atgPid, String aspPid, List<String> listePidNrGewaehlterKnotenImBaum) {
        _atgPid       = atgPid;
        _aspPid       = aspPid;
        _gewaehlteObj = listePidNrGewaehlterKnotenImBaum;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }

        if (obj == null) {
            return false;
        }

        if (!(obj instanceof TabellenObjekt)) {
            return false;
        }

        TabellenObjekt other = (TabellenObjekt) obj;

        if (_aspPid == null) {
            if (other._aspPid != null) {
                return false;
            }
        } else if (!_aspPid.equals(other._aspPid)) {
            return false;
        }

        if (_atgPid == null) {
            if (other._atgPid != null) {
                return false;
            }
        } else if (!_atgPid.equals(other._atgPid)) {
            return false;
        }

        if (_filterSpaltenAnzahl != other._filterSpaltenAnzahl) {
            return false;
        }

        if (_gewaehlteObj == null) {
            if (other._gewaehlteObj != null) {
                return false;
            }
        } else if (!_gewaehlteObj.equals(other._gewaehlteObj)) {
            return false;
        }

        if (_spaltenNamen == null) {
            if (other._spaltenNamen != null) {
                return false;
            }
        } else if (!_spaltenNamen.equals(other._spaltenNamen)) {
            return false;
        }

        if (_spaltenZahl != other._spaltenZahl) {
            return false;
        }

        if (_spo == null) {
            if (other._spo != null) {
                return false;
            }
        } else if (!_spo.equals(other._spo)) {
            return false;
        }

        if (_tds == null) {
            if (other._tds != null) {
                return false;
            }
        } else if (!_tds.equals(other._tds)) {
            return false;
        }

        return true;
    }

    /**
     * Erzeugt ein Systemobjekt, das eine Tabellenzeile reprsentiert.
     * @param art
     *      Die Art des Objektes.
     * @param date
     *      Das Datum.
     * @param archivDatenZeit   Archivdatenzeit.
     * @param attList
     *      Attributliste.
     * @param dataName
     *      Name des Objektes.
     * @param isArchivAnfrage  boolesche Variable die angibt, ob es sich um eine Archivanfrage handelt.
     * @return  ZeilenObjekt.
     */
    public ZeilenObjekt erzeugeSystemObjekt(String art, long date, long archivDatenZeit, List<String> attList, String dataName, boolean isArchivAnfrage) {
        ZeilenObjekt zeilenObjekt;
        int          zaehler;
        int          spaltenZahl;

        if (isArchivAnfrage) {
            spaltenZahl = getSpaltenNamen().size() + ARCHIV_MODUS;
        } else {
            spaltenZahl = getSpaltenNamen().size() + NORMAL_MODUS;
        }

        Object[] ar_zeilenObjektInhalt = new Object[spaltenZahl];

        ar_zeilenObjektInhalt[0] = art;

        // ar_zeilenObjektInhalt[1] = DatumFunktionen.getDatumZeitAlsString(date);
        ar_zeilenObjektInhalt[1] = new DatumObjekt(date);

        if (isArchivAnfrage) {

            // ar_zeilenObjektInhalt[2] = DatumFunktionen.getDatumZeitAlsString(archivDatenZeit);
            ar_zeilenObjektInhalt[2] = new DatumObjekt(archivDatenZeit);
            ar_zeilenObjektInhalt[3] = dataName;
            zaehler                  = ARCHIV_MODUS;
        } else {
            ar_zeilenObjektInhalt[2] = dataName;
            zaehler                  = NORMAL_MODUS;
        }

        try {
            for (String attribut : attList) {
                if (zaehler < spaltenZahl) {
                    ar_zeilenObjektInhalt[zaehler] = attribut;
                    zaehler++;
                } else {
                    break;
                }
            }

            if (attList.size() < getSpaltenNamen().size()) {
                while (zaehler < spaltenZahl) {
                    ar_zeilenObjektInhalt[zaehler] = "";
                    zaehler++;
                }
            }
        }
        catch (Exception e) {
            System.out.println("Probleme beim Fllen der Zeilenobjekte mit dem Attributlisteninhalt. " + e);
        }

        zeilenObjekt = new ZeilenObjekt(dataName, ar_zeilenObjektInhalt);

        return zeilenObjekt;
    }

    /**
     * Um die TabellenDatenstruktur festzulegen mssen alle SpaltenElemente bekannt sein,
     * die als Spaltenberschriften dienen, wie auch der Inhalt der Daten.
     * 1. Bestimme die Spaltenberschriften
     * 2. Melde die Klasse an, so dass sie ber nderungen informiert wird.
     *
     * @param anzahlVarAtt Anzahl der variablen Attribute.
     * @param isArchivAnfrage    boolesche Variable die angibt ob es sich um eine Archivanfrage handelt.
     * @return TabellenDatenStruktur.
     */
    public TabellenDatenStruktur erzeugeTabellenDatenStruktur(int anzahlVarAtt, boolean isArchivAnfrage) {
        List<String> spaltenNamen = new ArrayList<String>();
        int          zaehler;

        _spaltenNamen = spaltenNamen;

        // Fest bei der Erzeugung der Tabellenspalten sind nur Art, Zeit und Objekt, alle weiteren
        // Spalten ergeben sich je nach Datensatz.
        _spo             = new SpaltenObjekt(anzahlVarAtt);
        _spaltenNamen    = _spo.getSpaltenNamen(_atgPid);
        _spaltenNamenTyp = _spo.getSpaltenNamenTyp();

        if (!(_spaltenNamen.isEmpty())) {
            if (isArchivAnfrage) {
                _spaltenZahl = _spaltenNamen.size() + ARCHIV_MODUS;
            } else {
                _spaltenZahl = _spaltenNamen.size() + NORMAL_MODUS;
            }

            SpaltenElement[] spaltenTitel   = new SpaltenElement[_spaltenZahl];
            int[]            spaltenBreiten = new int[_spaltenZahl];
            Sortierung[]     sortierung     = new Sortierung[_spaltenZahl];

            // Die ersten drei Spalten sind fest vorbelegt, mit Art, Zeit und dem Objektnamen
            spaltenTitel[0]   = new SpaltenElement(_normalSpaltenAuswahl[0]);
            spaltenBreiten[0] = SPALTENBREITE_ART;
            sortierung[0]     = new Sortierung(0);

            //
            spaltenTitel[1]   = new SpaltenElement(_normalSpaltenAuswahl[1]);
            spaltenBreiten[1] = SPALTENBREITE;
            sortierung[1]     = new Sortierung(1);

            //
            if (isArchivAnfrage) {
                spaltenTitel[2]   = new SpaltenElement(_archivSpaltenAuswahl[2]);
                spaltenBreiten[2] = SPALTENBREITE;
                sortierung[2]     = new Sortierung(2);
                spaltenTitel[3]   = new SpaltenElement(_archivSpaltenAuswahl[3]);
                spaltenBreiten[3] = SPALTENBREITE;
                sortierung[3]     = new Sortierung(3);
                zaehler           = ARCHIV_MODUS;
            } else {
                spaltenTitel[2]   = new SpaltenElement(_normalSpaltenAuswahl[2]);
                spaltenBreiten[2] = SPALTENBREITE;
                sortierung[2]     = new Sortierung(2);
                zaehler           = NORMAL_MODUS;
            }

            for (String spN : _spaltenNamen) {
                spaltenTitel[zaehler]   = new SpaltenElement(spN);
                spaltenBreiten[zaehler] = SPALTENBREITE;
                sortierung[zaehler]     = new Sortierung(zaehler);
                zaehler++;
            }

            _tds = new TabellenDatenStruktur(_spaltenZahl);
            _tds.setSpaltenTitel(spaltenTitel);
            _tds.setSpaltenBreiten(spaltenBreiten);
            _tds.setSortierung(sortierung);
        }

        return _tds;
    }

    /**
     * Die Methode erzeugeTabellenDatenStrukturFilter erzeugt die Tabellendatenstruktur fr den Filter.
     * Dabei ist das erste Spaltenelement der Index, das Zweite der Titel der Spalte, das dritte Spaltenelement
     * entspricht dem Regulren Ausdruck, der auf die Spalte angewendet werden soll.
     * @return TabellenDatenStruktur.
     */
    public TabellenDatenStruktur erzeugeTabellenDatenStrukturFilter() {
        SpaltenElement[] spaltenTitelFilter   = new SpaltenElement[_filterSpaltenAnzahl];
        int[]            spaltenBreitenFilter = new int[_filterSpaltenAnzahl];
        Sortierung[]     sortierungFilter     = new Sortierung[_filterSpaltenAnzahl];

        // Die drei Spalten sind fest vorbelegt.
        spaltenTitelFilter[0]   = new SpaltenElement("Idx");
        spaltenBreitenFilter[0] = SPALTENBREITE_ART;
        sortierungFilter[0]     = new Sortierung(0);

        //
        spaltenTitelFilter[1]   = new SpaltenElement("Titel der Spalte");
        spaltenBreitenFilter[1] = SPALTENBREITE;
        sortierungFilter[1]     = new Sortierung(1);

        //
        spaltenTitelFilter[2]   = new SpaltenElement("Regulrer Ausdruck");
        spaltenBreitenFilter[2] = SPALTENBREITE;
        sortierungFilter[2]     = new Sortierung(2);
        _tds                    = new TabellenDatenStruktur(_filterSpaltenAnzahl);
        _tds.setSpaltenTitel(spaltenTitelFilter);
        _tds.setSpaltenBreiten(spaltenBreitenFilter);
        _tds.setSortierung(sortierungFilter);

        return _tds;
    }

    /**
     * Die Methode erzeugt ein Zeilenobjekt aus den bergebenen Parametern.
     * @param idx Index des Objektes.
     * @param spName Spaltenname.
     * @param regEx Regulre Ausdruck.
     * @return Zeilenobjekt.
     */
    public ZeilenObjekt erzeugeZeilenObjekt(int idx, String spName, String regEx) {
        ZeilenObjekt zeilenObjekt;
        Object[]     attribute = new Object[3];

        attribute[0] = idx;
        attribute[1] = spName;
        attribute[2] = regEx;
        zeilenObjekt = new ZeilenObjekt(spName, attribute);

        return zeilenObjekt;
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'spaltenNamen'.
     *
     * @return Wert der Eigenschaft 'spaltenNamen'.
     *
     * @see #_spaltenNamen
     */
    public List<String> getSpaltenNamen() {
        return _spaltenNamen;
    }

    /**
     *     Getter fr Eigenschaft 'spaltenNamenTyp'.
     *
     *     @return Wert der Eigenschaft 'spaltenNamenTyp'.
     *
     *     @see #_spaltenNamenTyp
     */
    public List<String> getSpaltenNamenTyp() {
        return _spaltenNamenTyp;
    }

    /**
     *     Getter fr Eigenschaft 'spo'.
     *
     *     @return Wert der Eigenschaft 'spo'.
     *
     *     @see #_spo
     */
    public SpaltenObjekt getSpo() {
        return _spo;
    }

    //~ METHODEN ==============================================================

    /*
     *  (non-Javadoc)
     *     @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final int prime  = 31;
        int       result = 1;

        result = prime * result + ((_aspPid == null) ? 0 : _aspPid.hashCode());
        result = prime * result + ((_atgPid == null) ? 0 : _atgPid.hashCode());
        result = prime * result + _filterSpaltenAnzahl;
        result = prime * result + ((_gewaehlteObj == null) ? 0 : _gewaehlteObj.hashCode());
        result = prime * result + ((_spaltenNamen == null) ? 0 : _spaltenNamen.hashCode());
        result = prime * result + _spaltenZahl;
        result = prime * result + ((_spo == null) ? 0 : _spo.hashCode());
        result = prime * result + ((_tds == null) ? 0 : _tds.hashCode());

        return result;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 12.09.12
