/*
 * @(#)GanglinienGUI.java   17.09.10
 *
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.visualisierung.ganglinien;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.kalender.DatumZeit;

import de.inovat.buv.inovat.lib.funktionen.datum.DatumFunktionen;

import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.ScrolledForm;
import org.eclipse.ui.forms.widgets.Section;

//~ JDK IMPORTE ===============================================================

import java.sql.Date;

//~ KLASSEN ===================================================================

/**
 * GUI fr die Sicht 'Ganglinien'.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 238 $ / $Date: 2012-10-23 16:56:21 +0200 (Di, 23 Okt 2012) $ / ($Author: LG $)
 */
public class GanglinienGUI extends Composite {

    /** FormToolkit. */
    private final FormToolkit _toolkit                   = new FormToolkit(Display.getCurrent());
    private DatumZeit[]       _arDzDatum                 = new DatumZeit[GanglinienGUIVew.ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS];
    private Button[]          _arCbtnDarzustellendeWerte = new Button[GanglinienGUIVew.ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS];
    private Button            _btnAktualisieren;
    private Button            _btnSpeichern;
    private Button            _cbtnQKfz;
    private Button            _cbtnQSgv;
    private Button            _cbtnQWerteManuell;

    /** Grafische Darstellung. */
    private GanglinienGrafik _grafik;
    private Group            _grpDarstellungsWerte;
    private Group            _grpZs;

    /** Instanz GUI-Verwaltung. */
    private GanglinienGUIVew   _guiVew;
    private Label              _lbDarstellungInfo;
    private Label              _lbQMax;
    private Label              _lbQMin;
    private Button             _rbtnJahrMq;
    private Button             _rbtnJahrZs;
    private Button             _rbtnMonatMq;
    private Button             _rbtnMonatZs;
    private Button             _rbtnTagBast;
    private Button             _rbtnTagTls;
    private Spinner            _spinnerQMax;
    private Spinner            _spinnerQMin;
    private CheckboxTreeViewer _treeViewerZs;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse GanglinienGUI.
     *
     * @param parent Parent-Composite.
     */
    public GanglinienGUI(Composite parent) {
        super(parent, SWT.NONE);

        //
        _guiVew = new GanglinienGUIVew(this);

        //
        _toolkit.adapt(this);
        _toolkit.paintBordersFor(this);

        //
        initGUI();

        //
        _guiVew.initialisiereGui();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Getter fr Eigenschaft 'arCbtnDarzustellendeWerte'.
     *
     * @return Wert der Eigenschaft 'arCbtnDarzustellendeWerte'.
     *
     * @see #_arCbtnDarzustellendeWerte
     */
    public Button[] getArCbtnDarzustellendeWerte() {
        return _arCbtnDarzustellendeWerte;
    }

    /**
     * Getter fr Eigenschaft 'arDzDatum'.
     *
     * @return Wert der Eigenschaft 'arDzDatum'.
     *
     * @see #_arDzDatum
     */
    public DatumZeit[] getArDzDatum() {
        return _arDzDatum;
    }

    /**
     * Getter fr Eigenschaft 'btnAktualisieren'.
     *
     * @return Wert der Eigenschaft 'btnAktualisieren'.
     *
     * @see #_btnAktualisieren
     */
    public Button getBtnAktualisieren() {
        return _btnAktualisieren;
    }

    /**
     * Getter fr Eigenschaft 'cbtnQKfz'.
     *
     * @return Wert der Eigenschaft 'cbtnQKfz'.
     *
     * @see #_cbtnQKfz
     */
    public Button getCbtnQKfz() {
        return _cbtnQKfz;
    }

    /**
     * Getter fr Eigenschaft 'cbtnQSgv'.
     *
     * @return Wert der Eigenschaft 'cbtnQSgv'.
     *
     * @see #_cbtnQSgv
     */
    public Button getCbtnQSgv() {
        return _cbtnQSgv;
    }

    /**
     * Getter fr Eigenschaft 'cbtnQWerteManuell'.
     *
     * @return Wert der Eigenschaft 'cbtnQWerteManuell'.
     *
     * @see #_cbtnQWerteManuell
     */
    public Button getCbtnQWerteManuell() {
        return _cbtnQWerteManuell;
    }

    /**
     * Getter fr Eigenschaft 'grafik'.
     *
     * @return Wert der Eigenschaft 'grafik'.
     *
     * @see #_grafik
     */
    public GanglinienGrafik getGrafik() {
        return _grafik;
    }

    /**
     * Getter fr Eigenschaft 'grpDarstellungsWerte'.
     *
     * @return Wert der Eigenschaft 'grpDarstellungsWerte'.
     *
     * @see #_grpDarstellungsWerte
     */
    public Group getGrpDarstellungsWerte() {
        return _grpDarstellungsWerte;
    }

    /**
     * Getter fr Eigenschaft 'grpZs'.
     *
     * @return Wert der Eigenschaft 'grpZs'.
     *
     * @see #_grpZs
     */
    public Group getGrpZs() {
        return _grpZs;
    }

    /**
     *     Getter fr Eigenschaft 'lbDarstellungInfo'.
     *
     *     @return Wert der Eigenschaft 'lbDarstellungInfo'.
     *
     *     @see #_lbDarstellungInfo
     */
    public Label getLbDarstellungInfo() {
        return _lbDarstellungInfo;
    }

    /**
     * Getter fr Eigenschaft 'lbQMax'.
     *
     * @return Wert der Eigenschaft 'lbQMax'.
     *
     * @see #_lbQMax
     */
    public Label getLbQMax() {
        return _lbQMax;
    }

    /**
     * Getter fr Eigenschaft 'lbQMin'.
     *
     * @return Wert der Eigenschaft 'lbQMin'.
     *
     * @see #_lbQMin
     */
    public Label getLbQMin() {
        return _lbQMin;
    }

    /**
     * Getter fr Eigenschaft 'rbtnJahrMq'.
     *
     * @return Wert der Eigenschaft 'rbtnJahrMq'.
     *
     * @see #_rbtnJahrMq
     */
    public Button getRbtnJahrMq() {
        return _rbtnJahrMq;
    }

    /**
     * Getter fr Eigenschaft 'rbtnJahrZs'.
     *
     * @return Wert der Eigenschaft 'rbtnJahrZs'.
     *
     * @see #_rbtnJahrZs
     */
    public Button getRbtnJahrZs() {
        return _rbtnJahrZs;
    }

    /**
     * Getter fr Eigenschaft 'rbtnMonatMq'.
     *
     * @return Wert der Eigenschaft 'rbtnMonatMq'.
     *
     * @see #_rbtnMonatMq
     */
    public Button getRbtnMonatMq() {
        return _rbtnMonatMq;
    }

    /**
     *     Getter fr Eigenschaft 'rbtnMonatZs'.
     *
     *     @return Wert der Eigenschaft 'rbtnMonatZs'.
     *
     *     @see #_rbtnMonatZs
     */
    public Button getRbtnMonatZs() {
        return _rbtnMonatZs;
    }

    /**
     * Getter fr Eigenschaft 'rbtnTagBast'.
     *
     * @return Wert der Eigenschaft 'rbtnTagBast'.
     *
     * @see #_rbtnTagBast
     */
    public Button getRbtnTagBast() {
        return _rbtnTagBast;
    }

    /**
     *     Getter fr Eigenschaft 'rbtnTagTls'.
     *
     *     @return Wert der Eigenschaft 'rbtnTagTls'.
     *
     *     @see #_rbtnTagTls
     */
    public Button getRbtnTagTls() {
        return _rbtnTagTls;
    }

    /**
     * Getter fr Eigenschaft 'spinnerQMax'.
     *
     * @return Wert der Eigenschaft 'spinnerQMax'.
     *
     * @see #_spinnerQMax
     */
    public Spinner getSpinnerQMax() {
        return _spinnerQMax;
    }

    /**
     * Getter fr Eigenschaft 'spinnerQMin'.
     *
     * @return Wert der Eigenschaft 'spinnerQMin'.
     *
     * @see #_spinnerQMin
     */
    public Spinner getSpinnerQMin() {
        return _spinnerQMin;
    }

    /**
     * Getter fr Eigenschaft 'toolkit'.
     *
     * @return Wert der Eigenschaft 'toolkit'.
     *
     * @see #_toolkit
     */
    public FormToolkit getToolkit() {
        return _toolkit;
    }

    /**
     * Getter fr Eigenschaft 'treeViewerZs'.
     *
     * @return Wert der Eigenschaft 'treeViewerZs'.
     *
     * @see #_treeViewerZs
     */
    public CheckboxTreeViewer getTreeViewerZs() {
        return _treeViewerZs;
    }

    //~ METHODEN ==============================================================

    /**
     * Initialisiere das GUI.
     */
    private void initGUI() {
        GridLayout gridLayout0 = new GridLayout(1, false);

        gridLayout0.marginWidth  = 0;
        gridLayout0.marginHeight = 0;
        setLayout(gridLayout0);

        //
        ScrolledComposite scompAll = new ScrolledComposite(this, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);

        scompAll.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        scompAll.setExpandHorizontal(true);
        scompAll.setExpandVertical(true);

        Composite  compAll    = new Composite(scompAll, SWT.NONE);
        GridLayout gl_compAll = new GridLayout(1, false);

        gl_compAll.marginWidth  = 0;
        gl_compAll.marginHeight = 0;
        compAll.setLayout(gl_compAll);

        ScrolledForm scrolledForm = _toolkit.createScrolledForm(compAll);

        scrolledForm.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        _toolkit.paintBordersFor(scrolledForm);
        scrolledForm.getBody().setLayout(new GridLayout(1, false));

        //
        Section sectionAuswahl = _toolkit.createSection(scrolledForm.getBody(), Section.TWISTIE | Section.TITLE_BAR);

        sectionAuswahl.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        _toolkit.paintBordersFor(sectionAuswahl);
        sectionAuswahl.setText("Einstellungen");

        Composite compAuswahl = _toolkit.createComposite(sectionAuswahl, SWT.NONE);

        _toolkit.paintBordersFor(compAuswahl);
        sectionAuswahl.setClient(compAuswahl);
        compAuswahl.setLayout(new GridLayout(6, false));

        Group grpGanglinienArt = new Group(compAuswahl, SWT.NONE);

        grpGanglinienArt.setText("Ganglinienart");
        grpGanglinienArt.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false, 1, 1));
        _toolkit.adapt(grpGanglinienArt);
        _toolkit.paintBordersFor(grpGanglinienArt);

        GridLayout gl_grpGanglinienArt = new GridLayout(1, false);

        gl_grpGanglinienArt.verticalSpacing = 9;
        gl_grpGanglinienArt.marginHeight    = 10;
        grpGanglinienArt.setLayout(gl_grpGanglinienArt);

        //
        _rbtnTagTls = _toolkit.createButton(grpGanglinienArt, "Erfasste Daten (FS/Tag)", SWT.RADIO);
        _rbtnTagTls.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnGanglinienArtSelektiert();
            }
        });
        _rbtnTagTls.setToolTipText("Erfasste Stundenintervalle");

        //
        _rbtnTagBast = _toolkit.createButton(grpGanglinienArt, "Messwertersetzte Daten (FS/Tag)", SWT.RADIO);
        _rbtnTagBast.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnGanglinienArtSelektiert();
            }
        });
        _rbtnTagBast.setToolTipText("Messwertersetzte Stundenintervalle nach BASt-Import Version 2004");

        //
        _rbtnMonatMq = _toolkit.createButton(grpGanglinienArt, "DTV Tag (MQ/Monat)", SWT.RADIO);
        _rbtnMonatMq.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnGanglinienArtSelektiert();
            }
        });

        //
        _rbtnJahrMq = _toolkit.createButton(grpGanglinienArt, "DTV Tag (MQ/Jahr)", SWT.RADIO);
        _rbtnJahrMq.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnGanglinienArtSelektiert();
            }
        });

        //
        _rbtnMonatZs = _toolkit.createButton(grpGanglinienArt, "DTV Tag (ZS/Monat)", SWT.RADIO);
        _rbtnMonatZs.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnGanglinienArtSelektiert();
            }
        });

        //
        _rbtnJahrZs = _toolkit.createButton(grpGanglinienArt, "DTV Tag (ZS/Jahr)", SWT.RADIO);
        _rbtnJahrZs.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.rbtnGanglinienArtSelektiert();
            }
        });
        _grpZs = new Group(compAuswahl, SWT.NONE);
        _grpZs.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
        _grpZs.setText("Auswahl Fahrstreifen/MQ");
        _toolkit.adapt(_grpZs);
        _toolkit.paintBordersFor(_grpZs);
        _grpZs.setLayout(new GridLayout(1, false));

        Button btnZsAuswahl = _toolkit.createButton(_grpZs, "Langzeitzhlstellenauswahl...", SWT.NONE);

        btnZsAuswahl.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
        btnZsAuswahl.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnZsAuswahlSelektiert();
            }
        });

        //
        _treeViewerZs = new CheckboxTreeViewer(_grpZs, SWT.BORDER);

        Tree     tree    = _treeViewerZs.getTree();
        GridData gd_tree = new GridData(SWT.LEFT, SWT.FILL, false, true, 1, 1);

        gd_tree.heightHint = 112;
        gd_tree.widthHint  = 282;
        tree.setLayoutData(gd_tree);
        _toolkit.paintBordersFor(tree);
        _treeViewerZs.setLabelProvider(_guiVew.new ZaehlstellenBaumLabelProvider());
        _treeViewerZs.setContentProvider(_guiVew.new ZaehlstellenBaumContentProvider());
        _treeViewerZs.addCheckStateListener(new ICheckStateListener() {
            public void checkStateChanged(CheckStateChangedEvent event) {
                _guiVew.treeViewerZsSelektiert(event);
            }
        });

        //
        Group grpZeitangabe = new Group(compAuswahl, SWT.NONE);

        grpZeitangabe.setToolTipText("Doppelklick auf der Gruppe setzt die Zeitangaben neu");
        grpZeitangabe.addMouseListener(new MouseAdapter() {
            @Override
            public void mouseDoubleClick(MouseEvent e) {
                _guiVew.grpZeitangabeMouseDoubleClick();
            }
        });
        grpZeitangabe.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
        grpZeitangabe.setText("Zeitangabe");
        _toolkit.adapt(grpZeitangabe);
        _toolkit.paintBordersFor(grpZeitangabe);
        grpZeitangabe.setLayout(new GridLayout(1, true));

        //
        long datum = DatumFunktionen.getVerschobenenDateTime(1 - GanglinienGUIVew.ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS, 0, 0, 0);

        for (int i = 0; i < GanglinienGUIVew.ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
            DatumZeit dzDatum = new DatumZeit(grpZeitangabe, SWT.BORDER, DatumZeit.Eingabetyp.datum, new Date(datum), false, false);

            dzDatum.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

            GridData gd_dzVon = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);

            gd_dzVon.widthHint = 100;
            dzDatum.setLayoutData(gd_dzVon);
            dzDatum.hinzufuegenSelektionsZuhoerer(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    _guiVew.dzDatumSelektiert();
                }
            });
            _arDzDatum[i] = dzDatum;
            datum         = DatumFunktionen.getNaechstenTag(datum);

            // Korrigiere die DatumDarstellung so, dass man imGUI das ganze Datum sieht:
            Control  cdt      = dzDatum.getChildren()[0];
            GridData gridData = new GridData();

            gridData.horizontalAlignment       = GridData.FILL;
            gridData.grabExcessHorizontalSpace = true;
            gridData.verticalAlignment         = GridData.CENTER;
            cdt.setLayoutData(gridData);
            dzDatum.pack(true);

            int breite = cdt.getSize().x;
            int hoehe  = cdt.getSize().y;

            cdt.setSize(breite + 8, hoehe);
            dzDatum.update();
        }

        //
        _btnAktualisieren = _toolkit.createButton(compAuswahl, "Aktualisieren  -->", SWT.NONE);
        _btnAktualisieren.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false, false, 1, 1));
        _btnAktualisieren.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnAktualisierenSelektiert();
            }
        });
        _grpDarstellungsWerte = new Group(compAuswahl, SWT.NONE);
        _grpDarstellungsWerte.setText("Darstellung");
        _grpDarstellungsWerte.setLayoutData(new GridData(SWT.LEFT, SWT.FILL, false, false, 1, 1));
        _toolkit.adapt(_grpDarstellungsWerte);
        _toolkit.paintBordersFor(_grpDarstellungsWerte);

        GridLayout gl__grpDarstellungsWerte = new GridLayout(1, false);

        gl__grpDarstellungsWerte.marginTop = 4;
        _grpDarstellungsWerte.setLayout(gl__grpDarstellungsWerte);
        _lbDarstellungInfo = _toolkit.createLabel(_grpDarstellungsWerte, "", SWT.NONE);

        Label lbSeparator = _toolkit.createSeparator(_grpDarstellungsWerte, SWT.HORIZONTAL);

        lbSeparator.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

        for (int i = 0; i < GanglinienGUIVew.ANZAHL_DER_TAGE_ODER_MQ_ODER_ZS; i++) {
            Button cbtn = new Button(_grpDarstellungsWerte, SWT.CHECK);

            _toolkit.adapt(cbtn, true, true);

            // cbtn.setText("Wert");
            cbtn.addSelectionListener(new SelectionAdapter() {
                @Override
                public void widgetSelected(SelectionEvent e) {
                    _guiVew.cbtnDarstellungsWertSelektiert();
                }
            });
            _arCbtnDarzustellendeWerte[i] = cbtn;
        }

        //
        Group grpQWerte = new Group(compAuswahl, SWT.NONE);

        grpQWerte.setText("Darzustellende Q-Werte");
        grpQWerte.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, false, 1, 1));
        _toolkit.adapt(grpQWerte);
        _toolkit.paintBordersFor(grpQWerte);

        GridLayout gl_grpQWerte = new GridLayout(2, false);

        gl_grpQWerte.marginHeight = 10;
        grpQWerte.setLayout(gl_grpQWerte);
        _cbtnQKfz = _toolkit.createButton(grpQWerte, "Q KFZ", SWT.CHECK);
        _cbtnQKfz.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
        _cbtnQKfz.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.cbtnQKfzSelektiert();
            }
        });
        _cbtnQSgv = _toolkit.createButton(grpQWerte, "Q SGV", SWT.CHECK);
        _cbtnQSgv.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
        _cbtnQSgv.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.cbtnQSgvSelektiert();
            }
        });

        Label lbSeparator2 = _toolkit.createSeparator(grpQWerte, SWT.HORIZONTAL);

        lbSeparator2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        _cbtnQWerteManuell = _toolkit.createButton(grpQWerte, "Grenzwerte manuell einstellen", SWT.CHECK);
        _cbtnQWerteManuell.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.cbtnQWerteManuellSelektiert();
            }
        });
        _cbtnQWerteManuell.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
        _lbQMax      = _toolkit.createLabel(grpQWerte, "max.", SWT.NONE);
        _spinnerQMax = new Spinner(grpQWerte, SWT.BORDER | SWT.WRAP);
        _spinnerQMax.setPageIncrement(10000);
        _spinnerQMax.setIncrement(1000);
        _spinnerQMax.setMaximum(200000);
        _spinnerQMax.setSelection(10000);
        _spinnerQMax.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.spinnerQMaxSelektiert();
            }
        });
        _toolkit.adapt(_spinnerQMax);
        _toolkit.paintBordersFor(_spinnerQMax);
        _lbQMin      = _toolkit.createLabel(grpQWerte, "min.", SWT.NONE);
        _spinnerQMin = new Spinner(grpQWerte, SWT.BORDER);
        _spinnerQMin.setPageIncrement(10000);
        _spinnerQMin.setMaximum(200000);
        _spinnerQMin.setIncrement(1000);
        _spinnerQMin.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.spinnerQMinSelektiert();
            }
        });
        _toolkit.adapt(_spinnerQMin);
        _toolkit.paintBordersFor(_spinnerQMin);

        Label lbSeparator3 = _toolkit.createSeparator(grpQWerte, SWT.HORIZONTAL);

        lbSeparator3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));

        //
        sectionAuswahl.setExpanded(true);

        //
        Composite compGrafik = _toolkit.createComposite(scrolledForm.getBody(), SWT.NONE);

        compGrafik.setLayout(new GridLayout(1, false));
        compGrafik.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
        _toolkit.paintBordersFor(compGrafik);

        //
        // $hide<<$
        _grafik = new GanglinienGrafik(compGrafik, _guiVew);
        _grafik.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

        // $hide>>$
        //
        _btnSpeichern = _toolkit.createButton(compGrafik, "Speichern...", SWT.NONE);
        _btnSpeichern.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                _guiVew.btnSpeichernSelektiert();
            }
        });
        _btnSpeichern.setToolTipText("Grafik in der Datei speichern");

        //
        scompAll.setContent(compAll);
        scompAll.setMinSize(compAll.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 23.10.13
