/*
 * Copyright 2008 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kni, Kln
 * <p>
 * This file is part of de.inovat.pat.datkat2html
 * <p>
 * de.inovat.pat.datkat2html is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <p>
 * de.inovat.pat.datkat2html is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.datkat2html; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * <p>
 * Contact Information:<br>
 * inovat, Dipl.-Ing. H. C. Kni<br>
 * Klner Strae 30<br>
 * 50859 Kln<br>
 * Phone: +49 2234 4301 800<br>
 * mailto: info@inovat.de
 */
package de.inovat.pat.datkat2html;

import org.eclipse.core.runtime.*;

import java.util.List;


/**
 * Die Klasse definiert die Meldungen fr das StandartView "ErrorLog".  
 * 
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 408 $ / $Date: 2010-04-06 08:14:07 +0200 (Di, 06. Apr 2010) $ / ($Author: HCK $)
 */

public class Log {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

	/**
	 * ID vom View "ErrorLog".
	 */
	public static final String LogViewID="org.eclipse.pde.runtime.LogView";

	/**
	 * Definiert den Status "OK". Gleich 0 ist.
	 */
	public static final int OK=IStatus.OK;
	/**
	 * Definiert den Status "INFO". Gleich 1 ist.
	 */
	public static final int INFO=IStatus.INFO;
	/**
	 * Definiert den Status "WARNUNG". Gleich 2 ist.
	 */
	public static final int WARNUNG=IStatus.WARNING;
	/**
	 * Definiert den Status "FEHLER". Gleich 4 ist.
	 */
	public static final int FEHLER=IStatus.ERROR;
	/**
	 * Definiert den Status "ABBRUCH". Gleich 8 ist.
	 */
	public static final int ABBRUCH=IStatus.CANCEL;

// ------------------------------------------------------------------------
// STATISCHE METHODEN
// ------------------------------------------------------------------------

	/**
	 * Erzeugt eine MultiMeldung fr "ErrorLog" mit der Kinder.
	 * Die Kinder knnen mit der Methode add oder addAll hinzugefgt werden.
	 * Die MultiMeldung kriegt den schlimmsten Status seiner Kinder.
	 * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
	 * @param meldungText Text der Meldung.
	 * @param kinder Liste der Meldungen. Als Kind kann auch MultiMeldung sein.
	 * @return die erzeugte MultiMeldung von Typ MultiStatus.
	 */
	public static MultiStatus erzeugeMultiMeldung(String pluginID, String meldungText, List<IStatus> kinder){
		MultiStatus multiStatus=new MultiStatus(pluginID, 0, kinder.toArray(new IStatus[0]), meldungText, null);
		return multiStatus;
	}

	/**
	 * Erzeugt eine MultiMeldung fr "ErrorLog" mit der Kinder.
	 * Die Kinder knnen mit der Methode add oder addAll hinzugefgt werden.
	 * Die MultiMeldung kriegt den schlimmsten Status seiner Kinder.
	 * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
	 * @param meldungText Text der Meldung.
	 * @param exception Exception, das beim Doppelklick auf der Meldung angezeigt wird.
	 * @param kinder Liste der Meldungen. Als Kind kann auch MultiMeldung sein.
	 * @return die erzeugte MultiMeldung von Typ MultiStatus.
	 */
	public static MultiStatus erzeugeMultiMeldung(String pluginID, String meldungText, Throwable  exception, List<IStatus> kinder){
		MultiStatus multiStatus=new MultiStatus(pluginID, 0, kinder.toArray(new IStatus[0]), meldungText, exception);
		return multiStatus;
	}

	/**
	 * Zeigt die Meldungen im "ErrorLog".
	 * @param listeMeldungen Liste der Meldungen, die angezeigt werden sollen.
	 */
	public static void zeige(List<IStatus> listeMeldungen){
		ILog logger=Activator.getDefault().getLog();
		for (IStatus status: listeMeldungen){
			logger.log(status);
		}
	}

	/**
	 * Erzeugt die Meldung vom Typ Warnung, mit dem Text "Interne Systemmeldung"
	 * und Exception und zeigt diese Exception.
	 * Wird fr die interne Meldungen benutzt, die fr den Entwickler relevant sind,
	 * um mgliche Fehler nachvollziehen zu knnen (z.B. beim Exception im try-catch Block
	 * oder man kann selber eine Exception erzeugen: new Throwable(...)).
	 *
	 * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
	 * @param exception Exception, das beim Doppelklick auf der Meldung angezeigt wird.
	 */
	public static void zeigeInterneMeldung(String pluginID, Throwable  exception){
		Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, pluginID, "Interne Systemmeldung", exception));
	}

	/**
	 * Zeigt die Meldung im "ErrorLog".
	 * @param meldung die zu zeigende Meldung.
	 */
	public static void zeige(IStatus meldung){
		Activator.getDefault().getLog().log(meldung);
	}

	/**
	 * Erzeugt eine Meldung fr "ErrorLog".
	 * @param typ kann eine der Konstanten Log.OK, Log.INFO, Log.WARNUNG, Log.FEHLER, Log.ABBRUCH sein.
	 * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
	 * @param meldungText Text der Meldung.
	 * @param exception Exception, das beim Doppelklick auf der Meldung angezeigt wird.
	 * @return die erzeugte Meldung von Typ Status.
	 */
	public static IStatus erzeugeMeldung(int typ, String pluginID, String meldungText, Throwable  exception){
		IStatus status=new Status(typ, pluginID, meldungText, exception);
		return status;
	}

	/**
	 * Erzeugt die MultiMeldung mit dem Text "Interne Systemmeldung" und einem Kind der Typ Warnung
	 * mit dem im Parameter angegebenen Text und zeigt die.
	 * Wird fr die interne Meldungen benutzt, die fr Entwickler relevant sind,
	 * um mgliche Fehler nachvollziehen zu knnen (z.B. wenn man einen Text beim Fehler anzeigen will).
	 *
	 * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
	 * @param meldungText Text der Meldung von Kind.
	 */
	public static void zeigeInterneMultiMeldung(String pluginID, String meldungText){
		MultiStatus meldung= erzeugeMultiMeldung(pluginID, "Interne Systemmeldung");
		meldung.add(erzeugeMeldung(Log.WARNUNG, pluginID, meldungText));
		Log.zeige(meldung);
	}

	/**
	 * Erzeugt eine MultiMeldung ohne Kinder fr "ErrorLog".
	 * Die Kinder knnen mit der Methode add oder addAll hinzugefgt werden.
	 * Die MultiMeldung kriegt den schlimmsten Status seiner Kinder.
	 * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
	 * @param meldungText Text der Meldung.
	 * @return die erzeugte MultiMeldung von Typ MultiStatus.
	 */
	public static MultiStatus erzeugeMultiMeldung(String pluginID, String meldungText){
		MultiStatus multiStatus=new MultiStatus(pluginID, 0, meldungText, null);
		return multiStatus;
	}

	/**
	 * Erzeugt eine Meldung fr "ErrorLog".
	 * @param typ kann eine der Konstanten Log.OK, Log.INFO, Log.WARNUNG, Log.FEHLER, Log.ABBRUCH sein.
	 * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
	 * @param meldungText Text der Meldung.
	 * @return die erzeugte Meldung von Typ IStatus.
	 */
	public static IStatus erzeugeMeldung(int typ, String pluginID, String meldungText){
		IStatus status=new Status(typ, pluginID, meldungText);
		return status;
	}

	/**
	 * Erzeugt die MultiMeldung mit dem Text "Interne Systemmeldung", mit Exception und einem Kind der Typ Warnung
	 * mit dem im Parameter angegebenen Text und zeigt die.
	 * Wird fr die interne Meldungen benutzt, die fr Entwickler relevant sind,
	 * um mgliche Fehler nachvollziehen zu knnen (z.B. beim Exception im try-catch Block und
	 * wenn man einen Text beim Fehler anzeigen will).
	 *
	 * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
	 * @param meldungText Text der Meldung von Kind.
	 */
	public static void zeigeInterneMultiMeldung(String pluginID, String meldungText, Throwable  exception){
		MultiStatus meldung= erzeugeMultiMeldung(pluginID, "Interne Systemmeldung", exception);
		meldung.add(erzeugeMeldung(Log.WARNUNG, pluginID, meldungText));
		Log.zeige(meldung);
	}

	/**
	 * Erzeugt eine MultiMeldung ohne Kinder fr "ErrorLog".
	 * Die Kinder knnen mit der Methode add oder addAll hinzugefgt werden.
	 * Die MultiMeldung kriegt den schlimmsten Status seiner Kinder.
	 * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
	 * @param meldungText Text der Meldung.
	 * @param exception Exception, das beim Doppelklick auf der Meldung angezeigt wird.
	 * @return die erzeugte MultiMeldung von Typ MultiStatus.
	 */
	public static MultiStatus erzeugeMultiMeldung(String pluginID, String meldungText, Throwable  exception){
		MultiStatus multiStatus=new MultiStatus(pluginID, 0, meldungText, exception);
		return multiStatus;
	}
}
