/*
 * Copyright 2008 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kni, Kln
 * <p>
 * This file is part of de.inovat.pat.datkat2html
 * <p>
 * de.inovat.pat.datkat2html is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <p>
 * de.inovat.pat.datkat2html is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.datkat2html; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * <p>
 * Contact Information:<br>
 * inovat, Dipl.-Ing. H. C. Kni<br>
 * Klner Strae 30<br>
 * 50859 Kln<br>
 * Phone: +49 2234 4301 800<br>
 * mailto: info@inovat.de
 */
package de.inovat.pat.datkat2html.ausgabe;

import de.inovat.pat.datkat2html.ausgabe.beschreibung.*;
import org.eclipse.core.runtime.IStatus;

import java.util.List;

/**
 * Interface fr die Ausgabe des Datenkatalogs.
 * Hier definiert man das Format der Ausgabe.
 * 
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 408 $ / $Date: 2010-04-06 08:14:07 +0200 (Di, 06. Apr 2010) $ / ($Author: HCK $)
 */

public interface IAusgabe {
// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * Abschliesse die Datei fr den Konfigurationsbereich.
	 */
	public void abschliessen();

	/**
	 * Abhngigkeitsmatrix der Konfigurationsbereich.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenAbhaengigkeitsmatrixKB(List<KonfigurationsBereichBeschreibung> listeKbBeschreibungen);

	/**
	 * Ausgabe der AspektDefinitionen.
	 *
	 * @param listeAspektDefBeschreibung Liste der AspektDefinitionBeschreibungen.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenAspektDefBeschreibungListe(List<AspektDefinitionBeschreibung> listeAspektDefBeschreibung);

	/**
	 * Ausgabe der AttributgruppenDefinitionen.
	 * Attribut, Attributliste und Attributauswahl sollen auf die Beschreibung referenzieren.
	 *
	 * @param listeAtgDefBeschreibung Liste der AttributgruppenDefinitionBeschreibungen.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenAtgDefBeschreibungListe(List<AttributgruppenDefinitionBeschreibung> listeAtgDefBeschreibung);

	/**
	 * Ausgabe der AttributlistenDefinitionen.
	 * Attribut, Attributliste und Attributauswahl sollen auf die Beschreibung referenzieren.
	 *
	 * @param listeAtlDefBeschreibung Liste der AttributlistenDefinitionBeschreibungen.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenAtlDefBeschreibungListe(List<AttributlistenDefinitionBeschreibung> listeAtlDefBeschreibung);

	/**
	 * Ausgabe der AttributauswahlDefinitionen.
	 * Auswhle sollen auf die Beschreibung referenzieren.
	 *
	 * @param listeAttributauswahlDefinitionBescreibung Liste der AttributauswahlDefinitionBeschreibungen.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenAttAuswahlDefBeschreibungListe(List <AttributauswahlDefinitionBeschreibung> listeAttributauswahlDefinitionBescreibung);

	/**
	 * Ausgabe der AttributtDefinitionen.
	 * objektReferenz soll auf die Beschreibung referenzieren.
	 *
	 * @param listeAttribbutDefinitionBeschreibung Liste der AttributDefinitionBeschreibungen.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenAttDefBeschreibungListe(List<AttributDefinitionBeschreibung> listeAttribbutDefinitionBeschreibung);

	/**
	 * Ausgabe des Titels von KonfigurationBereich.
	 *
	 * @param kbB KonfigurationsBereichBeschreibung.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenKBTitel(KonfigurationsBereichBeschreibung kbB);

	/**
	 * Ausgabe der KonfigurationsObjekten.
	 *
	 * @param listeKOB Liste der KonfigurationsObjektBeschreibungen.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenKOBeschreibungListe(List<KonfigurationsObjektBeschreibung> listeKOB);

	/**
	 * Tabellarische Ausgabe der nderungsbersicht.
	 *
	 * @param listeKAB Liste der KonfigurationsAenderungBeschreibungen.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenKonfigurationsAenderungenTabelle(List<KonfigurationsAenderungBeschreibung> listeKAB);


	/**
	 * Ausgabe der Langbezeichnung, Kommentar und Konvertierungsdatum des Datenkatalogs.
	 *
	 * @param kommentar Kommentar des Datenkatalogs.
	 * @param langBezeichung Langbezeichnung des Datenkatalogs.
	 */
	public void ausgebenKopfbereich(String kommentar, String langBezeichung);

	/**
	 * Ausgabe der MengenDefinitionen.
	 * Elemente sollen auf die Beschreibung referenzieren.
	 *
	 * @param listeMengenBeschreibung Liste der MengenDefinitionBeschreibungen.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenMengenDefinitionBeschreibungListe(List<MengenDefinitionBeschreibung> listeMengenBeschreibung);

	/**
	 * Ausgabe der Information ber das Modell:
	 *  - Dokumentation
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenModellTitel();


	/**
	 * Ausgabe der Titelinformation ber Objekte:
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenObjekteTitel();

	/**
	 * Tabellarische Auflistung mit den Angaben zu den Konfigurationsverantwortlichen
	 * sowie den Konfigurationsbereichen je Konfigurationsverantwortlichen des Datenkatalogs.
	 *
	 * @param listeKbBeschreibungen Liste von KonfigurationsBereichBeschreibungen.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenTabelleKvKb(List<KonfigurationsBereichBeschreibung> listeKbBeschreibungen);

	/**
	 * Ausgabe der TypDefinitionen.
	 *
	 * @param listeTypDefBeschreibung Liste der TypDefinitionBeschreibungen.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenTypDefBeschreibungListe(List<TypDefinitionBeschreibung> listeTypDefBeschreibung);

	/**
	 * Ausgabe der bersichttabelle.
	 * Name (PID) soll auf die Beschreibung referenzieren.
	 *
	 * @param listeUebersicht Liste der bersichten.
	 *
	 * @param listenName Name der Liste.
	 *
	 * @return StatusMeldungen.
	 */
	public IStatus ausgebenUebersichtListe(List<Uebersicht> listeUebersicht, String listenName);

	/**
	 * Gebe die StartDatei aus.
	 *
	 * @return StartDatei.
	 */
	public String ermittleStartDatei();

	/**
	 * Initialisiere die Datei fr den Konfigurationsbereich.
	 *
	 * @param KBPid PID des Konfigurationsbereichs.
	 */
	public void initialisieren(String KBPid);

	/**
	 * Vor der Neuerzeugung der Ausgabe lsche alte Datei.
	 */
	public void loescheAlteDatei();
}
