/*
 * Copyright 2008 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kni, Kln
 * <p>
 * This file is part of de.inovat.pat.datkat2html
 * <p>
 * de.inovat.pat.datkat2html is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <p>
 * de.inovat.pat.datkat2html is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.datkat2html; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * <p>
 * Contact Information:<br>
 * inovat, Dipl.-Ing. H. C. Kni<br>
 * Klner Strae 30<br>
 * 50859 Kln<br>
 * Phone: +49 2234 4301 800<br>
 * mailto: info@inovat.de
 */
package de.inovat.pat.datkat2html.ausgabe;

import de.inovat.pat.datkat2html.Activator;
import de.inovat.pat.datkat2html.Log;
import de.inovat.sys.funclib.bsvrzxml.BSVRZXML;

import java.util.*;

/**
 * Die Klasse enthlt die Information ber die Elementendefinition,
 * die verwendete in Konfigurationsbereichen Elemente und die Pfade zu den KBDatei.
 * 
 * Die liefert das PID von KonfigurationBereich, wo das Element definiert ist,
 * das PID von KonfigurationBereich, wo das Element verwendet wird,
 * und den Pfad zur XMLDatei, wo KB definiert ist.
 * 
 * In das Konstruktor wird die Liste aller zu betrachteten KBDateien bergeben.
 * 
 * Die Klasse wird fr die Referenzierung in der HTMLAusgabe 
 * und fr die Abhngigkeit zwischen den Konfigurationsbereichen benutzt.
 * 
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 408 $ / $Date: 2010-04-06 08:14:07 +0200 (Di, 06. Apr 2010) $ / ($Author: HCK $)
 */

public class ReferenzInformation {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Private ----------------------------------------------------------------
	/**
	 * Map enthlt alle KB, die in den Dateien aus der Liste aller KBDateien definiert sind.
	 * Schlssel: PID von KB, Wert: Pfad von XMLDatei, wo KB definiert ist.
	 */
	private Map<String, String> _mapDateiPfad = new HashMap<String, String>();

	/**
 	 * Map enthlt alle TypDefinitionen, die in den Dateien aus der Liste aller KBDatei definiert sind.
	 * Schlssel: PID von TypDefinition, Wert: PID von KB wo dieser Typ definiert ist.
	 */
	private Map<String, String> _mapAlleTypDefinitionen = new HashMap<String, String>();
	/**
 	 * Map enthlt alle MengenDefinitionen, die in den Dateien aus der Liste aller KBDatei definiert sind.
	 * Schlssel: PID von MengenDefinition, Wert: PID von KB wo diese Menge definiert ist.
	 */
	private Map<String, String> _mapAlleMengenDefinitionen = new HashMap<String, String>();
	/**
 	 * Map enthlt alle AttributgruppenDefinitionen, die in den Dateien aus der Liste aller KBDatei definiert sind.
	 * Schlssel: PID von AttributgruppenDefinition, Wert: PID von KB wo diese Attributgruppe definiert ist.
	 */
	private Map<String, String> _mapAlleAttributgruppenDefinitionen = new HashMap<String, String>();

	/**
 	 * Map enthlt alle AttributlistenDefinitionen, die in den Dateien aus der Liste aller KBDatei definiert sind.
	 *  Schlssel: PID von AttributlistenDefinition, Wert: PID von KB wo diese Attributliste definiert ist.
	 */
	private Map<String, String> _mapAlleAttributlistenDefinitionen = new HashMap<String, String>();
	/**
 	 * Map enthlt alle AttributDefinitionen, die in den Dateien aus der Liste aller KBDatei definiert sind.
	 * Schlssel: PID von AttributDefinition, Wert: PID von KB wo dieses Attribut definiert ist.
	 */
	private Map<String, String> _mapAlleAttributDefinitionen = new HashMap<String, String>();
	/**
 	 * Map enthlt alle AttributauswahlDefinitionen, die in den Dateien aus der Liste aller KBDatei definiert sind.
	 * Schlssel: PID von AttributauswahlDefinition, Wert: PID von KB wo diese Attributauswahl definiert ist.
	 */
	private Map<String, String> _mapAlleAttributauswahlDefinitionen = new HashMap<String, String>();
	/**
 	 * Map enthlt alle AspektDefinitionen, die in den Dateien aus der Liste aller KBDatei definiert sind.
	 * Schlssel: PID von AspektDefinition, Wert: PID von KB wo dieser Aspekt definiert ist.
	 */
	private Map<String, String> _mapAlleAspektDefinitionen = new HashMap<String, String>();
	/**
 	 * Map enthlt alle KonfigurationsObjektDefinitionen, die in den Dateien aus der Liste aller KBDatei definiert sind.
	 * Schlssel: PID von KonfigurationsObjektDefinition, Wert: PID von KB wo dieses KonfigurationsObjekt definiert ist.
	 */
	private Map<String, String> _mapAlleKonfigurationsObjektDefinitionen = new HashMap<String, String>();
	/**
 	 * Map enthlt PIDs alle Elemente, die in den Dateien aus der Liste aller KBDatei definiert sind.
 	 * Schlssel: PID von Element, Wert: PID von KB wo dieses Element definiert ist.
	 */
	private Map<String, String> _mapAlleReferenzen = new HashMap<String, String>();

	/**
	 * Map enthlt alle Elemente, die in den Dateien aus der Liste aller KBDatei verwendet werden.
	 *
	 * Als verwendete Elemente gelten:
	 * <erweitert> <attributgruppe> <menge> aus TypDefinition,
	 * <aspekt> <attribut> <attributliste>  <attributauswahl>aus AttributgruppenDefinition,
	 * <attribut> <attributliste> <attributauswahl> aus AttributlistenDefinition,
	 * <objektReferenz> aus AttrubutDefinition,
	 * <elemente> aus MengeDefinition,
	 * <attributgruppe> <aspekt> aus Datensatz,
	 * <element> aus ObjektMenge,
	 * <typ> aus KonfigurationsObjekt,
	 * <typ> <attributgruppe> aus DefaultParameter aus KonfigurationsObjekt.
	 *
	 * Schlssel: PID von Element, Wert: Liste von KB_PIDs wo dieses Element verwendet wird.
	 */
	private Map<String, List<String>> _mapAlleVerwendungen = new HashMap<String, List<String>>();

	/**
	 * Map enthlt alle KB aus der Liste aller KBDatei mit in diesen KB verwendeten Elementen.
	 *
	 * Als verwendete Elemente gelten:
	 * <erweitert> <attributgruppe> <menge> aus TypDefinition,
	 * <aspekt> <attribut> <attributliste>  <attributauswahl>aus AttributgruppenDefinition,
	 * <attribut> <attributliste> <attributauswahl> aus AttributlistenDefinition,
	 * <objektReferenz> aus AttrubutDefinition,
	 * <elemente> aus MengeDefinition,
	 * <attributgruppe> <aspekt> aus Datensatz,
	 * <element> aus ObjektMenge,
	 * <typ> aus KonfigurationsObjekt,
	 * <typ> <attributgruppe> aus DefaultParameter aus KonfigurationsObjekt.
	 *
	 * Schlssel: PID von KB, Wert: Liste von ElemenetenPIDs, die in diesem KB verwendet werden.
	 */
	private Map<String, List<String>> _mapAlleVerwendungenInKB = new HashMap<String, List<String>>();

	/**
	 * Map enthlt die den KV zu den KB.
	 * Schlssel: PID des KB, Wert: PID des KV.
	 */
	private Map<String, String> _mapAlleKbKv = new HashMap<String, String>();

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Erzeugt eine Instant der Klasse.
	 *
	 * @param listeKBDatei Liste aller zu betrachteten KBDateien.
	 * @param mapDateiPfad Map mit der Information fr jedes KB den Pfad zur zugehrigen XMLDatei.
	 */
	public ReferenzInformation(List<BSVRZXML> listeKBDatei, Map<String, String> mapDateiPfad) {
		_mapDateiPfad=mapDateiPfad;

		for(BSVRZXML kbDatei: listeKBDatei){
			String kbPID=kbDatei.getKb().getPid();
			String kvPID=kbDatei.getKb().getVerantwortlich();

			// Map fr KB und zugehrigem KV erzeugen.
			_mapAlleKbKv.put(kbPID, kvPID);


			// Addiere alle Maps von Definitionen aus BSVRZ.
			// Als Werte werden die PIDs von KB benutzt.
			// Hier wird auch berprft, ob ein Element des bestimmten Typs
			// zwei Mal in der verschiedenen KBDatei definiert ist.
			addiereSetZuMap(kbDatei.getMapAttributauswahlDefinition().keySet(), kbPID, _mapAlleAttributauswahlDefinitionen);
			addiereSetZuMap(kbDatei.getMapAttributDefinition().keySet(), kbPID, _mapAlleAttributDefinitionen);
			addiereSetZuMap(kbDatei.getMapAttributgruppenDefinition().keySet(), kbPID, _mapAlleAttributgruppenDefinitionen);
			addiereSetZuMap(kbDatei.getMapAttributlistenDefinition().keySet(), kbPID, _mapAlleAttributlistenDefinitionen);
			addiereSetZuMap(kbDatei.getMapMengenDefinition().keySet(), kbPID, _mapAlleMengenDefinitionen);
			addiereSetZuMap(kbDatei.getMapTypDefinition().keySet(), kbPID, _mapAlleTypDefinitionen);
			addiereSetZuMap(kbDatei.getMapAspektDefinition().keySet(), kbPID, _mapAlleAspektDefinitionen);
			addiereSetZuMap(kbDatei.getMapKonfigurationsObjekt().keySet(), kbPID, _mapAlleKonfigurationsObjektDefinitionen);

			// Addiere alle Maps von Verwendungen aus BSVRZ ins Map _mapAlleVerwendungen.
			// Als Werte werden die Liste von kbPIDs benutzt.
			addiereSetZuMapVonListen(kbDatei.getMapErweitert().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapAttributgruppe().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapMenge().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapAspekt().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapAttribut().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapAttributliste().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapAttributauswahl().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapObjektReferenz().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapMengenElemente().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapDatensatzAspekt().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapDatensatzAtg().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapElement().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapKonfigurationsObjektTyp().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapKonfigurationsObjektDefParTyp().keySet(), kbPID, _mapAlleVerwendungen);
			addiereSetZuMapVonListen(kbDatei.getMapKonfigurationsObjektDefParAtg().keySet(), kbPID, _mapAlleVerwendungen);

			// Addiere alle Maps von Verwendungen aus BSVRZ ins Map _mapAlleVerwendungenInKB.
			// Als Werte werden die Liste von in KB verwendeten ElementenPIDs benutzt.
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapErweitert().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapAttributgruppe().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapMenge().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapAspekt().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapAttribut().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapAttributliste().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapAttributauswahl().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapObjektReferenz().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapMengenElemente().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapDatensatzAspekt().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapDatensatzAtg().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapElement().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapKonfigurationsObjektTyp().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapKonfigurationsObjektDefParTyp().keySet(), kbPID, _mapAlleVerwendungenInKB);
			addiereSetZuMapVonElemenetenListen(kbDatei.getMapKonfigurationsObjektDefParAtg().keySet(), kbPID, _mapAlleVerwendungenInKB);
		}

		// Erstelle Map aller Referenzen.
		// Hier wird berprft, ob verschiedene Definitionen
		// das gleiche PID benutzten:
		_mapAlleReferenzen=erstelleMapAlleReferenzenen();
	}

	/**
	 * Fge die Elemente des Sets als Schlssel in Map ein.
	 * Als Wert kriegen diese alle Elemente kbPID.
	 *
	 * Das Element wird nicht eingefgt, falls das schon im Map existiert.
	 *
	 * @param setPID Set von Elementen.
	 * @param kbPID PID von KB, wo die Elemente definiert sind.
	 * @param map Das erstellte Map.
	 */
	private void addiereSetZuMap(Set<String> setPID, String kbPID, Map<String, String> map){
		for(String pid: setPID){
			if(map.get(pid)==null){
				map.put(pid, kbPID);
			}
			else{
				zeigeMeldungDoppelPid(kbPID, pid, map.get(pid));
			}
		}
	}

	/**
	 * Zeige die FehlerMeldung ber die wiederholte Definition des Elements.
	 *
	 * @param datei Datei, wo das Element wiederholt definiert wird.
	 * @param pid PID von Element.
	 */
	private void zeigeMeldungDoppelPid(String datei, String pid, String pidKBmitDef){
		Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
				"Das in der Datei ["+datei+"] definierte Element mit der PID ["+pid+
				"] wurde schon in der anderen Datei ["+pidKBmitDef+"] definiert"));
	}

	/**
	 * Fge die Elemente des Sets als Schlssel in Map ein.
	 * In die Liste von KB_PIDs (der Wert) fge kbPID ein,
	 * falls die kbPID noch nicht in der Liste liegt.
	 *
	 * @param setPID Set von Elementen.
	 * @param kbPID PID von KB, wo die Elemente verwendet werden.
	 * @param map Das erstellte Map.
	 */
	private void addiereSetZuMapVonListen(Set<String> setPID, String kbPID, Map<String, List<String>> map){
		for(String pid: setPID){
			List<String> listeKbPid=map.get(pid);
			if(listeKbPid==null){
				List<String> liste=new ArrayList<String>();
				liste.add(kbPID);
				map.put(pid, liste);
			}
			else{
				if(!listeKbPid.contains(kbPID)){
					listeKbPid.add(kbPID);
				}
			}
		}
	}

	private void addiereSetZuMapVonElemenetenListen(Set<String> setPID, String kbPID, Map<String, List<String>> map){
		// Erzeuge eine Liste der Elementen aus dem Set:
		List<String> listeElementenPid=new ArrayList<String>();
		for(String pid: setPID){
			listeElementenPid.add(pid);
		}
		// Addiere diese Liste zum Map (als den Wert):
		if(map.containsKey(kbPID)){
			map.get(kbPID).addAll(listeElementenPid);
		}
		else{
			map.put(kbPID, listeElementenPid);
		}
	}

	/**
	 * Erstelle ein Map von PIDs aller definierten Elementen.
	 * PID von Elementen als Schlssel und PID von KB als Wert.
	 *
	 * @return
	 */
	private Map<String, String> erstelleMapAlleReferenzenen() {
		Map<String, String> mapAlleReferenzen = new HashMap<String, String>();

		addiereMapZuMap(_mapAlleTypDefinitionen, mapAlleReferenzen);
		addiereMapZuMap(_mapAlleMengenDefinitionen, mapAlleReferenzen);
		addiereMapZuMap(_mapAlleAttributgruppenDefinitionen, mapAlleReferenzen);
		addiereMapZuMap(_mapAlleAttributlistenDefinitionen, mapAlleReferenzen);
		addiereMapZuMap(_mapAlleAttributDefinitionen, mapAlleReferenzen);
		addiereMapZuMap(_mapAlleAttributauswahlDefinitionen, mapAlleReferenzen);
		addiereMapZuMap(_mapAlleAspektDefinitionen, mapAlleReferenzen);
		addiereMapZuMap(_mapAlleKonfigurationsObjektDefinitionen, mapAlleReferenzen);

		return mapAlleReferenzen;
	}

	/**
	 * Addiere Map zum anderen Map.
	 * DoppelElemente werden nicht eingefgt.
	 *
	 * @param map Map, das addiert wird.
	 * @param mapAll Map, zum addiert wird.
	 */
	private void addiereMapZuMap(Map<String, String> map, Map<String, String> mapAll){
		if(map.size()>0){
			// Erstelle eine Kopie von map, um die Elemente lschen zu knnen:
			Map<String, String> mapKopie=new HashMap<String, String>();
			mapKopie.putAll(map);

			// Als Wert liegt in map PID von KB (das brauche ich fr die FehlerMeldung).
			String kbDatei=map.values().toArray(new String[0])[0];

			// Falls das Element schon im mapAll existiert,
			// wird das nicht eingefgt:
			for(String element: map.keySet()){
				if(mapAll.containsKey(element)){
					String pidKBmitDef=mapKopie.get(element);
					mapKopie.remove(element);
					zeigeMeldungDoppelPid(kbDatei, element, pidKBmitDef);
				}
			}

			mapAll.putAll(mapKopie);
		}
	}

// ------------------------------------------------------------------------
// GETTER / SETTER METHODEN
// ------------------------------------------------------------------------

	public Map<String, String> getMapAlleKbKv() {
		return _mapAlleKbKv;
	}

	/**
	 * Getter fr Eigenschaft '_mapAlleReferenzen'.
	 *
	 * @return Wert der Eigenschaft '_mapAlleReferenzen'.
	 *
	 * @see (#link _mapAlleReferenzen)
	 */
	public Map<String, String> getMapAlleReferenzen() {
		return _mapAlleReferenzen;
	}

	/**
	 * Getter fr Eigenschaft '_mapAlleVerwendungen'.
	 *
	 * @return Wert der Eigenschaft '_mapAlleVerwendungen'.
	 *
	 * @see (#link _mapAlleVerwendungen)
	 */
	public Map<String, List<String>> getMapAlleVerwendungen() {
		return _mapAlleVerwendungen;
	}

	/**
	 * Getter fr Eigenschaft 'mapAlleVerwendungenInKB'.
	 *
	 * @return Wert der Eigenschaft 'mapAlleVerwendungenInKB'.
	 *
	 * @see  {#link _mapAlleVerwendungenInKB}
	 */
	public Map<String, List<String>> getMapAlleVerwendungenInKB() {
		return _mapAlleVerwendungenInKB;
	}

	/**
	 * Getter fr Eigenschaft '_mapDateiPfad'.
	 *
	 * @return Wert der Eigenschaft '_mapDateiPfad'.
	 *
	 * @see (#link _mapDateiPfad)
	 */
	public Map<String, String> getMapDateiPfad() {
		return _mapDateiPfad;
	}

// ------------------------------------------------------------------------
// SONSTIGE METHODEN
// ------------------------------------------------------------------------

	/**
	 * Ermittelt PID von KBDatei, wo das Element definiert ist.
	 *
	 * @param elementPID PID vom Element, dessen Definition gesucht wird.
	 * @return PID von KB, wo das Element definiert ist
	 * oder leeres String, falls die Definition nicht gefunden wird.
	 */
	public String ermittleKbPid(String elementPID){
		String kbPID=_mapAlleReferenzen.get(elementPID);
		if(kbPID!=null)
			return kbPID;
		else
			return "";
	}

	/**
	 * Ermittelt eine Liste von Elementen, die in kb1 verwendet und in kb2 definiert wurden.
	 *
	 * @param pidKB1 PID von Konfigurationsbereich, wo die Elemente verwendet wurden.
	 * @param pidKB2 PID von Konfigurationsbereich, wo die Elemente definiert wurden.
	 *
	 * @return Liste von Elementen, die in kb1 verwendet und in kb2 definiert wurden.
	 */
	public List<String> ermittleVerwendungenInKb1DefinitionenInKb2(String pidKB1, String pidKB2){
		List<String> listeAbhaengigePIDs=new ArrayList<String>();
		List<String> alleElementenPIDs=_mapAlleVerwendungenInKB.get(pidKB1);
		if(alleElementenPIDs!=null){
			for(String elementPID: alleElementenPIDs){
				if((_mapAlleReferenzen.get(elementPID)!=null)&&
						(_mapAlleReferenzen.get(elementPID).equals(pidKB2))){
					listeAbhaengigePIDs.add(elementPID);
				}
			}
		}
		else{
			Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID,
					"Konfigurationsberech mit der PID ["+pidKB1+
			"] existiert nicht in der Liste der definierten Konfigurationsbereche!"));
		}
		return listeAbhaengigePIDs;
	}

	/**
	 * berprft, ob der Konfigurationsbereich mit dem PID pidKB1
	 * vom Konfigurationsbereich mit dem PID pidKB2 abhngig ist.
	 *
	 * @param pidKB1 Konfigurationsbereich, wo alle verwendete Elemente untersucht werden.
	 * @param pidKB2 Konfigurationsbereich, wo die Definitionen fr die in pidKB1
	 * verwendeten Elemente gesucht werden.
	 *
	 * @return true, falls der Konfigurationsbereich mit dem PID pidKB1
	 * vom Konfigurationsbereich mit dem PID pidKB2 abhngig ist.
	 * Sonst: false.
	 */
	public boolean istKBvomAnderenKBabhaengig(String pidKB1, String pidKB2){
		boolean ergebnis=false;
		List<String> alleElementenPIDs=_mapAlleVerwendungenInKB.get(pidKB1);
		if(alleElementenPIDs!=null){
			for(String elementPID: alleElementenPIDs){
				if((_mapAlleReferenzen.get(elementPID)!=null)&&
						(_mapAlleReferenzen.get(elementPID).equals(pidKB2))){
					ergebnis=true;
					break;
				}
			}
		}
		else{
			Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID,
					"Konfigurationsberech mit der PID ["+pidKB1+
					"] existiert nicht in der Liste der definierten Konfigurationsbereiche!"));
		}
		return ergebnis;
	}

	/**
	 * Prfe, ob es die im Konfigurationsbereich verwendeten Elemente gibt,
	 * fr die keine Definition gefunden werden kann.
	 *
	 * @param pidKB PID vom Konfigurationsbereich.
	 *
	 * @return true, falls es die im Konfigurationsbereich verwendeten Elemente gibt,
	 * fr die keine Definition gefunden werden kann,
	 * false sonst.
	 */
	public boolean sindUnbekannteAbhaengigkeitenVorhanden(String pidKB){
		boolean ergebnis=false;
		List<String> alleElementenPIDs=_mapAlleVerwendungenInKB.get(pidKB);

		if(alleElementenPIDs!=null){
			for(String elementPID: alleElementenPIDs){
				if(_mapAlleReferenzen.get(elementPID)==null){
					ergebnis=true;
					break;
				}
			}
		}
		else{
			Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID,
					"Konfigurationsberech mit der PID ["+pidKB+
					"] existiert nicht in der Liste der definierten Konfigurationsbereche!"));
		}

		return ergebnis;
	}
}
