/*
 * Copyright 2008 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kni, Kln
 * <p>
 * This file is part of de.inovat.pat.datkat2html
 * <p>
 * de.inovat.pat.datkat2html is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <p>
 * de.inovat.pat.datkat2html is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.datkat2html; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * <p>
 * Contact Information:<br>
 * inovat, Dipl.-Ing. H. C. Kni<br>
 * Klner Strae 30<br>
 * 50859 Kln<br>
 * Phone: +49 2234 4301 800<br>
 * mailto: info@inovat.de
 */
package de.inovat.pat.datkat2html.ausgabe.beschreibung;

import java.util.ArrayList;
import java.util.List;

/**
 * Klasse verwaltet dtd-Element typDefinition.
 * 
 * <!ELEMENT typDefinition ( info, (erweitert+ | basis), (attributgruppe | menge)*, defaultParameter* ) >
 * <!ATTLIST typDefinition
 *  	pid ID #REQUIRED
 *  	name CDATA #IMPLIED
 *  	objektNamenPermanent (ja|nein) "nein"
 *  	persistenzModus ( transient | persistent | persistentUndUngltigNachNeustart ) #IMPLIED
 * >
 *
 * <!ELEMENT basis EMPTY >
 * <!ATTLIST basis konfigurierend (ja|nein) #REQUIRED >
 * 
 * 
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 408 $ / $Date: 2010-04-06 08:14:07 +0200 (Di, 06. Apr 2010) $ / ($Author: HCK $)
 */

public class TypDefinitionBeschreibung {
// ------------------------------------------------------------------------
// VARIABLENDEKLARATIONEN
// ------------------------------------------------------------------------

// Private ----------------------------------------------------------------
	/** bersicht der TypDefinitionBeschreibung. */
	private Uebersicht _uebersicht;

	/** Ist ObjektNamen permanent (ja|nein)? */
	private String _objektNamenPermanent;

	/** PersistenzModus ( transient | persistent | persistentUndUngltigNachNeustart ) */
	private String _persistenzModus;

	/** Liste der Typen, die diesen Typ erweitern. */
	private List<TypBeschreibung> _listeErweitert=new ArrayList<TypBeschreibung>();

	/** Ist konfigurierend, falls BasisTyp (ja|nein)? */
	private String _basisKonfigurierend;

	/** Liste der AttributgruppenBeschreibungen. */
	private List<AttributgruppenBeschreibung> _listeAttributgruppenBeschreibung=new ArrayList<AttributgruppenBeschreibung>();

	/** Liste der MengenBeschreibungen. */
	private List<MengenBeschreibung> _listeMengenBeschreibung=new ArrayList<MengenBeschreibung>();

	/** Liste der DefaultParameterBeschreibungen. */
	private List<DefaultParameterBeschreibung> _listeDefaultParameterBeschreibung=new ArrayList<DefaultParameterBeschreibung>();

// ------------------------------------------------------------------------
// KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
// ------------------------------------------------------------------------

	/**
	 * Erzeugt eine Instanz der Klasse TypDefinitionBeschreibung.
	 *
	 * @param uebersicht
	 * @param objektNamenPermanent
	 * @param persistenzModus
	 * @param listeErweitert
	 * @param basisKonfigurierend
	 * @param listeAttributgruppenBeschreibung
	 * @param listeMengenBeschreibung
	 * @param listeDefaultParameterBeschreibung
	 */
	public TypDefinitionBeschreibung(Uebersicht uebersicht,
			String objektNamenPermanent, String persistenzModus,
			List<TypBeschreibung> listeErweitert, String basisKonfigurierend,
			List<AttributgruppenBeschreibung> listeAttributgruppenBeschreibung,
			List<MengenBeschreibung> listeMengenBeschreibung,
			List<DefaultParameterBeschreibung> listeDefaultParameterBeschreibung) {
		_uebersicht = uebersicht;
		_objektNamenPermanent = objektNamenPermanent;
		_persistenzModus = persistenzModus;
		if(listeErweitert!=null){
			_listeErweitert = listeErweitert;
		}
		_basisKonfigurierend = basisKonfigurierend;
		if(listeAttributgruppenBeschreibung!=null){
			_listeAttributgruppenBeschreibung = listeAttributgruppenBeschreibung;
		}
		if(listeMengenBeschreibung!=null){
			_listeMengenBeschreibung = listeMengenBeschreibung;
		}
		if(listeDefaultParameterBeschreibung!=null){
			_listeDefaultParameterBeschreibung = listeDefaultParameterBeschreibung;
		}
	}

// ------------------------------------------------------------------------
// GETTER / SETTER METHODEN
// ------------------------------------------------------------------------

	/**
	 * Getter fr Eigenschaft 'basisKonfigurierend'.
	 *
	 * @return Wert der Eigenschaft 'basisKonfigurierend'.
	 *
	 * @see  {#link _basisKonfigurierend}
	 */
	public String getBasisKonfigurierend() {
		return _basisKonfigurierend;
	}

	/**
	 * Getter fr Eigenschaft 'listeAttributgruppenBeschreibung'.
	 *
	 * @return Wert der Eigenschaft 'listeAttributgruppenBeschreibung'.
	 *
	 * @see  {#link _listeAttributgruppenBeschreibung}
	 */
	public List<AttributgruppenBeschreibung> getListeAttributgruppenBeschreibung() {
		return _listeAttributgruppenBeschreibung;
	}

	/**
	 * Getter fr Eigenschaft 'listeDefaultParameterBeschreibung'.
	 *
	 * @return Wert der Eigenschaft 'listeDefaultParameterBeschreibung'.
	 *
	 * @see  {#link _listeDefaultParameterBeschreibung}
	 */
	public List<DefaultParameterBeschreibung> getListeDefaultParameterBeschreibung() {
		return _listeDefaultParameterBeschreibung;
	}

	/**
	 * Getter fr Eigenschaft 'listeErweitert'.
	 *
	 * @return Wert der Eigenschaft 'listeErweitert'.
	 *
	 * @see  {#link _listeErweitert}
	 */
	public List<TypBeschreibung> getListeErweitert() {
		return _listeErweitert;
	}

	/**
	 * Getter fr Eigenschaft 'listeMengenBeschreibung'.
	 *
	 * @return Wert der Eigenschaft 'listeMengenBeschreibung'.
	 *
	 * @see  {#link _listeMengenBeschreibung}
	 */
	public List<MengenBeschreibung> getListeMengenBeschreibung() {
		return _listeMengenBeschreibung;
	}

	/**
	 * Getter fr Eigenschaft 'objektNamenPermanent'.
	 *
	 * @return Wert der Eigenschaft 'objektNamenPermanent'.
	 *
	 * @see  {#link _objektNamenPermanent}
	 */
	public String getObjektNamenPermanent() {
		return _objektNamenPermanent;
	}

	/**
	 * Getter fr Eigenschaft 'persistenzModus'.
	 *
	 * @return Wert der Eigenschaft 'persistenzModus'.
	 *
	 * @see  {#link _persistenzModus}
	 */
	public String getPersistenzModus() {
		return _persistenzModus;
	}

	/**
	 * Getter fr Eigenschaft 'uebersicht'.
	 *
	 * @return Wert der Eigenschaft 'uebersicht'.
	 *
	 * @see  {#link _uebersicht}
	 */
	public Uebersicht getUebersicht() {
		return _uebersicht;
	}
}
