/*
 * Copyright 2008 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kni, Kln
 * <p>
 * This file is part of de.inovat.pat.datkat2html
 * <p>
 * de.inovat.pat.datkat2html is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * <p>
 * de.inovat.pat.datkat2html is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.pat.datkat2html; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * <p>
 * Contact Information:<br>
 * inovat, Dipl.-Ing. H. C. Kni<br>
 * Klner Strae 30<br>
 * 50859 Kln<br>
 * Phone: +49 2234 4301 800<br>
 * mailto: info@inovat.de
 */
package de.inovat.pat.datkat2html.ausgabe.html;

import de.inovat.pat.datkat2html.Activator;
import de.inovat.pat.datkat2html.Log;
import de.inovat.pat.datkat2html.ausgabe.IAusgabe;
import de.inovat.pat.datkat2html.ausgabe.ReferenzInformation;
import de.inovat.pat.datkat2html.ausgabe.beschreibung.*;
import de.inovat.pat.datkat2html.ausgabe.beschreibung.Inhalt.InhaltTyp;
import org.eclipse.core.runtime.*;

import javax.xml.stream.*;
import java.io.*;
import java.net.URL;
import java.util.*;

/**
 * Klasse zur Konvertierung von Konfigurationsbereichen, die in einer
 * Konfigurationsvorgabe zusammengefat worden sind, von XML nach HTML (XHTML).
 * 
 * @author inovat
 * @author Dipl.-Inform. Joachim Collasius (JC)
 * @version $Revision: 408 $ / $Date: 2009-01-26 13:14:54 +0100 (Mo, 26 Jan
 *          2009) $ / ($Author: HCK $)
 */
public class HTMLAusgabe implements IAusgabe {
	// ------------------------------------------------------------------------
	// VARIABLENDEKLARATIONEN
	// ------------------------------------------------------------------------

	// Sonstige ---------------------------------------------------------------
	// String Konstante fr den Dokumententyp
	private static final String DOCTYPE = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\""
			+ "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">";

	// String Konstanten fr die Ausgabe von Log-Meldungen
	private static final String XML_STREAM_ERR = "Fehler bei der Ausgabe des XML Streams.";
	private static final String CREATE_STREAM_ERR = "Fehler beim Erzeugen des XML Streams.";
	private static final String UNBEKANNTER_TYP_ERR = "bei der HTML Ausgabe von 'Info' wurde der Inhalt eines Elements von unbekanntem Typ geparst, "
			+ "der aktuell nicht untersttzt wird.";
	private static final String TYP_INDEX_INFO = "Bei der HTML Ausgabe von 'Info' wurde der Inhalt eines Elements vom Typ 'index' geparst, "
			+ "der aktuell nicht untersttzt wird.";
	private static final String TYP_REFERENZ_INFO = "Bei der HTML Ausgabe von 'Info' wurde der Inhalt eines Elements vom Typ 'referenz' geparst, "
			+ "der aktuell nicht untersttzt wird.";
	private static final String TYP_HINWEIS_INFO = "Bei der HTML Ausgabe von 'Info' wurde der Inhalt eines Elements vom Typ 'hinweis' geparst, "
			+ "der aktuell nicht untersttzt wird.";
	private static final String TYP_KOMMENTAR_INFO = "Bei der HTML Ausgabe von 'Info' wurde der Inhalt eines Elements vom Typ 'kommentar' geparst, "
			+ "der aktuell nicht untersttzt wird.";
	private static final String OHNE_REF = "Referenzinformationen sind aus den analysierten Konfigurationsvorgaben nicht ermittelbar.";
	private static final String NICHT_ERMITTELBAR = "nicht ermittelbar";

	// Package fr die Bilder und die CSS Datei der HTML Ausgabe
	private static final String IMG_PACKAGE = Activator.PLUGIN_ID
			+ ".ausgabe.html.img";
	private static final String CSS_PACKAGE = Activator.PLUGIN_ID
			+ ".ausgabe.html.css";
	private static final String CSS_FILE = "HTMLAusgabe.css";

	// berschriften der bersicht bzw. der Einzelbeschreibung
	private static final String TYP_DEF = "TypDefinitionen";
	private static final String MENGE_DEF = "MengenDefinitionen";
	private static final String ATG_DEF = "AttributGruppenDefinitionen";
	private static final String ATL_DEF = "AttributListenDefinitionen";
	private static final String ATT_DEF = "AttributDefinitionen";
	private static final String ATA_DEF = "AttributAuswahlDefinitionen";
	private static final String ASP_DEF = "AspektDefinitionen";
	private static final String KFG_OBJ = "Konfigurationsobjekte";
	private static final String AEND_UEB = "nderungsbersicht";

	// Icons fr die Navigation im HTML-Dokument
	private static final String IMG_NAV_ERSTER = "img/navErster.gif";
	private static final String IMG_NAV_LETZTER = "img/navLetzter.gif";
	private static final String IMG_NAV_HOCH = "img/navHoch.gif";
	private static final String IMG_NAV_HOCH_UEBERSICHT = "img/navHochUebersicht.gif";
	private static final String IMG_NAV_VORGAENGER = "img/navVorgaenger.gif";
	private static final String IMG_NAV_NACHFOLGER = "img/navNachfolger.gif";
	// Texte fr das 'alt'-Attribut obiger Icons
	private static final String ALT_NAV_ERSTER = "kein Vorgnger";
	private static final String ALT_NAV_LETZTER = "kein Nachfolger";
	private static final String ALT_NAV_VORGAENGER = "zum Vorgnger";
	private static final String ALT_NAV_NACHFOLGER = "zum Nachfolger";

	private static final String TOOL_TIP_AG_AN = "Erstellt mit DatKat2HTML, Konvertierungsprogramm zur Darstellung des BSVRZ-Datenkatalogs in HTML-Darstellung.\n\n"
			+ "Auftraggeber: \n"
			+ "Regierungsprsidium Tbingen\n"
			+ "Landestelle fr Straentechnik\n"
			+ "Heilbronner Strae 300-302\n"
			+ "70469 Stuttgart\n"
			+ "http://www.rp.baden-wuerttemberg.de\n\n"
			+ "Erstellt durch: \n"
			+ "inovat\n"
			+ "innovative systeme - verkehr - tunnel - technik\n"
			+ "Dipl.-Ing. H. C. Kni\n"
			+ "Klner Strae 30\n"
			+ "50859 Kln\n"
			+ "http://www.inovat.de";

	// Icons fr die Abhngigkeitsmatrix:
	private static final String _iconAmAbhaengigIntern = "img/amAbhaengigIntern.png";
	private static final String _iconAmAbhaengigExtern = "img/amAbhaengigExtern.png";
	private static final String _iconAmUnabhaengig = "img/amUnabhaengig.png";
	private static final String _iconAmZyklischAbhaengigExtern = "img/amZyklischAbhaengigExtern.png";
	private static final String _iconAmZyklischAbhaengigIntern = "img/amZyklischAbhaengigIntern.png";
	private static final String _iconAmAbhaengigUnbekannt = "img/amAbhaengigUnbekannt.png";
	// Private ----------------------------------------------------------------;
	// Name des Konfigurationsverantwortlichen
	private String _kvName;
	// PID des Konfigurationsbereichs
	private String _kbPid;
	// vollstndige Pfadangabe des Zielordners fr die HTML Dateien
	private String _zielOrdner;
	// Instanz der ReferenzInformation
	private ReferenzInformation _refInfo;
	// die aktuell zu generierende HTML Datei
	private File _htmlDatei;
	// die zu genereierende HTML Startdatei
	private File _startDatei;
	// der XML Stream fr die HTML Ausgabe
	private XMLStreamWriter _ausgabe;

	// ------------------------------------------------------------------------
	// KONSTRUKTOREN (und vom Konstruktor verwende Methoden)
	// ------------------------------------------------------------------------

	/**
	 * Konstutruktor der HTML-ASusgabe
	 * 
	 * @param zielOrdner
	 *            Zielordner fr die Ausgabe
	 * @param refInfo
	 *            Referenz auf die Gesamte Information
	 */
	public HTMLAusgabe(String zielOrdner, ReferenzInformation refInfo) {
		_zielOrdner = zielOrdner;
		_refInfo = refInfo;
		_startDatei = new File(_zielOrdner + File.separator + "start.html");
		kopiereCSS();
		kopiereImages();
		try {
			BufferedWriter _puffer = new BufferedWriter(new FileWriter(
					_startDatei));
			XMLOutputFactory xmlFactory = XMLOutputFactory.newInstance();
			_ausgabe = xmlFactory.createXMLStreamWriter(_puffer);
			erzeugeHtmlHeader("Start");
		} catch (IOException e) {
			String meldung = "Fehler beim Erzeugen der HTML-Datei ["
					+ _startDatei + "].";
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung, e);
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
					CREATE_STREAM_ERR, e);
		}
	}

	private void kopiereCSS() {
		String quellOrdner = CSS_PACKAGE.replace('.', '/') + "/" + CSS_FILE;
		IPath pfad = new Path(quellOrdner);
		// ClassLoader mu benutzt werden, weil im Plugin 'jar' kein Verzeichnis
		// 'src' angelegt wird
		URL url = getClass().getClassLoader().getResource(quellOrdner);
		try {
			if (!(url == null)) {
				String cssPfad = FileLocator.toFileURL(url).getPath();
				File css = new File(cssPfad);
				File ziel = new File(_zielOrdner + File.separator + CSS_FILE);
				kopiereDatei(css, ziel);
			} else {
				String meldung = "Das erforderliche Quellverzeichnis [" + pfad
						+ "] konnte nicht gefunden werden.";
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung);
			}
		} catch (IOException e) {
			String meldung = "Fehler beim Erzeugen des absoluten Pfads aus der URL ["
					+ url.getFile() + "].";
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung, e);
		}
	}

	private void kopiereDatei(File quelle, File ziel) {
		try {
			ziel.createNewFile();
			FileInputStream in = new FileInputStream(quelle);
			FileOutputStream aus = new FileOutputStream(ziel);
			byte[] puffer = new byte[1024];
			int laenge;
			while ((laenge = in.read(puffer)) > 0) {
				aus.write(puffer, 0, laenge);
			}
			aus.close();
			in.close();
		} catch (IOException e) {
			String meldung = "Fehler beim Kopieren der Datei ["
					+ quelle.getName() + "]";
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung, e);
		}
	}

	private void kopiereImages() {
		String quellOrdner = IMG_PACKAGE.replace('.', '/');
		String zielOrdner = _zielOrdner + "/" + "img";
		File ziel = new File(zielOrdner);
		boolean erzeugt = true;
		if (!ziel.exists())
			erzeugt = ziel.mkdir();
		IPath pfad = new Path(quellOrdner);
		// URL url = FileLocator.find(Activator.getDefault().getBundle(), pfad,
		// null);
		// ClassLoader mu benutzt werden, weil im Plugin 'jar' kein Verzeichnis
		// 'src' angelegt wird
		URL url = getClass().getClassLoader().getResource(quellOrdner);
		try {
			if (url != null) {
				if (erzeugt) {
					String imgPfad = FileLocator.toFileURL(url).getPath();
					File[] dateien = new File(imgPfad).listFiles();
					for (File bild : dateien) {
						String dateiName = bild.getName();
						File zielDatei = new File(zielOrdner + "/" + dateiName);
						kopiereDatei(bild, zielDatei);
					}
				} else {
					String meldung = "Fehler beim Erzeugen des erforderlichen Verzeichnisses ["
							+ ziel.getPath() + "]";
					Log
							.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
									meldung);
				}
			} else {
				String meldung = "Das erforderliche Quellverzeichnis [" + pfad
						+ "] konnte nicht gefunden werden.";
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung);
			}
		} catch (IOException e) {
			String meldung = "Fehler beim Erzeugen des absoluten Pfads aus der URL ["
					+ url.getFile() + "].";
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung, e);
		}
	}

	private void erzeugeHtmlHeader(String kbPid) {
		try {
			_ausgabe.writeStartDocument("ISO-8859-1", "1.0");
			_ausgabe.writeDTD(DOCTYPE);
			// _ausgabe.writeEmptyElement("!" + DOCTYPE); //alternativ zu
			// writeDTD(?)
			_ausgabe.writeStartElement("html");
			_ausgabe.setDefaultNamespace("http://www.w3.org/1999/xhtml");
			_ausgabe.writeStartElement("head");

			_ausgabe.writeStartElement("title");
			_ausgabe.writeCharacters(kbPid);
			_ausgabe.writeEndElement(); // </title>

			_ausgabe.writeEmptyElement("link");
			_ausgabe.writeAttribute("rel", "stylesheet");
			_ausgabe.writeAttribute("href", "HTMLAusgabe.css");
			_ausgabe.writeAttribute("type", "text/css");

			_ausgabe.writeEndElement(); // </head>
			_ausgabe.writeStartElement("body");
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	// ------------------------------------------------------------------------
	// INTERFACE METHODEN
	// ------------------------------------------------------------------------

	// --------------------- Interface IAusgabe ---------------------

	@Override
	public void loescheAlteDatei() {
		Map<String, String> kbDateiPfade = _refInfo.getMapDateiPfad();
		File zielOrdner = new File(_zielOrdner);
		if (zielOrdner.exists()) {
			for (String kbPid : kbDateiPfade.keySet()) {
				String kbDatei = kbDateiPfade.get(kbPid);
				File datei = new File(kbDatei);
				if (datei.isFile()) {
					String dateiName = datei.getName().replace(".xml", ".html");
					File htmlDatei = new File(_zielOrdner + File.separator
							+ dateiName);
					boolean geloescht = htmlDatei.delete();
					if (!geloescht) {
						String meldung = "Die zu lschende HTML-Datei ["
								+ htmlDatei
								+ "] existiert nicht oder wurde bereits entfernt.";
						Log.erzeugeMeldung(Log.INFO, Activator.PLUGIN_ID,
								meldung);
					} else {
						// System.out.println("Datei gelscht: "+htmlDatei);
					}
				} else {
					String meldung = "Angabe der Datei [" + kbDatei
							+ "] zum Konfigurationsbereich [" + kbPid
							+ "] ist nicht korrekt.";
					Log
							.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
									meldung);
				}
			}
		} else {
			String meldung = "Der Zielordner ["
					+ zielOrdner
					+ "] mit den zu lschenden HTML-Dateien existiert nicht oder wurde bereits entfernt.";
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung);
		}
	}

	@Override
	public void initialisieren(String kbPid) {
		_kbPid = kbPid;
		if (pruefeZielOrdner()) {
			_htmlDatei = new File(_zielOrdner + File.separator + kbPid
					+ ".html");
			kopiereAbbildungen();
			try {
				BufferedWriter puffer = new BufferedWriter(new FileWriter(
						_htmlDatei));
				XMLOutputFactory xmlOutFact = XMLOutputFactory.newInstance();
				_ausgabe = xmlOutFact.createXMLStreamWriter(puffer);
				erzeugeHtmlHeader(kbPid);
			} catch (IOException e) {
				String meldung = "Fehler beim Erzeugen der HTML-Datei ["
						+ _htmlDatei + "].";
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung, e);
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						CREATE_STREAM_ERR, e);
			}
		}
	}

	@Override
	public void abschliessen() {
		try {
			// Erstellerhinweis einfgen
			Date datum = new Date();
			_ausgabe.writeStartElement("p");
			_ausgabe.writeAttribute("class", "mini");
			_ausgabe.writeCharacters("Erstellt am " + datum
					+ " mit DatKat2HTML, (C) 2008 inovat");
			_ausgabe.writeEndElement(); // </p>

			_ausgabe.writeEndElement(); // </body>
			_ausgabe.writeEndElement(); // </html>
			_ausgabe.writeEndDocument();
			_ausgabe.flush();
			_ausgabe.close();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	@Override
	public String ermittleStartDatei() {
		return _startDatei.getPath();
	}

	@Override
	public void ausgebenKopfbereich(String kommentar, String langBezeichung) {
		try {
			if (!kommentar.startsWith("   ")) {
				if (kommentar.startsWith("  ")) {
					einfuegenKopfzeileNwInovat();
				} else if (kommentar.startsWith(" ")) {
					einfuegenKopfzeileInovat();
				} else {
					einfuegenKopfzeileBwInovat();
				}
			}

			_ausgabe.writeStartElement("h1");
			// Anker fr Konfigurationsvorgabe
			_ausgabe.writeStartElement("a");
			_ausgabe.writeAttribute("name", "Konfigurationsvorgabe");
			_ausgabe.writeCharacters(langBezeichung);
			_ausgabe.writeEndElement(); // </a>
			_ausgabe.writeEndElement(); // </h1>
			_ausgabe.writeStartElement("p");
			_ausgabe.writeCharacters(kommentar);
			_ausgabe.writeEndElement(); // </p>
			_ausgabe.writeStartElement("h2");
			_ausgabe.writeCharacters("bersicht");
			_ausgabe.writeEndElement(); // </h2>
			// _ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	@Override
	public IStatus ausgebenTabelleKvKb(
			List<KonfigurationsBereichBeschreibung> listeKbBeschreibungen) {
		// ArrayList<Object[]> listeZeilen = new ArrayList<Object[]>();
		String[] arKopfzeile = new String[] { "Konfigurationsverantwortlicher",
				"Konfigurationsbereich", "PID" };
		erzeugeKopfzeile(arKopfzeile, "tabellekvkb");
		try {
			List<String> kbPidFeld = new ArrayList<String>();
			List<String> kbNameFeld = new ArrayList<String>();
			List<String[]> kbFeld = new ArrayList<String[]>();
			String alterKbV = null;
			String neuerKbV = null;
			String kbPid;
			String kbName;
			for (KonfigurationsBereichBeschreibung kbB : listeKbBeschreibungen) { // sortierte
																					// Liste
				neuerKbV = kbB.getVerantwortlich();
				if (!neuerKbV.equals(alterKbV)) {
					if (alterKbV != null) {
						ausgebenZeilenKvKb(alterKbV, kbFeld);
					}
				}
				kbPid = kbB.getUebersicht().getPid();
				kbPidFeld.add(kbPid);
				kbName = kbB.getUebersicht().getName();
				kbNameFeld.add(kbName);
				kbFeld.add(new String[] { kbName, kbPid });
				alterKbV = neuerKbV;
			}
			ausgebenZeilenKvKb(neuerKbV, kbFeld);
			// erzeugeTabelle(arKopfzeile, listeZeilen, "", "tabellekvkb");

			_ausgabe.writeEndElement(); // </table>
			// _ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
		return null;
	}

	@Override
	public IStatus ausgebenAbhaengigkeitsmatrixKB(
			List<KonfigurationsBereichBeschreibung> listeKbBeschreibungen) {
		MultiStatus meldung = Log
				.erzeugeMultiMeldung(
						Activator.PLUGIN_ID,
						"Es sind die folgenden zu bercksichtigten Abhngigkeiten zwischen den Konfigurationsbereichen gefunden worden:");

		// Liste mit dem Inhalt fr die Matrix.
		// Jede Zeile ist von Typ: {KV, KB, icon, kb von dem der KB abhngig
		// ist}:
		List<Object[]> listeMatrix = new ArrayList<Object[]>();

		// Anzahl der zeilen fr jeden KV:
		Map<String, Integer> mapAnzahlRowspanFuerKV = new HashMap<String, Integer>();
		// Anzahl der zeilen fr jeden KB:
		Map<String, Integer> mapAnzahlRowspanFuerKB = new HashMap<String, Integer>();

		String icon;

		String kv;
		String kv2;
		String kbPID;
		String kb2PID;
		Integer anzahlRowspanFurKV = 0;
		Integer anzahlRowspanFurKB = 0;
		String kvAlt = "";
		boolean keineAbhaengigkeit;

		// Erzeuge listeMatrix:
		for (KonfigurationsBereichBeschreibung kb : listeKbBeschreibungen) {
			kv = kb.getVerantwortlich();
			kbPID = kb.getUebersicht().getPid();

			keineAbhaengigkeit = true;

			// Suche alle KB, von dennen unserer kb abhngig ist:
			for (KonfigurationsBereichBeschreibung kb2 : listeKbBeschreibungen) {
				if (kb != kb2) {
					kb2PID = kb2.getUebersicht().getPid();
					if (_refInfo.istKBvomAnderenKBabhaengig(kbPID, kb2PID)) {
						keineAbhaengigkeit = false;
						anzahlRowspanFurKB = anzahlRowspanFurKB + 1;
						kv2 = kb2.getVerantwortlich();
						if (_refInfo.istKBvomAnderenKBabhaengig(kb2PID, kbPID)) {
							if (kv.equals(kv2)) {
								icon = _iconAmZyklischAbhaengigIntern;
								meldung.add(Log.erzeugeMeldung(Log.WARNUNG,
										Activator.PLUGIN_ID,
										"zyklische interne Abhngigkeit: "
												+ kbPID + " <---> " + kb2PID));
							} else {
								icon = _iconAmZyklischAbhaengigExtern;
								meldung
										.add(Log.erzeugeMeldung(Log.WARNUNG,
												Activator.PLUGIN_ID,
												"zyklische externe Abhngigkeit: "
														+ kbPID + " <<===>> "
														+ kb2PID));
							}
						} else {
							if (kv.equals(kv2)) {
								icon = _iconAmAbhaengigIntern;
							} else {
								icon = _iconAmAbhaengigExtern;
								meldung.add(Log.erzeugeMeldung(Log.WARNUNG,
										Activator.PLUGIN_ID,
										"externe Abhngigkeit: " + kbPID
												+ " ===>> " + kb2PID));
							}
						}
						listeMatrix.add(new String[] { kv, kbPID, icon,
								kb2PID + " <" + kv2 + ">" });
					}
				}
			}

			if (_refInfo.sindUnbekannteAbhaengigkeitenVorhanden(kbPID)) {
				anzahlRowspanFurKB = anzahlRowspanFurKB + 1;
				listeMatrix.add(new String[] { kv, kbPID,
						_iconAmAbhaengigUnbekannt,
						"nicht ermittelbaren Konfigurationsbereichen" });
				meldung
						.add(Log
								.erzeugeMeldung(
										Log.WARNUNG,
										Activator.PLUGIN_ID,
										kbPID
												+ " ===> nicht ermittelbare Konfigurationsbereiche"));
			} else if (keineAbhaengigkeit) {
				anzahlRowspanFurKB = anzahlRowspanFurKB + 1;
				listeMatrix.add(new String[] { kv, kbPID, _iconAmUnabhaengig,
						"keine Abhngigkeiten vorhanden" });
			}

			mapAnzahlRowspanFuerKB.put(kbPID, anzahlRowspanFurKB);

			if (kv.equals(kvAlt)) {
				anzahlRowspanFurKV = anzahlRowspanFurKV + anzahlRowspanFurKB;
			} else {
				mapAnzahlRowspanFuerKV.put(kvAlt, anzahlRowspanFurKV);
				anzahlRowspanFurKV = anzahlRowspanFurKB;
			}
			kvAlt = kv;

			anzahlRowspanFurKB = 0;
		}
		// Letzte KV:
		mapAnzahlRowspanFuerKV.put(kvAlt, anzahlRowspanFurKV);

		// HTML Ausgabe:
		try {
			erzeugeTabelleMatrixKV(listeMatrix, mapAnzahlRowspanFuerKV,
					mapAnzahlRowspanFuerKB);

			_ausgabe.writeEndElement(); // </body>
			_ausgabe.writeEndElement(); // </html>
			_ausgabe.writeEndDocument();
			_ausgabe.flush();
			_ausgabe.close();
		} catch (XMLStreamException e) {
			Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
					XML_STREAM_ERR, e));
		}

		if (meldung.getChildren().length == 0)
			meldung = null;

		return meldung;
	}

	@Override
	public IStatus ausgebenUebersichtListe(List<Uebersicht> listeUebersicht,
			String listenName) {
		List<Object[]> listeZeilen = new ArrayList<Object[]>();

		for (Uebersicht zeile : listeUebersicht) {
			// Ausgabe der gesamten Info, nicht nur Kurzinfo
			listeZeilen.add(new Object[] { zeile, zeile.getInfo() });
		}
		try {
			if (!listeUebersicht.isEmpty()) {
				String[] arKopfzeile = new String[] { "Name", "Info" };
				erzeugeTabelle(arKopfzeile, listeZeilen, listenName, listenName
						.toLowerCase());

				// Referenzierung auf den Anfang der Liste
				_ausgabe.writeStartElement("p");
				_ausgabe.writeAttribute("align", "center");
				_ausgabe.writeStartElement("a");
				_ausgabe.writeAttribute("href", _kbPid + ".html#" + listenName); // Anfang
																					// der
																					// Liste
				// _ausgabe.writeAttribute("href", _kbPid + ".html#" + _kbPid);
				// //Konfigurationsbereich
				_ausgabe.writeAttribute("title", listenName);
				_ausgabe.writeEmptyElement("img");
				_ausgabe.writeAttribute("src", "img/top.gif");
				_ausgabe.writeAttribute("alt", "...zum Anfang dieser Liste");
				_ausgabe.writeAttribute("title", "...zum Anfang dieser Liste");
				_ausgabe.writeAttribute("border", "0");

				_ausgabe.writeEndElement(); // </a>
				_ausgabe.writeEndElement(); // </p>
			}
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
		return null;
	}

	@Override
	public IStatus ausgebenKBTitel(KonfigurationsBereichBeschreibung kbB) {
		_kvName = kbB.getVerantwortlich();
		try {
			// Konfigurationsbereich als Sprungziel
			_ausgabe.writeStartElement("h1");
			einfuegenLinkZurKonfigurationsVorgabe();
			_ausgabe.writeStartElement("a");
			_ausgabe.writeAttribute("name", kbB.getUebersicht().getPid());
			_ausgabe.writeCharacters("Konfigurationsverantwortlicher: "
					+ kbB.getVerantwortlich());
			_ausgabe.writeEmptyElement("br");
			_ausgabe.writeCharacters("Konfigurationsbereich: ");
			_ausgabe.writeCharacters(kbB.getUebersicht().getName() + " ("
					+ kbB.getUebersicht().getPid() + ")");
			_ausgabe.writeEndElement(); // </a>
			_ausgabe.writeEndElement(); // </h1>

			ausgebenInfo(kbB.getUebersicht().getInfo());
			_ausgabe.writeStartElement("p");
			_ausgabe.writeAttribute("align", "center");
			_ausgabe.writeStartElement("a");
			_ausgabe.writeAttribute("href", _kbPid + ".html#"
					+ kbB.getUebersicht().getPid());
			_ausgabe.writeEmptyElement("img");
			_ausgabe.writeAttribute("src", "img/top.gif");
			_ausgabe.writeAttribute("alt", "...zum Anfang");
			_ausgabe.writeAttribute("title", "...zum Anfang");
			_ausgabe.writeAttribute("border", "0");
			_ausgabe.writeEndElement(); // </a>
			_ausgabe.writeEndElement(); // </p>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
		return null;
	}

	@Override
	public IStatus ausgebenModellTitel() {
		try {
			_ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("h1");
			einfuegenLinkZurKonfigurationsVorgabe();
			_ausgabe.writeCharacters("Modell");
			_ausgabe.writeEndElement(); // </h1>
			_ausgabe.writeStartElement("h2");
			einfuegenLinkZurKonfigurationsVorgabe();
			_ausgabe.writeCharacters("bersicht Modellbereich");
			_ausgabe.writeEndElement(); // </h2>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
		return null;
	}

	@Override
	public IStatus ausgebenObjekteTitel() {
		try {
			_ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("h1");
			einfuegenLinkZurKonfigurationsVorgabe();
			_ausgabe.writeCharacters("Objekte");
			_ausgabe.writeEndElement(); // </h1>
			_ausgabe.writeStartElement("h2");
			einfuegenLinkZurKonfigurationsVorgabe();
			_ausgabe.writeCharacters("bersicht Objektbereich");
			_ausgabe.writeEndElement(); // </h2>
			// _ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
		return null;
	}

	@Override
	public IStatus ausgebenKonfigurationsAenderungenTabelle(
			List<KonfigurationsAenderungBeschreibung> listeKAB) {
		if (!listeKAB.isEmpty()) {
			List<Object[]> listeZeilen = new ArrayList<Object[]>();

			/**
			 * <!ELEMENT konfigurationsAenderung (#PCDATA |
			 * %zeichenauszeichnung.klasse; | %absatzauszeichnung.klasse;)* >
			 * <!ATTLIST konfigurationsAenderung stand CDATA #REQUIRED version
			 * CDATA #IMPLIED autor CDATA #REQUIRED grund CDATA #IMPLIED>
			 */
			for (KonfigurationsAenderungBeschreibung kaB : listeKAB) {
				listeZeilen.add(new Object[] { kaB.getStand(),
						kaB.getVersion(), kaB.getAutor(), kaB.getGrund(),
						kaB.getInhalt() });
			}

			try {
				String[] arKopfzeile = new String[] { "Stand", "Version",
						"Autor", "Grund", "Info" };
				erzeugeTabelle(arKopfzeile, listeZeilen, AEND_UEB,
						"aenderungsuebersicht");
				_ausgabe.writeStartElement("p");
				_ausgabe.writeAttribute("align", "center");
				_ausgabe.writeStartElement("a");
				_ausgabe.writeAttribute("href", _kbPid + ".html#"
						+ "nderungsbersicht");
				_ausgabe.writeEmptyElement("img");
				_ausgabe.writeAttribute("src", "img/top.gif");
				_ausgabe.writeAttribute("alt", "...zum Anfang dieser Liste");
				_ausgabe.writeAttribute("title", "...zum Anfang dieser Liste");
				_ausgabe.writeAttribute("border", "0");
				_ausgabe.writeEndElement(); // </a>
				_ausgabe.writeEndElement(); // </p>
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
		return null;
	}

	@Override
	public IStatus ausgebenMengenDefinitionBeschreibungListe(
			List<MengenDefinitionBeschreibung> listeMengenDefinitionBeschreibung) {
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr die Mengendefinitionen", null);
		if (!listeMengenDefinitionBeschreibung.isEmpty()) {
			try {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h3");
				einfuegenLinkZurKonfigurationsVorgabe();
				_ausgabe.writeCharacters(MENGE_DEF);
				_ausgabe.writeEndElement(); // </h3>
				int index = 0;
				for (MengenDefinitionBeschreibung mdb : listeMengenDefinitionBeschreibung) {
					String pidVorgaenger = "";
					String pidNachfolger = "";
					if (index > 0) {
						pidVorgaenger = listeMengenDefinitionBeschreibung.get(
								index - 1).getUebersicht().getPid();
					}
					if (index < listeMengenDefinitionBeschreibung.size() - 1) {
						pidNachfolger = listeMengenDefinitionBeschreibung.get(
								index + 1).getUebersicht().getPid();
					}
					MultiStatus meldung = ausgebenMengenDefinitionBeschreibung(
							mdb, pidVorgaenger, pidNachfolger);
					if ((meldung != null) && (meldung.getChildren().length > 0)) {
						multiStatus.add(meldung);
					}
					index++;
					_ausgabe.writeStartElement("p");
					_ausgabe.writeAttribute("align", "center");
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("href", _kbPid + ".html#"
							+ mdb.getUebersicht().getPid());
					_ausgabe.writeEmptyElement("img");
					_ausgabe.writeAttribute("src", "img/top.gif");
					_ausgabe.writeAttribute("alt",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("title",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("border", "0");
					_ausgabe.writeEndElement(); // </a>
					_ausgabe.writeEndElement(); // </p>
				}
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
		return multiStatus;
	}

	@Override
	public IStatus ausgebenAttDefBeschreibungListe(
			List<AttributDefinitionBeschreibung> listeAttributDefinitionBeschreibung) {
		if (!listeAttributDefinitionBeschreibung.isEmpty()) {
			try {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h3");
				einfuegenLinkZurKonfigurationsVorgabe();
				_ausgabe.writeCharacters(ATT_DEF);
				_ausgabe.writeEndElement(); // </h3>
				int index = 0;
				for (AttributDefinitionBeschreibung adb : listeAttributDefinitionBeschreibung) {
					String pidVorgaenger = "";
					String pidNachfolger = "";
					if (index > 0) {
						pidVorgaenger = listeAttributDefinitionBeschreibung
								.get(index - 1).getUebersicht().getPid();
					}
					if (index < listeAttributDefinitionBeschreibung.size() - 1) {
						pidNachfolger = listeAttributDefinitionBeschreibung
								.get(index + 1).getUebersicht().getPid();
					}
					ausgebenAttDefBeschreibung(adb, pidVorgaenger,
							pidNachfolger);
					index++;
					_ausgabe.writeStartElement("p");
					_ausgabe.writeAttribute("align", "center");
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("href", _kbPid + ".html#"
							+ adb.getUebersicht().getPid());
					_ausgabe.writeEmptyElement("img");
					_ausgabe.writeAttribute("src", "img/top.gif");
					_ausgabe.writeAttribute("alt",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("title",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("border", "0");
					_ausgabe.writeEndElement(); // </a>
					_ausgabe.writeEndElement(); // </p>
				}
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
		return null;
	}

	@Override
	public IStatus ausgebenAttAuswahlDefBeschreibungListe(
			List<AttributauswahlDefinitionBeschreibung> listeAttributauswahlDefinitionBeschreibung) {
		if (!listeAttributauswahlDefinitionBeschreibung.isEmpty()) {
			try {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h3");
				_ausgabe.writeCharacters(ATA_DEF);
				_ausgabe.writeEndElement(); // </h3>
				int index = 0;
				for (AttributauswahlDefinitionBeschreibung adb : listeAttributauswahlDefinitionBeschreibung) {
					String pidVorgaenger = "";
					String pidNachfolger = "";
					if (index > 0) {
						pidVorgaenger = listeAttributauswahlDefinitionBeschreibung
								.get(index - 1).getUebersicht().getPid();
					}
					if (index < listeAttributauswahlDefinitionBeschreibung
							.size() - 1) {
						pidNachfolger = listeAttributauswahlDefinitionBeschreibung
								.get(index + 1).getUebersicht().getPid();
					}
					ausgebenUebersicht(adb.getUebersicht(), pidVorgaenger,
							pidNachfolger, ATA_DEF, "atadefuebersicht");
					if (adb.getTyp() != null) {
						_ausgabe.writeStartElement("table");
						_ausgabe.writeAttribute("class", "atadefspezial");
						_ausgabe.writeStartElement("tr");
						_ausgabe.writeStartElement("td");
						_ausgabe.writeCharacters("Typ:");
						_ausgabe.writeEndElement(); // <td>
						_ausgabe.writeStartElement("td");
						String wert = adb.getTyp();
						if (!wert.isEmpty())
							_ausgabe.writeCharacters(wert);
						else
							_ausgabe.writeCharacters("-");
						_ausgabe.writeEndElement(); // <td>
						_ausgabe.writeEndElement(); // <tr>
						_ausgabe.writeEndElement(); // </table>
					}
					ausgebenAttAuswahlBeschreibungListe(adb
							.getListeAuswahlBeschreibung());
					index++;
				}
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
		return null;
	}

	@Override
	public IStatus ausgebenAspektDefBeschreibungListe(
			List<AspektDefinitionBeschreibung> listeAspektDefBeschreibung) {
		if (!listeAspektDefBeschreibung.isEmpty()) {
			try {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h3");
				_ausgabe.writeCharacters(ASP_DEF);
				_ausgabe.writeEndElement(); // </h3>
				int index = 0;
				for (AspektDefinitionBeschreibung ab : listeAspektDefBeschreibung) {
					String pidVorgaenger = "";
					String pidNachfolger = "";
					if (index > 0) {
						pidVorgaenger = listeAspektDefBeschreibung.get(
								index - 1).getUebersicht().getPid();
					}
					if (index < listeAspektDefBeschreibung.size() - 1) {
						pidNachfolger = listeAspektDefBeschreibung.get(
								index + 1).getUebersicht().getPid();
					}
					ausgebenUebersicht(ab.getUebersicht(), pidVorgaenger,
							pidNachfolger, ASP_DEF, "aspdefuebersicht");
					index++;
					_ausgabe.writeStartElement("p");
					_ausgabe.writeAttribute("align", "center");
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("href", _kbPid + ".html#"
							+ ab.getUebersicht().getPid());
					_ausgabe.writeEmptyElement("img");
					_ausgabe.writeAttribute("src", "img/top.gif");
					_ausgabe.writeAttribute("alt",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("title",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("border", "0");
					_ausgabe.writeEndElement(); // </a>
					_ausgabe.writeEndElement(); // </p>
				}
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
		return null;
	}

	@Override
	public IStatus ausgebenAtlDefBeschreibungListe(
			List<AttributlistenDefinitionBeschreibung> listeAtlDefBeschreibung) {
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr die Attributlistendefinitionen", null);
		if (!listeAtlDefBeschreibung.isEmpty()) {
			try {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h3");
				einfuegenLinkZurKonfigurationsVorgabe();
				_ausgabe.writeCharacters(ATL_DEF);
				_ausgabe.writeEndElement(); // </h3>
				int index = 0;
				for (AttributlistenDefinitionBeschreibung atlDefB : listeAtlDefBeschreibung) {
					String pidVorgaenger = "";
					String pidNachfolger = "";
					if (index > 0) {
						pidVorgaenger = listeAtlDefBeschreibung.get(index - 1)
								.getUebersicht().getPid();
					}
					if (index < listeAtlDefBeschreibung.size() - 1) {
						pidNachfolger = listeAtlDefBeschreibung.get(index + 1)
								.getUebersicht().getPid();
					}
					MultiStatus meldung = ausgebenAtlDefBeschreibung(atlDefB,
							pidVorgaenger, pidNachfolger);
					if ((meldung != null) && (meldung.getChildren().length > 0)) {
						multiStatus.add(meldung);
					}
					index++;
					_ausgabe.writeStartElement("p");
					_ausgabe.writeAttribute("align", "center");
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("href", _kbPid + ".html#"
							+ atlDefB.getUebersicht().getPid());
					_ausgabe.writeEmptyElement("img");
					_ausgabe.writeAttribute("src", "img/top.gif");
					_ausgabe.writeAttribute("alt",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("title",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("border", "0");
					_ausgabe.writeEndElement(); // </a>
					_ausgabe.writeEndElement(); // </p>
				}
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
		return multiStatus;
	}

	@Override
	public IStatus ausgebenAtgDefBeschreibungListe(
			List<AttributgruppenDefinitionBeschreibung> listeAtgDefBeschreibung) {
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr die Attributgruppendefinitionen", null);
		if (!listeAtgDefBeschreibung.isEmpty()) {
			try {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h3");
				einfuegenLinkZurKonfigurationsVorgabe();
				_ausgabe.writeCharacters(ATG_DEF);
				_ausgabe.writeEndElement(); // </h3>
				int index = 0;
				for (AttributgruppenDefinitionBeschreibung atgDefB : listeAtgDefBeschreibung) {
					String pidVorgaenger = "";
					String pidNachfolger = "";
					if (index > 0) {
						pidVorgaenger = listeAtgDefBeschreibung.get(index - 1)
								.getUebersicht().getPid();
					}
					if (index < listeAtgDefBeschreibung.size() - 1) {
						pidNachfolger = listeAtgDefBeschreibung.get(index + 1)
								.getUebersicht().getPid();
					}
					MultiStatus meldung = ausgebenAtgDefBeschreibung(atgDefB,
							pidVorgaenger, pidNachfolger);
					if ((meldung != null) && (meldung.getChildren().length > 0)) {
						multiStatus.add(meldung);
					}
					index++;
					_ausgabe.writeStartElement("p");
					_ausgabe.writeAttribute("align", "center");
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("href", _kbPid + ".html#"
							+ atgDefB.getUebersicht().getPid());
					_ausgabe.writeEmptyElement("img");
					_ausgabe.writeAttribute("src", "img/top.gif");
					_ausgabe.writeAttribute("alt",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("title",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("border", "0");
					_ausgabe.writeEndElement(); // </a>
					_ausgabe.writeEndElement(); // </p>
				}
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
		return multiStatus;
	}

	@Override
	public IStatus ausgebenTypDefBeschreibungListe(
			List<TypDefinitionBeschreibung> listeTypDefBeschreibung) {
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr die Typdefinitionen", null);
		if (!listeTypDefBeschreibung.isEmpty()) {
			try {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h2");
				einfuegenLinkZurKonfigurationsVorgabe();
				_ausgabe.writeCharacters("Einzelbeschreibungen Modellbereich");
				_ausgabe.writeEndElement(); // </h2>
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h3");
				einfuegenLinkZurKonfigurationsVorgabe();
				_ausgabe.writeCharacters(TYP_DEF);
				_ausgabe.writeEndElement(); // </h3>
				int index = 0;
				for (TypDefinitionBeschreibung typDefB : listeTypDefBeschreibung) {
					String pidVorgaenger = "";
					String pidNachfolger = "";
					if (index > 0) {
						pidVorgaenger = listeTypDefBeschreibung.get(index - 1)
								.getUebersicht().getPid();
					}
					if (index < listeTypDefBeschreibung.size() - 1) {
						pidNachfolger = listeTypDefBeschreibung.get(index + 1)
								.getUebersicht().getPid();
					}
					MultiStatus meldung = ausgebenTypDefinitionBeschreibung(
							typDefB, pidVorgaenger, pidNachfolger);
					if ((meldung != null) && (meldung.getChildren().length > 0)) {
						multiStatus.add(meldung);
					}
					index++;
					_ausgabe.writeStartElement("p");
					_ausgabe.writeAttribute("align", "center");
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("href", _kbPid + ".html#"
							+ typDefB.getUebersicht().getPid());
					_ausgabe.writeEmptyElement("img");
					_ausgabe.writeAttribute("src", "img/top.gif");
					_ausgabe.writeAttribute("alt",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("title",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("border", "0");
					_ausgabe.writeEndElement(); // </a>
					_ausgabe.writeEndElement(); // </p>
				}
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
		return multiStatus;
	}

	@Override
	public IStatus ausgebenKOBeschreibungListe(
			List<KonfigurationsObjektBeschreibung> listeKOB) {
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr die Konfigurationsobjekte", null);

		if (!listeKOB.isEmpty()) {
			try {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h2");
				_ausgabe.writeCharacters("Einzelbeschreibungen Objektbereich");
				einfuegenLinkZurKonfigurationsVorgabe();
				_ausgabe.writeEndElement(); // </h2>
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h3");
				_ausgabe.writeCharacters(KFG_OBJ);
				_ausgabe.writeEndElement(); // </h3>
				int index = 0;
				for (KonfigurationsObjektBeschreibung koB : listeKOB) {
					String pidVorgaenger = "";
					String pidNachfolger = "";
					if (index > 0) {
						pidVorgaenger = listeKOB.get(index - 1).getUebersicht()
								.getPid();
					}
					if (index < listeKOB.size() - 1) {
						pidNachfolger = listeKOB.get(index + 1).getUebersicht()
								.getPid();
					}
					MultiStatus meldung = ausgebenKOB(koB, pidVorgaenger,
							pidNachfolger);
					if ((meldung != null) && (meldung.getChildren().length > 0)) {
						multiStatus.add(meldung);
					}
					index++;
					_ausgabe.writeStartElement("p");
					_ausgabe.writeAttribute("align", "center");
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("href", _kbPid + ".html#"
							+ koB.getUebersicht().getPid());
					_ausgabe.writeEmptyElement("img");
					_ausgabe.writeAttribute("src", "img/top.gif");
					_ausgabe.writeAttribute("alt",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("title",
							"...zum Anfang dieses Elements");
					_ausgabe.writeAttribute("border", "0");
					_ausgabe.writeEndElement(); // </a>
					_ausgabe.writeEndElement(); // </p>
				}
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
		return multiStatus;
	}

	// ------------------------------------------------------------------------
	// SONSTIGE METHODEN
	// ------------------------------------------------------------------------

	/**
	 * <!ELEMENT attributgruppenDefinition ( info, aspekt*,
	 * (attribut|attributliste|attributauswahl)+ ) > <!ATTLIST
	 * attributgruppenDefinition pid ID #REQUIRED name CDATA #IMPLIED
	 * konfigurierend (ja|nein) "nein" parametrierend (ja|nein) "nein" code
	 * CDATA #IMPLIED >
	 * 
	 * @param atgDefB
	 *            Beschreibeibung der ATL
	 * @param pidVorgaenger
	 *            Vorgnger PID
	 * @param pidNachfolger
	 *            Nachfolger PID
	 * 
	 * @return MultiStatus-Meldung.
	 */
	private MultiStatus ausgebenAtgDefBeschreibung(
			AttributgruppenDefinitionBeschreibung atgDefB,
			String pidVorgaenger, String pidNachfolger) {
		MultiStatus status = null;
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr die Attributgruppe "
						+ atgDefB.getUebersicht().getPid(), null);
		ausgebenUebersicht(atgDefB.getUebersicht(), pidVorgaenger,
				pidNachfolger, ATG_DEF, "atgdefuebersicht");
		ausgebenAtgDefBeschreibungSpezial(atgDefB);
		try {
			// Ausgabe Aspekte:
			List<Object[]> listeZeilen = new ArrayList<Object[]>();
			if (!atgDefB.getListeAspektBeschreibung().isEmpty()) {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h5");
				_ausgabe.writeCharacters("Aspekte");
				_ausgabe.writeEndElement();
				// _ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("p");
				_ausgabe
						.writeCharacters("Folgende Aspekte knnen bei dieser Attributgruppe verwendet werden:");
				_ausgabe.writeEndElement();

				for (AspektBeschreibung aspektB : atgDefB
						.getListeAspektBeschreibung()) {
					AspektDefinitionBeschreibung aspektDefB = aspektB
							.getAspektDefB();
					String konfigurationsModus = aspektB
							.getKonfigurationsModus();
					String onlineModus = aspektB.getOnlineModus();
					Uebersicht uebersicht = aspektB.getUebersicht();
					InfoBeschreibung infoB = aspektB.getUebersicht().getInfo();
					if (aspektDefB != null) {
						uebersicht = aspektDefB.getUebersicht();
						if (infoB == null) {
							Info info = new Info(uebersicht.getInfo());
							listeZeilen.add(new Object[] { uebersicht,
									konfigurationsModus, onlineModus, info });
						} else {
							listeZeilen.add(new Object[] { uebersicht,
									konfigurationsModus, onlineModus, infoB });
						}
					} else {
						listeZeilen.add(new Object[] { uebersicht,
								konfigurationsModus, onlineModus, OHNE_REF });
						multiStatus
								.add(new Status(
										IStatus.WARNING,
										Activator.PLUGIN_ID,
										"Referenz auf den Aspekt ["
												+ aspektB.getUebersicht()
														.getPid()
												+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
					}
				}
				String[] arKopfZeile = new String[] { "PID",
						"KonfigurationsModus", "OnlineModus", "Info" };
				erzeugeTabelle(arKopfZeile, listeZeilen, "", "atgdefaspekt");
			}
			_ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("h5");
			_ausgabe.writeCharacters("Attribute");
			_ausgabe.writeEndElement();
			// _ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("table");

			// Ausgabe (attribut|attributliste|attributauswahl)+
			String[] arKopfzeile = new String[] { "Name", "ObjektTyp", "Typ",
					"Anzahl", "Anzahl ist", "Info" };
			erzeugeKopfzeile(arKopfzeile, "atgdefelemente");
			status = ausgebenAttOderAtlOderAta(atgDefB, 0, arKopfzeile.length,
					multiStatus);
			_ausgabe.writeEndElement(); // </table>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
		return status;
	}

	private void ausgebenAtgDefBeschreibungSpezial(
			AttributgruppenDefinitionBeschreibung atgDefB) {
		try {
			_ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("table");
			_ausgabe.writeAttribute("class", "atgdefspezial");
			_ausgabe.writeStartElement("tr");
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters("Konfigurierend:");
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters(atgDefB.getKonfigurierend());
			_ausgabe.writeEndElement(); // <td>

			_ausgabe.writeEndElement(); // <tr>
			_ausgabe.writeStartElement("tr");
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters("Parametrierend:");
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters(atgDefB.getParametrierend());
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeEndElement(); // <tr>

			_ausgabe.writeStartElement("tr");
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters("Code:");
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeStartElement("td");
			String code = atgDefB.getCode();
			if (!code.isEmpty())
				_ausgabe.writeCharacters(code);
			else
				_ausgabe.writeCharacters("-");
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeEndElement(); // <tr>
			_ausgabe.writeEndElement(); // </table>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	/**
	 * <!ELEMENT attributlistenDefinition ( info,
	 * (attribut|attributliste|attributauswahl)+ ) > <!ATTLIST
	 * attributlistenDefinition pid ID #REQUIRED name CDATA #IMPLIED >
	 * 
	 * @param atlDefB
	 *            Beschreibeibung der ATL
	 * @param pidVorgaenger
	 *            Vorgnger PID
	 * @param pidNachfolger
	 *            Nachfolger PID
	 * 
	 * @return MultiStatus Meldung.
	 */
	private MultiStatus ausgebenAtlDefBeschreibung(
			AttributlistenDefinitionBeschreibung atlDefB, String pidVorgaenger,
			String pidNachfolger) {
		ausgebenUebersicht(atlDefB.getUebersicht(), pidVorgaenger,
				pidNachfolger, ATL_DEF, "atldefuebersicht");
		String[] arKopfzeile = new String[] { "Name", "ObjektTyp", "Typ",
				"Anzahl", "Anzahl ist", "Info" };
		erzeugeKopfzeile(arKopfzeile, "atldefatt");

		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr die Attributliste "
						+ atlDefB.getUebersicht().getPid(), null);
		MultiStatus status = ausgebenAttOderAtlOderAta(atlDefB, 0,
				arKopfzeile.length, multiStatus);

		try {
			_ausgabe.writeEndElement(); // </table>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
		return status;
	}

	/**
	 * Rekursive Methode. Ausgebe die Listen von Attributen, Attributauswahlen
	 * und Attributlisten (rekursiv).
	 * 
	 * Die Spalte 'Attributtyp' referenziert auf Definition.
	 * 
	 * (attribut|attributliste|attributauswahl)+
	 * 
	 * @param atlDefBoderAtgDefB
	 *            Instanz der Klasse AttributlistenDefinitionBeschreibung oder
	 *            AttributgruppenDefinitionBeschreibung, deren Listen ausgeben
	 *            werden.
	 * @param niveau
	 *            Definiert die Ebene des Aufruf.
	 * @param anzSpalten
	 *            Anzahl der Spalten
	 * @param multiStatus
	 *            MultiStatusmeldung
	 * 
	 * @return MulltiStatus Meldung
	 */
	private MultiStatus ausgebenAttOderAtlOderAta(Object atlDefBoderAtgDefB,
			int niveau, int anzSpalten, MultiStatus multiStatus) {
		// Definiere Listen von Attributen, Attributauswahlen und
		// Attributlisten:
		List<AttributBeschreibung> listeAttB = new ArrayList<AttributBeschreibung>();
		List<AttributauswahlBeschreibung> listeAttAuswahlB = new ArrayList<AttributauswahlBeschreibung>();
		List<AttributlistenBeschreibung> listeAtlB = new ArrayList<AttributlistenBeschreibung>();
		ElementTyp elementTyp = null;

		if (atlDefBoderAtgDefB instanceof AttributlistenDefinitionBeschreibung) {
			AttributlistenDefinitionBeschreibung atlDefB = (AttributlistenDefinitionBeschreibung) atlDefBoderAtgDefB;
			listeAttB = atlDefB.getListeAttributBeschreibung();
			listeAttAuswahlB = atlDefB.getListeAttributAuswahlBeschreibung();
			listeAtlB = atlDefB.getListeAttributListenBeschreibung();
			elementTyp = ElementTyp.atl;
		} else if (atlDefBoderAtgDefB instanceof AttributgruppenDefinitionBeschreibung) {
			AttributgruppenDefinitionBeschreibung atgDefB = (AttributgruppenDefinitionBeschreibung) atlDefBoderAtgDefB;
			listeAttB = atgDefB.getListeAttributBeschreibung();
			listeAttAuswahlB = atgDefB.getListeAttributAuswahlBeschreibung();
			listeAtlB = atgDefB.getListeAttributListenBeschreibung();
			elementTyp = ElementTyp.atg;
		} else {
			String meldung = "Methode 'ausgebenAttOderAtlOderAta' mit falschem Argument 'atlDefBoderAtgDefB' aufgerufen: "
					+ atlDefBoderAtgDefB.toString();
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung);
		}

		List<Object[]> listeZeilen = new ArrayList<Object[]>();
		// Ausgabe von Attributen:
		if (!listeAttB.isEmpty()) {
			for (AttributBeschreibung attB : listeAttB) {
				AttributDefinitionBeschreibung attDefB = attB
						.getAttributDefinitionBeschreibung();
				String name = attB.getUebersicht().getName();
				Name attName = new Name(name, niveau, ElementTyp.att);
				String attTyp = NICHT_ERMITTELBAR;
				String typ = "";
				InfoBeschreibung infoB = attB.getUebersicht().getInfo();
				if (attDefB != null) {
					Uebersicht uebersicht = attDefB.getUebersicht();
					typ = attDefB.getArrAttributTyp()[0][1];
					if (infoB == null) {
						Info info = new Info(uebersicht.getInfo());
						listeZeilen.add(new Object[] { attName, uebersicht,
								typ, attB.getAnzahl(), attB.getAnzahlIst(),
								info });
					} else {
						listeZeilen.add(new Object[] { attName, uebersicht,
								typ, attB.getAnzahl(), attB.getAnzahlIst(),
								infoB });
					}
				} else {
					listeZeilen.add(new Object[] { attName, attTyp, typ,
							attB.getAnzahl(), attB.getAnzahlIst(), infoB });
					multiStatus
							.add(new Status(
									IStatus.WARNING,
									Activator.PLUGIN_ID,
									"Referenz auf das Attribut ["
											+ attB.getUebersicht().getPid()
											+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
				}
			}
			String klasse = "";
			if (elementTyp == ElementTyp.atg)
				klasse = "atgdefatt";
			else if (elementTyp == ElementTyp.atl)
				klasse = "atldefatt";
			erzeugeTabelleOhneKopfzeile(anzSpalten, listeZeilen, klasse);
		}

		// Ausgabe von Attributauswahl:
		listeZeilen = new ArrayList<Object[]>();
		if (!listeAttAuswahlB.isEmpty()) {
			for (AttributauswahlBeschreibung ataB : listeAttAuswahlB) {
				AttributauswahlDefinitionBeschreibung ataDefB = ataB
						.getAttAuswahlDefinitionBescreibung();
				String name = ataB.getUebersicht().getName();
				Name ataName = new Name(name, niveau, ElementTyp.ata);
				String attTyp = NICHT_ERMITTELBAR;
				String typ = "";
				String anzahl = "";
				String anzIst = "";
				InfoBeschreibung infoB = ataB.getUebersicht().getInfo();
				if (ataDefB != null) {
					Uebersicht uebersicht = ataDefB.getUebersicht();
					typ = ataDefB.getTyp();
					if (infoB == null) {
						Info info = new Info(uebersicht.getInfo());
						listeZeilen.add(new Object[] { ataName, uebersicht,
								typ, anzahl, anzIst, info });
					} else {
						listeZeilen.add(new Object[] { ataName, uebersicht,
								typ, anzahl, anzIst, infoB });
					}
				} else {
					listeZeilen.add(new Object[] { ataName, attTyp, typ,
							anzahl, anzIst, infoB });
					multiStatus
							.add(new Status(
									IStatus.WARNING,
									Activator.PLUGIN_ID,
									"Referenz auf die Attributauswahl ["
											+ ataB.getUebersicht().getPid()
											+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
				}
			}
			String klasse = "";
			if (elementTyp == ElementTyp.atg)
				klasse = "atgdefata";
			else if (elementTyp == ElementTyp.atl)
				klasse = "atldefata";
			erzeugeTabelleOhneKopfzeile(anzSpalten, listeZeilen, klasse);
		}

		// Ausgabe von Attributlisten:
		listeZeilen = new ArrayList<Object[]>();
		if (!listeAtlB.isEmpty()) {
			for (AttributlistenBeschreibung atlB : listeAtlB) {
				AttributlistenDefinitionBeschreibung atlDefB_2 = atlB
						.getAttListenDefinitionBeschreibung();
				// Uebersicht uebersicht = null;
				String name = atlB.getUebersicht().getName();
				Name atlName = new Name(name, niveau, ElementTyp.atl);
				String attTyp = NICHT_ERMITTELBAR;
				String typ = "";
				InfoBeschreibung infoB = atlB.getUebersicht().getInfo();
				if (atlDefB_2 != null) {
					Uebersicht uebersicht = atlDefB_2.getUebersicht();
					if (infoB == null) {
						Info info = new Info(uebersicht.getInfo());
						listeZeilen.add(new Object[] { atlName, uebersicht,
								typ, atlB.getAnzahl(), atlB.getAnzahlIst(),
								info });
					} else {
						listeZeilen.add(new Object[] { atlName, uebersicht,
								typ, atlB.getAnzahl(), atlB.getAnzahlIst(),
								infoB });
					}
				} else {
					listeZeilen.add(new Object[] { atlName, attTyp, typ,
							atlB.getAnzahl(), atlB.getAnzahlIst(), infoB });
					multiStatus
							.add(new Status(
									IStatus.WARNING,
									Activator.PLUGIN_ID,
									"Referenz auf die Attributliste ["
											+ atlB.getUebersicht().getPid()
											+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
				}
				String klasse = "";
				if (elementTyp == ElementTyp.atg)
					klasse = "atgdefatl";
				else if (elementTyp == ElementTyp.atl)
					klasse = "atldefatl";
				erzeugeTabelleOhneKopfzeile(anzSpalten, listeZeilen, klasse);

				listeZeilen.remove(listeZeilen.size() - 1);
				// rekursiver Aufruf:
				if (atlDefB_2 != null) {
					ausgebenAttOderAtlOderAta(atlDefB_2, niveau + 1,
							anzSpalten, multiStatus);
				}
			}
		}
		return multiStatus;
	}

	private void ausgebenAttAuswahlBeschreibungListe(
			List<AuswahlBeschreibung> listeAuswahlBeschreibung) {
		if (!listeAuswahlBeschreibung.isEmpty()) {
			List<Object[]> listeZeilen = new ArrayList<Object[]>();

			for (AuswahlBeschreibung ab : listeAuswahlBeschreibung) {
				// Uebersicht uebersicht = null;
				// String name = ab.getUebersicht().getName();
				// String attTyp = NICHT_ERMITTELBAR;
				// String typ = "";
				// InfoBeschreibung info = ab.getUebersicht().getInfo();
				// uebersicht = ab.getUebersicht();

				listeZeilen.add(new Object[] { ab.getUebersicht(),
						ab.getWert(), ab.getArt(), ab.getAnzahl(),
						ab.getAnzahlIst(), ab.getUebersicht().getInfo() });
			}

			String[] arKopfzeile = new String[] { "Name", "Wert", "Art",
					"Anzahl", "Anzahl ist", "Info" };
			erzeugeTabelle(arKopfzeile, listeZeilen, "Attribute", "atadefatt");
		}
	}

	private void ausgebenAttDefBeschreibung(AttributDefinitionBeschreibung adb,
			String pidVorgaenger, String pidNachfolger) {
		/*
		 * <!ELEMENT attributDefinition ( info, ( zeichenkette | ganzzahl |
		 * zeitstempel | objektReferenz | kommazahl ), default? ) > <!ATTLIST
		 * attributDefinition pid ID #REQUIRED name CDATA #IMPLIED >
		 */
		try {
			ausgebenUebersicht(adb.getUebersicht(), pidVorgaenger,
					pidNachfolger, ATT_DEF, "attdefuebersicht");
			if (adb.getDefault() != null) {
				_ausgabe.writeStartElement("table");
				_ausgabe.writeAttribute("class", "attdefspezial");
				_ausgabe.writeStartElement("tr");
				_ausgabe.writeStartElement("td");
				_ausgabe.writeCharacters("Default Wert:");
				_ausgabe.writeEndElement(); // <td>
				_ausgabe.writeStartElement("td");
				String wert = adb.getDefault();
				if (!wert.isEmpty())
					_ausgabe.writeCharacters(wert);
				else
					_ausgabe.writeCharacters("-");
				_ausgabe.writeEndElement(); // <td>
				_ausgabe.writeEndElement(); // <tr>
				_ausgabe.writeEndElement(); // </table>
			}

			List<Object[]> listeZeilen = new ArrayList<Object[]>();
			int anzSpalten = 0;
			boolean istRef = false;
			if (adb.getArrAttributTyp() != null) {
				for (String[] zeile : adb.getArrAttributTyp()) {
					anzSpalten = zeile.length;
					if (istRef) {
						Uebersicht uebersicht = new Uebersicht(zeile[1], null,
								null);
						listeZeilen.add(new Object[] { zeile[0], uebersicht });
					} else {
						listeZeilen.add(new Object[] { zeile[0], zeile[1] });
					}
					if (zeile[1]
							.equals(AttributDefinitionBeschreibung.OBJEKT_REFERENZ)) {
						istRef = true;
					} else {
						istRef = false;
					}
				}
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h5");
				_ausgabe.writeCharacters("Typbeschreibung");
				_ausgabe.writeEndElement(); // </h5>
				_ausgabe.writeStartElement("table");
				_ausgabe.writeAttribute("class", "attdeftyp");
				erzeugeTabelleOhneKopfzeile(anzSpalten, listeZeilen,
						"attdeftyp");
				_ausgabe.writeEndElement(); // </table>
			}
			/*
			 * <!ELEMENT bereich ( info? ) > <!ATTLIST bereich skalierung CDATA
			 * #IMPLIED > <!ATTLIST bereich minimum CDATA #REQUIRED > <!ATTLIST
			 * bereich maximum CDATA #IMPLIED > <!ATTLIST bereich einheit CDATA
			 * #IMPLIED >
			 */
			if (!adb.getBereiche().isEmpty()) {
				listeZeilen = new ArrayList<Object[]>();
				for (BereichBeschreibung bereich : adb.getBereiche()) {
					String skalierung = bereich.getSkalierung();
					if (skalierung == null) {
						skalierung = "";
					}
					String maximum = bereich.getMaximum();
					if (maximum == null) {
						maximum = "";
					}
					String einheit = bereich.getEinheit();
					if (einheit == null) {
						einheit = "";
					}
					listeZeilen.add(new Object[] { bereich.getMinimum(),
							maximum, skalierung, einheit, bereich.getInfo() });
				}
				erzeugeTabelle(new String[] { "Minimum", "Maximum",
						"Skalierung", "Einheit", "Info" }, listeZeilen,
						"Bereiche", "attdefbereich");
			}
			/*
			 * <!ELEMENT zustand ( info? ) > <!ATTLIST zustand name CDATA
			 * #REQUIRED > <!ATTLIST zustand wert CDATA #REQUIRED >
			 */
			if (!adb.getZustaende().isEmpty()) {
				listeZeilen = new ArrayList<Object[]>();
				for (ZustandBeschreibung zustand : adb.getZustaende()) {
					listeZeilen.add(new Object[] { zustand.getName(),
							zustand.getWert(), zustand.getInfo() });
				}
				erzeugeTabelle(new String[] { "Name", "Wert", "Info" },
						listeZeilen, "Zustnde", "attdefzustand");
			}
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	private MultiStatus ausgebenKOB(KonfigurationsObjektBeschreibung koB,
			String pidVorgaenger, String pidNachfolger) {
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr das Konfigurationsobjekt "
						+ koB.getUebersicht().getPid(), null);

		ausgebenUebersicht(koB.getUebersicht(), pidVorgaenger, pidNachfolger,
				KFG_OBJ, "konfigobjuebersicht");
		List<Object[]> listeZeilen = new ArrayList<Object[]>();
		// Zeile fr ID
		listeZeilen.add(new Object[] { "ID:", koB.getId() });
		// Zeile fr Referenz auf Typ
		if (koB.getTyp().getTypDefinitionBeschreibung() == null) {
			listeZeilen.add(new Object[] { "Typ:",
					koB.getTyp().getPid() + " nicht referenzierbar" });
			multiStatus
					.add(new Status(
							IStatus.WARNING,
							Activator.PLUGIN_ID,
							"Referenz auf den Typ ["
									+ koB.getTyp().getPid()
									+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
		} else {
			listeZeilen
					.add(new Object[] {
							"Typ:",
							koB.getTyp().getTypDefinitionBeschreibung()
									.getUebersicht() });
		}

		try {
			_ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("table");
			_ausgabe.writeAttribute("class", "konfigobjspezial");
			erzeugeTabelleOhneKopfzeile(2, listeZeilen, "konfigobjspezial");
			_ausgabe.writeEndElement(); // </table>

			if (!koB.getListeDatensatzBeschreibung().isEmpty()) {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h5");
				_ausgabe.writeCharacters("Datenstze");
				_ausgabe.writeEndElement();
				_ausgabe.writeStartElement("ul");
				_ausgabe.writeAttribute("class", "konfigobjdatensatz");

				for (DatensatzBeschreibung datensatzB : koB
						.getListeDatensatzBeschreibung()) {
					_ausgabe.writeStartElement("li");
					MultiStatus meldung = ausgebenDatensatzBeschreibung(datensatzB);
					if ((meldung != null) && (meldung.getChildren().length > 0)) {
						multiStatus.add(meldung);
					}
					_ausgabe.writeEndElement(); // </li>
				}
				_ausgabe.writeEndElement(); // </ul>
			}

			if (!koB.getListeObjektMengeBeschreibung().isEmpty()) {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h5");
				_ausgabe.writeCharacters("Objektmengen");
				_ausgabe.writeEndElement();
				_ausgabe.writeStartElement("ul");
				_ausgabe.writeAttribute("class", "konfigobjmenge");

				for (ObjektMengeBeschreibung objektMengeB : koB
						.getListeObjektMengeBeschreibung()) {
					_ausgabe.writeStartElement("li");
					MultiStatus meldung = ausgebenObjektMengeBeschreibung(objektMengeB);
					if ((meldung != null) && (meldung.getChildren().length > 0)) {
						multiStatus.add(meldung);
					}
					_ausgabe.writeEndElement(); // </li>
				}
				_ausgabe.writeEndElement(); // </ul>
			}

			if (!koB.getListeDefaultParameterBeschreibung().isEmpty()) {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h5");
				_ausgabe.writeCharacters("Default Parameter");
				_ausgabe.writeEndElement();
				_ausgabe.writeStartElement("ul");
				_ausgabe.writeAttribute("class", "konfigobjdefaultparam");

				for (DefaultParameterBeschreibung defParB : koB
						.getListeDefaultParameterBeschreibung()) {
					_ausgabe.writeStartElement("li");
					ausgebenDefaultParameterBeschreibung(defParB,
							"konfigobjdefaultparam");
					_ausgabe.writeEndElement(); // </li>
				}
				_ausgabe.writeEndElement(); // </ul>
			}

			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
		return multiStatus;
	}

	private MultiStatus ausgebenDatensatzBeschreibung(
			DatensatzBeschreibung datensatzB) {
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr die Datenstze ", null);
		List<Object[]> listeZeilen = new ArrayList<Object[]>();
		AttributgruppenBeschreibung atgB = datensatzB.getAtgB();
		String pid = "-";
		String atg = "-";
		if (atgB != null) {
			// Zeile fr PID
			pid = atgB.getPid();
			listeZeilen.add(new Object[] { "PID:", atgB.getPid() });
			// Zeile fr Referenz auf Attributgruppe
			if (atgB.getAttributgruppenDefinitionBeschreibung() != null) {
				Uebersicht uebersicht = atgB
						.getAttributgruppenDefinitionBeschreibung()
						.getUebersicht();
				listeZeilen.add(new Object[] { "Attributgruppe:", uebersicht });
				if (uebersicht != null)
					atg = uebersicht.getName();
			} else {
				listeZeilen.add(new Object[] { "Attributgruppe:",
						NICHT_ERMITTELBAR });
				multiStatus
						.add(new Status(
								IStatus.WARNING,
								Activator.PLUGIN_ID,
								"Referenz auf die Attributgruppe ["
										+ atgB.getPid()
										+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
			}
		}

		// Zeile fr Referenz auf Aspekt
		AspektBeschreibung aspektB = datensatzB.getAspektB();
		if (aspektB != null) {
			Uebersicht uebersicht = datensatzB.getAspektB().getUebersicht();
			if (uebersicht != null) {
				listeZeilen.add(new Object[] { "Aspekt:",
						datensatzB.getAspektB().getUebersicht() });
			} else {
				listeZeilen.add(new Object[] { "Aspekt:", NICHT_ERMITTELBAR });
				// keine PID in Meldung auszugeben(?)
				// multiStatus.add( new Status(IStatus.WARNING,
				// Activator.PLUGIN_ID,
				// "Referenz auf den Aspekt [" +
				// datensatzB.getAspektB().getUebersicht.getPid() + "] kann in
				// der aktuellen Konfigurationsvorgabe nicht aufgelst
				// werden."));
			}
		}

		try {
			// PID und Name der Attributgruppe ausgeben
			_ausgabe.writeStartElement("h6");
			_ausgabe.writeCharacters(pid + " : " + atg);
			_ausgabe.writeEndElement(); // </h6>
			// _ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("table");
			_ausgabe.writeAttribute("class", "konfigobjdatensatzspezial");
			erzeugeTabelleOhneKopfzeile(2, listeZeilen,
					"konfigobjdatensatzspezial");
			_ausgabe.writeEndElement(); // </table>

			String[] arKopfzeile = new String[] { "Name", "Wert" };
			erzeugeKopfzeile(arKopfzeile, "konfigobjdatensatz");
			// status =
			// ausgebenDatumOderDListeOderDFeld(datensatzB.getListeDatumBeschreibung(),
			ausgebenDatumOderDListeOderDFeld(datensatzB
					.getListeDatumBeschreibung(), datensatzB
					.getListeDatenlisteBeschreibung(), datensatzB
					.getListeDatenfeldBeschreibung(), 0, arKopfzeile.length,
					"konfigobjdatensatz");

			_ausgabe.writeEndElement(); // </table>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
		return multiStatus;
	}

	private MultiStatus ausgebenObjektMengeBeschreibung(
			ObjektMengeBeschreibung objektMengeB) {
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr die Objektmengen ", null);
		List<Object[]> listeZeilen = new ArrayList<Object[]>();
		// Name der Objektmenge ausgeben
		String name = objektMengeB.getName();
		// Zeile fr Name
		listeZeilen.add(new Object[] { "Name:", objektMengeB.getName() });
		// Zeile fr Verwaltung
		listeZeilen.add(new Object[] { "Verwaltung:",
				objektMengeB.getVerwaltung() });

		try {
			_ausgabe.writeStartElement("h6");
			_ausgabe.writeCharacters(name);
			_ausgabe.writeEndElement(); // </h6>
			// _ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("table");
			_ausgabe.writeAttribute("class", "konfigobjmenge");
			erzeugeTabelleOhneKopfzeile(2, listeZeilen, "konfigobjmenge");
			_ausgabe.writeEndElement(); // </table>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}

		if (!objektMengeB.getListeElementBeschreibung().isEmpty()) {
			listeZeilen = new ArrayList<Object[]>();
			for (ElementBeschreibung elementB : objektMengeB
					.getListeElementBeschreibung()) {
				// Zeile fr Referenz auf Element
				String pid = elementB.getPid();
				Uebersicht uebersicht = elementB.getUebersicht();
				if (uebersicht != null) {
					listeZeilen.add(new Object[] { pid, uebersicht });
				} else {
					listeZeilen.add(new Object[] { pid, NICHT_ERMITTELBAR });
					multiStatus
							.add(new Status(
									IStatus.WARNING,
									Activator.PLUGIN_ID,
									"Referenz auf das Objekt ["
											+ pid
											+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
				}
			}
			erzeugeTabelle(new String[] { "PID", "Element" }, listeZeilen, "",
					"konfigobjmengeelement");
		}
		return multiStatus;
	}

	private MultiStatus ausgebenMengenDefinitionBeschreibung(
			MengenDefinitionBeschreibung mdb, String pidVorgaenger,
			String pidNachfolger) {
		List<Object[]> listeZeilen = new ArrayList<Object[]>();
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr die Menge " + mdb.getUebersicht().getPid(),
				null);
		ausgebenUebersicht(mdb.getUebersicht(), pidVorgaenger, pidNachfolger,
				MENGE_DEF, "mengedefuebersicht");
		listeZeilen.add(new Object[] { mdb.getAenderbar(), mdb.getMindestens(),
				mdb.getHoechstens(), mdb.getReferenzierungsart() });
		String[] arKopfzeile = new String[] { "nderbar", "mindestens",
				"hchstens", "referenzierungsart" };
		erzeugeTabelle(arKopfzeile, listeZeilen, "", "mengedef");

		// Elemente der Menge ausgeben
		if (!mdb.getElementeUebersicht().isEmpty()) {
			try {
				_ausgabe.writeStartElement("p");
				_ausgabe
						.writeCharacters("Objekte von den folgenden Typen (oder von Typen, die von diesen Typen abgeleitet sind) knnen in der Menge enthalten sein:");
				_ausgabe.writeEndElement(); // </p>

				List<Object[]> listeElemente = new ArrayList<Object[]>();
				for (Uebersicht uebersicht : mdb.getElementeUebersicht()) {
					String name = uebersicht.getName();
					if (!name.isEmpty()) {
						listeElemente.add(new Object[] { uebersicht,
								uebersicht.getInfo() });
					} else {
						// TODO CSS-Style Attribut class?
						listeElemente
								.add(new Object[] { uebersicht, OHNE_REF });
						multiStatus
								.add(new Status(
										IStatus.WARNING,
										Activator.PLUGIN_ID,
										"Referenz auf den Typ ["
												+ uebersicht.getPid()
												+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
					}
				}
				erzeugeTabelle(new String[] { "Typ", "Info" }, listeElemente,
						"", "mengedefelement");
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
		return multiStatus;
	}

	private void ausgebenUebersicht(Uebersicht uebersicht,
			String pidVorgaenger, String pidNachfolger, String listenName,
			String klasse) {
		try {
			_ausgabe.writeStartElement("h4");
			// Link zur bersichtsliste
			_ausgabe.writeStartElement("a");
			_ausgabe.writeAttribute("href", _kbPid + ".html#" + listenName);
			_ausgabe.writeEmptyElement("img");
			_ausgabe.writeAttribute("src", IMG_NAV_HOCH);
			_ausgabe.writeAttribute("alt", "hoch");
			_ausgabe.writeAttribute("title", listenName);
			_ausgabe.writeAttribute("border", "0");
			_ausgabe.writeEndElement(); // </a>
			_ausgabe.writeEmptyElement("img");
			_ausgabe.writeAttribute("src", "img/einrueckung.png");
			_ausgabe.writeAttribute("alt", "");
			_ausgabe.writeAttribute("border", "0");
			// Link zum Vorgaenger
			if (!pidVorgaenger.isEmpty()) {
				_ausgabe.writeStartElement("a");
				_ausgabe.writeAttribute("href", _kbPid + ".html#"
						+ pidVorgaenger);
				_ausgabe.writeEmptyElement("img");
				_ausgabe.writeAttribute("src", IMG_NAV_VORGAENGER);
				_ausgabe.writeAttribute("alt", ALT_NAV_VORGAENGER);
				_ausgabe.writeAttribute("title", pidVorgaenger);
				_ausgabe.writeAttribute("border", "0");
				_ausgabe.writeEndElement(); // </a>
			} else {
				_ausgabe.writeEmptyElement("img");
				_ausgabe.writeAttribute("src", IMG_NAV_ERSTER);
				_ausgabe.writeAttribute("alt", ALT_NAV_ERSTER);
				_ausgabe.writeAttribute("title", ALT_NAV_ERSTER);
				_ausgabe.writeAttribute("border", "0");
			}
			// Link zum Nachfolger
			if (!pidNachfolger.isEmpty()) {
				_ausgabe.writeStartElement("a");
				_ausgabe.writeAttribute("href", _kbPid + ".html#"
						+ pidNachfolger);
				_ausgabe.writeEmptyElement("img");
				_ausgabe.writeAttribute("src", IMG_NAV_NACHFOLGER);
				_ausgabe.writeAttribute("alt", ALT_NAV_NACHFOLGER);
				_ausgabe.writeAttribute("title", pidNachfolger);
				_ausgabe.writeAttribute("border", "0");
				_ausgabe.writeEndElement(); // </a>
			} else {
				_ausgabe.writeEmptyElement("img");
				_ausgabe.writeAttribute("src", IMG_NAV_LETZTER);
				_ausgabe.writeAttribute("alt", ALT_NAV_LETZTER);
				_ausgabe.writeAttribute("title", ALT_NAV_LETZTER);
				_ausgabe.writeAttribute("border", "0");
			}
			_ausgabe.writeEntityRef("nbsp");
			// Anker fr Definition
			_ausgabe.writeStartElement("a");
			_ausgabe.writeAttribute("name", uebersicht.getPid());
			_ausgabe.writeCharacters(uebersicht.getName());
			_ausgabe.writeEndElement(); // </a>
			_ausgabe.writeEndElement(); // </h4>
			// _ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("table");
			_ausgabe.writeAttribute("class", klasse);

			_ausgabe.writeStartElement("tr");
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters("PID:");
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters(uebersicht.getPid());
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeEndElement(); // <tr>

			_ausgabe.writeStartElement("tr");
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters("Name:");
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters(uebersicht.getName());
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeEndElement(); // <tr>

			_ausgabe.writeStartElement("tr");
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters("Info:");
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeStartElement("td");
			ausgebenInfo(uebersicht.getInfo());
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeEndElement(); // <tr>
			_ausgabe.writeEndElement(); // </table>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	private void ausgebenInfo(InfoBeschreibung info) {
		if (info != null) {
			try {
				if (info.getKurzinfo() != null) {
					_ausgabe.writeStartElement("div");
					_ausgabe.writeAttribute("class", "kurzinfo");
					erzeugeAusgabeFuerInhalt(info.getKurzinfo());
					_ausgabe.writeEndElement(); // </div>
				}
				if (info.getBeschreibung() != null) {
					_ausgabe.writeEmptyElement("br");
					_ausgabe.writeStartElement("div");
					_ausgabe.writeAttribute("class", "beschreibung");
					erzeugeAusgabeFuerInhalt(info.getBeschreibung());
					_ausgabe.writeEndElement(); // </div>
				}
				_ausgabe.flush();
			} catch (XMLStreamException e) {
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
						XML_STREAM_ERR, e);
			}
		}
	}

	/**
	 * Rekursive Methode. Erzeuge die Ausgabe fr die Instanz der Klasse Inhalt.
	 * 
	 * @param inhalt
	 *            Die Liste _listeInhalte vom Parameter inhalt wird untersucht
	 *            und Textangabe von jedem Typ wird zu Ausgabe addiert.
	 * 
	 * @return erzeugte Ausgabe.
	 */
	private void erzeugeAusgabeFuerInhalt(Inhalt inhalt) {
		// Formatierung
		/*
		 * tief, hoch, code, adresse, pre, wichtig, zitat, absatz, index,
		 * hinweis, kommentar, verweis, anker, abbildung, liste, numListe,
		 * referenz, tabelle, titel, listenPunkt, spalte, kopf, rumpf, zeile,
		 * zelle, startInfo, unbekannt
		 */

		try {
			if (inhalt != null) {
				InhaltTyp typ = inhalt.getTyp();

				switch (typ) {
				case startInfo: // markiert Start einer Info (ohne Inhalt)
					break;
				case string:
					// Alle Tabs und Leerzeichen am Anfang und Ende werden von
					// Text gelscht.
					// (fr code, pre sollte die Formatierung bleiben.)
					// ausgabe=ausgabe+inhalt.getText().replaceAll("\t",
					// "").trim(); //irrelevant fr HTML
					_ausgabe.writeCharacters(inhalt.getText());
					break;
				case tief:
					_ausgabe.writeStartElement("sub");
					break;
				case hoch:
					_ausgabe.writeStartElement("sup");
					break;
				case code:
					_ausgabe.writeStartElement("code");
					break;
				case pre:
					_ausgabe.writeStartElement("pre");
					break;
				case adresse:
					_ausgabe.writeStartElement("address");
					break;
				case index: // HTML-Tag unklar(?)
					String sichtbar = inhalt.getIndexSichtbar();
					String id = inhalt.getIndexID();
					// ausgabe.writeStartElement("<?>"); //HTML-Tag(?)
					Log.erzeugeMeldung(Log.INFO, Activator.PLUGIN_ID,
							TYP_INDEX_INFO);
					break;
				case hinweis: // HTML-Tag unklar(?)
					String hinweisPid = inhalt.getHinweisPID();
					String hinweisTyp = inhalt.getHinweisTyp();
					Log.erzeugeMeldung(Log.INFO, Activator.PLUGIN_ID,
							TYP_HINWEIS_INFO);
					break;
				case kommentar: // HTML-Tag unklar(?)
					String titel = inhalt.getKommentarTitel();
					String status = inhalt.getKommentarStatus();
					Log.erzeugeMeldung(Log.INFO, Activator.PLUGIN_ID,
							TYP_KOMMENTAR_INFO);
					break;
				case verweis:
					String pid = inhalt.getVerweisPID();
					String text = inhalt.getVerweisTxt();
					String num = inhalt.getVerweisNum(); // HTML Attribut(?)
					String klammer = inhalt.getVerweisKlammer(); // HTML
																	// Attribut(?)
					String direkt = inhalt.getVerweisDirekt(); // HTML
																// Attribut(?)
					String tooltip = inhalt.getVerweisTooltip();
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("href", pid);
					if (tooltip != null)
						_ausgabe.writeAttribute("title", tooltip);
					if (text != null)
						_ausgabe.writeCharacters(text);
					else
						_ausgabe.writeCharacters(pid);
					if (tooltip != null)
						_ausgabe.writeAttribute("title", tooltip);
					break;
				case anker:
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("name", inhalt.getAnkerPID());
					if (inhalt.getAnkerTxt() != null)
						_ausgabe.writeCharacters(inhalt.getAnkerTxt());
					else
						_ausgabe.writeCharacters(inhalt.getAnkerPID());
					break;
				case wichtig:
					_ausgabe.writeStartElement("em"); // wichtig
					// falls <em> kursiv bedeutet(?)
					// ausgabe.writeStartElement("strong"); //besonders wichtig
					break;
				case zitat:
					_ausgabe.writeStartElement("blockquote");
					break;
				case absatz:
					_ausgabe.writeStartElement("p");
					break;
				case abbildung:
					_ausgabe.writeEmptyElement("img");
					_ausgabe.writeAttribute("src", inhalt.getAbbildungScr());
					if (inhalt.getAbbildungPID() != null)
						_ausgabe.writeAttribute("name", inhalt
								.getAbbildungPID());
					if (inhalt.getAbbildungAlt() != null)
						_ausgabe
								.writeAttribute("alt", inhalt.getAbbildungAlt());
					if (inhalt.getAbbildungHoehe() != null)
						_ausgabe.writeAttribute("height", inhalt
								.getAbbildungHoehe());
					if (inhalt.getAbbildungWeite() != null) {
						_ausgabe.writeAttribute("width", inhalt
								.getAbbildungWeite());
					} else {
						// Ausgabe der Bilder auf 100%, wenn keine andere Angabe
						// vorhanden.
						// _ausgabe.writeAttribute("width", "100%");
					}
					break;
				case liste:
					String zeichen = inhalt.getListeZeichen();
					_ausgabe.writeStartElement("ul");
					if (zeichen.equals("punkt"))
						_ausgabe.writeAttribute("type", "disc");
					else if (zeichen.equals("eckig"))
						_ausgabe.writeAttribute("type", "square");
					else if (zeichen.equals("kreis"))
						_ausgabe.writeAttribute("type", "circle");
					break;
				case numListe:
					_ausgabe.writeStartElement("ol");
					_ausgabe.writeAttribute("type", inhalt.getNumListeTyp());
					if (inhalt.getNumListeStart() != null)
						_ausgabe.writeAttribute("start", inhalt
								.getNumListeStart());
					break;
				case listenPunkt:
					_ausgabe.writeStartElement("li");
					break;
				case referenz: // HTML-Tag unklar(?)
					String refPid = inhalt.getReferenzPID();
					String link = inhalt.getReferenzLink();
					String ref = inhalt.getReferenzInReferenzverzeichnis();
					Log.erzeugeMeldung(Log.INFO, Activator.PLUGIN_ID,
							TYP_REFERENZ_INFO);
					break;
				case titel: // HTML-Attribut(?)
					_ausgabe.writeStartElement("p");
					_ausgabe.writeAttribute("class", "titel");
					break;
				case tabelle:
					String tabPid = inhalt.getTabellePID();
					String transponiert = inhalt.getTabelleTransponiert(); // HTML-Attribut(?)
					_ausgabe.writeStartElement("table");
					if (tabPid != null)
						_ausgabe.writeAttribute("id", tabPid);
					_ausgabe
							.writeAttribute("border", inhalt.getTabelleRahmen());
					if (inhalt.getTabelleAusrichtung().equals("l"))
						_ausgabe.writeAttribute("align", "left");
					else if (inhalt.getTabelleAusrichtung().equals("r"))
						_ausgabe.writeAttribute("align", "right");
					else if (inhalt.getTabelleAusrichtung().equals("m"))
						_ausgabe.writeAttribute("align", "center");
					else if (inhalt.getTabelleAusrichtung().equals("s")) {
						// _ausgabe.writeAttribute("align", "???");
						// //HTML-Entsprechung(?)
						String meldung = "Bei der HTML-Ausgabe von 'Info' fr den Typ 'tabelle' gibt es gibt keine Entsprechung in HTML fr die Tabellenausrichtung 's'.";
						Log.erzeugeMeldung(Log.INFO, Activator.PLUGIN_ID,
								meldung);
					}
					break;
				case spalte:
					_ausgabe.writeStartElement("col");
					if (inhalt.getSpalteBreite() != null)
						_ausgabe.writeAttribute("width", inhalt
								.getSpalteBreite());
					if (inhalt.getSpalteHintergrund() != null)
						_ausgabe.writeAttribute("bgcolor", inhalt
								.getSpalteHintergrund());
					break;
				case kopf:
					_ausgabe.writeStartElement("thead");
					break;
				case rumpf:
					_ausgabe.writeStartElement("tbody");
					break;
				case zeile:
					_ausgabe.writeStartElement("tr");
					break;
				case zelle:
					if (inhalt.getZelleKopf() != null) {
						if (inhalt.getZelleKopf().equals("j")) {
							_ausgabe.writeStartElement("th");
						}
					} else {
						_ausgabe.writeStartElement("td");
					}
					if (inhalt.getZelleAusrichtung() != null) {
						if (inhalt.getZelleAusrichtung().equals("l"))
							_ausgabe.writeAttribute("align", "left");
						else if (inhalt.getTabelleAusrichtung().equals("r"))
							_ausgabe.writeAttribute("align", "right");
						else if (inhalt.getTabelleAusrichtung().equals("m"))
							_ausgabe.writeAttribute("align", "center");
					}
					if (inhalt.getZelleVertikal() != null) {
						if (inhalt.getZelleVertikal().equals("o"))
							_ausgabe.writeAttribute("valign", "top");
						else if (inhalt.getZelleVertikal().equals("m"))
							_ausgabe.writeAttribute("valign", "middle");
						else if (inhalt.getZelleVertikal().equals("u"))
							_ausgabe.writeAttribute("valign", "bottom");
					}
					if (inhalt.getZelleHintergrund() != null) {
						_ausgabe.writeAttribute("bgcolor", inhalt
								.getZelleHintergrund());
					}
					if (inhalt.getZelleBreite() != null) {
						_ausgabe.writeAttribute("width", inhalt
								.getZelleBreite());
					}
					if (inhalt.getZelleAnzSpalten() != null) {
						_ausgabe.writeAttribute("colspan", inhalt
								.getZelleAnzSpalten());
					}
					if (inhalt.getZelleAnzZeilen() != null) {
						_ausgabe.writeAttribute("rowspan", inhalt
								.getZelleAnzZeilen());
					}
					break;
				case sektion:
					String sektionPid = inhalt.getSektionPid();
					if (sektionPid != null) {
						_ausgabe.writeStartElement("div");
						_ausgabe.writeAttribute("id", sektionPid);
					}
					break;
				case unbekannt:
					Log.erzeugeMeldung(Log.INFO, Activator.PLUGIN_ID,
							UNBEKANNTER_TYP_ERR);
					break;
				default:
					// _ausgabe.writeCharacters("default typ: "+typ);
					_ausgabe.writeEntityRef("nbsp");
				}

				for (Inhalt inhalt2 : inhalt.getListeInhalte()) {
					erzeugeAusgabeFuerInhalt(inhalt2);
				}
				/*
				 * // Keinen Zeilenumbruch fr folgende Typen: automatisch in
				 * HTML(?) if ((typ != InhaltTyp.tief) && (typ !=
				 * InhaltTyp.hoch) && (typ != InhaltTyp.adresse) && (typ !=
				 * InhaltTyp.wichtig)) { if (!ausgabe2.equals("")) {
				 * _ausgabe.writeCharacters("\n");
				 * //_ausgabe.writeEmptyElement("br"); } }
				 */
				switch (typ) {
				case tief:
					_ausgabe.writeEndElement(); // </sub>
					break;
				case hoch:
					_ausgabe.writeEndElement(); // </sup>
					break;
				case code:
					_ausgabe.writeEndElement(); // </code>
					break;
				case pre:
					_ausgabe.writeEndElement(); // </pre>
					break;
				case adresse:
					_ausgabe.writeEndElement(); // </address>
					break;
				case index: // HTML-Tag unklar(?)
					// _ausgabe.writeEndElement();
					break;
				case verweis:
					_ausgabe.writeEndElement(); // </a>
					break;
				case anker:
					_ausgabe.writeEndElement(); // </a>
					break;
				case wichtig:
					_ausgabe.writeEndElement(); // </em>
					break;
				case zitat:
					_ausgabe.writeEndElement(); // </blockquote>
					break;
				case absatz:
					_ausgabe.writeEndElement(); // </p>
					break;
				case abbildung:
					// _ausgabe.writeEndElement(); //</img>
					break;
				case liste:
					_ausgabe.writeEndElement(); // </ul>
					break;
				case numListe:
					_ausgabe.writeEndElement(); // </ol>
					break;
				case listenPunkt:
					_ausgabe.writeEndElement(); // </li>
					break;
				case referenz: // HTML-Tag unklar(?)
					// keine Verarbeitung(?)
					// ausgabe.writeEndElement();
					break;
				case titel:
					_ausgabe.writeEndElement(); // </p>
					break;
				case tabelle:
					_ausgabe.writeEndElement(); // </table>
					break;
				case spalte:
					_ausgabe.writeEndElement(); // </col>
					break;
				case kopf:
					_ausgabe.writeEndElement(); // </thead>
					break;
				case rumpf:
					_ausgabe.writeEndElement(); // </tbody>
					break;
				case zeile:
					_ausgabe.writeEndElement(); // <tr>
					break;
				case zelle:
					_ausgabe.writeEndElement(); // </td> bzw. </th>
					break;
				case sektion:
					if (inhalt.getSektionPid() != null)
						_ausgabe.writeEndElement(); // </div>
					break;
				}
			}
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	/**
	 * Erzeugt eine Tabelle mit der Kopfzeile.
	 * 
	 * @param arKopfzeile
	 *            Titel jeder Spalte.
	 * @param listeZeilen
	 *            Liste der Zeilen. Jede Zeile ist ein Array von Objekten (jedes
	 *            element kann von Typ String, oder KurzInfo sein).
	 */
	private void erzeugeTabelle(String[] arKopfzeile,
			List<Object[]> listeZeilen, String bezeichnung, String klasse) {
		try {
			if (!bezeichnung.isEmpty()) {
				if (bezeichnung.equals(ASP_DEF) || bezeichnung.equals(TYP_DEF)
						|| bezeichnung.equals(MENGE_DEF)
						|| bezeichnung.equals(ATG_DEF)
						|| bezeichnung.equals(ATL_DEF)
						|| bezeichnung.equals(ATT_DEF)
						|| bezeichnung.equals(KFG_OBJ)
						|| bezeichnung.equals(AEND_UEB)) {
					_ausgabe.writeStartElement("h3");
					// Link zur Konfigurationsvorgabe
					einfuegenLinkZurKonfigurationsVorgabe();
					_ausgabe.writeEntityRef("nbsp");
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("name", bezeichnung);
					_ausgabe.writeCharacters(bezeichnung);
					_ausgabe.writeEndElement(); // </a>
				} else if (bezeichnung.equals("Zustnde")
						|| bezeichnung.equals("Bereiche")) {
					_ausgabe.writeStartElement("h6");
					_ausgabe.writeCharacters(bezeichnung);
				} else {
					_ausgabe.writeStartElement("h5");
					_ausgabe.writeEntityRef("nbsp");
					_ausgabe.writeStartElement("a");
					_ausgabe.writeAttribute("name", bezeichnung);
					_ausgabe.writeCharacters(bezeichnung);
					_ausgabe.writeEndElement(); // </a>
				}

				_ausgabe.writeEndElement(); // </h...>
			}
			erzeugeKopfzeile(arKopfzeile, klasse);
			erzeugeTabelleOhneKopfzeile(arKopfzeile.length, listeZeilen, klasse);
			_ausgabe.writeEndElement(); // </table>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	/**
	 * Fgt einen Link auf die Konfigurationsvorgabe (Startdatei) in den
	 * Ausgabestream ein.
	 */
	private void einfuegenLinkZurKonfigurationsVorgabe() {
		try {
			// Link zur Konfigurationsvorgabe
			_ausgabe.writeStartElement("a");
			_ausgabe.writeAttribute("href", "start.html#"
					+ "Konfigurationsvorgabe");
			_ausgabe.writeEmptyElement("img");
			_ausgabe.writeAttribute("src", IMG_NAV_HOCH_UEBERSICHT);
			_ausgabe.writeAttribute("alt", "hoch");
			_ausgabe.writeAttribute("title", "...zur Gesamtbersicht");
			_ausgabe.writeAttribute("border", "0");
			_ausgabe.writeEndElement(); // </a>
		} catch (XMLStreamException e) {
			System.out.println(XML_STREAM_ERR);
			e.printStackTrace();
		}
	}

	/**
	 * Erzeuge die Kopfzeile einer Tabelle.
	 * 
	 * @param arKopfzeile
	 *            Titel jeder Spalte.
	 * @param klasse
	 *            die Bezeichnung des Attributs 'class' fr den CSS-Style
	 */
	private void erzeugeKopfzeile(String[] arKopfzeile, String klasse) {
		try {
			_ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("table");
			_ausgabe.writeAttribute("class", klasse);
			_ausgabe.writeStartElement("tr");

			for (int i = 0; i < arKopfzeile.length; i++) {
				_ausgabe.writeStartElement("th");
				_ausgabe.writeCharacters(arKopfzeile[i]);
				_ausgabe.writeEndElement(); // </th>
			}
			_ausgabe.writeEndElement(); // </tr>
			// _ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	/**
	 * Erzeugt eine Tabelle ohne Kopfzeile.
	 * 
	 * @param spaltenAnzahl
	 *            Anzahl der Spalten.
	 * @param listeZeilen
	 *            Liste der Zeilen. Jede Zeile ist ein Array von Objekten (jedes
	 *            element kann von Typ String, oder KurzInfo sein).
	 * @param klasse
	 *            CSS-Klasse fr dieses Element.
	 */
	private void erzeugeTabelleOhneKopfzeile(int spaltenAnzahl,
			List<Object[]> listeZeilen, String klasse) {
		// Ausgabe der Zeilen:
		for (Object[] zeile : listeZeilen) {
			if (zeile.length != spaltenAnzahl) {
				String meldung = "Anzahl der Spalten in der Zeile ist ungleich der Spaltenanzahl in der auszugebenden Tabelle!";
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung);
			} else {
				try {
					_ausgabe.writeStartElement("tr");
					// optional: Formatierung mittels CSS Klasse
					// _ausgabe.writeAttribute("class", klasse);
					for (int i = 0; i < spaltenAnzahl; i++) {
						_ausgabe.writeStartElement("td");
						// optional: Formatierung mittels CSS Klasse
						// _ausgabe.writeAttribute("class", klasse);
						// Ausgabe von Name des Elements bei Einrckung:
						if (zeile[i] instanceof Name) {
							Name name = (Name) zeile[i];
							ElementTyp typ = name.getTyp();
							if (name != null) {
								if (typ != null) {
									int leerzeichen = 0;
									while (leerzeichen < name.getNiveau()) {
										// _ausgabe.writeEntityRef("nbsp");
										// falls Einrckung durch leeres Image
										_ausgabe.writeEmptyElement("img");
										_ausgabe.writeAttribute("src",
												"img/einrueckung.png");
										_ausgabe.writeAttribute("alt",
												"Einrckung");
										_ausgabe.writeAttribute("title",
												"Einrckung");
										// }
										leerzeichen++;
									}
									_ausgabe.writeEmptyElement("img");
									switch (typ) {
									case att:
										_ausgabe.writeAttribute("src",
												"img/gelb.png");
										_ausgabe.writeAttribute("alt",
												"Attribut");
										_ausgabe.writeAttribute("title",
												"Attribut");
										break;
									case ata:
										_ausgabe.writeAttribute("src",
												"img/grau.png");
										_ausgabe.writeAttribute("alt",
												"Attributauswahl");
										_ausgabe.writeAttribute("title",
												"Attributauswahl");
										break;
									case atl:
										_ausgabe.writeAttribute("src",
												"img/blau.png");
										_ausgabe.writeAttribute("alt",
												"Attributliste");
										_ausgabe.writeAttribute("title",
												"Attributliste");
										break;
									case datum:
										_ausgabe.writeAttribute("src",
												"img/gelb.png");
										_ausgabe.writeAttribute("alt", "Datum");
										_ausgabe.writeAttribute("title",
												"Datum");
										break;
									case dfeld:
										_ausgabe.writeAttribute("src",
												"img/grau.png");
										_ausgabe.writeAttribute("alt",
												"Datenfeld");
										_ausgabe.writeAttribute("title",
												"Datenfeld");
										break;
									case dliste:
										_ausgabe.writeAttribute("src",
												"img/blau.png");
										_ausgabe.writeAttribute("alt",
												"Datenliste");
										_ausgabe.writeAttribute("title",
												"Datenliste");
										break;
									}
								}
								_ausgabe.writeEntityRef("nbsp");
								_ausgabe.writeCharacters(name.getName());
							} else {
								_ausgabe.writeEntityRef("nbsp");
							}
						}
						// Ausgabe von InfoBeschreibung:
						else if (zeile[i] instanceof InfoBeschreibung) {
							// _ausgabe.writeStartElement("td");
							InfoBeschreibung infoB = (InfoBeschreibung) zeile[i];
							if (infoB != null) {
								// KurzInfo:
								if (infoB.getKurzinfo() != null) {
									_ausgabe.writeStartElement("div");
									_ausgabe
											.writeAttribute("class", "kurzinfo");
									erzeugeAusgabeFuerInhalt(infoB
											.getKurzinfo());
									_ausgabe.writeEndElement(); // </div>
								}
								// Beschreibung:
								if (infoB.getBeschreibung() != null) {
									_ausgabe.writeStartElement("div");
									_ausgabe.writeAttribute("class",
											"beschreibung");
									erzeugeAusgabeFuerInhalt(infoB
											.getBeschreibung());
									_ausgabe.writeEndElement(); // </div>
								}
							} else {
								_ausgabe.writeEntityRef("nbsp");
							}
							// _ausgabe.writeEndElement(); //</td>
						}
						// Ausgabe von Info, falls aus Definition des Elements:
						else if (zeile[i] instanceof Info) {
							_ausgabe.writeAttribute("class", "infoelementdef"); // ("td");
							InfoBeschreibung infoB = ((Info) zeile[i])
									.getInfo();
							if (infoB != null) {
								// KurzInfo:
								if (infoB.getKurzinfo() != null) {
									erzeugeAusgabeFuerInhalt(infoB
											.getKurzinfo());
								}
								// Beschreibung:
								if (infoB.getBeschreibung() != null) {
									erzeugeAusgabeFuerInhalt(infoB
											.getBeschreibung());
								}
							} else {
								_ausgabe.writeEntityRef("nbsp");
							}
							// _ausgabe.writeEndElement(); //</td>
						}
						// Ausgabe von Inhalt-Element:
						else if (zeile[i] instanceof Inhalt) {
							// _ausgabe.writeStartElement("td");
							Inhalt inhalt = (Inhalt) zeile[i];
							if (inhalt != null) {
								erzeugeAusgabeFuerInhalt(inhalt);
							} else {
								_ausgabe.writeEntityRef("nbsp");
							}
						}
						// Ausgabe von Referenz PID/Name
						else if (zeile[i] instanceof Uebersicht) {
							String pid = ((Uebersicht) zeile[i]).getPid();
							String kbPid = _refInfo.ermittleKbPid(pid);
							String kvPid = _refInfo.getMapAlleKbKv().get(kbPid);

							if (!kbPid.isEmpty()) {
								String name = ((Uebersicht) zeile[i]).getName();
								_ausgabe.writeStartElement("a");
								_ausgabe.writeAttribute("href", kbPid
										+ ".html#" + pid);
								if (name == null || name.isEmpty()) {
									einfuegenToolTipKbKv(kbPid, kvPid);
									_ausgabe.writeCharacters(pid);
								} else {
									einfuegenToolTipKbKv(kbPid, kvPid);
									_ausgabe.writeCharacters(name);
								}
								_ausgabe.writeEndElement(); // </a>
							} else {
								// _ausgabe.writeCharacters(pid);
								_ausgabe.writeCharacters(pid
										+ " nicht referenzierbar");
							}
							// _ausgabe.writeEndElement(); //</td>
						}
						// Ausgabe von Symbol 'Konfigurierend' bei
						// Attributgruppen
						else if (zeile[i] instanceof AtgKonfigurierend) {
							_ausgabe.writeAttribute("align", "center");
							_ausgabe.writeEmptyElement("img");

							switch ((AtgKonfigurierend) zeile[i]) {
							case ja:
								_ausgabe.writeAttribute("src", "img/ja.gif");
								_ausgabe.writeAttribute("alt", "ja");
								_ausgabe.writeAttribute("title", "ja");
								break;
							case nein:
								_ausgabe.writeAttribute("src", "img/nein.png");
								_ausgabe.writeAttribute("alt", "nein");
								_ausgabe.writeAttribute("title", "nein");
								break;
							case unbekannt:
								_ausgabe.writeAttribute("src",
										"img/unbekannt.png");
								_ausgabe.writeAttribute("alt", "unbekannt");
								_ausgabe.writeAttribute("title", "unbekannt");
							}
						}
						// Ausgabe von Symbol 'Parametrierend' bei
						// Attributgruppen
						else if (zeile[i] instanceof AtgParametrierend) {
							_ausgabe.writeAttribute("align", "center");
							_ausgabe.writeEmptyElement("img");

							switch ((AtgParametrierend) zeile[i]) {
							case ja:
								_ausgabe.writeAttribute("src", "img/ja.gif");
								_ausgabe.writeAttribute("alt", "ja");
								_ausgabe.writeAttribute("title", "ja");
								break;
							case nein:
								_ausgabe.writeAttribute("src", "img/nein.png");
								_ausgabe.writeAttribute("alt", "nein");
								_ausgabe.writeAttribute("title", "nein");
								break;
							case unbekannt:
								_ausgabe.writeAttribute("src",
										"img/unbekannt.png");
								_ausgabe.writeAttribute("alt", "unbekannt");
								_ausgabe.writeAttribute("title", "unbekannt");
							}
						}
						// Ausgabe von PID bei TypBeschreibung:
						else if (zeile[i] instanceof TypBeschreibung) {
							String pid = ((TypBeschreibung) zeile[i]).getPid();
							// _ausgabe.writeStartElement("td");
							_ausgabe.writeCharacters(pid);
							// _ausgabe.writeEndElement();
						}
						// Ausgabe von PID bei AttributgruppenBeschreibung:
						else if (zeile[i] instanceof AttributgruppenBeschreibung) {
							String pid = ((AttributgruppenBeschreibung) zeile[i])
									.getPid();
							// _ausgabe.writeStartElement("td");
							_ausgabe.writeCharacters(pid);
							// _ausgabe.writeEndElement();
						}
						// Ausgabe von PID/Name bei MengenBeschreibung:
						else if (zeile[i] instanceof MengenBeschreibung) {
							String pid = ((MengenBeschreibung) zeile[i])
									.getUebersicht().getPid();
							String name = ((MengenBeschreibung) zeile[i])
									.getUebersicht().getName();
							if (name.isEmpty())
								_ausgabe.writeCharacters(pid);
							else
								_ausgabe.writeCharacters(name);
						}
						// Ausgabe von PID/Name bei AspektBeschreibung:
						else if (zeile[i] instanceof AspektBeschreibung) {
							String pid = ((AspektBeschreibung) zeile[i])
									.getUebersicht().getPid();
							String name = ((AspektBeschreibung) zeile[i])
									.getUebersicht().getName();
							if (name.isEmpty())
								_ausgabe.writeCharacters(pid);
							else
								_ausgabe.writeCharacters(name);
						}
						// Ausgabe von Element der Zeile:
						else if (zeile[i] instanceof String) {
							String text = (String) zeile[i];
							// _ausgabe.writeStartElement("td");
							if (!text.isEmpty())
								_ausgabe.writeCharacters(text);
							else
								_ausgabe.writeEntityRef("nbsp");
							// _ausgabe.writeEndElement();
						} else if (zeile[i] == null) {
							// _ausgabe.writeStartElement("td");
							_ausgabe.writeEntityRef("nbsp");
							// _ausgabe.writeEndElement();
						}
						_ausgabe.writeEndElement(); // </td>
					}
					_ausgabe.writeEndElement(); // </tr>
					_ausgabe.flush();
				} catch (XMLStreamException e) {
					Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
							XML_STREAM_ERR, e);
				}
			}
		}
	}

	/**
	 * Fgt einen ToolTip mit KB und KV in den Ausgabestream ein.
	 */
	private void einfuegenToolTipKbKv(String kbPid, String kvPid) {
		String toolTip = "KB: " + kbPid + "\nKV: " + kvPid;

		try {
			_ausgabe.writeAttribute("title", toolTip);
		} catch (XMLStreamException e) {
			System.out.println(XML_STREAM_ERR);
			e.printStackTrace();
		}
	}

	/**
	 * <!ELEMENT typDefinition ( info, (erweitert+ | basis), (attributgruppe |
	 * menge)*, defaultParameter* ) > <!ATTLIST typDefinition pid ID #REQUIRED
	 * name CDATA #IMPLIED objektNamenPermanent (ja|nein) "nein" persistenzModus (
	 * transient | persistent | persistentUndUngltigNachNeustart ) #IMPLIED >
	 * 
	 * <!ELEMENT basis EMPTY > <!ATTLIST basis konfigurierend (ja|nein)
	 * #REQUIRED >
	 * 
	 * @param typDefB
	 *            Typdefinition
	 * @param pidNachfolger
	 *            PID des Nachfolgers
	 * @param pidVorgaenger
	 *            Pid Ddes Vrorgngers
	 * 
	 * @return MultiStatus Meldung
	 */
	private MultiStatus ausgebenTypDefinitionBeschreibung(
			TypDefinitionBeschreibung typDefB, String pidVorgaenger,
			String pidNachfolger) {
		MultiStatus multiStatus = new MultiStatus(Activator.PLUGIN_ID, 0,
				"Referenzen fr den Typ " + typDefB.getUebersicht().getPid(),
				null);
		List<Object[]> listeZeilen = new ArrayList<Object[]>();

		ausgebenUebersicht(typDefB.getUebersicht(), pidVorgaenger,
				pidNachfolger, TYP_DEF, "typdefuebersicht");
		ausgebenTypDefBeschreibungSpezial(typDefB);

		try {
			if (typDefB.getBasisKonfigurierend().equals("ja")) {
				_ausgabe.writeStartElement("p");
				_ausgabe
						.writeCharacters("Dieser Typ ist ein konfigurierender Basistyp.");
				_ausgabe.writeEndElement();
			}
			if (!typDefB.getListeErweitert().isEmpty()) {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h5");
				_ausgabe.writeCharacters("Vererbung");
				_ausgabe.writeEndElement();
				// _ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("p");
				_ausgabe
						.writeCharacters("Dieser Typ erweitert folgende Typen:");
				_ausgabe.writeEndElement();

				for (TypBeschreibung typB : typDefB.getListeErweitert()) {
					if (typB.getTypDefinitionBeschreibung() != null) {
						Uebersicht uebersicht = typB
								.getTypDefinitionBeschreibung().getUebersicht();
						InfoBeschreibung info = uebersicht.getInfo();
						listeZeilen.add(new Object[] { uebersicht, info });
					} else {
						listeZeilen.add(new Object[] { typB, OHNE_REF });
						multiStatus
								.add(new Status(
										IStatus.WARNING,
										Activator.PLUGIN_ID,
										"Referenz auf den erweiterten Typ ["
												+ typB.getPid()
												+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
					}
				}
				erzeugeTabelle(new String[] { "Name", "Info" }, listeZeilen,
						"", "typdeftyp");
			}

			listeZeilen = new ArrayList<Object[]>();
			if (!typDefB.getListeAttributgruppenBeschreibung().isEmpty()) {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h5");
				_ausgabe.writeCharacters("Attributgruppen");
				_ausgabe.writeEndElement();
				// _ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("p");
				_ausgabe
						.writeCharacters("Folgende Attributgruppen sind bei diesem Objekttyp erlaubt:");
				_ausgabe.writeEndElement();

				for (AttributgruppenBeschreibung atgB : typDefB
						.getListeAttributgruppenBeschreibung()) {
					AttributgruppenDefinitionBeschreibung atgDefB = atgB
							.getAttributgruppenDefinitionBeschreibung();
					if (atgDefB != null) {
						AtgKonfigurierend atgKonfig;
						String konfigurierend = atgDefB.getKonfigurierend();
						if (konfigurierend.equals("ja"))
							atgKonfig = AtgKonfigurierend.ja;
						else if (konfigurierend.equals("nein"))
							atgKonfig = AtgKonfigurierend.nein;
						else
							atgKonfig = AtgKonfigurierend.unbekannt;

						AtgParametrierend atgParam;
						String parametrierend = atgDefB.getParametrierend();
						if (parametrierend.equals("ja"))
							atgParam = AtgParametrierend.ja;
						else if (konfigurierend.equals("nein"))
							atgParam = AtgParametrierend.nein;
						else
							atgParam = AtgParametrierend.unbekannt;

						Uebersicht uebersicht = atgDefB.getUebersicht();
						InfoBeschreibung info = uebersicht.getInfo();
						listeZeilen.add(new Object[] { atgKonfig, atgParam,
								uebersicht, info });
					} else {
						listeZeilen.add(new Object[] {
								AtgKonfigurierend.unbekannt,
								AtgParametrierend.unbekannt, atgB, OHNE_REF });
						multiStatus
								.add(new Status(
										IStatus.WARNING,
										Activator.PLUGIN_ID,
										"Referenz auf die Attributgruppe ["
												+ atgB.getPid()
												+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
					}
				}
				String[] arKopfzeile = new String[] { "Konfigurierend",
						"Parametrierend", "Name", "Info" };
				erzeugeTabelle(arKopfzeile, listeZeilen, "", "typdefatg");
			}

			listeZeilen = new ArrayList<Object[]>();
			if (!typDefB.getListeMengenBeschreibung().isEmpty()) {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h5");
				_ausgabe.writeCharacters("Mengen");
				_ausgabe.writeEndElement();
				// _ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("p");
				_ausgabe
						.writeCharacters("Folgende Mengen mssen (knnen) bei diesem Objekttyp vorhanden sein:");
				_ausgabe.writeEndElement();

				for (MengenBeschreibung mengenB : typDefB
						.getListeMengenBeschreibung()) {
					Uebersicht uebersicht = mengenB.getUebersicht();
					String name = mengenB.getUebersicht().getName();
					String erforderlich = mengenB.getErforderlich();
					InfoBeschreibung infoB = mengenB.getUebersicht().getInfo();
					// Name und Info evtl. aus MengenDefinitionBeschreibung
					// holen
					if (mengenB.getMengenDefinitionBeschreibung() != null) {
						// Kann laut DTD nie der Fall sein!!!
						// if (name.isEmpty()) uebersicht =
						// mengenB.getMengenDefinitionBeschreibung().getUebersicht();
						// if (name.isEmpty()) name = uebersicht.getName();
						if (infoB == null) {
							Info info = new Info(uebersicht.getInfo());
							listeZeilen.add(new Object[] { uebersicht,
									erforderlich, info });
						} else if (infoB != null) {
							listeZeilen.add(new Object[] { uebersicht,
									erforderlich, infoB });
						}
					} else {
						// Keine Info
						listeZeilen.add(new Object[] { mengenB, erforderlich,
								OHNE_REF });
						multiStatus
								.add(new Status(
										IStatus.WARNING,
										Activator.PLUGIN_ID,
										"Referenz auf die Mengen ["
												+ mengenB.getUebersicht()
														.getPid()
												+ "] kann in der aktuellen Konfigurationsvorgabe nicht aufgelst werden."));
					}
				}
				// _ausgabe.writeEmptyElement("br");
				String[] arKopfzeile = new String[] { "Name", "erforderlich",
						"Info" };
				erzeugeTabelle(arKopfzeile, listeZeilen, "", "typdefmenge");
			}

			if (!typDefB.getListeDefaultParameterBeschreibung().isEmpty()) {
				_ausgabe.writeEmptyElement("br");
				_ausgabe.writeStartElement("h5");
				_ausgabe.writeCharacters("Default Parameter");
				_ausgabe.writeEndElement();
				for (DefaultParameterBeschreibung defParB : typDefB
						.getListeDefaultParameterBeschreibung()) {
					ausgebenDefaultParameterBeschreibung(defParB,
							"typdefdefaultparam");
				}
			}
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
		return multiStatus;
	}

	private void ausgebenTypDefBeschreibungSpezial(
			TypDefinitionBeschreibung typDefB) {
		try {
			_ausgabe.writeEmptyElement("br");
			_ausgabe.writeStartElement("table");
			_ausgabe.writeAttribute("class", "typdefspezial");
			/*
			 * // Ausgabe optional, falls das Attribut ausgegeben werden soll
			 * //if (typDefB.getBasisKonfigurierend().equals("ja")) {
			 * _ausgabe.writeStartElement("tr");
			 * _ausgabe.writeStartElement("td"); _ausgabe.writeCharacters("Basis
			 * konfigurierend:"); _ausgabe.writeEndElement(); //<td>
			 * _ausgabe.writeStartElement("td");
			 * _ausgabe.writeCharacters(typDefB.getBasisKonfigurierend());
			 * _ausgabe.writeEndElement(); //<td> _ausgabe.writeEndElement(); //<tr> }
			 */
			_ausgabe.writeStartElement("tr");
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters("ObjektNamenPermanent:");
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters(typDefB.getObjektNamenPermanent());
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeEndElement(); // <tr>

			_ausgabe.writeStartElement("tr");
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters("PersistenzModus:");
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters(typDefB.getPersistenzModus());
			_ausgabe.writeEndElement(); // <td>
			_ausgabe.writeEndElement(); // <tr>
			_ausgabe.writeEndElement(); // </table>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	/**
	 * <!ELEMENT defaultParameter ((datum|datenliste|datenfeld)+) > <!ATTLIST
	 * defaultParameter typ NMTOKEN #IMPLIED attributgruppe NMTOKEN #REQUIRED >
	 * 
	 * @param defParB
	 * @param klasse
	 *            TODO
	 */
	private void ausgebenDefaultParameterBeschreibung(
			DefaultParameterBeschreibung defParB, String klasse) {
		try {
			List<Object[]> listeZeilen = new ArrayList<Object[]>();
			// Referenz fr Typ
			Uebersicht typUebersicht = defParB.getTypUebersicht();
			String typ = "-";
			if (typUebersicht != null) {
				typ = typUebersicht.getName();
				listeZeilen.add(new Object[] { "Typ:", typUebersicht });
			} else {
				listeZeilen.add(new Object[] { "Typ:", NICHT_ERMITTELBAR });
			}

			// Referenz fr Attributgruppe
			String atg = "-";
			Uebersicht atgUebersicht = null;
			AttributgruppenBeschreibung atgB = defParB.getAttributgruppe();
			if (atgB != null) {
				AttributgruppenDefinitionBeschreibung atgDefB = atgB
						.getAttributgruppenDefinitionBeschreibung();
				if (atgDefB != null) {
					atgUebersicht = atgB
							.getAttributgruppenDefinitionBeschreibung()
							.getUebersicht();
				}
			}
			if (atgUebersicht != null) {
				atg = atgUebersicht.getName();
				listeZeilen
						.add(new Object[] { "Attributgruppe:", atgUebersicht });
			} else {
				listeZeilen.add(new Object[] { "Attributgruppe:",
						NICHT_ERMITTELBAR });
			}
			// Typ und Name der Attributgruppe ausgeben
			_ausgabe.writeStartElement("h6");
			_ausgabe.writeCharacters(typ + " : " + atg);
			_ausgabe.writeEndElement(); // </h6>
			_ausgabe.writeStartElement("table");
			_ausgabe.writeAttribute("class", klasse + "spezial");
			erzeugeTabelleOhneKopfzeile(2, listeZeilen, klasse + "spezial");
			_ausgabe.writeEndElement(); // </table>

			String[] arKopfzeile = new String[] { "Name", "Wert" };
			erzeugeKopfzeile(arKopfzeile, klasse);
			ausgebenDatumOderDListeOderDFeld(defParB
					.getListeDatumBeschreibung(), defParB
					.getListeDatenlisteBeschreibung(), defParB
					.getListeDatenfeldBeschreibung(), 0, arKopfzeile.length,
					klasse);

			_ausgabe.writeEndElement(); // </table>
			_ausgabe.flush();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	/**
	 * Rekursive Methode.
	 * 
	 * Erzeugt die Tabelle mit den Daten der Art: (datum|datenliste|datenfeld)+
	 * 
	 * <!ELEMENT datum EMPTY > <!ATTLIST datum name CDATA #REQUIRED wert CDATA
	 * #REQUIRED >
	 * 
	 * <!ELEMENT datenliste ((datum|datenliste|datenfeld)+) > <!ATTLIST
	 * datenliste name CDATA #REQUIRED >
	 * 
	 * <!ELEMENT datenfeld ((datum|datenliste)*) > <!ATTLIST datenfeld name
	 * CDATA #REQUIRED >
	 * 
	 * @param listeDatumB
	 * @param listeDListeB
	 * @param listeDFeldB
	 * @param niveau
	 * @param anzSpalten
	 * @param klasse
	 *            Klasse in CSS
	 */
	private void ausgebenDatumOderDListeOderDFeld(
			List<DatumBeschreibung> listeDatumB,
			List<DatenlisteBeschreibung> listeDListeB,
			List<DatenfeldBeschreibung> listeDFeldB, int niveau,
			int anzSpalten, String klasse) {
		List<Object[]> listeZeilen = new ArrayList<Object[]>();

		// Ausgabe von datum:
		if (listeDatumB.size() > 0) {
			for (DatumBeschreibung datumB : listeDatumB) {
				Name name = new Name(datumB.getName(), niveau, ElementTyp.datum);
				listeZeilen.add(new Object[] { name, datumB.getWert() });
			}
			erzeugeTabelleOhneKopfzeile(anzSpalten, listeZeilen, klasse
					+ "datum");
		}

		// Ausgabe von datenliste:
		if (listeDListeB.size() > 0) {
			listeZeilen.clear();
			for (DatenlisteBeschreibung datenlisteB : listeDListeB) {
				Name name = new Name(datenlisteB.getName(), niveau,
						ElementTyp.dliste);
				listeZeilen.add(new Object[] { name, "" });
				erzeugeTabelleOhneKopfzeile(anzSpalten, listeZeilen, klasse
						+ "datenliste");
				listeZeilen.clear();
				ausgebenDatumOderDListeOderDFeld(datenlisteB
						.getListeDatumBeschreibung(), datenlisteB
						.getListeDatenlisteBeschreibung(), datenlisteB
						.getListeDatenfeldBeschreibung(), niveau + 1,
						anzSpalten, klasse);
			}
		}

		// Ausgabe von datenfeld:
		if (listeDFeldB.size() > 0) {
			listeZeilen.clear();
			for (DatenfeldBeschreibung datenfeldB : listeDFeldB) {
				Name name = new Name(datenfeldB.getName(), niveau,
						ElementTyp.dfeld);
				listeZeilen.add(new Object[] { name, "" });
				erzeugeTabelleOhneKopfzeile(anzSpalten, listeZeilen, klasse
						+ "datenfeld");
				listeZeilen.clear();
				ausgebenDatumOderDListeOderDFeld(datenfeldB
						.getListeDatumBeschreibung(), datenfeldB
						.getListeDatenlisteBeschreibung(),
						new ArrayList<DatenfeldBeschreibung>(), niveau + 1,
						anzSpalten, klasse);
			}
		}
	}

	private void ausgebenZeilenKvKb(String kbV, List<String[]> kbFeld) {
		int rowspan = kbFeld.size();
		try {
			// Ausgabe Konfigurationsverantwortlicher
			_ausgabe.writeStartElement("tr");
			_ausgabe.writeStartElement("td");
			_ausgabe.writeAttribute("rowspan", String.valueOf(rowspan));
			_ausgabe.writeCharacters(kbV);
			_ausgabe.writeEndElement(); // </td>

			// Ausgabe Konfigurationsbereiche pro Konfigurationsverantwortlichen
			for (String[] kbNamePid : kbFeld) {
				_ausgabe.writeStartElement("td");
				_ausgabe.writeStartElement("a");
				// _ausgabe.writeAttribute("href", kbNamePid[1] + ".html#" +
				// kbNamePid[1]);
				_ausgabe.writeAttribute("href", kbNamePid[1] + ".html#" + kbV);
				if (kbNamePid[0] == null || kbNamePid[0].isEmpty())
					_ausgabe.writeCharacters(kbNamePid[1]);
				else
					_ausgabe.writeCharacters(kbNamePid[0]);
				_ausgabe.writeEndElement(); // </a>
				_ausgabe.writeEndElement(); // </td>

				_ausgabe.writeStartElement("td");
				_ausgabe.writeCharacters(kbNamePid[1]);
				_ausgabe.writeEndElement(); // </td>
				_ausgabe.writeEndElement(); // </tr>
				_ausgabe.writeStartElement("tr");
			}
			_ausgabe.writeEndElement(); // </tr>
			kbFeld.clear();
		} catch (XMLStreamException e) {
			Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, XML_STREAM_ERR,
					e);
		}
	}

	/**
	 * Fgt eine Kopfzeile am Anfang des Datenkatalogs ein mit Logos von BW und
	 * inovat.
	 */
	private void einfuegenKopfzeileBwInovat() {
		try {
			// <div align="right"
			// style="background:#fff url('img/BW_200x74.gif') no-repeat;
			// border-bottom:1px solid #09407F;">
			// <img src="img/inovat_200x74.gif" alt="inovat-Logo">
			// </div>

			_ausgabe.writeStartElement("div");
			_ausgabe.writeAttribute("align", "right");
			_ausgabe.writeAttribute("title", TOOL_TIP_AG_AN);
			_ausgabe
					.writeAttribute(
							"style",
							"background:#fff url('img/BW_200x74.gif') no-repeat; border-bottom:1px solid #09407F;");
			_ausgabe.writeEmptyElement("img");
			_ausgabe.writeAttribute("src", "img/inovat_200x74.gif");
			_ausgabe.writeAttribute("alt", "inovat-Logo");
			_ausgabe.writeAttribute("title", TOOL_TIP_AG_AN);
			_ausgabe.writeEndElement(); // </div>
		} catch (XMLStreamException e) {
			System.out.println(XML_STREAM_ERR);
			e.printStackTrace();
		}
	}

	/**
	 * Fgt eine Kopfzeile am Anfang des Datenkatalogs ein mit Logos von Nw und
	 * inovat.
	 */
	private void einfuegenKopfzeileNwInovat() {
		try {
			// <div align="right"
			// style="background:#fff url('img/BW_200x74.gif') no-repeat;
			// border-bottom:1px solid #09407F;">
			// <img src="img/inovat_200x74.gif" alt="inovat-Logo">
			// </div>

			_ausgabe.writeStartElement("div");
			_ausgabe.writeAttribute("align", "right");
			_ausgabe.writeAttribute("title", TOOL_TIP_AG_AN);
			_ausgabe
					.writeAttribute(
							"style",
							"background:#fff url('img/NW_200x74.gif') no-repeat; border-bottom:1px solid #09407F;");
			_ausgabe.writeEmptyElement("img");
			_ausgabe.writeAttribute("src", "img/inovat_200x74.gif");
			_ausgabe.writeAttribute("alt", "inovat-Logo");
			_ausgabe.writeAttribute("title", TOOL_TIP_AG_AN);
			_ausgabe.writeEndElement(); // </div>
		} catch (XMLStreamException e) {
			System.out.println(XML_STREAM_ERR);
			e.printStackTrace();
		}
	}

	/**
	 * Fgt eine Kopfzeile am Anfang des Datenkatalogs ein (analog zur
	 * Darstellung auf www.inovat.de).
	 */
	private void einfuegenKopfzeileInovat() {
		try {
			// <div align="right"
			// style="background:#fff url('img/kopfzeile.png') no-repeat;
			// border-bottom:1px solid #09407F;height:124px;">
			// <img src="img/inovat_logo.png" height="124" alt="inovat-Logo">
			// </div>

			_ausgabe.writeStartElement("div");
			_ausgabe.writeAttribute("align", "right");
			_ausgabe
					.writeAttribute(
							"style",
							"background:#fff url('img/kopfzeile.png') no-repeat; border-bottom:1px solid #09407F;height:124px;");
			_ausgabe.writeAttribute("title", TOOL_TIP_AG_AN);
			_ausgabe.writeEmptyElement("img");
			_ausgabe.writeAttribute("src", "img/inovat_logo.png");
			_ausgabe.writeAttribute("height", "124");
			_ausgabe.writeAttribute("alt", "inovat-Logo");
			_ausgabe.writeAttribute("title", TOOL_TIP_AG_AN);
			_ausgabe.writeEndElement(); // </div>
		} catch (XMLStreamException e) {
			System.out.println(XML_STREAM_ERR);
			e.printStackTrace();
		}
	}

	/**
	 * Erzeuge eine Tabelle mit der Abhngigkeitsmatrix.
	 * 
	 * @param listeMatrix
	 *            Liste der Matrixelemente
	 * @param mapAnzahlRowspanFurKV
	 *            KV
	 * @param mapAnzahlRowspanFurKB
	 *            KB
	 * 
	 * @throws XMLStreamException
	 *             Excception beim einfgen
	 */
	private void erzeugeTabelleMatrixKV(List<Object[]> listeMatrix,
			Map<String, Integer> mapAnzahlRowspanFurKV,
			Map<String, Integer> mapAnzahlRowspanFurKB)
			throws XMLStreamException {
		// Titel:
		_ausgabe.writeStartElement("h2");
		_ausgabe.writeCharacters("Abhngigkeitsmatrix");
		_ausgabe.writeEmptyElement("br");
		_ausgabe.writeEndElement(); // </h2>

		// Tabelle Kopfzeile:
		String[] arKopfzeile = new String[] { "Konfigurationsverantwortlicher",
				"Konfigurationsbereich", "", "abhngig von" };

		erzeugeKopfzeile(arKopfzeile, "tabelleabhaengigkeitsmatrix");

		// Ausgabe der Matrix:
		String kvAlt = "";
		String kv;
		String kbAlt = "";
		String kb = "";
		String icon;

		for (Object[] zeile : listeMatrix) {
			kv = zeile[0].toString();
			_ausgabe.writeStartElement("tr");

			if (!kvAlt.equals(kv)) {
				kvAlt = kv;
				// Ausgabe Konfigurationsverantwortlicher
				_ausgabe.writeStartElement("td");
				_ausgabe.writeAttribute("rowspan", String
						.valueOf(mapAnzahlRowspanFurKV.get(kv)));
				_ausgabe.writeCharacters(kv);
				_ausgabe.writeEndElement(); // </td>
			}

			// Ausgabe Konfigurationsbereiche pro Konfigurationsverantwortlichen
			kb = zeile[1].toString();

			if (!kbAlt.equals(kb)) {
				kbAlt = kb;
				_ausgabe.writeStartElement("td");
				_ausgabe.writeAttribute("rowspan", String
						.valueOf(mapAnzahlRowspanFurKB.get(kb)));
				_ausgabe.writeCharacters(kb);
				_ausgabe.writeEndElement(); // </td>
			}

			// Ausgabe Konfigurationsbereiche von dennen kb abhngt:
			icon = zeile[2].toString();
			_ausgabe.writeStartElement("td");
			_ausgabe.writeEmptyElement("img");
			_ausgabe.writeAttribute("src", icon);
			// TooltipText:
			if (icon.equals(_iconAmAbhaengigIntern)) {
				_ausgabe.writeAttribute("alt",
						"abhngig von internem Konfigurationsbereich");
				_ausgabe.writeAttribute("title",
						"abhngig von internem Konfigurationsbereich");
			} else if (icon.equals(_iconAmAbhaengigExtern)) {
				_ausgabe.writeAttribute("alt",
						"abhngig von externem Konfigurationsbereich");
				_ausgabe.writeAttribute("title",
						"abhngig von externem Konfigurationsbereich");
			} else if (icon.equals(_iconAmAbhaengigUnbekannt)) {
				_ausgabe
						.writeAttribute("alt",
								"abhngig von nicht ermittelbaren Konfigurationsbereichen");
				_ausgabe
						.writeAttribute("title",
								"abhngig von nicht ermittelbaren Konfigurationsbereichen");
			} else if (icon.equals(_iconAmUnabhaengig)) {
				_ausgabe.writeAttribute("alt", "unabhngig");
				_ausgabe.writeAttribute("title", "unabhngig");
			} else if (icon.equals(_iconAmZyklischAbhaengigExtern)) {
				_ausgabe.writeAttribute("alt",
						"zyklisch abhngig von externem Konfigurationsbereich");
				_ausgabe.writeAttribute("title",
						"zyklisch abhngig von externem Konfigurationsbereich");
			} else if (icon.equals(_iconAmZyklischAbhaengigIntern)) {
				_ausgabe.writeAttribute("alt",
						"zyklisch abhngig von internem Konfigurationsbereich");
				_ausgabe.writeAttribute("title",
						"zyklisch abhngig von internem Konfigurationsbereich");
			}
			_ausgabe.writeCharacters("");
			_ausgabe.writeEndElement(); // </td>

			_ausgabe.writeStartElement("td");
			_ausgabe.writeCharacters(zeile[3].toString());
			_ausgabe.writeEndElement(); // </td>

			_ausgabe.writeEndElement(); // </tr>
		}
	}

	private void kopiereAbbildungen() {
		Map<String, String> kbDateiPfade = _refInfo.getMapDateiPfad();
		Set<File> quellOrdnerMenge = new HashSet<File>();
		for (String kbPid : kbDateiPfade.keySet()) {
			String kbDatei = kbDateiPfade.get(kbPid);
			File datei = new File(kbDatei);
			if (datei.isFile()) {
				String pfad = datei.getParent();
				File quelle = new File(pfad + File.separator + "abbildungen");
				if (quelle.exists()) {
					if (!quellOrdnerMenge.contains(quelle)) {
						quellOrdnerMenge.add(quelle);
						String abbildOrdner = _zielOrdner + File.separator
								+ "abbildungen";
						File ziel = new File(abbildOrdner);
						boolean erzeugt = true;
						if (!ziel.exists())
							erzeugt = ziel.mkdir();
						if (erzeugt) {
							File[] dateien = quelle.listFiles();
							for (File abbild : dateien) {
								String dateiName = abbild.getName();
								File zielDatei = new File(abbildOrdner
										+ File.separator + dateiName);
								kopiereDatei(abbild, zielDatei);
							}
						} else {
							String meldung = "Fehler beim Erzeugen des erforderlichen Abbildungsverzeichnisses ["
									+ ziel.getPath() + "]";
							Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
									meldung);
						}
					}
				} else {
					String meldung = "Der Quellordner ["
							+ quelle
							+ "] mit den zu kopierenden Abbildungen existiert nicht.";
					Log
							.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID,
									meldung);
				}
			} else {
				String meldung = "Angabe der Datei [" + kbDatei
						+ "] zum Konfigurationsbereich [" + kbPid
						+ "] ist nicht korrekt.";
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung);
			}
		}
	}

	/**
	 * Prft zuerst, ob der vom Anwender angegebene Zielordner bereits existiert
	 * oder erst erzeugt werden mu. Falls der Zielordner nicht erzeugt werden
	 * kann, wird eine IOException geworfen.
	 * 
	 * @return boolscher Wert gibt an, ob die der Zielordner erzeugt werden
	 *         konnte oder nicht.
	 */
	private boolean pruefeZielOrdner() {
		boolean erzeugt = false;
		File zielOrdner = new File(_zielOrdner);
		if (!zielOrdner.exists()) {
			try {
				erzeugt = zielOrdner.mkdirs();
				if (!erzeugt) {
					throw new IOException();
				}
			} catch (IOException e) {
				String meldung = "Fehler beim Erstellen des Zielordners.";
				Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, meldung, e);
			}
		} else {
			erzeugt = true;
		}
		return erzeugt;
	}

	// ------------------------------------------------------------------------
	// ENUMERATIONEN
	// ------------------------------------------------------------------------

	// Aufzhlungstypen:
	// - ob eine Attributgruppe vom konfigurierenden Typ ist
	private static enum AtgKonfigurierend {
		ja, nein, unbekannt
	};

	// - ob eine Attributgruppe vom parameterierenden Typ ist
	private static enum AtgParametrierend {
		ja, nein, unbekannt
	};

	// - um welchen Typ es sich bei dem Element handelt:
	// - att: Attribut
	// - ata: Attributauswahl
	// - atl: Attributliste
	// - atg: Attributgruppe
	// - datum: einzelnes Datum
	// - dfeld: Datenfeld
	// - dliste: Datenliste
	private static enum ElementTyp {
		att, ata, atl, atg, datum, dfeld, dliste
	}

	// ------------------------------------------------------------------------
	// INNERE KLASSEN
	// ------------------------------------------------------------------------

	// Innere Klasse zum Erzeugen eines Objekts vom Typ 'Info'
	// wird gebraucht fr den Fall, falls die Info aus der Definition eines
	// Elements geholt wird.
	class Info {
		// der eigentliche Inhalt der Info
		private InfoBeschreibung info;

		// der Konstruktor
		Info(InfoBeschreibung info) {
			this.info = info;
		}

		/**
		 * @return info
		 */
		private InfoBeschreibung getInfo() {
			return info;
		}

		/**
		 * @param info
		 *            info
		 */
		private void setInfo(InfoBeschreibung info) {
			this.info = info;
		}
	}

	// Innere Klasse zum Erzeugen eines Objekts vom Typ 'Name'
	// wird gebraucht von Tabellen zur Darstellung, Aufzhlung und Einrckung
	// von unterschiedlichen Elementen
	class Name {
		// der eigentliche Name des Elements
		private String name;
		// die Rekursionsebene entspricht dem Grad der Einrckung des Elements
		// innerhalb der Tabelle
		private int niveau;
		// der Typ des Elements aus obigem Aufzhlungstyp
		private ElementTyp typ;

		// der Konstruktor
		Name(String name, int niveau, ElementTyp typ) {
			this.name = name;
			this.niveau = niveau;
			this.typ = typ;
		}

		/**
		 * @return name
		 */
		private String getName() {
			return name;
		}

		/**
		 * @param name
		 *            name
		 */
		private void setName(String name) {
			this.name = name;
		}

		/**
		 * @return niveau
		 */
		private int getNiveau() {
			return niveau;
		}

		/**
		 * @param niveau
		 *            niveau
		 */
		private void setNiveau(int niveau) {
			this.niveau = niveau;
		}

		/**
		 * @return typ
		 */
		private ElementTyp getTyp() {
			return typ;
		}

		/**
		 * @param typ
		 *            typ
		 */
		private void setTyp(ElementTyp typ) {
			this.typ = typ;
		}
	}
}
