/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.gui.pruefung;

//~ NICHT JDK IMPORTE =========================================================

import org.junit.Test;

import de.inovat.buv.inovat.lib.gui.pruefung.PruefungDialogFelder;

import static org.junit.Assert.*;

//~ KLASSEN ===================================================================

/**
 * Die Klasse Testet die statische Methoden aus PruefungDialogFelder,
 * die fr die allgemeine Prfung der Felder im Dialog dienen.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class PruefungDialogFelderTest3 {
    @Test
    public final void testFormatiereTextAlsPositiveZahl() {
        assertEquals("Leeres String wird als 0 dargestellt", "0", PruefungDialogFelder.formatiereTextAlsPositiveZahl("", 0, 0));
        assertEquals("String 9;8 wird als 10 dargestellt " + "(eine Zahle mit 0 Nachkommazahl)", "10", PruefungDialogFelder.formatiereTextAlsPositiveZahl("9;8", 0, 0));
        assertEquals("String [ 9;8] wird als 10 dargestellt " + "(eine Zahle mit 0 Nachkommazahl)", "10", PruefungDialogFelder.formatiereTextAlsPositiveZahl(" 9;8", 0, 0));
        assertEquals("String 9;8 wird als 9,80 dargestellt" + " (eine Zahle mit 2 Nachkommazahl)", "9,80", PruefungDialogFelder.formatiereTextAlsPositiveZahl("9;8", 0, 2));
        assertEquals("String -9;8 wird als 9,80 dargestellt" + " (eine Zahle mit 2 Nachkommazahl)", "9,80", PruefungDialogFelder.formatiereTextAlsPositiveZahl("-9;8", 0, 2));
        assertEquals("String +9;8 wird als 9,80 dargestellt" + " (eine Zahle mit 2 Nachkommazahl)", "9,80", PruefungDialogFelder.formatiereTextAlsPositiveZahl("+9;8", 0, 2));
        assertEquals("String 49.8 wird als 49,800 dargestellt" + " (eine Zahle mit 3 Nachkommazahl)", "49,800", PruefungDialogFelder.formatiereTextAlsPositiveZahl("49.8", 0, 3));
        assertEquals("String 49.8 wird als 0049,800 dargestellt " + "(eine Zahle mit 3 Nachkommazahl und 4 Vorkommazahl)", "0049,800", PruefungDialogFelder.formatiereTextAlsPositiveZahl("49.8", 4, 3));
        assertEquals("String 49.8 wird als leeres String dargestellt" + " (eine Zahle mit 3 Nachkommazahl und 1 Vorkommazahl)", "", PruefungDialogFelder.formatiereTextAlsPositiveZahl("49.8", 1, 3));
    }

    @Test
    public final void testFormatiereTextAlsZahl() {
        assertEquals("Leeres String wird als 0 dargestellt", "0", PruefungDialogFelder.formatiereTextAlsZahl("", 0, 0, true));
        assertEquals("10", PruefungDialogFelder.formatiereTextAlsZahl("9,8", 0, 0, true));
        assertEquals("10", PruefungDialogFelder.formatiereTextAlsZahl("+9,8", 0, 0, true));
        assertEquals("-10", PruefungDialogFelder.formatiereTextAlsZahl("-9,8", 0, 0, true));
        assertEquals("-9,8000", PruefungDialogFelder.formatiereTextAlsZahl("-9,8", 0, 4, true));
    }

    @Test
    public final void testIsEmail() {
        assertTrue("Leeres String ist eine EMail", PruefungDialogFelder.isEmail(""));
        assertFalse("vorname-name@ ist keine EMail", PruefungDialogFelder.isEmail("vorname-name@"));
        assertFalse("vorname-name@de ist keine EMail", PruefungDialogFelder.isEmail("vorname-name@de"));
        assertTrue("vorname-name@firma.de ist eine EMail", PruefungDialogFelder.isEmail("vorname-name@firma.de"));
        assertTrue("vorname-name@firma.info.de ist eine EMail", PruefungDialogFelder.isEmail("vorname-name@firma.info.de"));
        assertTrue("vorname-name@firma-info.de ist eine EMail", PruefungDialogFelder.isEmail("vorname-name@firma-info.de"));
        assertFalse("vorname-name@firma_info.de ist keine EMail", PruefungDialogFelder.isEmail("vorname-name@firma_info.de"));
        assertTrue("vorname_name@firma-info.de ist eine EMail", PruefungDialogFelder.isEmail("vorname_name@firma-info.de"));
    }

    @Test
    public final void testIsZahl() {
        assertFalse("Leeres String ist keine Zahl", PruefungDialogFelder.isZahl("", 0));
        assertFalse("Leeres String ist keine Zahl der Gre 10", PruefungDialogFelder.isZahl("", 10));
        assertTrue("0 ist eine Zahl", PruefungDialogFelder.isZahl("0", 0));
        assertTrue("000 ist eine Zahl", PruefungDialogFelder.isZahl("000", 0));
        assertTrue("000 ist eine Zahl der Gre (Anzahl der Symbolen) 3", PruefungDialogFelder.isZahl("000", 3));
        assertTrue("12000 ist eine Zahl der Gre (Anzahl der Symbolen) 5", PruefungDialogFelder.isZahl("12000", 5));
        assertFalse("12000 ist eine Zahl aber der Gre " + "(Anzahl der Symbolen) ist 5 (nicht 1200)", PruefungDialogFelder.isZahl("12000", 12000));
        assertFalse("12f ist keine Zahl", PruefungDialogFelder.isZahl("12f", 0));
        assertFalse("-12 ist keine (positive) Zahl", PruefungDialogFelder.isZahl("-12", 0));
    }

    @Test
    public final void testKonvertiereDoubleInText() {
        assertEquals("Aus 0 (0 Nachkommazahl) bekomme [0]", "0", PruefungDialogFelder.konvertiereDoubleInText(0, 0));
        assertEquals("Aus 5f (0 Nachkommazahl) bekomme [5]", "5", PruefungDialogFelder.konvertiereDoubleInText(5f, 0));
        assertEquals("Aus 5d (2 Nachkommazahl) bekomme [5,00]", "5,00", PruefungDialogFelder.konvertiereDoubleInText(5d, 2));
        assertEquals("Aus -4.8 (1 Nachkommazahl) bekomme [-4,8]", "-4,8", PruefungDialogFelder.konvertiereDoubleInText(-4.8, 1));
        assertEquals("Aus 4.78 (1 Nachkommazahl) bekomme [4,8]", "4,8", PruefungDialogFelder.konvertiereDoubleInText(4.78, 1));
        assertEquals("Aus 444.78 (1 Nachkommazahl) bekomme [444,8]", "444,8", PruefungDialogFelder.konvertiereDoubleInText(444.78, 1));
    }

    @Test
    public final void testKonvertiereTextAlsDouble1() {

        // erfolgreicher Aufruf:
        assertEquals("Aus [0] bekomme 0", 0, PruefungDialogFelder.konvertiereTextInDouble("0"), 0);
        assertEquals("Aus [0,489] bekomme 0.489", 0.489, PruefungDialogFelder.konvertiereTextInDouble("0,489"), 0);
        assertEquals("Aus [20,489] bekomme 20.489", 20.489, PruefungDialogFelder.konvertiereTextInDouble("20,489"), 0);
        assertEquals("Aus [0.7] bekomme 0.7", 0.7, PruefungDialogFelder.konvertiereTextInDouble("0.7"), 0);
        assertEquals("Aus [-0.7] bekomme -0.7", -0.7, PruefungDialogFelder.konvertiereTextInDouble("-0.7"), 0);

        // wenn das String als Double oder Float angegeben ist:
        assertEquals("Aus [5d] bekomme 5", 5, PruefungDialogFelder.konvertiereTextInDouble("5d"), 0);
        assertEquals("Aus [5f] bekomme 5", 5, PruefungDialogFelder.konvertiereTextInDouble("5f"), 0);
    }

    @Test(expected = NumberFormatException.class)
    public final void testKonvertiereTextAlsDouble2() {
        assertEquals("Beim leeren String - NumberFormatException", 0, PruefungDialogFelder.konvertiereTextInDouble(""), 0);
    }

    @Test(expected = NumberFormatException.class)
    public final void testKonvertiereTextAlsDouble3() {
        assertEquals("Beim [7ttt] - NumberFormatException", 0, PruefungDialogFelder.konvertiereTextInDouble("7ttt"), 0);
    }

    @Test(expected = NumberFormatException.class)
    public final void testKonvertiereTextAlsDouble4() {
        assertEquals("Beim [7.8,0] - NumberFormatException", 0, PruefungDialogFelder.konvertiereTextInDouble("7.8,0"));
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 10.06.10
